@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Fumail Project
+#   Copyright Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -41,11 +41,12 @@ class Addrcheck(object):
             Addrcheck.__instance.set("Default")
         return Addrcheck.__instance
 
-    def set(self, name):
+    def set(self, name: str, maxsize: int = 64) -> None:
         """
         Sets method to be used in valid - function to validate an address
         Args:
             name (String): String with name of validator
+            maxsize (int): maximum number of characters allowed in localpart (64 according to RFC 3696)
         """
         if name == "Default":
             self._method = Default()
@@ -54,15 +55,15 @@ class Addrcheck(object):
         elif name == "NoCheck":
             self._method = NoCheck()
         elif name == "AsciiOnly":
-            self._method = AsciiOnly()
+            self._method = AsciiOnly(maxsize)
         elif name == "PrintableAsciiOnly":
-            self._method = PrintableAsciiOnly()
+            self._method = PrintableAsciiOnly(maxsize)
         else:
             logger = logging.getLogger("%s.Addrcheck" % __package__)
-            logger.warning("Mail address check \"%s\" not valid, using default..."%name)
+            logger.warning(f'Mail address check "{name}" not valid, using default...')
             self._method = Default()
 
-    def valid(self, address: str, allow_postmaster: bool = False):
+    def valid(self, address: str, allow_postmaster: bool = False) -> bool:
         """
 
         Args:
@@ -85,74 +86,89 @@ class Addrcheckint(object):
     """
     Functor interface for method called by Addrcheck
     """
-    def __init__(self):
-        self._re_at = re.compile(r"[^@]+@[^@]+$")
-    
-    def __call__(self, mailAddress):
+
+    def __init__(self, maxsize=64):
+        pass
+
+    def __call__(self, mailAddress: str) -> bool:
         raise NotImplemented
+    
+    def check_startsendsdot(self, mailAddress):
+        localpart = mailAddress.rsplit('@',1)[0]
+        return localpart.startswith('.') or localpart.endswith('.')
 
 
 class Default(Addrcheckint):
     """
     Default implementation (and backward compatible) which does not allow more than one '@'
     """
-    def __init__(self):
-        super(Default, self).__init__()
-        
-    def __call__(self,mailAddress):
-        leg = (mailAddress !='' and self._re_at.match(mailAddress))
-        return leg
+
+    def __init__(self, maxsize: int = 64):
+        super().__init__(maxsize)
+        self._re_at = re.compile(r"[^@]{1,%u}@[^@]+$" % maxsize)
+
+    def __call__(self, mailAddress: str) -> bool:
+        return mailAddress != '' \
+            and self._re_at.match(mailAddress) \
+            and not self.check_startsendsdot(mailAddress)
 
 
-class LazyLocalPart(Addrcheckint):
+class LazyLocalPart(Default):
     """
     Allows '@' in local part if quoted
     """
-    def __init__(self):
-        super(LazyLocalPart, self).__init__()
-        self._re_u00_7f = re.compile(r"^\"[\x00-\x7f]+\"@[^@]+$")
-        
-    def __call__(self,mailAddress):
-        leg = ( mailAddress !='' and  (      self._re_at.match(mailAddress)
-                                          or self._re_u00_7f.match(mailAddress) ))
-        return leg
+
+    def __init__(self, maxsize: int = 64):
+        super().__init__(maxsize)
+        self._re_u00_7f = re.compile(r"^\"[\x00-\x7f]{1,%u}\"@[^@]+$" % maxsize)
+
+    def __call__(self, mailAddress: str) -> bool:
+        return mailAddress != '' and \
+            (self._re_at.match(mailAddress) or self._re_u00_7f.match(mailAddress)) \
+            and not self.check_startsendsdot(mailAddress)
 
 
 class AsciiOnly(Addrcheckint):
     """
     Allow ascii characters only, "@" in local part has to be quoted, max 64 chars in localpart
     """
-    def __init__(self):
-        super().__init__()
-        self.ascii_noadd = re.compile(r"^[\x00-\x3f\x41-\x7f]{1,64}@[\x00-\x3f\x41-\x7f]+$", flags=re.IGNORECASE)
-        self.ascii_quoted = re.compile(r"^\"[\x00-\x7f]{0,62}\"@[\x00-\x3f\x41-\x7f]+$", flags=re.IGNORECASE)
 
-    def __call__(self, mailAddress):
+    def __init__(self, maxsize: int = 64):
+        super().__init__(maxsize)
+        self.ascii_noadd = re.compile(r"^[\x00-\x3f\x41-\x7f]{1,%u}@[\x00-\x3f\x41-\x7f]+$" % maxsize, flags=re.IGNORECASE)
+        self.ascii_quoted = re.compile(r"^\"[\x00-\x7f]{0,%u}\"@[\x00-\x3f\x41-\x7f]+$" % (maxsize-2), flags=re.IGNORECASE)
+
+    def __call__(self, mailAddress: str) -> bool:
         # there has to be an "@" in the address
-        return mailAddress and \
-               (self.ascii_noadd.match(mailAddress) or self.ascii_quoted.match(mailAddress))
+        return mailAddress != '' and \
+            (self.ascii_noadd.match(mailAddress) or self.ascii_quoted.match(mailAddress)) \
+            and not self.check_startsendsdot(mailAddress)
 
 
 class PrintableAsciiOnly(Addrcheckint):
     """
     Allow ascii printable characters only, "@" in local part has to be quoted, max 64 chars in localpart
     """
-    def __init__(self):
-        super().__init__()
-        self.ascii_noadd = re.compile(r"^[\x20-\x3f\x41-\x7e]{1,64}@[\x20-\x3e\x41-\x7e]+$", flags=re.IGNORECASE)
-        self.ascii_quoted = re.compile(r"^\"[\x20-\x7e]{0,62}\"@[\x20-\x3f\x41-\x7e]+$", flags=re.IGNORECASE)
 
-    def __call__(self, mailAddress):
+    def __init__(self, maxsize: int = 64):
+        super().__init__(maxsize)
+        self.ascii_noadd = re.compile(r"^[\x20-\x3f\x41-\x7e]{1,%u}@[\x20-\x3e\x41-\x7e]+$" % maxsize, flags=re.IGNORECASE)
+        self.ascii_quoted = re.compile(r"^\"[\x20-\x7e]{0,%u}\"@[\x20-\x3f\x41-\x7e]+$" % (maxsize-2), flags=re.IGNORECASE)
+
+    def __call__(self, mailAddress: str) -> bool:
         # there has to be an "@" in the address
-        return mailAddress and \
-               (self.ascii_noadd.match(mailAddress) or self.ascii_quoted.match(mailAddress))
+        return mailAddress != '' and \
+            (self.ascii_noadd.match(mailAddress) or self.ascii_quoted.match(mailAddress)) \
+            and not self.check_startsendsdot(mailAddress)
 
 
 class NoCheck(Addrcheckint):
     """
     Disable check
     """
-    def __init__(self):
-        super(NoCheck, self).__init__()
-    def __call__(self,mailAddress):
+
+    def __init__(self, maxsize: int = 64):
+        super(NoCheck, self).__init__(maxsize)
+
+    def __call__(self, mailAddress: str) -> bool:
         return True
