@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -60,8 +60,8 @@ from fuglu.debug import ControlServer
 
 
 class ProcManager(object):
-    def __init__(self, logQueue, numprocs = None, queuesize=100, config = None):
-        self._child_id_counter=0
+    def __init__(self, logQueue, numprocs=None, queuesize=100, config=None):
+        self._child_id_counter = 0
         self._logQueue = logQueue
         self.manager = multiprocessing.Manager()
         self.shared_state = self._init_shared_state()
@@ -132,8 +132,8 @@ class ProcManager(object):
             self.logger.exception(e)
 
     def _create_worker(self):
-        self._child_id_counter +=1
-        worker_name = "Worker-%s"%self._child_id_counter
+        self._child_id_counter += 1
+        worker_name = "Worker-%s" % self._child_id_counter
         worker = aioprocessing.AioProcess(target=fuglu_async_process_worker, name=worker_name,
                                           args=(self.tasks, self.config, self.shared_state,
                                                 self.child_to_server_messages, self._logQueue)
@@ -248,7 +248,7 @@ class ProcManager(object):
 
 class MessageListener(threading.Thread):
     def __init__(self, message_queue):
-        threading.Thread.__init__(self)
+        super().__init__()
         self.name = "Process Message Listener"
         self.message_queue = message_queue
         self.stayalive = True
@@ -261,7 +261,7 @@ class MessageListener(threading.Thread):
             if message is None:
                 break
             event_type = message['event_type']
-            if event_type == 'statsdelta': # increase statistics counters
+            if event_type == 'statsdelta':  # increase statistics counters
                 try:
                     delta = StatDelta(**message)
                     self.statskeeper.increase_counter_values(delta)
@@ -295,7 +295,7 @@ def fuglu_async_process_worker(queue, config, shared_state, child_to_server_mess
 
     logtools.client_configurer(logQueue)
     logging.basicConfig(level=logging.DEBUG)
-
+    pidinfo = logtools.createPIDinfo()
     async_coroutines = config.getint('performance', 'async_coroutines', fallback=1)
 
     async_threadpoolworkers = config.getint('performance', 'async_threadpool', fallback=None)
@@ -303,24 +303,20 @@ def fuglu_async_process_worker(queue, config, shared_state, child_to_server_mess
         async_threadpoolworkers = max(1, int(async_coroutines/2))
     async_prepare_timeout = config.getfloat('performance', 'async_prepare_timeout', fallback=None)
 
-    workerstate = WorkerStateWrapper(shared_state,'loading configuration', async_coroutines=async_coroutines)
+    workerstate = WorkerStateWrapper(shared_state, 'loading configuration', async_coroutines=async_coroutines)
     logger = logging.getLogger('fuglu.process.%s(%u)' % (workerstate.process.name, workerstate.process.pid))
-    logger.debug(f"New worker: {logtools.createPIDinfo()}, "
-                 f"coroutines={async_coroutines} with threadpool{async_threadpoolworkers}")
-    logger.debug(f"async timeouts: "
+    logger.debug(f"{pidinfo}: New worker, coroutines={async_coroutines} with threadpool{async_threadpoolworkers}")
+    logger.debug(f"{pidinfo}: async timeouts: "
                  f"coroutines={async_coroutines}, "
                  f"threadworkers={async_threadpoolworkers}, "
                  f"preparetimeout={async_prepare_timeout}")
 
     # Setup address compliance checker
     # -> Due to default linux forking behavior this should already
-    #    have the correct setup but it's better not to rely on this
-    try:
-        address_check = config.get('main', 'address_compliance_checker')
-    except Exception as e:
-        # might happen for some tests which do not propagate defaults
-        address_check = "Default"
-    Addrcheck().set(address_check)
+    #    have the correct setup, but it's better not to rely on this
+    address_check = config.get('main', 'address_compliance_checker', fallback='Default')
+    maxsize = config.getint('main', 'address_compliance_checker_maxsize', fallback=64)
+    Addrcheck().set(address_check, maxsize)
 
     try:
         logtimings = config.getboolean('main', 'scantimelogger', False)
@@ -328,7 +324,7 @@ def fuglu_async_process_worker(queue, config, shared_state, child_to_server_mess
         logtimings = False
 
     # load config and plugins
-    logger.debug("Create MainController")
+    logger.debug(f"{pidinfo}: Create MainController")
     controller = fuglu.core.MainController(config, logQueue=logQueue, nolog=True)
     controller.load_extensions()
     controller.load_plugins()
@@ -342,32 +338,32 @@ def fuglu_async_process_worker(queue, config, shared_state, child_to_server_mess
         pid = os.getpid()
         if pid:
             cport = f"{cport}.{pid}"
-            logger.info(f"Creating processor-local control server with socket: {cport}")
+            logger.info(f"{pidinfo}: Creating processor-local control server with socket: {cport}")
             control = ControlServer(controller, address=config.get('main', 'bindaddress'), port=cport)
             ctrl_server_thread = threading.Thread(name='Control server', target=control.serve, args=())
             ctrl_server_thread.daemon = True
             ctrl_server_thread.start()
             controller.controlserver = control
     else:
-        logger.debug(f"Not creating processor-local control server because socket is int: {cport}")
+        logger.debug(f"{pidinfo}: Not creating processor-local control server because socket is int: {cport}")
 
     # forward statistics counters to parent process
     stats = Statskeeper()
     stats.stat_listener_callback.append(lambda event: child_to_server_messages.put(event.as_message()))
 
-    logger.debug("%s: Enter service loop..." % logtools.createPIDinfo())
+    logger.debug(f"{pidinfo}: Enter service loop...")
 
     loop = None
     try:
         if uvloop:
             loop = uvloop.new_event_loop()
             asyncio.set_event_loop(loop)
-            logger.debug("Set uvloop as event loop!")
+            logger.debug(f"{pidinfo}: Set uvloop as event loop!")
         else:
-            logger.debug("use asyncio as event loop!")
+            logger.debug(f"{pidinfo}: use asyncio as event loop!")
             loop: tp.Optional[asyncio.AbstractEventLoop] = asyncio.get_event_loop()
             if async_coroutines > 5:
-                logger.warning("More than 5 coroutines only seems to have problems with aioredis.\n"
+                logger.warning(f"{pidinfo}: More than 5 coroutines only seems to have problems with aioredis.\n"
                                "-> reduced to 5\n"
                                "-> install module uvloop to use more coroutines per processor")
 
@@ -378,24 +374,25 @@ def fuglu_async_process_worker(queue, config, shared_state, child_to_server_mess
 
         with ThreadPoolExecutor(max_workers=async_threadpoolworkers) as pool:
             # create worker coroutines for async loop
-            asworkers = [fuglu_async_process_worker_main(queue, workerstate, logger, controller, i, logtimings, pool, async_prepare_timeout) for i in range(1, 1+async_coroutines)]
-            logger.debug(f"We put {len(asworkers)} into the loop...")
+            asworkers = [fuglu_async_process_worker_main(queue, workerstate, logger, controller, i, logtimings, pool,
+                                                         async_prepare_timeout) for i in range(1, 1+async_coroutines)]
+            logger.debug(f"{pidinfo}: We put {len(asworkers)} into the loop...")
             if sys.version_info > (3, 10):
                 # loop parameter has been removed for python-3.10
                 loop.run_until_complete(asyncio.gather(*asworkers, return_exceptions=True))
             else:
                 loop.run_until_complete(asyncio.gather(*asworkers, loop=loop))
-            logger.debug("Gathered result...")
+            logger.debug(f"{pidinfo}: Gathered result...")
     except KeyboardInterrupt:
         workerstate.workerstate = 'ended (keyboard interrupt)'
-        logger.debug("Keyboard interrupt")
+        logger.debug(f"{pidinfo}: Keyboard interrupt")
     except Exception as e:
-        logger.error("Exception in worker process: %s" % str(e), exc_info=e)
+        logger.error(f"{pidinfo}: Exception in worker process: {str(e)}", exc_info=e)
         workerstate.workerstate = 'crashed'
     finally:
         # this process will not put any object in queue
         if loop:
-            logger.debug(f"Stop async event loop: {loop.is_running()}")
+            logger.debug(f"{pidinfo}: Stop async event loop: {loop.is_running()}")
             if loop.is_running():
                 loop.stop()
                 logger.debug("-> stopped")
@@ -403,20 +400,20 @@ def fuglu_async_process_worker(queue, config, shared_state, child_to_server_mess
             if not loop.is_closed():
                 loop.close()
                 logger.debug("-> closed")
-            logger.debug("Async event loop stopped & closed")
-        logger.debug("Close multiprocessing queue")
+            logger.debug(f"{pidinfo}: Async event loop stopped & closed")
+        logger.debug(f"{pidinfo}: Close multiprocessing queue")
         queue.close()
         try:
-            logger.debug("Multiprocessing queue closed, shutdown processor-local controller")
+            logger.debug(f"{pidinfo}: Multiprocessing queue closed, shutdown processor-local controller")
         except BrokenPipeError:
             pass
         controller.shutdown()
         try:
-            logger.debug("Processor-local controller closed")
+            logger.debug(f"{pidinfo}: Processor-local controller closed")
         except BrokenPipeError:
             pass
 
-        logger.debug("%s: async proc worker finished & complete..." % logtools.createPIDinfo())
+        logger.debug(f"{pidinfo}: async proc worker finished & complete...")
 
 
 def prepare_session_task_reader(asyncid: int, workerstate, logger, task, loop):
@@ -510,7 +507,7 @@ async def fuglu_async_process_worker_main(queue, workerstate, logger, controller
 
             logger.debug(f"Prepare session (connection, timeout={async_prepare_timeout})")
             if async_prepare_timeout:
-                transport, _ = await asyncio.wait_for(future , timeout=async_prepare_timeout)
+                transport, _ = await asyncio.wait_for(future, timeout=async_prepare_timeout)
             else:
                 transport, _ = await future
 
@@ -546,7 +543,6 @@ async def fuglu_async_process_worker_main(queue, workerstate, logger, controller
                                                                       transport=transport, protocol=protocol,
                                                                       reader=reader, loop=loop, port=port)
 
-
             # handle session
             workerstate.set_workerstate(f'{msess.id} Handle scan session...', id=asyncid)
             logger.debug(f"{logtools.createPIDinfo()}({asyncid}): Handle scan session {msess.id}")
@@ -616,7 +612,7 @@ def debug_procpoolworkermemory(logger, config):
     suspectobjects = objgraph.by_type('Suspect')
     if len(suspectobjects) > 0:
         if writedebuggraphs:
-            objgraph.show_backrefs(suspectobjects[-1], max_depth=5,refcounts=True,
+            objgraph.show_backrefs(suspectobjects[-1], max_depth=5, refcounts=True,
                                    filename=os.path.join(config.get('main', 'tempdir'), 'suspects.dot'))
         logger.info("Refcounts on last subject: %u" % sys.getrefcount(suspectobjects[-1]))
     mailattachmentobjects = objgraph.by_type('Mailattachment')
@@ -637,10 +633,10 @@ def debug_procpoolworkermemory(logger, config):
     allobjects = suspectobjects + mailattachmentobjects + mailattachmentmanagerobjects
     if len(allobjects) > 0:
         logger.error('objects in memory: Suspect: %u, MailAttachments: %u, MailAttachment_mgr: %u'
-                     % (len(suspectobjects),len(mailattachmentobjects),len(mailattachmentmanagerobjects)))
+                     % (len(suspectobjects), len(mailattachmentobjects), len(mailattachmentmanagerobjects)))
     else:
         logger.debug('objects in memory: Suspect: %u, MailAttachments: %u, MailAttachment_mrt: %u'
-                     % (len(suspectobjects),len(mailattachmentobjects),len(mailattachmentmanagerobjects)))
+                     % (len(suspectobjects), len(mailattachmentobjects), len(mailattachmentmanagerobjects)))
     del suspectobjects
     del mailattachmentobjects
     del mailattachmentmanagerobjects
@@ -688,4 +684,3 @@ class WorkerStateWrapper(object):
 
     def get_workerstates(self) -> tp.List[str]:
         return self._state
-
