@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +18,6 @@ import smtplib
 import logging
 import os
 import email
-import socket
 import re
 import ssl
 import asyncio
@@ -34,12 +33,15 @@ try:
 except ImportError:
     class aioSMTP(object):
         pass
+
     class SMTPException(Exception):
         pass
     HAVE_AIOSMTP = False
 
 
-requeue_rgx = re.compile("""2.0.0 Ok: queued as (?P<requeueid>[A-Za-z0-9]{12,18}|[A-Z0-9]{10,12})""")
+requeue_rgx = re.compile(r"(?:2.0.0 Ok:|ok,) queued as\s+(?P<requeueid>[A-Za-z0-9]{12,18}|[A-Z0-9]{10,12})")
+
+
 def queueid_from_postfixreply(logline):
     queueid = None
     m = requeue_rgx.search(logline)
@@ -58,9 +60,9 @@ class FugluSMTPClient(smtplib.SMTP):
     queueid = None
     lastserveranswer = None
     lastservercode = None
-        
-    
+
     # usually no need to call this manually (may block infinitely)
+
     def getreply(self):
         code, response = smtplib.SMTP.getreply(self)
         self.lastserveranswer = response
@@ -69,18 +71,17 @@ class FugluSMTPClient(smtplib.SMTP):
         if queueid is not None:
             self.queueid = queueid
         return code, response
-    
-    
+
     def xclient(self, xclient_args=None):
         self.ehlo_or_helo_if_needed()
         if not 'xclient' in self.esmtp_features:
             raise smtplib.SMTPNotSupportedError("SMTP XCLIENT extension not supported by server.")
-        
+
         if xclient_args:
             xclient_cmds = self.esmtp_features['xclient'].split()
             if 'ADDR' in xclient_args and ':' in xclient_args['ADDR'] and not xclient_args['ADDR'].startswith('IPV6:'):
                 xclient_args['ADDR'] = 'IPV6:%s' % xclient_args['ADDR']
-            values = [f'{k.upper()}={force_uString(v)}' for k,v in xclient_args.items() if v and k.upper() in xclient_cmds]
+            values = [f'{k.upper()}={force_uString(v)}' for k, v in xclient_args.items() if v and k.upper() in xclient_cmds]
             xclient_str = ' '.join(values)
             if xclient_str:
                 return self.docmd('XCLIENT', xclient_str)
@@ -100,8 +101,7 @@ class FugluAioSMTPClient(aioSMTP):
             ctx.verify_mode = ssl.CERT_NONE
             kwargs['tls_context'] = ctx
         return await aioSMTP.starttls(self, *args, **kwargs)
-    
-    
+
     async def xclient(self, xclient_args=None):
         await self._ehlo_or_helo_if_needed()
         if not self.supports_extension('xclient'):
@@ -127,22 +127,20 @@ class Bounce(object):
         self.config = config
         self.nobounce = None
         self.event_loop = None
-    
-    
+
     def _init_nobounce(self):
         if self.nobounce is None:
             filepath = self.config.get('main', 'nobouncefile', fallback=None)
             if filepath and os.path.exists(filepath):
                 self.nobounce = FileList(filepath)
             elif filepath:
-                self.logger.warning('nobouncefile %s not found' % filepath)
-    
-    
+                self.logger.warning(f'nobouncefile {filepath} not found')
+
     def _add_required_headers(self, recipient, messagecontent):
         """add headers required for sending automated mail"""
 
         msgrep = email.message_from_bytes(force_bString(messagecontent))
-        msgrep.set_charset("utf-8") # define unicode because the messagecontent is unicode
+        msgrep.set_charset("utf-8")  # define unicode because the messagecontent is unicode
 
         if not 'to' in msgrep:
             msgrep['To'] = Header(f'<{recipient}>').encode()
@@ -160,8 +158,7 @@ class Bounce(object):
             msgrep['Message-ID'] = make_msgid()
 
         return msgrep.as_string()
-    
-    
+
     def send_template_file(self, recipient, templatefile, suspect, values):
         """Send a E-Mail Bounce Message
 
@@ -176,7 +173,7 @@ class Bounce(object):
         """
 
         if not os.path.exists(templatefile):
-            self.logger.error('Template file does not exist: %s' % templatefile)
+            self.logger.error(f'{suspect.id} Template file does not exist: {templatefile}')
             return
 
         with open(templatefile) as fp:
@@ -184,8 +181,7 @@ class Bounce(object):
 
         queueid = self.send_template_string(recipient, filecontent, suspect, values)
         return queueid
-    
-    
+
     def send_template_string(self, recipient, templatecontent, suspect, values):
         """Send a E-Mail Bounce Message
 
@@ -199,16 +195,16 @@ class Bounce(object):
             values       : Values to apply to the template
         """
         if suspect.get_tag('nobounce'):
-            self.logger.info('Not sending bounce to %s - bounces disabled by plugin' % recipient)
+            self.logger.info(f'{suspect.id} Not sending bounce to {recipient} - bounces disabled by plugin')
             return
 
         message = apply_template(templatecontent, suspect, values)
         try:
             message = self._add_required_headers(recipient, message)
         except Exception as e:
-            self.logger.warning('Bounce message template could not be verified: %s' % str(e))
+            self.logger.warning(f'{suspect.id} Bounce message template could not be verified: {e.__class__.__name__}: {str(e)}')
 
-        self.logger.debug('Sending bounce message to %s' % recipient)
+        self.logger.debug(f'{suspect.id} Sending bounce message to {recipient}')
         fromaddress = "<>"
         queueid = self.send(fromaddress, recipient, message)
         return queueid
@@ -230,8 +226,7 @@ class Bounce(object):
         if not targethost or (isinstance(targethost, str) and not targethost.strip()):
             raise ValueError("Bouncer: No targethost defined for Bouncer, please define valid outgoinghost in config!")
         return targethost
-    
-    
+
     def _send_sync(self, fromaddress, toaddress, message):
         """really send message"""
         targethost = self._get_targethost()
@@ -240,13 +235,14 @@ class Bounce(object):
         try:
             smtp_server = FugluSMTPClient(host=targethost, port=outgoingport, local_hostname=helostring)
             smtp_server.sendmail(fromaddress, toaddress, message)
-        except smtplib.SMTPException as e:
-            self.logger.error(f'failed to send mail from={fromaddress} to={toaddress} via={targethost} port={outgoingport} due to {e.__class__.__name__} {str(e)}')
+        except (smtplib.SMTPException, OSError) as e:
+            self.logger.error(
+                f'failed to send mail from={fromaddress} to={toaddress} via={targethost} port={outgoingport} helo={helostring} due to {e.__class__.__name__} {str(e)}')
             raise
         smtp_server.quit()
         return smtp_server.queueid
-    
-    
+
+    @deprecated
     def _send_async(self, fromaddress, toaddress, message):
         targethost = self._get_targethost()
         outgoingport = self.config.getint('main', 'outgoingport')
@@ -256,22 +252,22 @@ class Bounce(object):
             smtp_server.connect()
             send_resp = smtp_server.sendmail(fromaddress, toaddress, message)
         except SMTPException as e:
-            self.logger.error(f'failed to send mail from={fromaddress} to={toaddress} via={targethost} port={outgoingport} due to {e.__class__.__name__} {str(e)}')
+            self.logger.error(
+                f'failed to send mail from={fromaddress} to={toaddress} via={targethost} port={outgoingport} due to {e.__class__.__name__} {str(e)}')
             raise
         smtp_server.quit()
         return smtp_server.parse_postfixreply(send_resp[1])
-    
-    
+
     def send(self, fromaddress, toaddress, message):
-        if self.config.getboolean('main', 'disablebounces'):
-            self.logger.info('Bounces are disabled in config - not sending message to %s' % toaddress)
+        if (not fromaddress or fromaddress == '<>') and self.config.getboolean('main', 'disablebounces'):
+            self.logger.info(f'Bounces are disabled in config - not sending message to {toaddress}')
             return
-        
+
         self._init_nobounce()
         if self.nobounce and extract_domain(toaddress) in self.nobounce.get_list():
-            self.logger.info('Bounces to this rcpt are disabled - not sending message to %s' % toaddress)
+            self.logger.info(f'Bounces to this rcpt are disabled - not sending message to {toaddress}')
             return
-        
+
         disable_aiosmtp = self.config.getboolean('performance', 'disable_aiosmtp')
         if HAVE_AIOSMTP and not disable_aiosmtp:
             if self.event_loop is None:
@@ -284,14 +280,12 @@ class Bounce(object):
         else:
             queueid = self._send_sync(fromaddress, toaddress, message)
         return queueid
-    
-    
+
     @deprecated
     def _send(self, fromaddress, toaddress, message):
         """deprecated version of send()"""
         self.send(fromaddress, toaddress, message)
-    
-    
+
     def lint(self):
         from fuglu.funkyconsole import FunkyConsole
         fc = FunkyConsole()
