@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project #
+#   Copyright Oli Schacher, Fumail Project #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -57,7 +57,7 @@ class ESMTPHandler(ProtocolHandler):
     protoname = 'ESMTP (before queue)'
 
     def __init__(self, socket, config):
-        ProtocolHandler.__init__(self, socket, config)
+        super().__init__(socket, config)
         self.sess = ESMTPPassthroughSession(socket, config)
 
     def re_inject(self, suspect):
@@ -88,12 +88,12 @@ class ESMTPHandler(ProtocolHandler):
         sess = self.sess
         fromaddr = sess.from_address
         tempfilename = sess.tempfilename
-        
+
         try:
             suspect = Suspect(fromaddr, sess.recipients, tempfilename, att_cachelimit=self._att_mgr_cachesize,
                               att_defaultlimit=self._att_defaultlimit, att_maxlimit=self._att_maxlimit)
         except ValueError as e:
-            if len(sess.recipients)>0:
+            if len(sess.recipients) > 0:
                 toaddr = sess.recipients[0]
             else:
                 toaddr = ''
@@ -135,7 +135,8 @@ class ESMTPHandler(ProtocolHandler):
 class ESMTPServer(BasicTCPServer):
 
     def __init__(self, controller, port=10125, address="127.0.0.1"):
-        BasicTCPServer.__init__(self, controller, port, address, ESMTPHandler)
+        super().__init__(controller, port, address, ESMTPHandler)
+
 
 class ESMTPPassthroughSession(object):
     ST_INIT = 0
@@ -176,10 +177,10 @@ class ESMTPPassthroughSession(object):
                     cmd = data[0:4]
                     cmd = cmd.upper()
                     if cmd == b"QUIT":
-                        self.socket.send(force_bString("%s %s\r\n" % (220, "BYE")))
+                        self.socket.send(b"220 BYE\r\n")
                         self.closeconn()
                         return
-                    self.socket.send(force_bString("%s %s\r\n" % (421, "Cannot accept further commands")))
+                    self.socket.send(b"421 Cannot accept further commands\r\n")
                     self.closeconn()
                     return
             else:
@@ -194,14 +195,14 @@ class ESMTPPassthroughSession(object):
             pass
         finally:
             self.socket.close()
-        
+
     def _close_tempfile(self):
         if self.tempfile and not self.tempfile.closed:
             self.tempfile.close()
 
     def getincomingmail(self):
         """return true if mail got in, false on error Session will be kept open"""
-        self.socket.send(force_bString("220 fuglu scanner ready \r\n"))
+        self.socket.send(b"220 fuglu scanner ready \r\n")
 
         while True:
             rawdata = b''
@@ -231,7 +232,7 @@ class ESMTPPassthroughSession(object):
                                 # data finished.. keep connection open though
                                 self.logger.debug('incoming message finished')
                                 return True
-                        
+
                         self.socket.send(force_bString(rsp + "\r\n"))
                         if keep == 0:
                             self.closeconn()
@@ -243,13 +244,13 @@ class ESMTPPassthroughSession(object):
 
     def forwardCommand(self, command):
         """forward a esmtp command to outgoing postfix instance
-        
+
         Args:
             command (str): command in unicode
-        
+
         Returns (str): reply from outgoing server
         """
-        
+
         command = command.strip()
         if self.forwardconn is None:
             targethost = self.config.get('main', 'outgoinghost')
@@ -299,7 +300,7 @@ class ESMTPPassthroughSession(object):
         cmd = cmd.upper()
         keep = 1
 
-        if cmd in["EHLO", 'HELO']:
+        if cmd in ["EHLO", 'HELO']:
             self.state = ESMTPPassthroughSession.ST_HELO
         elif cmd == "RSET":
             self.from_address = None
@@ -417,13 +418,15 @@ class ESMTPPassthroughSession(object):
         else:
             self.tempfile.write(data)
             return None
-    
+
     _re_dots = re.compile(rb'(?m)^\.\.')
+
     def unquoteData(self, data):
         """two leading dots at the beginning of a line must be unquoted to a single dot"""
         return self._re_dots.sub(b'.', data)
-    
+
     _re_at = re.compile(r"^[^@]+@[^@]+\.[^@]+$")
+
     def stripAddress(self, address):
         """
         Strip the leading & trailing <> from an address.  Handy for
