@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Fumail Project
+#   Copyright Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,6 +31,7 @@ from fuglu.protocolbase import ProtocolHandler, BasicTCPServer
 from fuglu.stringencode import force_bString, force_uString
 from fuglu.shared import Suspect
 from configparser import ConfigParser
+from .asyncmilterconnector import MILTERMACROS, MILTERMACRO_TYPES
 
 try:
     import libmilter as lm
@@ -57,6 +58,7 @@ except ImportError:
     class lm:
         MilterProtocol = object
         SMFIF_ALLOPTS = None
+
         @staticmethod
         def noReply(self):
             pass
@@ -83,7 +85,7 @@ STR2RETCODE = {
 }
 
 
-RETCODE2STR = dict([(v, k) for k,v in STR2RETCODE.items()])
+RETCODE2STR = dict([(v, k) for k, v in STR2RETCODE.items()])
 
 
 # states
@@ -95,11 +97,12 @@ HEADER = "header"
 EOH = "eoh"
 EOB = "eob"
 
+
 class MilterHandler(ProtocolHandler):
     protoname = 'MILTER V6'
 
     def __init__(self, sock, config):
-        ProtocolHandler.__init__(self, sock, config)
+        super().__init__(sock, config)
 
         # Milter can keep the connection and handle several suspect in one session
         self.keep_connection = True
@@ -132,8 +135,8 @@ class MilterHandler(ProtocolHandler):
         self.sess = MilterSession(sock, config, options=sess_options)
 
         self.logger.debug("Milter mode: auto=%s, readonly=%s, tags=%s, autoheaders=%s" %
-                         (self.enable_mode_auto, self.enable_mode_readonly, self.enable_mode_tags,
-                          self.enable_mode_autoheaders))
+                          (self.enable_mode_auto, self.enable_mode_readonly, self.enable_mode_tags,
+                           self.enable_mode_autoheaders))
 
         # options (can be combined into a string): "all" "body" "headers" "from" "to"
         try:
@@ -142,9 +145,9 @@ class MilterHandler(ProtocolHandler):
             self.milter_mode_options = []
 
         self.logger.debug("Milter config fixed replacements: all=%s, body=%s, headers=%s, from=%s, to=%s" %
-                         ("all" in self.milter_mode_options, "body" in self.milter_mode_options,
-                          "headers" in self.milter_mode_options, "from" in self.milter_mode_options,
-                          "to" in self.milter_mode_options))
+                          ("all" in self.milter_mode_options, "body" in self.milter_mode_options,
+                           "headers" in self.milter_mode_options, "from" in self.milter_mode_options,
+                           "to" in self.milter_mode_options))
 
     def get_suspect(self, **kwargs):
 
@@ -155,7 +158,7 @@ class MilterHandler(ProtocolHandler):
 
         # already generate Suspect id
         id = Suspect.generate_id()
-        # set in session already for logging
+        # set in session already for logging,
         # so we can link milter logs in different states to final suspect
         self.sess.id = id
         # set milter plugins dictionary
@@ -177,26 +180,24 @@ class MilterHandler(ProtocolHandler):
 
         # If there is a filename but no file
         if temp_filename and not os.path.exists(temp_filename):
-            self.logger.warning("File '%s' not found for suspect creation! from: %s, to: %s"
-                                % (temp_filename, from_address, recipients))
+            self.logger.warning(f"{id} File '{temp_filename}' not found for suspect creation! from: {from_address}, to: {recipients}")
             return None
 
         suspect = Suspect(from_address, recipients, temp_filename, att_cachelimit=self._att_mgr_cachesize,
                           att_defaultlimit=self._att_defaultlimit, att_maxlimit=self._att_maxlimit,
-                          sasl_login=sess.sasl_login, sasl_sender=sess.sasl_sender, sasl_method=sess.sasl_method,
-                          queue_id=sess.queueid, id=id)
+                          queue_id=sess.queueid, id=id, milter_macros=sess.milter_macros)
         suspect.timestamp = sess.timestamp
-        
+
         # add headers
         for hdrname, hdrval in self.sess.addheaders.items():
-            suspect.add_header(key=hdrname, value=hdrval, immediate=False)
+            immediate = hdrname in self.sess._addheaders_immediate
+            suspect.add_header(key=hdrname, value=hdrval, immediate=immediate)
 
         # add session tags to Suspect
         if self.sess.tags:
             suspect.tags.update(self.sess.tags)
 
-        logging.getLogger('fuglu.MilterHandler.queueid').info(
-            '"%s" "%s"' % (suspect.id, sess.queueid if sess.queueid else "NOQUEUE"))
+        logging.getLogger('fuglu.MilterHandler.queueid').info(f'"{id}" "{sess.queueid if sess.queueid else "NOQUEUE"}"')
 
         if sess.heloname is not None and sess.addr is not None and sess.fcrdns is not None:
             suspect.clientinfo = force_uString(sess.heloname), force_uString(sess.addr), force_uString(sess.fcrdns)
@@ -418,9 +419,8 @@ class MilterHandler(ProtocolHandler):
                 self.add_rcpt(recipient)
 
         if (self.enable_mode_auto or self.enable_mode_autoheaders) and not replace_headers:
-            self.logger.warning(f"{suspect.id} Modify({len(suspect.added_headers)})/add({len(suspect.modified_headers)}"
-                                f" headers according to modification track in suspect")
-
+            lvl = logging.WARNING if len(suspect.modified_headers) > 0 else logging.INFO
+            self.logger.log(lvl, f"{suspect.id} Modify {len(suspect.modified_headers)} headers according to modification track in suspect")
             for key, val in iter(suspect.modified_headers.items()):
                 hdr = Header(val, header_name=key, continuation_ws=' ')
                 self.changeheader(key, hdr.encode())
@@ -438,7 +438,8 @@ class MilterHandler(ProtocolHandler):
         # --
         # headers to add, same as for the other connectors
         # --
-        self.logger.info(f"{suspect.id} Add {len(suspect.addheaders)} headers as defined in suspect")
+        lvl = logging.WARNING if len(suspect.addheaders) > 0 else logging.INFO
+        self.logger.log(lvl, f"{suspect.id} Add {len(suspect.addheaders)} headers as defined in suspect")
         for key, val in iter(suspect.addheaders.items()):
             hdr = Header(val, header_name=key, continuation_ws=' ')
             self.logger.debug("Add suspect header-> %s: %s" % (key, val))
@@ -481,7 +482,7 @@ class MilterHandler(ProtocolHandler):
 
         # Create the body of the message (a plain-text and an HTML version).
         text = "Hi!\nBad luck, your message has been replaced completely :-("
-        html = u"""\
+        html = """\
         <html>
           <head></head>
           <body>
@@ -563,6 +564,7 @@ class MilterSession(lm.MilterProtocol):
         self.addr = None
         self.fcrdns = None
         self.ptr = None
+        self.milter_macros = {}
 
         self.recipients = []
         self.sender = None
@@ -585,6 +587,7 @@ class MilterSession(lm.MilterProtocol):
         # plugins
         self.milterplugins = {}
         self.addheaders = {}
+        self._addheaders_immediate = set()
         # tags (will be passed to Suspect)
         self.tags = copy.deepcopy(Suspect._default_tags)
 
@@ -618,9 +621,11 @@ class MilterSession(lm.MilterProtocol):
             pass
         return res
 
-    def add_header(self, key: str, value: str):
+    def add_header(self, key: str, value: str, immediate: bool = False):
         """ Headers to add to mail (if allowed) """
         self.addheaders[key] = value
+        if immediate:
+            self._addheaders_immediate.add(key)
 
     def get_templ_dict(self) -> tp.Dict[str, tp.Any]:
         templdict = {}
@@ -638,7 +643,7 @@ class MilterSession(lm.MilterProtocol):
             templdict['queueid'] = force_uString(self.queueid)
         if self.sasl_login is not None:
             templdict['sasl_login'] = force_uString(self.sasl_login)
-        if self.sasl_user is not None: # same as sasl_login
+        if self.sasl_user is not None:  # same as sasl_login
             templdict['sasl_user'] = force_uString(self.sasl_user)
         if self.sasl_sender is not None:
             templdict['sasl_sender'] = force_uString(self.sasl_sender)
@@ -659,7 +664,9 @@ class MilterSession(lm.MilterProtocol):
         if self.tags:
             for tkey, tval in self.tags.items():
                 templdict[f'tag_{tkey}'] = f"{tval}"
-
+                
+        for item in self.milter_macros:
+                templdict[item] = force_uString(self.milter_macros[item])
         return templdict
 
     def reset_connection(self):
@@ -815,7 +822,7 @@ class MilterSession(lm.MilterProtocol):
         # actually setReply needs all bytes
         return super(__class__, self).setReply(force_bString(rcode), force_bString(xcode), force_bString(msg))
 
-    def handle_milter_plugin_reply(self, res: Union[bytes, Tuple[bytes, str]] ):
+    def handle_milter_plugin_reply(self, res: Union[bytes, Tuple[bytes, str]]):
         """Handle reply from plugin which might contain a message to set for the reply"""
         try:
             outres, message = res
@@ -895,6 +902,17 @@ class MilterSession(lm.MilterProtocol):
     def store_info_from_dict(self, command_dict):
         """Extract and store additional info passed by dict"""
         if command_dict:
+            for item in MILTERMACROS:
+                if not force_uString(item) in self.milter_macros:
+                    value = command_dict.get(item)
+                    if value:
+                        conv = MILTERMACRO_TYPES.get(item, force_uString)
+                        try:
+                            value = conv(value)
+                        except (TypeError, ValueError):
+                            value = None
+                        self.milter_macros[force_uString(item)] = value
+            
             if not self.queueid:
                 queueid = command_dict.get(b'i', None)
                 if queueid:
@@ -924,25 +942,25 @@ class MilterSession(lm.MilterProtocol):
 
     @staticmethod
     def dict_unicode(command_dict):
-        commanddictstring = u""
+        commanddictstring = ''
         if command_dict:
-            for key,value in iter(command_dict.items()):
-                commanddictstring += force_uString(key) + u": " + force_uString(value) + u", "
+            for key, value in iter(command_dict.items()):
+                commanddictstring += force_uString(key) + ": " + force_uString(value) + ", "
         return commanddictstring
 
     def connect(self, hostname, family, ip, port, command_dict):
         self.log('Connect from %s:%d (%s) with family: %s, dict: %s' % (ip, port,
-                                                              hostname, family, str(command_dict)))
+                                                                        hostname, family, str(command_dict)))
         self.store_info_from_dict(command_dict)
         if family not in (b'4', b'6', b'U'):
             self.logger.warning('Return temporary fail since family is: %s' % force_uString(family))
-            self.logger.warning(u'command dict is: %s' % MilterSession.dict_unicode(command_dict))
+            self.logger.warning('command dict is: %s' % MilterSession.dict_unicode(command_dict))
             return lm.TEMPFAIL
         elif family == b"U":
             # we handle unix socket, but print debug message since it's not tested well
             self.logger.debug(f'Handle unix socket connection')
-        if hostname is None or force_uString(hostname) == u'[%s]' % force_uString(ip):
-            hostname = u'unknown'
+        if hostname is None or force_uString(hostname) == '[%s]' % force_uString(ip):
+            hostname = 'unknown'
 
         self.fcrdns = hostname
         self.addr = ip
@@ -1124,6 +1142,6 @@ class MilterSession(lm.MilterProtocol):
 class MilterServer(BasicTCPServer):
 
     def __init__(self, controller, port=10125, address="127.0.0.1"):
-        BasicTCPServer.__init__(self, controller, port, address, MilterHandler)
+        super().__init__(controller, port, address, MilterHandler)
         if not LIMBMILTER_AVAILABLE:
             raise ImportError("libmilter not available, not possible to use MilterServer")
