@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,12 +25,13 @@ import os
 import socket
 import email
 
+
 class NCHandler(ProtocolHandler):
     protoname = 'NETCAT'
 
-    def __init__(self, socket, config):
-        ProtocolHandler.__init__(self, socket, config)
-        self.sess = NCSession(socket, config)
+    def __init__(self, sock, config):
+        super().__init__(sock, config)
+        self.sess = NCSession(sock, config)
 
     def get_suspect(self, **kwargs):
         success = self.sess.getincomingmail()
@@ -41,11 +42,11 @@ class NCHandler(ProtocolHandler):
         sess = self.sess
         fromaddr = "unknown@example.org"
         toaddr = ["unknown@example.org"]
-        
+
         # use envelope from/to from ncsession if available
         if sess.from_address:
             fromaddr = sess.from_address
-        if sess.recipients:    
+        if sess.recipients:
             toaddr = sess.recipients
 
         tempfilename = sess.tempfilename
@@ -53,13 +54,13 @@ class NCHandler(ProtocolHandler):
         # Select a Suspect class
         SuspectClass = HealthCheckSuspect if sess.healthcheck else Suspect
         suspect = SuspectClass(fromaddr, toaddr, tempfilename, att_cachelimit=self._att_mgr_cachesize,
-                          att_defaultlimit=self._att_defaultlimit, att_maxlimit=self._att_maxlimit)
+                               att_defaultlimit=self._att_defaultlimit, att_maxlimit=self._att_maxlimit)
 
         if sess.tags:
             # update suspect tags with the ones receivec
             # during ncsession
             suspect.tags.update(sess.tags)
-            
+
         return suspect
 
     def commitback(self, suspect):
@@ -78,9 +79,10 @@ class NCHandler(ProtocolHandler):
     def healthcheck_reply(self):
         self.sess.endsession("DUNNO: healthcheck")
 
+
 class NCServer(BasicTCPServer):
     def __init__(self, controller, port=10125, address="127.0.0.1"):
-        BasicTCPServer.__init__(self, controller, port, address, NCHandler)
+        super().__init__(controller, port, address, NCHandler)
 
 
 class NCSession(object):
@@ -137,11 +139,11 @@ class NCSession(object):
                 break
             else:
                 collect_lumps.append(data)
-                
+
         data = b"".join(collect_lumps)
-        
+
         data = self.parse_remove_env_data(data)
-        
+
         self.tempfile.write(data)
         self.tempfile.close()
         if not data and not self.healthcheck:
@@ -150,7 +152,7 @@ class NCSession(object):
         else:
             self.logger.debug('Incoming message received')
         return True
-    
+
     def parse_remove_env_data(self, data):
         """
         Check if there is envelop data prepend to the message. If yes, parse it and store sender, receivers, ...
@@ -164,10 +166,10 @@ class NCSession(object):
         """
         start_tag_deprecated = b"<ENV_DATA_PREPEND>"
         end_tag = b"</ENV_DATA_PREPEND>"
-        
+
         start_tag_header = b"X-DATA-PREPEND-START"
         end_tag_header = b"X-DATA-PREPEND-END"
-        
+
         if start_tag_deprecated == data[:len(start_tag_deprecated)]:
             self.logger.error('Deprecated env start tag found, please update fuglu\n'
                               'Header first 200 chars:\n'
@@ -192,7 +194,7 @@ class NCSession(object):
         else:
             self.logger.debug('No prepend envelope data found')
         return data
-    
+
     def parse_env_data_header(self, env_buffer):
         """
         Parse envelope data string and store data internally
