@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -70,7 +70,7 @@ def smtp_strip_address(address: str) -> tp.Tuple[tp.Optional[str], tp.Optional[s
     retaddr = address[start:end]
     retaddr = retaddr.strip()
 
-    remaining = u""
+    remaining = ''
     if end + 1 < len(address):
         remaining = address[end + 1:]
         remaining = remaining.strip()
@@ -103,9 +103,9 @@ def buildmsgsource(suspect):
 class SMTPHandler(ProtocolHandler):
     protoname = 'SMTP (after queue)'
 
-    def __init__(self, socket, config):
-        ProtocolHandler.__init__(self, socket, config)
-        self.sess = SMTPSession(socket, config)
+    def __init__(self, sock, config):
+        super().__init__(sock, config)
+        self.sess = SMTPSession(sock, config)
 
     def re_inject(self, suspect):
         """Send message back to postfix"""
@@ -118,7 +118,6 @@ class SMTPHandler(ProtocolHandler):
         else:
             msgcontent = buildmsgsource(suspect)
 
-
         targethost: str = self.config.get('main', 'outgoinghost')
         if targethost == '${injecthost}':
             targethost = self.socket.getpeername()[0]
@@ -145,20 +144,18 @@ class SMTPHandler(ProtocolHandler):
                 try:
                     force_uString(suspect.from_address).encode(encoding="ascii", errors="strict")
                 except UnicodeEncodeError:
-                    self.logger.warning("%s reinject: trying to send from %s without SMTPUTF8 option enabled"
-                                        % (suspect.id, suspect.from_address))
+                    self.logger.warning(f'{suspect.id} reinject: trying to send from {suspect.from_address} without SMTPUTF8 option enabled')
                     add_option = True
             for rcpt in suspect.to_address:
                 if rcpt:
                     try:
                         force_uString(rcpt).encode(encoding="ascii", errors="strict")
                     except UnicodeEncodeError:
-                        self.logger.warning("%s reinject: trying to send to %s without SMTPUTF8 option enabled"
-                                            % (suspect.id, rcpt))
+                        self.logger.warning(f'{suspect.id} reinject: trying to send to {rcpt} without SMTPUTF8 option enabled')
                         add_option = True
             if add_option:
                 mail_options.append("SMTPUTF8")
-                self.logger.warning("%s reinject: enable SMTPUTF8 option!" % suspect.id)
+                self.logger.warning(f'{suspect.id} reinject: enable SMTPUTF8 option!')
 
         serveranswer = None
         responsecode = None
@@ -168,7 +165,7 @@ class SMTPHandler(ProtocolHandler):
                 client.ehlo(helostring)
             else:
                 client.helo(helostring)
-            
+
             # for sending, make sure the string to sent is byte string
             client.sendmail(force_uString(suspect.from_address),
                             force_uString(suspect.recipients),
@@ -191,13 +188,13 @@ class SMTPHandler(ProtocolHandler):
             try:
                 client.quit()
             except Exception as e:
-                self.logger.warning('Exception while quitting re-inject session: %s' % str(e))
+                self.logger.warning(f'{suspect.id} Exception while quitting re-inject session to {targethost}: {e.__class__.__name__}: {str(e)}')
 
         if responsecode is None:
-            self.logger.warning('Re-inject: could not get server response code.')
+            self.logger.warning(f'{suspect.id} Re-inject: could not get server response code from {targethost}')
             responsecode = 451
         if serveranswer is None:
-            self.logger.warning('Re-inject: could not get server answer.')
+            self.logger.warning(f'{suspect.id} Re-inject: could not get server answer from {targethost}')
             serveranswer = ''
 
         # make sure serveranswer is unicode (could be bytes, error, ...)
@@ -227,7 +224,7 @@ class SMTPHandler(ProtocolHandler):
                 toaddr = sess.recipients[0]
             else:
                 toaddr = ''
-            self.logger.error('failed to initialise suspect with from=<%s> to=<%s> : %s' % (fromaddr, toaddr, str(e)))
+            self.logger.error(f'failed to initialise suspect with from=<{fromaddr}> to=<{toaddr}> : {str(e)}')
             raise
         return suspect
 
@@ -262,7 +259,7 @@ class SMTPHandler(ProtocolHandler):
 class SMTPServer(BasicTCPServer):
 
     def __init__(self, controller, port=10125, address="127.0.0.1"):
-        BasicTCPServer.__init__(self, controller, port, address, SMTPHandler)
+        super().__init__(controller, port, address, SMTPHandler)
 
 
 class SMTPSession(object):
@@ -272,8 +269,7 @@ class SMTPSession(object):
     ST_RCPT = 3
     ST_DATA = 4
     ST_QUIT = 5
-    
-    
+
     def __init__(self, socket, config):
         self.config = config
         self.from_address = None
@@ -292,9 +288,8 @@ class SMTPSession(object):
         # size optionally sent in MAIL FROM command
         self.mfsize = None
 
-    
     def endsession(self, code, message):
-        self.socket.send(force_bString("%s %s\r\n" % (code, message)))
+        self.socket.send(force_bString(f'{code} {message}\r\n'))
 
         rawdata = b''
         while True:
@@ -303,22 +298,21 @@ class SMTPSession(object):
             if len(lump):
 
                 rawdata += lump
-                if (len(rawdata) >= 2) and rawdata[-2:] == force_bString('\r\n'):
+                if (len(rawdata) >= 2) and rawdata[-2:] == b'\r\n':
                     cmd = rawdata[0:4]
                     cmd = cmd.upper()
-                    if cmd == force_bString("QUIT"):
-                        self.socket.send(force_bString("%s %s\r\n" % (220, "BYE")))
+                    if cmd == b"QUIT":
+                        self.socket.send(b"220 BYE\r\n")
                         self.closeconn()
                         return
 
-                    self.socket.send(force_bString("%s %s\r\n" % (421, "Cannot accept further commands")))
+                    self.socket.send(b"421 Cannot accept further commandsr\r\n")
                     self.closeconn()
                     return
             else:
                 self.closeconn()
                 return
-    
-    
+
     def closeconn(self):
         try:
             self.socket.shutdown(socket.SHUT_RDWR)
@@ -326,16 +320,14 @@ class SMTPSession(object):
             pass
         finally:
             self.socket.close()
-    
-    
+
     def _close_tempfile(self):
         if self.tempfile and not self.tempfile.closed:
             self.tempfile.close()
-    
 
     def getincomingmail(self):
         """return true if mail got in, false on error Session will be kept open"""
-        self.socket.send(force_bString("220 fuglu scanner ready \r\n"))
+        self.socket.send(b"220 fuglu scanner ready \r\n")
 
         while True:
             completeLine = 0
@@ -378,19 +370,17 @@ class SMTPSession(object):
 
                         else:
                             try:
-                                #directly use raw bytes-string data
+                                # directly use raw bytes-string data
                                 rsp = self.doData(rawdata)
                             except IOError:
-
-                                self.endsession(
-                                    421, "Could not write to temp file")
+                                self.endsession(421, "Could not write to temp file")
                                 self._close_tempfile()
                                 return False
 
                             if rsp is None:
                                 continue
                             else:
-                                # data finished.. keep connection open though
+                                # data finished. keep connection open though
                                 return True
 
                         self.socket.send(force_bString(rsp + "\r\n"))
@@ -402,8 +392,7 @@ class SMTPSession(object):
                     # EOF
                     self.logger.warning("EOF, something went wrong!")
                     return False
-    
-    
+
     def doCommand(self, data):
         """Process a single SMTP Command"""
         cmd = data[0:4]
@@ -465,8 +454,7 @@ class SMTPSession(object):
             return "505 Bad SMTP command", 1
 
         return rv, keep
-    
-    
+
     def doData(self, data):
         """Store data in temporary file
 
@@ -484,7 +472,7 @@ class SMTPSession(object):
         # unquote data here after storing in dataAccum (which is used to detect end of message)
         data = self.unquoteData(data)
 
-        if len(self.dataAccum) > 4 and self.dataAccum[-5:] == force_bString('\r\n.\r\n'):
+        if len(self.dataAccum) > 4 and self.dataAccum[-5:] == b'\r\n.\r\n':
             # check if there is more data to write to the file
             if len(data) > 4:
                 self.tempfile.write(data[0:-5])
@@ -496,14 +484,13 @@ class SMTPSession(object):
         else:
             self.tempfile.write(data)
             return None
-    
-    
+
     _re_dots = re.compile(rb'(?m)^\.\.')
+
     def unquoteData(self, data):
         """two leading dots at the beginning of a line must be unquoted to a single dot"""
         return self._re_dots.sub(b'.', force_bString(data))
-    
-    
+
     def stripAddress(self, address):
         """
         Strip the leading & trailing <> from an address.  Handy for
