@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -56,6 +56,8 @@ EXIT_FATAL = 66
 
 
 _re_replycode = re.compile(r'^127\.0\.0\.(?P<replycode>\d{1,4})$')
+
+
 def check_version_status(lint=False):
     import pkg_resources
     """Check our version string in DNS for known issues and warn about them
@@ -94,7 +96,7 @@ def check_version_status(lint=False):
 
     if version_obj is None:
         logging.getLogger("fuglu.check_version_status")\
-            .warning("Version string %s could not be parsed to pkg_resources version object", FUGLU_VERSION)
+            .warning(f"Version string {FUGLU_VERSION} could not be parsed to pkg_resources version object")
         return
 
     (major, minor, micro) = version_obj.release
@@ -103,7 +105,7 @@ def check_version_status(lint=False):
     if version_obj.dev:
         stage_str = version_obj.dev[0]
         stage_id = version_obj.dev[1]
-    
+
     if version_obj.pre:
         stage_str = version_obj.pre[0]
         stage_id = version_obj.pre[1]
@@ -133,9 +135,9 @@ def check_version_status(lint=False):
 
     ret = _re_replycode.match(result)
     if ret is not None:
-        code = int(ret.groupdict()['replycode'])
+        rcode = int(ret.groupdict()['replycode'])
         for bitmask, message in bitmaskmap.items():
-            if code & bitmask == bitmask:
+            if rcode & bitmask == bitmask:
                 logging.warning(message)
                 if lint:
                     fc = FunkyConsole()
@@ -148,7 +150,7 @@ def sigterm(signum, frame):
 
 
 class MainController(DefConfigMixin):
-    """main class to startup and control the app"""
+    """main class to start up and control the app"""
 
     def __init__(self, config, logQueue=None, logProcessFacQueue=None, nolog=False):
         """
@@ -166,7 +168,7 @@ class MainController(DefConfigMixin):
         """
 
         # initialise mixin
-        DefConfigMixin.__init__(self, config)
+        super().__init__(config)
 
         self.requiredvars = {
             # main section
@@ -181,11 +183,11 @@ class MainController(DefConfigMixin):
                 'description': "run as a daemon? (fork)",
                 'default': "1",
             },
-            
-            'scantimelogger':{
-                'section':'main',
-                'description':"Enable session scantime logger",
-                'default':"0",
+
+            'scantimelogger': {
+                'section': 'main',
+                'description': "Enable session scantime logger",
+                'default': "0",
             },
 
             'user': {
@@ -283,7 +285,7 @@ class MainController(DefConfigMixin):
                 'description': "if this is set to True/1/yes , no Bounces will be sent from Fuglu eg. after a blocked attachment has been detected\nThis may be used for debugging/testing to make sure fuglu can not produce backscatter",
                 'default': "0",
             },
-            
+
             'nobouncefile': {
                 'section': 'main',
                 'description': 'list of domains to which no bounces will be sent',
@@ -307,7 +309,7 @@ class MainController(DefConfigMixin):
                 'description': "write suspect ID to every mail",
                 'default': "1",
             },
-            
+
             'suspectidreturn': {
                 'section': 'main',
                 'description': "append suspect ID to return message (e.g. reject message)",
@@ -343,6 +345,11 @@ class MainController(DefConfigMixin):
                 'description': "Method to check mail address validity (\"Default\",\"LazyLocalPart\")",
                 'default': "Default",
             },
+            'address_compliance_checker_maxsize': {
+                'section': 'main',
+                'description': "localpart max size",
+                'default': "64",
+            },
             'address_compliance_fail_action': {
                 'section': 'main',
                 'description': "Action to perform if address validity check fails (\"defer\",\"reject\",\"discard\")",
@@ -359,6 +366,32 @@ class MainController(DefConfigMixin):
                 'default': '1',
             },
 
+            # milter section
+            'milter_mode': {
+                'section': 'milter',
+                'description': "available options: readonly,manual,auto,tags,replace_demo,autoheaders",
+                'default': 'readonly',
+            },
+
+            'milter_mode_options': {
+                'section': 'milter',
+                'description': "available options: all,body,headers,from,to,prepend",
+                'default': '',
+            },
+
+            'ignoreclient': {
+                'section': 'milter',
+                'description': "do not write clientinfo",
+                'default': '0',
+            },
+
+            'milter_debug': {
+                'section': 'milter',
+                'description': "extra verbose debug output",
+                'default': '0',
+            },
+
+
             # performance section
             'minthreads': {
                 'default': "2",
@@ -411,7 +444,7 @@ class MainController(DefConfigMixin):
                 'section': 'performance',
                 'description': "Upper limit for maximum number of files to be extracted from archives (default: 500)"
             },
-            
+
             'disable_aiosmtp': {
                 'default': "False",
                 'section': 'performance',
@@ -463,7 +496,7 @@ class MainController(DefConfigMixin):
                 'section': 'databaseconfig',
                 'description': """read runtime configuration values from database types specified in this order. currently supported db types: sql, rest, yaml""",
             },
-            
+
             'value_cache_ttl': {
                 'default': "30",
                 'section': 'databaseconfig',
@@ -476,13 +509,13 @@ class MainController(DefConfigMixin):
                 'description': """read runtime configuration values from SQL database. requires sqlalchemy to be installed""",
                 'confidential': True,
             },
-            
+
             'sql': {
                 'default': """SELECT value FROM fugluconfig WHERE `section`=:section AND `scope` IN (:globalscope,CONCAT('%',:to_domain),:to_address) AND `option`=:option ORDER BY `scope` DESC LIMIT 1""",
                 'section': 'databaseconfig',
                 'description': """sql query that returns a configuration value override. sql placeholders are ':section',':option' in addition the usual suspect filter default values like ':to_domain', ':to_address' etc\nif the statement returns more than one row/value only the first value in the first row is used""",
             },
-            
+
             'sqlsection': {
                 'default': """SELECT value, option, scope FROM fugluconfig WHERE `section`=:section AND `scope` IN (:globalscope,CONCAT('%',:to_domain),:to_address) ORDER BY `scope` DESC""",
                 'section': 'databaseconfig',
@@ -494,7 +527,7 @@ class MainController(DefConfigMixin):
                 'section': 'databaseconfig',
                 'description': """name of global scope""",
             },
-    
+
             'option_field': {
                 'default': "option",
                 'section': 'databaseconfig',
@@ -517,7 +550,7 @@ class MainController(DefConfigMixin):
                 'description': """REST API base URI""",
                 'confidential': True,
             },
-            
+
             'restapi_endpoint': {
                 'default': "",
                 'section': 'databaseconfig',
@@ -531,25 +564,25 @@ class MainController(DefConfigMixin):
                 'description': """list of additional REST API headers, e.g. for authentication""",
                 'confidential': True,
             },
-            
+
             'restapi_verify': {
                 'default': "True",
                 'section': 'databaseconfig',
                 'description': """verify REST API server's SSL certificate""",
             },
-            
+
             'restapi_timeout': {
                 'default': "10",
                 'section': 'databaseconfig',
                 'description': """REST API http connection timeout""",
             },
-            
+
             'restapi_cachettl': {
                 'default': "300",
                 'section': 'databaseconfig',
                 'description': """REST API caching timeout. Set to 0 to disable caching (not recommended)""",
             },
-            
+
             # environment
             'boundarydistance': {
                 'default': "0",
@@ -566,6 +599,16 @@ class MainController(DefConfigMixin):
                 'section': 'environment',
                 'description': """Optional regex that should be applied to received headers to skip trusted (local) mta transfers (for example LMTP).\nOnly required if plugins need to have information about the last untrusted host and the message doesn't pass a fixed amount of hops to reach this system in your network""",
             },
+            'skiponerror': {
+                'default': '0',
+                'section': 'environment',
+                'description': """Skip broken/unparseable received headers""",
+            },
+            'skipsamedomain': {
+                'default': '0',
+                'section': 'environment',
+                'description': """Skip received headers if helo/ptr domain is same domain as host in 'by' section""",
+            },
 
             #  plugin alias
             'debug': {
@@ -663,6 +706,11 @@ class MainController(DefConfigMixin):
                 'section': 'PluginAlias',
             },
 
+            'arcsign': {
+                'default': "fuglu.plugins.domainauth.ARCSignPlugin",
+                'section': 'PluginAlias',
+            },
+
             'arcverify': {
                 'default': "fuglu.plugins.domainauth.ARCVerifyPlugin",
                 'section': 'PluginAlias',
@@ -814,7 +862,7 @@ class MainController(DefConfigMixin):
             },
 
             'sssp': {
-                'default': "fuglu.plugins.sssp.SSSPPlugin",
+                'default': "fuglu.plugins.antivirus.SSSPPlugin",
                 'section': 'PluginAlias',
             },
 
@@ -822,7 +870,7 @@ class MainController(DefConfigMixin):
                 'default': "fuglu.plugins.antivirus.FprotPlugin",
                 'section': 'PluginAlias',
             },
-            
+
             'cyren': {
                 'default': "fuglu.plugins.antivirus.CyrenPlugin",
                 'section': 'PluginAlias',
@@ -930,9 +978,9 @@ class MainController(DefConfigMixin):
 
     def _logger(self):
         myclass = self.__class__.__name__
-        loggername = "fuglu.%s" % (myclass,)
+        loggername = f"fuglu.{myclass}"
         return logging.getLogger(loggername)
-    
+
     @staticmethod
     def get_connectorinfo(portspec, default_protocol, default_bindaddress):
         protocol = default_protocol
@@ -949,7 +997,7 @@ class MainController(DefConfigMixin):
         elif portformat == 2:
             protocol, bindaddress, port = port.split(':')
         else:
-            raise ValueError("Error in bind definition: %s"%portspec)
+            raise ValueError(f"Error in bind definition: {portspec}")
         return protocol, bindaddress, port
 
     def start_connector(self, portspec):
@@ -957,7 +1005,7 @@ class MainController(DefConfigMixin):
         bindaddress = self.config.get('main', 'bindaddress')
         protocol, bindaddress, port = MainController.get_connectorinfo(portspec, protocol, bindaddress)
 
-        self.logger.info("starting connector %s/%s/%s" % (protocol, bindaddress, port))
+        self.logger.info(f"starting connector {protocol}/{bindaddress}/{port}")
         try:
             port = int(port)
             if protocol == 'smtp':
@@ -993,9 +1041,9 @@ class MainController(DefConfigMixin):
                 tr.start()
                 self.servers.append(ncserver)
             else:
-                self.logger.error('Unknown Interface Protocol: %s, ignoring server on port %s' % (protocol, port))
+                self.logger.error(f'Unknown Interface Protocol: {protocol}, ignoring server on port {port}')
         except Exception as e:
-            self.logger.error("could not start connector %s/%s : %s" % (protocol, port, str(e)))
+            self.logger.error(f"could not start connector {protocol}/{port} : {e.__class__.__name__}: {str(e)}")
 
     def _start_stats_thread(self):
         self.logger.info("Init Stat Engine")
@@ -1028,8 +1076,8 @@ class MainController(DefConfigMixin):
             numprocs = max(int(multiprocessing.cpu_count() * abs(numprocs)), 1)
             self.logger.info(f"Set number of worker processors for procpool automatically ({abs(numprocs_in)}*#procs) to {numprocs}")
 
-        self.logger.info("Init process pool with %s worker processes"%(numprocs))
-        pool = fuglu.procpool.ProcManager(self._logQueue, numprocs=numprocs, config = self.config)
+        self.logger.info(f"Init process pool with {numprocs} worker processes")
+        pool = fuglu.procpool.ProcManager(self._logQueue, numprocs=numprocs, config=self.config)
         return pool
 
     def _start_async_processpool(self):
@@ -1158,7 +1206,7 @@ class MainController(DefConfigMixin):
         serversocket.bind((address, port))
         serversocket.listen(1)
         clientsocket, _ = serversocket.accept()  # client socket
-        self.logger.info("Interactive python connection from %s/%s" % (address, address))
+        self.logger.info(f"Interactive python connection from {address}/{port}")
 
         class sw(object):  # socket wrapper
             def __init__(self, s):
@@ -1172,7 +1220,7 @@ class MainController(DefConfigMixin):
 
             def readline(self):
                 return self.read(256)
-        
+
         sw = sw(clientsocket)
         sys.stdin = sw
         sys.stdout = sw
@@ -1183,18 +1231,18 @@ class MainController(DefConfigMixin):
             terp.interact("Fuglu Python Shell - MainController available as 'mc'")
         except Exception:
             pass
-        self.logger.info("done talking to %s - closing interactive shell on %s/%s" % (address, address, port))
+        self.logger.info(f"done talking to {address} - closing interactive shell on {address}/{port}")
         sys.stdin = old_stdin
         sys.stdout = old_stdout
         sys.stderr = old_stderr
         try:
             clientsocket.close()
         except Exception as e:
-            self.logger.warning("Failed to close shell client socket: %s" % str(e))
+            self.logger.warning(f'Failed to close shell client socket: {e.__class__.__name__}: {str(e)}')
         try:
             serversocket.close()
         except Exception as e:
-            self.logger.warning("Failed to close shell server socket: %s" % str(e))
+            self.logger.warning(f'Failed to close shell server socket: {e.__class__.__name__}: {str(e)}')
 
     def reload(self):
         """apply config changes"""
@@ -1266,19 +1314,19 @@ class MainController(DefConfigMixin):
                     break
 
             if not alreadyRunning:
-                self.logger.info('start new connector at %s' % str(portspec))
+                self.logger.info(f'start new connector at {str(portspec)}')
                 self.start_connector(portspec)
             else:
-                self.logger.info('keep connector at %s' % str(portspec))
+                self.logger.info(f'keep connector at {str(portspec)}')
 
         servercopy = self.servers[:]
-        for serv in servercopy:
-            if serv.port not in portlist:
-                self.logger.info('Closing server socket on port %s' % serv.port)
-                serv.shutdown()
-                self.servers.remove(serv)
+        for server in servercopy:
+            if server.port not in portlist:
+                self.logger.info(f'Closing server socket on port {server.port}')
+                server.shutdown()
+                self.servers.remove(server)
             else:
-                self.logger.info('Keep server socket on port %s' % serv.port)
+                self.logger.info(f'Keep server socket on port {server.port}')
 
         self.logger.info('Config changes applied')
 
@@ -1286,7 +1334,7 @@ class MainController(DefConfigMixin):
         if self.statsthread:
             self.statsthread.stayalive = False
         for server in self.servers:
-            self.logger.info('Closing server socket on port %s' % server.port)
+            self.logger.info(f'Closing server socket on port {server.port}')
             server.shutdown()
 
         if self.controlserver is not None:
@@ -1311,23 +1359,23 @@ class MainController(DefConfigMixin):
 
         self.stayalive = False
         self.logger.info('Shutdown complete')
-        self.logger.info('Remaining threads: %s' % threading.enumerate())
+        self.logger.info(f'Remaining threads: {threading.enumerate()}')
 
     def _lint_dependencies(self, fc):
         print(fc.strcolor('Checking dependencies...', 'magenta'))
         try:
             import sqlalchemy
-            print(fc.strcolor('sqlalchemy: Version %s installed' % sqlalchemy.__version__, 'green'))
+            print(fc.strcolor(f'sqlalchemy: Version {sqlalchemy.__version__} installed', 'green'))
         except ImportError:
             print(fc.strcolor('sqlalchemy: not installed', 'yellow') +
                   " Optional dependency, required if you want to enable any database lookups")
 
         if HAVE_BEAUTIFULSOUP:
             import bs4 as BeautifulSoup
-            print(fc.strcolor('BeautifulSoup: Version %s installed' % BeautifulSoup.__version__, 'green'))
+            print(fc.strcolor(f'BeautifulSoup: Version {BeautifulSoup.__version__} installed', 'green'))
             try:
                 from lxml import etree
-                if etree.LXML_VERSION <= (2,2):
+                if etree.LXML_VERSION <= (2, 2):
                     print(fc.strcolor('WARNING: your lxml version is prone to segfaults. An update is recommended.', 'red'))
             except ImportError:
                 print(fc.strcolor('ERROR: missing lxml as required dependency for BeautifulSoup', 'yellow'))
@@ -1339,14 +1387,14 @@ class MainController(DefConfigMixin):
             import magic
 
             if hasattr(magic, 'open'):
-                magic_vers = "python-file/libmagic bindings (http://www.darwinsys.com/file/)"
-                print(fc.strcolor('magic: found %s' % magic_vers, 'green'))
+                magic_vers = "python-file/libmagic bindings (https://www.darwinsys.com/file/)"
+                print(fc.strcolor(f'magic: found {magic_vers}', 'green'))
             elif hasattr(magic, 'from_buffer'):
                 magic_vers = "python-magic (https://github.com/ahupp/python-magic)"
-                print(fc.strcolor('magic: found %s' % magic_vers, 'green'))
+                print(fc.strcolor(f'magic: found {magic_vers}', 'green'))
             else:
                 print(fc.strcolor('magic: unsupported version', 'yellow') +
-                      " File type detection requires either the python bindings from http://www.darwinsys.com/file/ or python magic from https://github.com/ahupp/python-magic")
+                      " File type detection requires either the python bindings from https://www.darwinsys.com/file/ or python magic from https://github.com/ahupp/python-magic")
         except ImportError:
             print(fc.strcolor('magic: not installed', 'yellow') +
                   " Optional dependency, without python-file or python-magic the attachment plugin's automatic file type detection will easily be fooled")
@@ -1354,7 +1402,7 @@ class MainController(DefConfigMixin):
         try:
             import libmilter
             libmilter_vers = libmilter.__version__
-            print(fc.strcolor('libmilter: found %s' % libmilter_vers, 'green'))
+            print(fc.strcolor(f'libmilter: found {libmilter_vers}', 'green'))
         except ImportError:
             print(fc.strcolor('libmilter: not installed', 'yellow') +
                   " Optional dependency, only needed if fuglu runs as milter")
@@ -1373,16 +1421,16 @@ class MainController(DefConfigMixin):
                 penabled = fc.strcolor('enabled', 'green')
             else:
                 penabled = fc.strcolor('disabled', 'red')
-            print("%s: %s (%s)" % (pname, penabled, status))
+            print(f"{pname}: {penabled} ({status})")
 
         print(fc.strcolor('Loading plugins...', 'magenta'))
         if not self.load_plugins():
             print(fc.strcolor('At least one plugin failed to load', 'red'))
-            errors +=1
+            errors += 1
         print(fc.strcolor('Plugin loading complete', 'magenta'))
 
         print("Linting ", fc.strcolor("main configuration", 'cyan'))
-        if not self.checkConfig():
+        if not self.check_config():
             print(fc.strcolor("ERROR", "red"))
             errors += 1
         else:
@@ -1390,7 +1438,7 @@ class MainController(DefConfigMixin):
 
         trashdir = self.config.get('main', 'trashdir').strip()
         if trashdir != "" and not os.path.isdir(trashdir):
-            print(fc.strcolor("Trashdir %s does not exist" % trashdir, 'red'))
+            print(fc.strcolor(f"Trashdir {trashdir} does not exist", 'red'))
             errors += 1
 
         # sql config override
@@ -1407,14 +1455,13 @@ class MainController(DefConfigMixin):
                     att_defaultlimit=self.config.getint('performance', 'att_mgr_default_maxextract'),
                     att_maxlimit=self.config.getint('performance', 'att_mgr_hard_maxextract')
                 )
-                sqlvars = dict(
-                    section='testsection', option='testoption', scope='$GLOBAL')
+                sqlvars = dict(section='testsection', option='testoption', scope='$GLOBAL')
                 default_template_values(tempsuspect, sqlvars)
                 sess.execute(self.config.get('databaseconfig', 'sql'), sqlvars)
                 sess.remove()
                 print(fc.strcolor("OK", 'green'))
             except Exception as e:
-                print(fc.strcolor("Failed %s" % str(e), 'red'))
+                print(fc.strcolor(f"SQL connection failed {e.__class__.__name__}: {str(e)}", 'red'))
                 errors += 1
 
         allplugins = self.plugins + self.prependers + self.appenders
@@ -1428,7 +1475,7 @@ class MainController(DefConfigMixin):
                 result = plugin.lint()
             except Exception as e:
                 CrashStore.store_exception()
-                print("ERROR: %s" % e)
+                print(f"ERROR: {e.__class__.__name__}: {str(e)}")
                 result = False
 
             if result:
@@ -1437,7 +1484,7 @@ class MainController(DefConfigMixin):
                 perrors += 1
                 errors += 1
                 print(fc.strcolor("ERROR", "red"))
-        print("%s plugins reported errors." % perrors)
+        print(f"{perrors} plugins reported errors.")
 
         if "milter" in self.config.get('main', 'incomingport') \
                 and self.config.get('performance', 'backend') != 'process':
@@ -1457,12 +1504,12 @@ class MainController(DefConfigMixin):
             for plugin in allplugins:
                 print()
                 print(f"Linting MPlugin (state={state})", fc.strcolor(str(plugin), 'cyan'),
-                       'Config section:', fc.strcolor(str(plugin.section), 'cyan'))
+                      'Config section:', fc.strcolor(str(plugin.section), 'cyan'))
                 try:
                     result = plugin.lint(state=state)
                 except Exception as e:
                     CrashStore.store_exception()
-                    print("ERROR: %s" % e)
+                    print(f"ERROR: {e.__class__.__name__}: {str(e)}")
                     result = False
 
                 if result:
@@ -1504,7 +1551,7 @@ class MainController(DefConfigMixin):
         """
         MainController.propagate_defaults(self.requiredvars, self.config, 'main')
 
-    def checkConfig(self):
+    def check_config(self):
         """Check if all required options without default are in the config file
         Fill missing values with defaults if possible
         """
@@ -1516,7 +1563,7 @@ class MainController(DefConfigMixin):
                 var = self.config.get(section, config)
 
                 if 'validator' in infodic and not infodic["validator"](var):
-                    print("Validation failed for [%s] :: %s" % (section, config))
+                    print(f"Validation failed for [{section}] :: {config}")
                     all_ok = False
 
             except configparser.NoSectionError:
@@ -1526,7 +1573,7 @@ class MainController(DefConfigMixin):
             except configparser.NoOptionError:
                 print(fc.strcolor(f"Missing configuration value without default [{section}] :: {config}", "red"))
                 all_ok = False
-                
+
         # missing sections -> this is only a warning since section is not required
         # as long as there are no required variables without default values...
         if all_ok:
@@ -1538,7 +1585,7 @@ class MainController(DefConfigMixin):
 
             for section in missingsections:
                 print(fc.strcolor(f"Missing configuration section [{section}] :: "
-                              f"All variables will use default values", "yellow"))
+                                  f"All variables will use default values", "yellow"))
         return all_ok
 
     def load_extensions(self):
@@ -1546,7 +1593,7 @@ class MainController(DefConfigMixin):
         ret = []
         import fuglu.extensions
         for extension in fuglu.extensions.__all__:
-            mod = __import__('fuglu.extensions.%s' % extension)
+            mod = __import__(f'fuglu.extensions.{extension}')
             ext = getattr(mod, 'extensions')
             fl = getattr(ext, extension)
             enabled = getattr(fl, 'ENABLED')
@@ -1562,10 +1609,10 @@ class MainController(DefConfigMixin):
             pluginalias = self.config.get("PluginAlias", pluginalias)
         except (configparser.NoSectionError, configparser.NoOptionError):
             pass
-        
+
         return pluginalias
 
-    def load_plugins(self, propagate_plugin_config: bool=False):
+    def load_plugins(self, propagate_plugin_config: bool = False):
         """
         load plugins defined in config
 
@@ -1576,13 +1623,13 @@ class MainController(DefConfigMixin):
         plugindirs = self.config.getlist('main', 'plugindir')
         for plugindir in plugindirs:
             if os.path.isdir(plugindir):
-                self.logger.debug('Searching for additional plugins in %s' % plugindir)
+                self.logger.debug(f'Searching for additional plugins in {plugindir}')
                 if plugindir not in sys.path:
                     sys.path.insert(0, plugindir)
             else:
-                self.logger.warning('Plugin directory %s not found' % plugindir)
+                self.logger.warning(f'Plugin directory {plugindir} not found')
 
-        self.logger.debug('Module search path %s' % sys.path)
+        self.logger.debug(f'Module search path {sys.path}')
         self.logger.debug('Loading scanner plugins')
         newplugins, loadok = self._load_all(self.config.getlist('main', 'plugins', fallback=[]))
         if not loadok:
@@ -1623,8 +1670,9 @@ class MainController(DefConfigMixin):
                 self._propagate_plugin_defaults_to_config()
 
         return allOK
-    
+
     _re_config = re.compile(r"^(?P<structured_name>[a-zA-Z0-9._-]+)(?:\((?P<config_override>[a-zA-Z0-9._-]+)\))?$")
+
     def _load_all(self, plugins):
         """load all plugins from config string. returns tuple ([list of loaded instances],allOk)"""
         pluglist = []
@@ -1634,7 +1682,7 @@ class MainController(DefConfigMixin):
                 continue
             m = self._re_config.match(plug)
             if m is None:
-                self.logger.error('Invalid Plugin Syntax: %s' % plug)
+                self.logger.error(f'Invalid Plugin Syntax: {plug}')
                 allOK = False
                 continue
             structured_name, configoverride = m.groups()
@@ -1644,13 +1692,13 @@ class MainController(DefConfigMixin):
                 pluglist.append(plugininstance)
             except (configparser.NoSectionError, configparser.NoOptionError):
                 CrashStore.store_exception()
-                self.logger.error("The plugin %s is accessing the config in __init__ -> can not load default values" % structured_name)
+                self.logger.error(f"The plugin {structured_name} is accessing the config in __init__ -> can not load default values")
             except (ValueError, ModuleNotFoundError) as e:
-                self.logger.error('Could not load plugin %s : %s' % (structured_name, e))
+                self.logger.error(f'Could not load plugin {structured_name} : {str(e)}')
                 allOK = False
             except Exception as e:
                 CrashStore.store_exception()
-                self.logger.error('Could not load plugin %s : %s' % (structured_name, e))
+                self.logger.error(f'Could not load plugin {structured_name} : {e.__class__.__name__}: {str(e)}')
                 exc = traceback.format_exc()
                 self.logger.error(exc)
                 allOK = False
@@ -1672,7 +1720,7 @@ class MainController(DefConfigMixin):
             if 'section' in inspect.getfullargspec(mod.__init__)[0]:
                 plugininstance = mod(self.config, section=configsection)
             else:
-                raise Exception('Cannot set Config Section %s : Plugin %s does not support config override' % (configsection, mod))
+                raise Exception(f'Cannot set Config Section {configsection} : Plugin {mod} does not support config override')
         return plugininstance
 
     def _propagate_plugin_defaults_to_config(self):
@@ -1712,24 +1760,22 @@ class MainController(DefConfigMixin):
 
             for plug in remove_plugin:
                 if plug in self.plugins:
-                    msg_string = "Warning: Removing plugin %s from plugin-list" % plug
+                    msg_string = f"Warning: Removing plugin {plug} from plugin-list"
                     self.logger.warning(msg_string)
                     print(fc.strcolor(msg_string, "yellow"))
-
                     self.plugins.remove(plug)
                 elif plug in self.prependers:
-                    msg_string = "Warning: Removing plugin %s from prependers-list" % plug
+                    msg_string = f"Warning: Removing plugin {plug} from prependers-list"
                     self.logger.warning(msg_string)
                     print(fc.strcolor(msg_string, "yellow"))
-
                     self.prependers.remove(plug)
                 elif plug in self.appenders:
-                    msg_string = "Warning: Removing plugin %s from appenders-list" % plug
+                    msg_string = f"Warning: Removing plugin {plug} from appenders-list"
                     self.logger.warning(msg_string)
                     print(fc.strcolor(msg_string, "yellow"))
                     self.appenders.remove(plug)
                 else:
-                    msg_string = "Error: Could not remove plugin %s, not found in any list!" % str(plug)
+                    msg_string = f"Error: Could not remove plugin {plug}, not found in any list!"
                     self.logger.error(msg_string)
                     print(fc.strcolor(msg_string, "red"))
-                    raise ValueError("Plugin %s with bad config section name not found in any list!" % str(plug))
+                    raise ValueError(f"Plugin {plug} with bad config section name not found in any list!")
