@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -33,10 +33,12 @@ except ImportError:
     OBJGRAPH_EXTENSION_ENABLED = False
     # dummy object so we can keep objgraph._long_typename
     # as default input method in buildfilter
+
     class objgraph:
         _long_typename = None
         pass
 
+
 class ControlServer(object):
 
     def __init__(self, controller, port=None, address="127.0.0.1"):
@@ -91,7 +93,7 @@ class ControlServer(object):
         try:
             self._socket.shutdown(socket.SHUT_RDWR)
             self._socket.close()
-            #time.sleep(3) # why wait here?
+            # time.sleep(3) # why wait here?
         except Exception:
             pass
 
@@ -204,7 +206,7 @@ class ControlSession(object):
     def workerlist(self, args):
         """list of mail scanning workers"""
         threadpool = self.controller.threadpool
-        res=""
+        res = ""
         if threadpool is not None:
             workerlist = "\n%s" % '\n*******\n'.join(map(repr, threadpool.workers))
             res += "Total %s worker threads\n%s" % (len(threadpool.workers), workerlist)
@@ -212,7 +214,7 @@ class ControlSession(object):
         procpool = self.controller.procpool
         if procpool is not None:
             childstate_dict = procpool.shared_state
-            workerlist = "\n%s" % '\n*******\n'.join(["%s: %s"%(procname,procstate) for procname,procstate in childstate_dict.items()])
+            workerlist = "\n%s" % '\n*******\n'.join(["%s: %s" % (procname, procstate) for procname, procstate in childstate_dict.items()])
             res += "Total %s worker processes\n%s" % (len(procpool.workers), workerlist)
 
         asyncprocpool = self.controller.asyncprocpool
@@ -738,6 +740,7 @@ Block:\t\t${blockedcount}
                         self.ids = set(ids)
                         self.logger = logger
                         self.logger.debug("allids: %s" % ",".join([str(i) for i in self.ids]))
+
                     def __call__(self, x):
                         out = id(x) in self.ids
                         return out
@@ -746,7 +749,7 @@ Block:\t\t${blockedcount}
 
                 max_depth = max(map(len, chains)) - 1
                 objgraph.show_backrefs([chain[-1] for chain in chains], max_depth=max_depth,
-                              filter=in_chains, filename=inputdict["filename"])
+                                       filter=in_chains, filename=inputdict["filename"])
 
                 res += 'Graph for one object of type(s) %s written to %s' % (",".join(inputdict["typelist"]),
                                                                              inputdict['filename'])
@@ -883,7 +886,6 @@ Block:\t\t${blockedcount}
 
                 res += 'Graph for one object of type(s) %s written to %s' % (",".join(inputdict["typelist"]), inputdict['filename'])
 
-
             except Exception as e:
                 res += u"Exception: %s" % force_uString(e)
                 self.logger.exception(e)
@@ -977,7 +979,7 @@ Block:\t\t${blockedcount}
         if jsonstring:
             pyobject = json.loads(jsonstring)
         if ForcedType is not None:
-            if not isinstance(pyobject,ForcedType):
+            if not isinstance(pyobject, ForcedType):
                 pyobject = ForcedType()
         return pyobject
 
@@ -1082,7 +1084,7 @@ Block:\t\t${blockedcount}
 
             filters.append(TmpConstruct(must_contain))
 
-        finalfilter = lambda o: all(x(input_to_string(o)) for x in filters)
+        def finalfilter(o): return all(x(input_to_string(o)) for x in filters)
 
         return finalfilter
 
@@ -1097,7 +1099,7 @@ class CrashStore(object):
 
         if thread is None:
             thread = threading.current_thread()
-        
+
         info = ""
         if hasattr(thread, 'threadinfo'):
             info = thread.threadinfo
@@ -1107,4 +1109,3 @@ class CrashStore(object):
         CrashStore.exceptions.append((exc_info, time.time(), desc),)
         while len(CrashStore.exceptions) > maxtracebacks:
             CrashStore.exceptions.pop(0)
-
