@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Fumail Project
+#   Copyright Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,12 +15,15 @@
 #
 #
 
+import typing as tp
+import logging
+import ipaddress
+
+
 class _DummyExc(Exception):
     pass
 
 
-import ipaddress
-
 STATUS = "not loaded"
 _NXDOMAIN = _DummyExc
 DNS_TIMEOUT = TimeoutError
@@ -28,25 +31,27 @@ _PYDNSEXC = _DummyExc
 _NONAMESERVER = _DummyExc
 _NOANSWER = _DummyExc
 try:
-    from dns import resolver, exception
-    HAVE_DNSPYTHON=True
+    from dns import resolver, exception, flags
+    HAVE_DNSPYTHON = True
     STATUS = "available"
     _NXDOMAIN = resolver.NXDOMAIN
     DNS_TIMEOUT = exception.Timeout
     _NONAMESERVER = resolver.NoNameservers
     _NOANSWER = resolver.NoAnswer
+    myresolver = resolver.Resolver()
 except ImportError:
-    resolver = None
-    HAVE_DNSPYTHON=False
+    resolver = exception = flags = None
+    HAVE_DNSPYTHON = False
 
-HAVE_PYDNS=False
+HAVE_PYDNS = False
 if not HAVE_DNSPYTHON:
     try:
         import DNS
-        HAVE_PYDNS=True
+        HAVE_PYDNS = True
         DNS.DiscoverNameServers()
         STATUS = "available"
         _PYDNSEXC = DNS.Base.ServerError
+        logging.warning('support for pydns is deprecated in fuglu, please install dnspython')
     except ImportError:
         DNS = None
         STATUS = "DNS not installed"
@@ -54,7 +59,6 @@ if not HAVE_DNSPYTHON:
 ENABLED = DNSQUERY_EXTENSION_ENABLED = HAVE_DNSPYTHON or HAVE_PYDNS
 
 
-
 QTYPE_A = 'A'
 QTYPE_AAAA = 'AAAA'
 QTYPE_MX = 'MX'
@@ -67,40 +71,73 @@ QTYPE_SRV = 'SRV'
 QTYPE_SOA = 'SOA'
 QTYPE_CAA = 'CAA'
 QTYPE_DS = 'DS'
+QTYPE_CDS = 'CDS'
 QTYPE_DNSKEY = 'DNSKEY'
 QTYPE_SSHFP = 'SSHFP'
 QTYPE_TLSA = 'TLSA'
-
+QTYPE_NSEC = 'NSEC'
+QTYPE_NSEC3 = 'NSEC3'
+QTYPE_NSEC3PARAM = 'NSEC3PARAM'
+QTYPE_RRSIG = 'RRSIG'
 
 
 class FuNXDOMAIN(Exception):
     pass
 
+
 class FuTIMEOUT(Exception):
     pass
 
+
 class FuSERVFAIL(Exception):
     pass
 
+
 class FuNoNameserver(Exception):
     pass
 
+
 class FuNoAnswer(Exception):
     pass
 
 
-def lookup(hostname, qtype=QTYPE_A, reraise=False):
+class FuNoDNSSec(Exception):
+    pass
+
+
+def lookup(hostname: str, qtype: str = QTYPE_A, reraise: bool = False, dnssec: bool = None, timeout: float = None) -> tp.Optional[tp.List[str]]:
+    """
+    performs dns lookup
+    :param hostname: string: the fqdn to look up
+    :param qtype: string: the qtype (A, AAAA, MX, ...)
+    :param reraise: boolean: if True reraise lookup exceptions. if False suppress and return None.
+    :param dnssec: boolean: if None no special handling of dnssec. if True result must be dnssec secured, else raises NoFuDNSSec exception. if False return response even if dnssec validation would fail.
+    :param timeout: float: maximum query time
+    :return: List of DNS replies (string) on successful lookup, empty List if no DNS answer, None on error
+    """
+    # noinspection PyExceptClausesOrder
     try:
         if HAVE_DNSPYTHON:
+            if dnssec is True:
+                myresolver.flags = flags.RD | flags.AD
+            elif dnssec is False:
+                myresolver.flags = flags.RD | flags.CD
+
+            arequest = myresolver.resolve(hostname, qtype, lifetime=timeout)
+
+            if dnssec is True and not arequest.response.flags & flags.AD:
+                raise FuNoDNSSec
+
             arecs = []
-            arequest = resolver.resolve(hostname, qtype)
             for rec in arequest:
                 arecs.append(rec.to_text())
             return arecs
 
         elif HAVE_PYDNS:
+            logger = logging.getLogger('fuglu.DNSQuery')
+            logger.warning('support for pydns is deprecated in fuglu, please install dnspython')
             return DNS.dnslookup(hostname, qtype)
-    
+
     except _NXDOMAIN:
         if reraise:
             raise FuNXDOMAIN
@@ -125,18 +162,24 @@ def lookup(hostname, qtype=QTYPE_A, reraise=False):
     return None
 
 
-
-def mxlookup(domain):
+def mxlookup(domain: str, reraise: bool=False, dnssec: bool=None, timeout: float = None) -> tp.Optional[tp.List[str]]:
+    """
+    perform MX lookup of given domain
+    :param domain: see lookup
+    :param reraise: see lookup
+    :param dnssec: see lookup
+    :param timeout: see lookup
+    :return: List of MX servers on successful lookup, empty List if no DNS answer, None on error
+    """
     try:
         if HAVE_DNSPYTHON:
-            mxrecs = []
-            mxrequest = resolver.resolve(domain, QTYPE_MX)
-            for rec in mxrequest:
-                mxrecs.append(rec.to_text())
+            mxrecs = lookup(domain, qtype=QTYPE_MX, reraise=reraise, dnssec=dnssec, timeout=timeout)
             mxrecs.sort()  # automatically sorts by priority
-            return [x.split(None, 1)[-1] for x in mxrecs]
+            return [x.split(None, 1)[-1] for x in mxrecs]  # only return server name
 
         elif HAVE_PYDNS:
+            logger = logging.getLogger('fuglu.DNSQuery')
+            logger.warning('support for pydns is deprecated in fuglu, please install dnspython')
             mxrecs = []
             mxrequest = DNS.mxlookup(domain)
             for dataset in mxrequest:
@@ -147,13 +190,20 @@ def mxlookup(domain):
             return [x[1] for x in mxrecs]
 
     except Exception:
-        return None
+        pass
 
     return None
 
 
-
-def revlookup(ip, reraise=False):
+def revlookup(ip: str, reraise: bool = False, dnssec: bool = None, timeout: float = None) -> tp.Optional[tp.List[str]]:
+    """
+    reverse lookup of IP address
+    :param ip: string of IPv4 or IPv6 address
+    :param reraise: see lookup
+    :param dnssec: see lookup
+    :param timeout: see lookup
+    :return: see lookup
+    """
     ipaddr = ipaddress.ip_address(ip)
     revip = ipaddr.reverse_pointer
-    return lookup(revip, qtype=QTYPE_PTR, reraise=reraise)
+    return lookup(revip, qtype=QTYPE_PTR, reraise=reraise, dnssec=dnssec, timeout=timeout)
