@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*- #
-#   Copyright 2009-2022 Fumail Project
+#   Copyright Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ import zipfile
 import tarfile
 import re
 import os.path
+from typing import List, Union
 from unittest.mock import MagicMock
 
 STATUS = "available: zip, tar"
@@ -137,8 +138,9 @@ class Archive_int(object):
 # Don't forget to add new implementations to the dict "archive_impl" and "archive_avail"
 # below the implementations in class Archivehandle
 
+
 class Archive_zip(Archive_int):
-    def __init__(self,filedescriptor,archivename=None):
+    def __init__(self, filedescriptor, archivename=None):
         super(Archive_zip, self).__init__(filedescriptor, archivename)
         self._handle = zipfile.ZipFile(filedescriptor)
         if self._archivename is None:
@@ -146,7 +148,6 @@ class Archive_zip(Archive_int):
                 self._archivename = os.path.basename(str(filedescriptor))
             except Exception:
                 self._archivename = "generic.zip"
-                
 
     def namelist(self):
         """ Get archive file list
@@ -180,6 +181,7 @@ class Archive_zip(Archive_int):
         """
         return self._handle.getinfo(path).file_size
 
+
 class Archive_rar(Archive_int):
     def __init__(self, filedescriptor, archivename=None):
         super(Archive_rar, self).__init__(filedescriptor, archivename)
@@ -225,6 +227,7 @@ class Archive_rar(Archive_int):
         """
         return self._handle.getinfo(path).file_size
 
+
 class Archive_tar(Archive_int):
     def __init__(self, filedescriptor, archivename=None):
         super(Archive_tar, self).__init__(filedescriptor, archivename)
@@ -246,7 +249,7 @@ class Archive_tar(Archive_int):
         Returns:
             (list) Returns a list of file paths within the archive
         """
-        return self._handle.getnames()
+        return self._handle.getnames() if self._handle else []
 
     def extract(self, path, archivecontentmaxsize):
         """extract a file from the archive into memory
@@ -389,10 +392,11 @@ class Archive_7zr(Archive_int):
             (list) Returns a list of file paths within the archive
         """
 
-        return self._handle.getnames()
+        return self._handle.getnames() if self._handle else []
 
     def extract(self, path, archivecontentmaxsize):
-
+        if not path.isascii():
+            return None
         if archivecontentmaxsize is not None and self.filesize(path) > archivecontentmaxsize:
             return None
         output = self._handle.read(path)
@@ -402,7 +406,8 @@ class Archive_7zr(Archive_int):
         return None
 
     def filesize(self, path):
-        """get extracted file size
+        """
+        get extracted file size
 
         Args:
             path (str): is the filename in the archive as returned by namelist
@@ -410,11 +415,6 @@ class Archive_7zr(Archive_int):
             (int) file size in bytes
         """
         filesize = 0
-        try:
-            list_in_archive = files_list
-        except Exception:
-            list_in_archive = []
-
         for filecontent in self._handle.files:
             try:
                 if filecontent.filename == path:
@@ -444,14 +444,14 @@ class Archive_gz(Archive_int):
         # Python 3 gzip.open handles both filename and file object
         # --
         self._handle = gzip.open(filedescriptor)
-        if isinstance(filedescriptor,(str,bytes)):
+        if isinstance(filedescriptor, (str, bytes)):
             try:
                 self._archivename = os.path.basename(str(filedescriptor))
             except Exception:
                 self._archivename = "generic.gz"
         else:
             if self._archivename is None:
-                # if there is not archive name defined yet
+                # if there is no archive name defined yet
                 try:
                     # eventually it is possible to get the filename from
                     # the GzipFile object
@@ -527,10 +527,11 @@ class Archive_gz(Archive_int):
         """
         try:
             return len(self.extract(path, None))
-        except Exception as e:
+        except Exception:
             return 0
 
-class Archive_tgz():
+
+class Archive_tgz(object):
     r"""
     Archive combining tar and gzip extractor
 
@@ -554,15 +555,16 @@ class Archive_tgz():
     This extractor will first try tar extractor and if this fails the gz extractor. This has the advantage
     we can keep content detection for gzip and be backward compatible.
     """
+
     def __init__(self, filedescriptor, archivename=None):
         # first try extract using tar
         # check if it is possible to extract filenames
         try:
-            self._archive_impl = Archive_tar(filedescriptor,archivename)
+            self._archive_impl = Archive_tar(filedescriptor, archivename)
             filenamelist = self._archive_impl.namelist()
         except Exception as e:
             if GZIP_AVAILABLE:
-                self._archive_impl = Archive_gz(filedescriptor,archivename)
+                self._archive_impl = Archive_gz(filedescriptor, archivename)
             else:
                 raise e
 
@@ -585,6 +587,8 @@ class Archive_tgz():
 # inspired by:
 # https://stackoverflow.com/questions/128573/using-property-on-classmethods
 # Working for static getter implementation in Py2 and Py3
+
+
 class classproperty(property):
     def __get__(self, obj, objtype=None):
         return super(classproperty, self).__get__(objtype)
@@ -592,9 +596,11 @@ class classproperty(property):
 #--------------------------------------------------------------------------#
 #- The pubic available factory class to produce the archive handler class -#
 #--------------------------------------------------------------------------#
+
+
 class Archivehandle(object):
     """
-    Archivehandle is the actually the factory for the archive handle implementations.
+    Archivehandle is actually the factory for the archive handle implementations.
     Besides being the factory, Archivehandle provides also dicts and lists of implemented
     and available archives based on different keys (for example file extension).
 
@@ -624,18 +630,17 @@ class Archivehandle(object):
     archive_impl = {"zip": Archive_zip,
                     "rar": Archive_rar,
                     "tar": Archive_tar,
-                    "7z" : Archive_7z if SEVENZIP_PACKAGE == "py7zlib" else Archive_7zr,
+                    "7z": Archive_7z if SEVENZIP_PACKAGE == "py7zlib" else Archive_7zr,
                     "tgz": Archive_tgz,
-                    "gz" : Archive_gz}
+                    "gz": Archive_gz}
 
     # Dict storing if archive type is available
-    archive_avail= {"zip": True,
-                    "rar": (RARFILE_AVAILABLE > 0),
-                    "tar": True,
-                    "7z" : (SEVENZIP_AVAILABLE > 0),
-                    "gz" : (GZIP_AVAILABLE > 0),
-                    "tgz": (GZIP_AVAILABLE > 0)}
-
+    archive_avail = {"zip": True,
+                     "rar": (RARFILE_AVAILABLE > 0),
+                     "tar": True,
+                     "7z": (SEVENZIP_AVAILABLE > 0),
+                     "gz": (GZIP_AVAILABLE > 0),
+                     "tgz": (GZIP_AVAILABLE > 0)}
 
     # key: regex matching content type as returned by file magic, value: archive type
     implemented_archive_ctypes = {
@@ -645,10 +650,9 @@ class Archivehandle(object):
         r'^application\/x-bzip2': 'tar',
         r'^application\/gzip': 'tgz',
         r'^application\/x-rar': 'rar',         # available only if RARFILE_AVAILABLE > 0
-        r'^application\/x-7z-compressed': '7z' # available only if SEVENZIP_AVAILABLE > 0
+        r'^application\/x-7z-compressed': '7z'  # available only if SEVENZIP_AVAILABLE > 0
     }
 
-
     # key: file ending, value: archive type
     implemented_archive_extensions = {
         'zip': 'zip',
@@ -664,10 +668,10 @@ class Archivehandle(object):
         '7z': '7z',    # available only if SEVENZIP_AVAILABLE > 0
     }
 
-    #--
+    # --
     # dicts and lists containing information about available
-    # archives are setup automatically (see below in metaclass)
-    #--
+    # archives are set up automatically (see below in metaclass)
+    # --
 
     # "avail_archives_list" is a list of available archives based on available implementations
     _avail_archives_list = None
@@ -708,19 +712,18 @@ class Archivehandle(object):
         # first time this list has to be created based on what's available
         if cls._avail_archives_list is None:
             tlist = []
-            for atype,available in iter(Archivehandle.archive_avail.items()):
+            for atype, available in iter(Archivehandle.archive_avail.items()):
                 if available:
                     tlist.append(atype)
             cls._avail_archives_list = tlist
         return cls._avail_archives_list
 
-
     @classproperty
     def avail_archive_ctypes(cls):
         # first time this dict has to be created based on what's available
         if cls._avail_archive_ctypes is None:
             newDict = {}
-            for regex,atype in iter(Archivehandle.implemented_archive_ctypes.items()):
+            for regex, atype in iter(Archivehandle.implemented_archive_ctypes.items()):
                 if Archivehandle.avail(atype):
                     newDict[regex] = atype
             cls._avail_archive_ctypes = newDict
@@ -732,7 +735,7 @@ class Archivehandle(object):
         # first time this list has to be created based on what's available
         if cls._avail_archive_ctypes_list is None:
             tlist = []
-            for ctype,atype in iter(Archivehandle.avail_archive_ctypes.items()):
+            for ctype, atype in iter(Archivehandle.avail_archive_ctypes.items()):
                 if Archivehandle.avail(atype):
                     tlist.append(ctype)
             cls._avail_archive_ctypes_list = tlist
@@ -743,7 +746,7 @@ class Archivehandle(object):
         # first time this dict has to be created based on what's available
         if cls._avail_archive_extensions is None:
             newDict = {}
-            for regex,atype in iter(Archivehandle.implemented_archive_extensions.items()):
+            for regex, atype in iter(Archivehandle.implemented_archive_extensions.items()):
                 if Archivehandle.avail(atype):
                     newDict[regex] = atype
             cls._avail_archive_extensions = newDict
@@ -755,7 +758,7 @@ class Archivehandle(object):
         # first time this dict has to be created based on what's available
         if cls._avail_archive_extensionlist4type is None:
             newDict = {}
-            for regex,atype in iter(Archivehandle.implemented_archive_extensions.items()):
+            for regex, atype in iter(Archivehandle.implemented_archive_extensions.items()):
                 # regex is the file extension
                 # atype is the archive type
                 if Archivehandle.avail(atype):
@@ -768,7 +771,34 @@ class Archivehandle(object):
             cls._avail_archive_extensionlist4type = newDict
         return cls._avail_archive_extensionlist4type
 
-        return cls._avail_archive_extensions
+    @classmethod
+    def register(cls,
+                 archivemanager: Archive_int,
+                 archive_type: str,
+                 archive_ctypes: Union[List[str], str] = (),
+                 archive_extensions: [List[str], str] = (),
+                 ):
+        """Method to add custom archive managers"""
+        archive_type = archive_type.lower().strip()
+        archive_ctypes = [archive_ctypes] if isinstance(archive_ctypes, str) else archive_ctypes
+        archive_extensions = [archive_extensions] if isinstance(archive_extensions, str) else archive_extensions
+
+        if not cls.archive_avail.get(archive_type, False):
+            cls.archive_avail[archive_type] = True
+            cls.archive_impl[archive_type] = archivemanager
+            for ctype in archive_ctypes:
+                cls.implemented_archive_ctypes[archive_type] = ctype
+            for ext in archive_extensions:
+                cls.implemented_archive_extensions[archive_type] = ext
+
+            # reset calculated class properties
+            cls._avail_archives_list = None
+            cls._avail_archive_ctypes_list = None
+            cls._avail_archive_ctypes = None
+            cls._avail_archive_extensions_list = None
+            cls._avail_archive_extensions = None
+            cls._avail_archive_extensionlist4type = None
+
     @staticmethod
     def impl(archive_type):
         """
@@ -798,7 +828,7 @@ class Archivehandle(object):
         return Archivehandle.archive_avail[archive_type]
 
     @staticmethod
-    def archive_type_from_content_type(content_type, all_impl = False, custom_ctypes_dict = None):
+    def archive_type_from_content_type(content_type, all_impl=False, custom_ctypes_dict=None):
         """
         Return the corresponding archive type if the content type matches a regex , None otherwise
 
@@ -831,14 +861,14 @@ class Archivehandle(object):
         return archive_type
 
     @staticmethod
-    def archive_type_from_extension(att_name, all_impl = False, custom_extensions_dict = None):
+    def archive_type_from_extension(att_name, all_impl=False, custom_extensions_dict=None):
         """
         Return the corresponding archive type if the extension matches regex , None otherwise
 
         Args:
             att_name (str): filename
             all_impl (bool): check all implementations, not only the ones available
-            custom_ctypes_dict (dict): dict with custom mapping (key: regex matching content type as returned by file magic, value: archive type)
+            custom_extensions_dict (dict): dict with custom mapping (key: regex matching content type as returned by file magic, value: archive type)
 
         Returns:
             (str or None) archive type
@@ -862,12 +892,12 @@ class Archivehandle(object):
 
         archive_type = None
         for arext in sorted_ext_list:
-            if att_name.lower().endswith('.%s' % arext):
+            if att_name.lower().endswith(f'.{arext}'):
                 archive_type = sorted_ext_dict[arext]
                 break
         return archive_type
 
-    def __new__(cls,archive_type, filedescriptor, archivename=None):
+    def __new__(cls, archive_type, filedescriptor, archivename=None):
         """
         Factory method that will produce and return the correct implementation depending
         on the archive type
@@ -877,7 +907,6 @@ class Archivehandle(object):
             filedescriptor (): file-like object (io.BytesIO) or path-like object (str or bytes with filename including path)
         """
 
-        assert Archivehandle.impl(archive_type), "Archive type %s not in list of supported types: %s" % (archive_type, ",".join(Archivehandle.archive_impl.keys()))
-        assert Archivehandle.avail(archive_type), "Archive type %s not in list of available types: %s" % (archive_type, ",".join(Archivehandle.avail_archives_list))
-        return Archivehandle.archive_impl[archive_type](filedescriptor,archivename)
-
+        assert Archivehandle.impl(archive_type), f'Archive type {archive_type} not in list of supported types: {",".join(Archivehandle.archive_impl.keys())}'
+        assert Archivehandle.avail(archive_type), f'Archive type {archive_type} not in list of available types: {",".join(Archivehandle.avail_archives_list)}'
+        return Archivehandle.archive_impl[archive_type](filedescriptor, archivename)
