@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Fumail Project
+#   Copyright Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,19 @@
 #
 #
 import logging
+import os
 import threading
 import time
 
 try:
     import redis
     from redis import StrictRedis
-    STATUS = "redis installed, version: {}".format(redis.__version__)
+    from redis import __version__ as redisversion
+    STATUS = f"redis installed, version: {redisversion}"
     ENABLED = True
-    REDIS2 = redis.__version__.startswith('2')
+    REDIS2 = redisversion.startswith('2')
     try:
-        redisversion = [int(x) for x in redis.__version__.split('.')]
+        redisversion = [int(x) for x in redisversion.split('.')]
         REDIS33X = (redisversion[0] == 3 and redisversion[1] >= 3) or redisversion[0] > 3
     except Exception:
         REDIS33X = None
@@ -35,6 +37,7 @@ except ImportError:
     ENABLED = False
     StrictRedis = object
     redis = None
+    redisversion = 'unkonwn'
     REDIS2 = False
     REDIS33X = None
 
@@ -47,7 +50,7 @@ class RedisKeepAlive(StrictRedis):
 
     Issue on github:
     https://github.com/andymccurdy/redis-py/issues/722
-    
+
     this should no longer be necessary as newer version of redispy
     bring their own health_check_interval parameter
     """
@@ -65,12 +68,12 @@ class RedisKeepAlive(StrictRedis):
                 # the pinginterval manually implemented before...
                 super(RedisKeepAlive, self).__init__(*args, **kwargs, health_check_interval=self._pinginterval)
                 logging.getLogger("fuglu.RedisKeepAlive").debug(f"Set ping interval {self._pinginterval} "
-                                                                f"for redis {redis.__version__} using "
+                                                                f"for redis {redisversion} using "
                                                                 f"parameter health_check_interval")
             except Exception as e:
                 if REDIS33X:
                     logging.getLogger("fuglu.RedisKeepAlive").error(f"Setting ping interval {self._pinginterval} "
-                                                                    f"for redis {redis.__version__}: {str(e)}")
+                                                                    f"for redis {redisversion}: {e.__class__.__name__} {str(e)}")
                 super(RedisKeepAlive, self).__init__(*args, **kwargs)
 
                 # start a thread which will ping the server to keep
@@ -91,16 +94,15 @@ class RedisKeepAlive(StrictRedis):
             time.sleep(self._pinginterval)
 
 
-
 class RedisPooledConn(object):
-    def __init__(self, redis_url:str=None, **args):
+    def __init__(self, redis_url: str = None, **args):
         self.logger = logging.getLogger('fuglu.RedisPooledConn')
         if 'password' in args.keys() and args['password']:
             self.logger.warning(f'deprecated redis password config - include it in redis_conn URL')
         elif 'password' in args.keys() and not args['password']:
             del args['password']
         self._pinginterval = args.pop("pinginterval", 0)
-        
+
         if 'retry_on_timeout' not in args.keys():
             args['retry_on_timeout'] = True
         if 'socket_keepalive' not in args.keys():
@@ -109,7 +111,9 @@ class RedisPooledConn(object):
             args['health_check_interval'] = 10
         if 'socket_connect_timeout' not in args.keys():
             args['socket_connect_timeout'] = 5
-        
+        if 'client_name' not in args.keys():
+            args['client_name'] = f'fuglu'
+
         if not redis_url:
             self.pool = None
         elif redis is None:
@@ -123,14 +127,12 @@ class RedisPooledConn(object):
             host, port, db = redis_url.split(':')
             self.pool = redis.ConnectionPool(host=host, port=port, db=int(db), **args)
 
-
-    def get_conn(self) -> RedisKeepAlive:
+    def get_conn(self) -> StrictRedis:
         args = {}
         if self._pinginterval > 0:
             args['pinginterval'] = self._pinginterval
         return RedisKeepAlive(connection_pool=self.pool, **args)
-    
-    
+
     def check_connection(self) -> bool:
         if self.pool is None:
             return False
@@ -139,16 +141,14 @@ class RedisPooledConn(object):
             return redisconn.ping()
 
 
-
 class ExpiringCounter(object):
-    
-    def __init__(self, redis_pool:RedisPooledConn=None, ttl:int=0, maxcount:int=0):
+
+    def __init__(self, redis_pool: RedisPooledConn = None, ttl: int = 0, maxcount: int = 0):
         self.redis_pool = redis_pool or RedisPooledConn()
         self.ttl = ttl
         self.maxcount = maxcount
-    
-    
-    def _to_int(self, value, default:int=0) -> int:
+
+    def _to_int(self, value, default: int = 0) -> int:
         """
         Convert to integer if possible
 
@@ -166,9 +166,8 @@ class ExpiringCounter(object):
         except (ValueError, TypeError):
             value = default
         return value
-    
-    
-    def increase(self, key:str, value:int=1):
+
+    def increase(self, key: str, value: int = 1):
         """
         Given the identifier, create a new entry for current time with given value.
         Args:
@@ -191,16 +190,15 @@ class ExpiringCounter(object):
         try:
             ts = int(time.time()) + self.ttl
             pipe = redisconn.pipeline()
-            # increase the the value of 'ts' by 'value' for hash 'key'
-            pipe.hincrby(key, ts, value) # returns the value of redis[key][ts], usually same as param value
+            # increase the value of 'ts' by 'value' for hash 'key'
+            pipe.hincrby(key, str(ts), value)  # returns the value of redis[key][ts], usually same as param value
             pipe.expire(key, self.ttl)  # returns None, this is a safety measure to avoid stale keys
             result = pipe.execute()
             return result[0]
         except redis.exceptions.TimeoutError:
             return 0
-    
-    
-    def get_count(self, key:str, cleanup:bool=True) -> int:
+
+    def get_count(self, key: str, cleanup: bool = True) -> int:
         """
         Get value. This is the sum of the count values within the ttl value stored in the class.
         Args:
@@ -220,16 +218,15 @@ class ExpiringCounter(object):
                 count += self._to_int(v)
             elif cleanup:  # mark stale keys for cleanup
                 delkeys.append(k)
-        
+
         if delkeys and len(delkeys) == len(values):
             # all keys are stale
             redisconn.delete(key)
         elif delkeys:
             redisconn.hdel(key, *delkeys)
-        
+
         return count
-    
-    
+
     def cleanup(self) -> None:
         """
         Remove stale entries from redis database
@@ -251,4 +248,4 @@ class ExpiringCounter(object):
                 elif delkeys:
                     redisconn.hdel(key, *delkeys)
             if delete:
-                redisconn.delete(key)
\ No newline at end of file
+                redisconn.delete(key)
