@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,7 +23,7 @@ import logging
 import traceback
 import weakref
 from fuglu.shared import default_template_values, FuConfigParser, Suspect, deprecated, get_default_cache
-from fuglu import get_version
+from fuglu.utils.version import get_main_version
 import typing as tp
 import time
 import os
@@ -32,10 +32,9 @@ modlogger = logging.getLogger('fuglu.extensions.sql')
 STATUS = "not loaded"
 try:
     from sqlalchemy import create_engine, text
-    from sqlalchemy.orm import scoped_session, sessionmaker
-    from sqlalchemy.ext.declarative import declarative_base
+    from sqlalchemy.orm import scoped_session, sessionmaker, declarative_base
     DeclarativeBase = declarative_base()
-    
+
     SQL_EXTENSION_ENABLED = True
     STATUS = "available"
 except ImportError:
@@ -46,7 +45,7 @@ except ImportError:
     def text(string):
         return string
 
-ENABLED = SQL_EXTENSION_ENABLED # fuglu compatibility
+ENABLED = SQL_EXTENSION_ENABLED  # fuglu compatibility
 
 try:
     import requests
@@ -101,7 +100,7 @@ def lint_session(connectstring):
         dbsession = get_session(connectstring)
         dbsession.execute('SELECT 1')
     except Exception as e:
-        print('ERROR: failed to connect to SQL database: %s' % str(e))
+        print(f'ERROR: failed to connect to SQL database: {e.__class__.__name__}: {str(e)}')
         return False
     return True
 
@@ -112,19 +111,19 @@ class DBFile(object):
 
     def __init__(self, connectstring, query):
         self.connectstring = connectstring
-        # eg. "select action,regex,description FROM tblname where scope=:scope
+        # e.g. "select action,regex,description FROM tblname where scope=:scope
         self.query = query
         self.logger = logging.getLogger('fuglu.sql.dbfile')
 
     def getContent(self, templatevars=None):
         """Get the content from the database as a list of lines. If the query returns multiple columns, they are joined together with a space as separator
-        templatevars: replace placeholders in the originalquery , eg. select bla from bla where domain=:domain
+        templatevars: replace placeholders in the originalquery , e.g. select bla from bla where domain=:domain
         """
         if templatevars is None:
             templatevars = {}
         sess = get_session(self.connectstring)
         res = sess.execute(self.query, templatevars)
-        self.logger.debug('Executing query %s with vars %s' % (self.query, templatevars))
+        self.logger.debug(f'Executing query {self.query} with vars {templatevars}')
         buff = []
         for row in res:
             line = " ".join(filter(None, row))
@@ -138,36 +137,38 @@ class RESTAPIError(Exception):
 
 
 class RESTAPIConfig(object):
-    def __init__(self):
+    def __init__(self, config: FuConfigParser, suspectid: str = '<unknown>'):
         self.logger = logging.getLogger('fuglu.sql.restconfig')
-    
+        self.config = config
+        self.suspectid = suspectid
+
     _re_title = re.compile('<title>(.{1,1000})</title>', re.I)
+
     def _get_errormsg(self, reply):
         errmsg = reply.text
-        rgxtitle =  self._re_title.search(errmsg)
+        rgxtitle = self._re_title.search(errmsg)
         if rgxtitle:
             errmsg = rgxtitle.groups()[0]
         return errmsg
-    
-    
-    def _http_request(self, uri, headers, suspectid, verify, timeout, retry=3):
-        reply = requests.get(uri, headers=headers, verify=verify, timeout=timeout)
+
+    def _http_request(self, reqtype, jsondata, uri, headers, verify, timeout, retry=3):
+        req = getattr(requests, reqtype.lower())
+        reply = req(uri, headers=headers, json=jsondata, verify=verify, timeout=timeout)
         if reply.status_code != 200:
             sleeptime = 0.1
             if reply.status_code == 503:
                 sleeptime = 2
             if retry > 0:
                 time.sleep(sleeptime * (4-retry))
-                self.logger.debug(f'{suspectid} got unexpected status code {reply.status_code} from {uri} retry {retry}')
-                reply = self._http_request(uri, headers, suspectid, verify, timeout, retry=retry-1)
+                self.logger.debug(f'{self.suspectid} got unexpected status code {reply.status_code} from {uri} retry {retry}')
+                reply = self._http_request(reqtype, jsondata, uri, headers, verify, timeout, retry=retry-1)
             else:
-                self.logger.warning(f'{suspectid} got unexpected status code {reply.status_code} {self._get_errormsg(reply)} from {uri}')
+                self.logger.warning(f'{self.suspectid} got unexpected status code {reply.status_code} {self._get_errormsg(reply)} from {uri}')
         return reply
-    
-    
-    def _get(self, restapi_uri, headers, suspectid, restapi_verify, timeout):
+
+    def _call(self, reqtype, jsondata, restapi_uri, headers, restapi_verify, timeout):
         try:
-            reply = self._http_request(restapi_uri, headers, suspectid, restapi_verify, timeout)
+            reply = self._http_request(reqtype, jsondata, restapi_uri, headers, restapi_verify, timeout)
             if reply.status_code != 200:
                 try:
                     err = reply.json()
@@ -179,37 +180,58 @@ class RESTAPIConfig(object):
         except Exception as e:
             raise RESTAPIError(f'{e.__class__.__name__}: {str(e)}')
         return content
-    
-    
-    def get(self, config:FuConfigParser, endpoint:str, suspectid:str, timeout:int = 0):
-        if not REQUESTS_AVAILABLE:
-            return []
-        if not endpoint:
-            return []
-        
-        restapi_uri = config.get('databaseconfig', 'restapi_uri', resolve_env=True)
-        restapi_headers = config.getlist('databaseconfig', 'restapi_headers', resolve_env=True)
-        restapi_verify = config.getboolean('databaseconfig', 'restapi_verify', fallback=False)
-        restapi_timeout = timeout or config.getfloat('databaseconfig', 'restapi_timeout', fallback=10)
-        
+
+    def get_headers(self):
+        restapi_headers = self.config.getlist('databaseconfig', 'restapi_headers', resolve_env=True)
         headers = {
-            'User-Agent': f'Fuglu/{get_version().strip()}',
+            'User-Agent': f'Fuglu/{get_main_version().strip()}',
             'accept': 'application/json',
         }
         for item in restapi_headers:
             if not ':' in item:
-                self.logger.warning(f'{suspectid} invalid header definition {item}')
+                self.logger.warning(f'{self.suspectid} invalid header definition {item}')
             else:
                 hdr, value = item.split(':', 1)
                 headers[hdr] = value
-        
+        return headers
+
+    def get(self, endpoint: str, timeout: int = 0):
+        if not REQUESTS_AVAILABLE:
+            return []
+        if not endpoint:
+            return []
+
+        restapi_uri = self.config.get('databaseconfig', 'restapi_uri', resolve_env=True)
+        restapi_verify = self.config.getboolean('databaseconfig', 'restapi_verify', fallback=False)
+        restapi_timeout = timeout or self.config.getfloat('databaseconfig', 'restapi_timeout', fallback=10)
+
+        headers = self.get_headers()
+
         if not restapi_uri.endswith('/'):
             restapi_uri = f'{restapi_uri}/'
         restapi_uri = f'{restapi_uri}{endpoint}'
-        
-        content = self._get(restapi_uri, headers, suspectid, restapi_verify, restapi_timeout)
+
+        content = self._call('GET', None, restapi_uri, headers, restapi_verify, restapi_timeout)
+        return content
+
+    def put(self, endpoint: str, jsondata, timeout: int = 0):
+        if not REQUESTS_AVAILABLE:
+            raise RESTAPIError('Requests unavailable')
+        if not endpoint:
+            raise RESTAPIError('no endpoint specified')
+
+        restapi_uri = self.config.get('databaseconfig', 'restapi_uri', resolve_env=True)
+        restapi_verify = self.config.getboolean('databaseconfig', 'restapi_verify', fallback=False)
+        restapi_timeout = timeout or self.config.getfloat('databaseconfig', 'restapi_timeout', fallback=10)
+
+        headers = self.get_headers()
+
+        if not restapi_uri.endswith('/'):
+            restapi_uri = f'{restapi_uri}/'
+        restapi_uri = f'{restapi_uri}{endpoint}'
+
+        content = self._call('PUT', jsondata, restapi_uri, headers, restapi_verify, restapi_timeout)
         return content
-    
 
 
 class SectionCacheRow(object):
@@ -222,7 +244,6 @@ class SectionCacheRow(object):
         setattr(self, valuefield, value)
 
 
-
 class DBConfig(FuConfigParser):
 
     """
@@ -239,8 +260,7 @@ class DBConfig(FuConfigParser):
         self._clone_from(config)
         self.cache = get_default_cache()
         self.yamldata = {}
-    
-    
+
     def set_suspect(self, suspect: tp.Optional[Suspect]) -> None:
         if suspect is None:
             suspect = Suspect('', 'postmaster', '/dev/null')
@@ -251,13 +271,11 @@ class DBConfig(FuConfigParser):
         # object does not get freed until a (manual or automatic) run of the
         # garbage collector "gc.collect()"
         self.suspect = weakref.ref(suspect)
-    
-    
+
     def set_rcpt(self, recipient: str) -> None:
         self._tempsuspect = Suspect('', recipient, '/dev/null')
         self.set_suspect(self._tempsuspect)
-    
-    
+
     def _clone_from(self, config: FuConfigParser) -> None:
         """Clone this object from a FuConfigParser"""
         stringout = StringIO()
@@ -270,7 +288,6 @@ class DBConfig(FuConfigParser):
         self.markers = config.markers.copy()
         self._defaultdict = config._defaultdict
 
-    
     def load_section(self, section: str) -> bool:
         """
         load section into local cache.
@@ -295,7 +312,7 @@ class DBConfig(FuConfigParser):
                     self.sectioncache[section] = result
                 except Exception:
                     trb = traceback.format_exc()
-                    self.logger.error("Error getting database config override section data: %s" % trb)
+                    self.logger.error(f'Error getting database config override section data: {trb}')
                 loaded = True
 
         if REQUESTS_AVAILABLE and self.has_section('databaseconfig') \
@@ -318,7 +335,7 @@ class DBConfig(FuConfigParser):
                     else:
                         self.logger.debug(f'ignoring bogus section {section} in loaded rest data')
             loaded = True
-        
+
         if YAML_AVAILABLE and self.has_section('databaseconfig') \
                 and self.has_option('databaseconfig', 'yaml_filepath') \
                 and os.path.exists(self.parentget('databaseconfig', 'yaml_filepath')):
@@ -333,20 +350,19 @@ class DBConfig(FuConfigParser):
                     except KeyError:
                         self.sectioncache[section] = cachable_sectiondata
             loaded = True
-        
+
         return loaded
-    
-    
+
     def _get_cached(self, section: str, option: str):
         suspect = self.suspect()
         if suspect is None:
             return None
         todomain = f'%{suspect.to_domain}'
-        
+
         optionfield = self.parentget('databaseconfig', 'option_field', fallback='option')
         scopefield = self.parentget('databaseconfig', 'scope_field', fallback='scope')
         valuefield = self.parentget('databaseconfig', 'value_field', fallback='value')
-        
+
         try:
             for row in self.sectioncache.get(section, []):
                 scope = getattr(row, scopefield)
@@ -354,20 +370,19 @@ class DBConfig(FuConfigParser):
                 if dboption == option and scope in {suspect.to_address, todomain}:
                     return getattr(row, valuefield)
         except AttributeError as e:
-            self.logger.error('database layout does not match databaseconfig settings: %s' % str(e))
+            self.logger.error(f'database layout does not match databaseconfig settings: {str(e)}')
         return None
-    
-    
+
     def get_cached_options(self, section):
         options = []
         suspect = self.suspect()
         if suspect is None:
             return options
-        
+
         todomain = f'%{suspect.to_domain}'
         optionfield = self.parentget('databaseconfig', 'option_field', fallback='option')
         scopefield = self.parentget('databaseconfig', 'scope_field', fallback='scope')
-        
+
         for row in self.sectioncache.get(section, []):
             scope = getattr(row, scopefield)
             dboption = getattr(row, optionfield)
@@ -375,14 +390,13 @@ class DBConfig(FuConfigParser):
                 options.append(dboption)
         return options
 
-
     def get_override(self, section: str, option: str, **kwargs) -> tp.Any:
         result = None
         priorities = []
         if (self.has_section('databaseconfig')) and (self.has_option('databaseconfig', 'dbpriority')):
             priorities = self.parentgetlist('databaseconfig', 'dbpriority', lower=True)
             #cache_ttl = self.parentgetint('databaseconfig', 'value_cache_ttl', fallback=30)
-            #if not 'cache' in priorities and cache_ttl > 0:
+            # if not 'cache' in priorities and cache_ttl > 0:
             #    priorities.insert(0, 'cache')
         for priority in priorities:
             if priority in {'cache', 'sql', 'rest', 'yaml'}:
@@ -404,6 +418,11 @@ class DBConfig(FuConfigParser):
         result = super().getint(section, option, override=override, **kwargs)
         return result
 
+    def getfloat(self, section: str, option: str, **kwargs):
+        override = self.get_override(section, option, **kwargs)
+        result = super().getfloat(section, option, override=override, **kwargs)
+        return result
+
     def getboolean(self, section: str, option: str, **kwargs):
         override = self.get_override(section, option, **kwargs)
         result = self.parentgetboolean(section, option, override=override, **kwargs)
@@ -414,28 +433,27 @@ class DBConfig(FuConfigParser):
         result = self.parentgetlist(section, option, override=override, **kwargs)
         return result
 
-    
     def _get_sql(self, section: str, option: str, **kwargs) -> tp.Tuple[tp.Any, bool]:
         if not SQL_EXTENSION_ENABLED or (not self.has_section('databaseconfig')) \
                 or (not self.has_option('databaseconfig', 'dbconnectstring')):
             return None, False
-        
+
         connectstring = self.parentget('databaseconfig', 'dbconnectstring')
         if connectstring.strip() == '':
             return None, False
-        
+
         query = self.parentget('databaseconfig', 'sql')
         if query.strip() == '':
             return None, False
-        
+
         if section in self.sectioncache:
             return self._get_cached(section, option)
-        
+
         if (self.has_section('databaseconfig')) and (self.has_option('databaseconfig', 'globalscope')):
             globalscope = self.parentget('databaseconfig', 'globalscope')
         else:
             globalscope = None
-        
+
         session = get_session(connectstring)
         sqlvalues = {
             'section': section,
@@ -444,7 +462,7 @@ class DBConfig(FuConfigParser):
         }
         suspect = self.suspect()
         default_template_values(self.suspect(), sqlvalues)
-        
+
         result = None
         try:
             #self.logger.debug("Executing query '%s' with vars %s"%(query,sqlvalues))
@@ -453,7 +471,7 @@ class DBConfig(FuConfigParser):
             trb = traceback.format_exc()
             self.logger.error(f'{suspect.id} Error getting database config override: {trb}')
         session.remove()
-        
+
         cache_ttl = self.parentgetint('databaseconfig', 'value_cache_ttl', fallback=30)
         if result is not None:
             value = result[0]
@@ -464,8 +482,7 @@ class DBConfig(FuConfigParser):
             self.cache.put_cache(self._mkcachekey(False, f'%{suspect.to_domain}', section, option), True, cache_ttl)
             self.cache.put_cache(self._mkcachekey(False, globalscope or '$GLOBAL', section, option), True, cache_ttl)
             return None, False
-    
-    
+
     def _get_rest(self, section: str, option: tp.Optional[str], **kwargs) -> tp.Tuple[tp.Any, bool]:
         if (not REQUESTS_AVAILABLE) or (not self.has_section('databaseconfig')) \
                 or (not self.has_option('databaseconfig', 'restapi_uri')) \
@@ -474,12 +491,15 @@ class DBConfig(FuConfigParser):
                 return self.parentget(section, option, **kwargs), False
             except (configparser.NoSectionError, configparser.NoOptionError):
                 return None, False
-        
+
         suspect = self.suspect()
         restapi_uri = self.parentget('databaseconfig', 'restapi_uri', resolve_env=True)
         restapi_endpoint = self.parentget('databaseconfig', 'restapi_endpoint')
         if not restapi_uri or not restapi_endpoint:
             return None, False
+        if suspect is None:
+            raise RESTAPIError('suspect not set')
+
         restapi_headers = self.parentgetlist('databaseconfig', 'restapi_headers', resolve_env=True)
         restapi_verify = self.parentgetboolean('databaseconfig', 'restapi_verify', fallback=True)
         restapi_timeout = self.parentgetfloat('databaseconfig', 'restapi_timeout', fallback=10)
@@ -488,20 +508,20 @@ class DBConfig(FuConfigParser):
             globalscope = self.parentget('databaseconfig', 'globalscope')
         else:
             globalscope = None
-        headers = {'user-agent': 'fuglu', 'accept': 'application/json', }
+        headers = {'user-agent': f'Fuglu/{get_main_version().strip()}', 'accept': 'application/json', }
         for item in restapi_headers:
             if not ':' in item:
                 self.logger.warning(f'{suspect.id} invalid header definition {item}')
             else:
                 hdr, hdrval = item.split(':', 1)
                 headers[hdr] = hdrval
-        
+
         if not restapi_uri.endswith('/'):
             restapi_uri = f'{restapi_uri}/'
         restapi_uri = f'{restapi_uri}{restapi_endpoint}'
         if not restapi_uri.endswith('/'):
             restapi_uri = f'{restapi_uri}/'
-        
+
         value = None
         scopes = []
         if suspect.to_address and '@' in suspect.to_address:
@@ -510,20 +530,20 @@ class DBConfig(FuConfigParser):
             scopes.append(f'%{suspect.to_domain.lower()}')
         if globalscope:
             scopes.append(globalscope)
-            
+
         content = None
         if restapi_cachettl > 0:
             content = self.cache.get_cache(f'rest-{restapi_uri}')
         if content is None:
-            restapi = RESTAPIConfig()
+            restapi = RESTAPIConfig(config=FuConfigParser(), suspectid=suspect.id)
             # returns data for all scopes from this endpoint
-            content = restapi._get(restapi_uri, headers, suspect.id, restapi_verify, restapi_timeout)
+            content = restapi._call('GET', None, restapi_uri, headers, restapi_verify, restapi_timeout)
             self.logger.debug(f'{suspect.id} fetched restapi data')
             if restapi_cachettl > 0:
                 self.cache.put_cache(f'rest-{restapi_uri}', content, restapi_cachettl)
         else:
             self.logger.debug(f'{suspect.id} using cached restapi data')
-        
+
         alldata = {}
         for scope in scopes:
             scopedata = content.get(scope, {})
@@ -542,21 +562,20 @@ class DBConfig(FuConfigParser):
         if option is None:
             return alldata, False
         return value, False
-    
-    
+
     def _get_yaml(self, section: str, option: tp.Optional[str], **kwargs) -> tp.Tuple[tp.Any, bool]:
         """
         get config override value from yaml file. yaml file should look like this:
         Section:
           option:
             scope: value
-        
+
         e.g.
         FileTypePlugin:
           sendbounce:
             user@example.com: false
         """
-        
+
         if (not YAML_AVAILABLE) or (not self.has_section('databaseconfig')) \
                 or (not self.has_option('databaseconfig', 'yaml_filepath')):
             try:
@@ -577,7 +596,7 @@ class DBConfig(FuConfigParser):
                 return self.parentget(section, option, **kwargs), False
             except (configparser.NoSectionError, configparser.NoOptionError):
                 return None, False
-        
+
         if not yaml_filepath in self.yamldata:
             with open(yaml_filepath, 'r') as fp:
                 self.yamldata[yaml_filepath] = yaml.load(fp, Loader=yaml.FullLoader)
@@ -595,7 +614,7 @@ class DBConfig(FuConfigParser):
                     except KeyError:
                         alldata[s.lower()] = {o.lower: v}
             return alldata, False
-        
+
         scopes = []
         if suspect.to_address and '@' in suspect.to_address:
             scopes.append(suspect.to_address.lower())
@@ -603,7 +622,7 @@ class DBConfig(FuConfigParser):
             scopes.append(suspect.to_domain.lower())
         if not globalscope:
             scopes.append(globalscope)
-            
+
         value = None
         for scope in scopes:
             sectiondata = self.yamldata[yaml_filepath].get(section, {})
@@ -612,18 +631,17 @@ class DBConfig(FuConfigParser):
                 value = str(value)
                 break
         return value, False
-    
-    
+
     def _get_cache(self, section: str, option: str, **kwargs) -> tp.Tuple[tp.Any, bool]:
         value = None
         negcache = False
         suspect = self.suspect()
-        
+
         if section in self.sectioncache:
             value = self._get_cached(section, option)
             if value is not None:
                 return value, negcache
-        
+
         scopes = []
         if suspect.to_address and '@' in suspect.to_address:
             scopes.append(suspect.to_address.lower())
@@ -633,34 +651,32 @@ class DBConfig(FuConfigParser):
         globalscope = self.parentget('databaseconfig', 'globalscope', fallback='$GLOBAL')
         if globalscope:
             scopes.append(globalscope)
-        
+
         for scope in scopes:
             value = self.cache.get_cache(self._mkcachekey(True, scope, section, option))
             if value is None:
-                negcache = self.cache.get_cache(self._mkcachekey(False, scope, section, option)) is True # True if negcached, False if not negcached
+                negcache = self.cache.get_cache(self._mkcachekey(False, scope, section, option)) is True  # True if negcached, False if not negcached
         return value, negcache
-    
-    
-    def _mkcachekey(self, pos:bool, scope:str, section: str, option: str) -> str:
+
+    def _mkcachekey(self, pos: bool, scope: str, section: str, option: str) -> str:
         if pos:
             prefix = 'dbconfig-pos'
         else:
             prefix = 'dbconfig-neg'
         return f'{prefix}-{scope}-{section}-{option}'
-    
-    
+
     def parentget(self, section: str, option: str, **kwargs) -> str:
         return super().get(section, option, **kwargs)
-    
+
     def parentgetboolean(self, section: str, option: str, **kwargs) -> bool:
         return self._convert_to_boolean(super().get(section, option, **kwargs))
-    
+
     def parentgetfloat(self, section: str, option: str, **kwargs) -> float:
         return float(super().get(section, option, **kwargs))
-    
+
     def parentgetint(self, section: str, option: str, **kwargs) -> int:
         return int(super().get(section, option, **kwargs))
-    
+
     def parentgetlist(self, section: str, option: str, **kwargs) -> tp.List[str]:
         return super().getlist(section, option, **kwargs)
 
@@ -671,10 +687,10 @@ def get_domain_setting(domain, dbconnection, sqlquery, cache, cachename, default
     if logger is None:
         logger = logging.getLogger()
 
-    cachekey = '%s-%s' % (cachename, domain)
+    cachekey = f'{cachename}-{domain}'
     cached = cache.get_cache(cachekey)
     if cached is not None:
-        logger.debug("got cached setting for %s" % domain)
+        logger.debug(f"got cached setting for {domain}")
         return cached
 
     settings = default_value
@@ -686,15 +702,15 @@ def get_domain_setting(domain, dbconnection, sqlquery, cache, cachename, default
         dom = session.execute(sqlquery, {'domain': domain}).fetchall()
 
         if not dom or not dom[0] or len(dom[0]) == 0:
-            logger.debug("Can not load domain setting - domain %s not found. Using default settings." % domain)
+            logger.debug(f"Can not load domain setting - domain {domain} not found. Using default settings.")
         else:
             settings = dom[0][0]
 
         session.remove()
 
     except Exception as e:
-        logger.error("Exception while loading setting for %s : %s" % (domain, str(e)))
+        logger.error(f"Exception while loading setting for {domain} : {e.__class__.__name__}: {str(e)}")
 
     cache.put_cache(cachekey, settings)
-    logger.debug("refreshed setting for %s" % domain)
+    logger.debug(f"refreshed setting for {domain}")
     return settings
