@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Fumail Project
+#   Copyright Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@ import signal
 import sys
 
 
-def logFactoryProcess(listenerQueue,logQueue):
+def logFactoryProcess(listenerQueue, logQueue):
     """
     This process is responsible for creating the logger listener for the given queue.
     This Listener process is responsible for actually handling all the log messages in the queue.
@@ -43,8 +43,8 @@ def logFactoryProcess(listenerQueue,logQueue):
     received from other processes will not necessarily apply the correct debug level if this has been changed.
 
     The only "clean" solution found so far is to always create the logging process from a clean process which does not
-    have any logging structure stored yet. Therefore, the logFactoryProcess is created at the very beginning and it
-    can produce new clean logging processes later that will properly setup with all propagation and level changes.
+    have any logging structure stored yet. Therefore, the logFactoryProcess is created at the very beginning, and it
+    can produce new clean logging processes later that will properly set up with all propagation and level changes.
 
     Args:
         listenerQueue (multiprocessing.Queue): Queue where the logFactoryProcess will receive a configuration for
@@ -60,32 +60,33 @@ def logFactoryProcess(listenerQueue,logQueue):
     loggerProcess = None
     while True:
         try:
-            logConfig = listenerQueue.get()
+            log_config = listenerQueue.get()
 
             # if existing stop last logger process
             if loggerProcess:
                 # try to close the old logger
                 try:
                     logQueue.put_nowait(None)
-                    loggerProcess.join(10) # wait 10 seconds max
+                    loggerProcess.join(10)  # wait 10 seconds max
                 except Exception:
                     loggerProcess.terminate()
                 finally:
                     loggerProcess = None
 
-            if logConfig is None:  # We send this as a sentinel to tell the listener to quit.
+            if log_config is None:  # We send this as a sentinel to tell the listener to quit.
                 break
 
             # create new logger process
-            loggerProcess = multiprocessing.Process(target=listener_process, args=(logConfig,logQueue))
+            loggerProcess = multiprocessing.Process(target=listener_process, args=(log_config, logQueue))
             loggerProcess.daemon = True
             loggerProcess.start()
 
         except KeyboardInterrupt:
             print("logFactoryProcess: Listener process received KeyboardInterrupt")
-            #break
+            # break
         except Exception:
-            import sys, traceback
+            import sys
+            import traceback
             print('LogFactoryProcess: Problem:', file=sys.stderr)
             traceback.print_exc(file=sys.stderr)
 
@@ -94,9 +95,10 @@ def logFactoryProcess(listenerQueue,logQueue):
         # try to close the old logger
         try:
             logQueue.put_nowait(None)
-            loggerProcess.join(10) # wait 10 seconds max
+            loggerProcess.join(10)  # wait 10 seconds max
         except Exception:
-            import sys, traceback
+            import sys
+            import traceback
             loggerProcess.terminate()
             print('LogFactoryProcess: Problem:', file=sys.stderr)
             traceback.print_exc(file=sys.stderr)
@@ -106,6 +108,7 @@ class logConfig(object):
     """
     Conig class to easily distinguish logging configuration for lint, foreground and production (from file)
     """
+
     def __init__(self, lint=False, logConfigFile="none", foreground=False, level=None):
         """
         Setup in lint mode of using a config file
@@ -153,7 +156,7 @@ class logConfig(object):
             logConfig._configure4screen(self._level, timeinformat=self._foreground)
         elif self._configFile != "none":
             if not os.path.exists(self._configFile):
-                raise FileNotFoundError("Logging config file %s does not exist!" % self._configFile)
+                raise FileNotFoundError(f"Logging config file {self._configFile} does not exist!")
 
             logConfig._configure(self._configFile)
         else:
@@ -188,14 +191,14 @@ class logConfig(object):
         root = logging.getLogger()
 
 
-def listener_process(configurer,queue):
+def listener_process(configurer, queue):
     """
     This is the listener process top-level loop: wait for logging events
     (LogRecords) on the queue and handle them, quit when you get a None for a
     LogRecord.
 
     Args:
-        configurer (logConfig): instance lof logConfig class setting up logging on configure call
+        configurer (logConfig): instance of logConfig class setting up logging on configure call
         queue (multiprocessing.Queue): The queue where log messages will be received and processed by this same process
     """
 
@@ -222,7 +225,7 @@ def listener_process(configurer,queue):
                 break
 
             if logLogger:
-                logLogger.debug("Approx queue size: %u, received record to process -> %s"%(queue.qsize(),record))
+                logLogger.debug(f"Approx queue size: {queue.qsize()}, received record to process -> {record}")
 
             if queue.full():
                 root.error("QUEUE IS FULL!!!")
@@ -239,7 +242,7 @@ def listener_process(configurer,queue):
         except KeyboardInterrupt:
             print("listener_process: Listener process received KeyboardInterrupt")
             root.warning("listener_process: Listener received KeyboardInterrupt ")
-            #break
+            # break
         except EOFError:
             print("listener_process: Listener process received EOFError -> continue")
             root.warning("listener_process: Listener received EOFError -> continue ")
@@ -270,12 +273,7 @@ def client_configurer(queue, level="NOTSET"):
     name = createPIDinfo()
 
     if numRootHandlers == 0:
-        try:
-            # Python 3
-            h = logging.handlers.QueueHandler(queue)  # Just the one handler needed
-        except AttributeError:
-            # Python 2
-            h = QueueHandlerPy3Copy(queue)
+        h = logging.handlers.QueueHandler(queue)  # Just the one handler needed
         root.addHandler(h)
         # send all messages
         if level == "DEBUG":
@@ -286,91 +284,24 @@ def client_configurer(queue, level="NOTSET"):
             root.setLevel(logging.ERROR)
         else:
             root.setLevel(logging.NOTSET)
-            root.info("Log-Level \"%s\" not vaild, setting NOTSET" % level)
-        root.info("(%s) Queue handler added to root logger" % name)
+            root.info(f"Log-Level \"{level}\" not valid, setting NOTSET")
+        root.info(f"({name}) Queue handler added to root logger")
     else:
         # on linux config is taken from father process automatically
-        root.info("(%s) Root already has a handler -> not adding Queue handler" % name)
-
-
-#----------------------------------------#
-#-- Copied from Python 3 - handlers.py --#
-#----------------------------------------#
-class QueueHandlerPy3Copy(logging.Handler):
-    """
-    This handler sends events to a queue. Typically, it would be used together
-    with a multiprocessing Queue to centralise logging to file in one process
-    (in a multi-process application), so as to avoid file write contention
-    between processes.
-
-    This code is new in Python 3.2, but this class can be copy pasted into
-    user code for use with earlier Python versions.
-    """
-
-    def __init__(self, queue):
-        """
-        Initialise an instance, using the passed queue.
-        """
-        logging.Handler.__init__(self)
-        self.queue = queue
-
-    def enqueue(self, record):
-        """
-        Enqueue a record.
-
-        The base implementation uses put_nowait. You may want to override
-        this method if you want to use blocking, timeouts or custom queue
-        implementations.
-        """
-        self.queue.put_nowait(record)
-
-    def prepare(self, record):
-        """
-        Prepares a record for queuing. The object returned by this method is
-        enqueued.
-
-        The base implementation formats the record to merge the message
-        and arguments, and removes unpickleable items from the record
-        in-place.
-
-        You might want to override this method if you want to convert
-        the record to a dict or JSON string, or send a modified copy
-        of the record while leaving the original intact.
-        """
-        # The format operation gets traceback text into record.exc_text
-        # (if there's exception data), and also puts the message into
-        # record.message. We can then use this to replace the original
-        # msg + args, as these might be unpickleable. We also zap the
-        # exc_info attribute, as it's no longer needed and, if not None,
-        # will typically not be pickleable.
-        self.format(record)
-        record.msg = record.message
-        record.args = None
-        record.exc_info = None
-        return record
-
-    def emit(self, record):
-        """
-        Emit a record.
-
-        Writes the LogRecord to the queue, preparing it for pickling first.
-        """
-        try:
-            self.enqueue(self.prepare(record))
-        except Exception:
-            self.handleError(record)
+        root.info(f"({name}) Root already has a handler -> not adding Queue handler")
 
 
 def createPIDinfo():
     infoString = ""
     if hasattr(os, 'getppid'):  # only available on Unix
-        infoString += 'parent process: %u, ' % os.getppid()
-    infoString += 'process id: %u' % os.getpid()
+        infoString += f'parent process: {os.getppid()}, '
+    infoString += f'process id: {os.getpid()}'
     return infoString
 
 
 class PrependLoggerMsg(object):
     """Prepend something to all log messages of original logger, for example fuglu id"""
+
     def __init__(self, origlogger, prepend,
                  prependseparator=" ",
                  maxlevel=None,
@@ -385,10 +316,10 @@ class PrependLoggerMsg(object):
         maxlevel = maxlevel if maxlevel is not None else 1000
 
         self.debuglevel = max(min(logging.DEBUG, maxlevel), minlevel)
-        self.infolevel= max(min(logging.INFO, maxlevel), minlevel)
-        self.warninglevel= max(min(logging.WARNING, maxlevel), minlevel)
-        self.errorlevel= max(min(logging.ERROR, maxlevel), minlevel)
-        self.criticallevel= max(min(logging.CRITICAL, maxlevel), minlevel)
+        self.infolevel = max(min(logging.INFO, maxlevel), minlevel)
+        self.warninglevel = max(min(logging.WARNING, maxlevel), minlevel)
+        self.errorlevel = max(min(logging.ERROR, maxlevel), minlevel)
+        self.criticallevel = max(min(logging.CRITICAL, maxlevel), minlevel)
 
         self.origroutines = {
             logging.DEBUG: self._origlogger.debug,
@@ -399,47 +330,48 @@ class PrependLoggerMsg(object):
         }
 
     def debug(self, msg, *args, **kwargs):
-        self.origroutines[self.debuglevel]("%s%s%s" % (self.prepend, self.prependseparator, msg), *args, **kwargs)
+        self.origroutines[self.debuglevel](f'{self.prepend}{self.prependseparator}{msg}', *args, **kwargs)
 
     def info(self, msg, *args, **kwargs):
-        self.origroutines[self.infolevel]("%s%s%s" % (self.prepend, self.prependseparator, msg), *args, **kwargs)
+        self.origroutines[self.infolevel](f'{self.prepend}{self.prependseparator}{msg}', *args, **kwargs)
 
     def warning(self, msg, *args, **kwargs):
-        self.origroutines[self.warninglevel]("%s%s%s" % (self.prepend, self.prependseparator, msg), *args, **kwargs)
+        self.origroutines[self.warninglevel](f'{self.prepend}{self.prependseparator}{msg}', *args, **kwargs)
 
     def error(self, msg, *args, **kwargs):
-        self.origroutines[self.errorlevel]("%s%s%s" % (self.prepend, self.prependseparator, msg), *args, **kwargs)
+        self.origroutines[self.errorlevel](f'{self.prepend}{self.prependseparator}{msg}', *args, **kwargs)
 
     def critical(self, msg, *args, **kwargs):
-        self.origroutines[self.criticallevel]("%s%s%s" % (self.prepend, self.prependseparator, msg), *args, **kwargs)
+        self.origroutines[self.criticallevel](f'{self.prepend}{self.prependseparator}{msg}', *args, **kwargs)
 
     def exception(self, msg, *args, **kwargs):
-        self._origlogger.exception("%s%s%s" % (self.prepend, self.prependseparator, msg), *args, **kwargs)
+        self._origlogger.exception(f'{self.prepend}{self.prependseparator}{msg}', *args, **kwargs)
 
     def __getattr__(self, attr):
         """if attribute doesn't exist __getattr__ is called, redirect to wrapped logger"""
         return getattr(self._origlogger, attr)
 
 
-class LoggingContext(object):     
+class LoggingContext(object):
     """to be used for 'with'-statements to temporarily change a logger"""
-    def __init__(self, logger, level=None, handler=None, close=True):     
-        self.logger = logger     
-        self.level = level     
-        self.handler = handler     
-        self.close = close     
-    
-    def __enter__(self):     
-        if self.level is not None:     
-            self.old_level = self.logger.level     
-            self.logger.setLevel(self.level)     
-        if self.handler:     
-            self.logger.addHandler(self.handler)     
-    
-    def __exit__(self, et, ev, tb):     
-        if self.level is not None:     
-            self.logger.setLevel(self.old_level)     
-        if self.handler:     
-            self.logger.removeHandler(self.handler)     
-        if self.handler and self.close:     
+
+    def __init__(self, logger, level=None, handler=None, close=True):
+        self.logger = logger
+        self.level = level
+        self.handler = handler
+        self.close = close
+
+    def __enter__(self):
+        if self.level is not None:
+            self.old_level = self.logger.level
+            self.logger.setLevel(self.level)
+        if self.handler:
+            self.logger.addHandler(self.handler)
+
+    def __exit__(self, et, ev, tb):
+        if self.level is not None:
+            self.logger.setLevel(self.old_level)
+        if self.handler:
+            self.logger.removeHandler(self.handler)
+        if self.handler and self.close:
             self.handler.close()
