@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Fumail Project
+#   Copyright Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -53,7 +53,7 @@ DEFECT_ATTPARSEERROR = "<<ATTPARSEERROR>>"
 # tuple containing errors preventing correct decoding of buffer
 BAD_DEFECTS = (email.errors.InvalidBase64LengthDefect,)
 # The contenttype returned for an attachment with parsing error
-ATTPARSEERROR_CONTENTTYPE= "application/unknown"
+ATTPARSEERROR_CONTENTTYPE = "application/unknown"
 
 
 class NoExtractInfo(object):
@@ -100,14 +100,14 @@ class NoExtractInfo(object):
             minus_filters = []
 
         output = []
-        for filter in plus_filters:
-            assert filter in NoExtractInfo.valid_causes
-        for filter in minus_filters:
-            assert filter in NoExtractInfo.valid_causes
+        for fltr in plus_filters:
+            assert fltr in NoExtractInfo.valid_causes
+        for fltr in minus_filters:
+            assert fltr in NoExtractInfo.valid_causes
 
-        for filter in plus_filters:
-            if filter not in minus_filters:
-                output.extend(self.infolists[filter])
+        for fltr in plus_filters:
+            if fltr not in minus_filters:
+                output.extend(self.infolists[fltr])
         return output
 
 
@@ -116,12 +116,11 @@ class Mailattachment(Cachelimits):
     Mail attachment object or a file contained in the attachment.
     """
     objectCounter = 0
-    
+
     HASH_DEFAULTS = ['md5', 'sha1', 'sha256']
     HASHENC_HEX = 'hex'
     HASHENC_B64 = 'b64'
     HASHENC_B32 = 'b32'
-    
 
     def __init__(self, buffer, filename, mgr, fugluid, filesize=None, in_obj=None, contenttype_mime=None,
                  maintype_mime=None, subtype_mime=None, ismultipart_mime=None, content_charset_mime=None,
@@ -151,13 +150,11 @@ class Mailattachment(Cachelimits):
         self.filesize = filesize
         self.buffer = force_bString(buffer)
         self._buffer_archobj = {}
+        self._buffer_noextractinfo = {}
         self.fugluid = fugluid
         self.defects = defects if defects else []  # make an empty list for None
         self.filename_generated = filename_generated
-
-        myclass = self.__class__.__name__
-        loggername = "fuglu.%s" % myclass
-        self.logger = logging.getLogger(loggername)
+        self.logger = logging.getLogger(f"fuglu.{self.__class__.__name__}")
 
         # use weak reference to avoid cyclic dependency
         self.in_obj = weakref.ref(in_obj) if in_obj is not None else None
@@ -171,10 +168,13 @@ class Mailattachment(Cachelimits):
         self.is_inline = is_inline
 
         # use weak reference to avoid cyclic dependency
-        self._mgr = weakref.ref(mgr) if mgr is not None else None # keep only weak reference
+        self._mgr = weakref.ref(mgr) if mgr is not None else None  # keep only weak reference
 
         # store exception creating archive handle (if any)
         self._archive_handle_exc = None
+        
+        # archive extension for internal use
+        self._arext = None
 
         # try to increment object counter in manager for each object created.
         # this helps debugging and testing caching...
@@ -182,10 +182,9 @@ class Mailattachment(Cachelimits):
             self._mgr()._increment_ma_objects()
         except (AttributeError, TypeError):
             pass
-    
-    
-    def content_fname_check(self,maintype=None,ismultipart=None,subtype=None,contenttype=None,contenttype_start=None,
-                            name_end=None,contenttype_contains=None,name_contains=None,
+
+    def content_fname_check(self, maintype=None, ismultipart=None, subtype=None, contenttype=None, contenttype_start=None,
+                            name_end=None, contenttype_contains=None, name_contains=None,
                             ctype=None, ctype_start=None, ctype_contains=None):
         """
         Test content or filename for options or a set of options. All inputs except 'ismultipart' allow
@@ -194,7 +193,7 @@ class Mailattachment(Cachelimits):
         """
         try:
             if maintype is not None:
-                if isinstance(maintype,(list,tuple)):
+                if isinstance(maintype, (list, tuple)):
                     if not self.maintype_mime in maintype:
                         return False
                 else:
@@ -206,7 +205,7 @@ class Mailattachment(Cachelimits):
                     return False
 
             if subtype is not None:
-                if isinstance(subtype,(list,tuple)):
+                if isinstance(subtype, (list, tuple)):
                     if not self.subtype_mime in subtype:
                         return False
                 else:
@@ -214,7 +213,7 @@ class Mailattachment(Cachelimits):
                         return False
 
             if contenttype is not None:
-                if isinstance(contenttype,(list,tuple)):
+                if isinstance(contenttype, (list, tuple)):
                     if not self.contenttype_mime in contenttype:
                         return False
                 else:
@@ -222,7 +221,7 @@ class Mailattachment(Cachelimits):
                         return False
 
             if ctype is not None:
-                if isinstance(ctype,(list,tuple)):
+                if isinstance(ctype, (list, tuple)):
                     if not self.contenttype in ctype:
                         return False
                 else:
@@ -230,7 +229,7 @@ class Mailattachment(Cachelimits):
                         return False
 
             if contenttype_start is not None:
-                if isinstance(contenttype_start,list):
+                if isinstance(contenttype_start, list):
                     if not self.contenttype_mime.startswith(tuple(contenttype_start)):
                         return False
                 else:
@@ -238,7 +237,7 @@ class Mailattachment(Cachelimits):
                         return False
 
             if ctype_start is not None:
-                if isinstance(ctype_start,list):
+                if isinstance(ctype_start, list):
                     if not self.contenttype.startswith(tuple(ctype_start)):
                         return False
                 else:
@@ -246,7 +245,7 @@ class Mailattachment(Cachelimits):
                         return False
 
             if name_end is not None:
-                if isinstance(name_end,list):
+                if isinstance(name_end, list):
                     if not self.filename.endswith(tuple(name_end)):
                         return False
                 else:
@@ -254,7 +253,7 @@ class Mailattachment(Cachelimits):
                         return False
 
             if contenttype_contains is not None:
-                if isinstance(contenttype_contains,(list,tuple)):
+                if isinstance(contenttype_contains, (list, tuple)):
                     if not any((a in self.contenttype_mime for a in contenttype_contains)):
                         return False
                 else:
@@ -262,7 +261,7 @@ class Mailattachment(Cachelimits):
                         return False
 
             if ctype_contains is not None:
-                if isinstance(ctype_contains,(list,tuple)):
+                if isinstance(ctype_contains, (list, tuple)):
                     if not any((a in self.contenttype for a in ctype_contains)):
                         return False
                 else:
@@ -270,7 +269,7 @@ class Mailattachment(Cachelimits):
                         return False
 
             if name_contains is not None:
-                if isinstance(name_contains,(list,tuple)):
+                if isinstance(name_contains, (list, tuple)):
                     if not any((a in self.filename for a in name_contains)):
                         return False
                 else:
@@ -281,9 +280,8 @@ class Mailattachment(Cachelimits):
             return False
 
         return True
-    
-    
-    @smart_cached_property(inputs=['buffer','in_obj','content_charset_mime'])
+
+    @smart_cached_property(inputs=['buffer', 'in_obj', 'content_charset_mime'])
     def decoded_buffer_text(self):
         """
         (Cached Member Function)
@@ -308,7 +306,6 @@ class Mailattachment(Cachelimits):
 
         return decoded_buffer
 
-
     def _mkchecksum(self, value, method, encoding=HASHENC_HEX, strip=False):
         """
         generate checksum of value using given method and encoding
@@ -322,7 +319,7 @@ class Mailattachment(Cachelimits):
             raise KeyError(f'unsupported hash algorithm: {method}')
         if not encoding in [self.HASHENC_HEX, self.HASHENC_B64, self.HASHENC_B32]:
             raise KeyError(f'unsupported hash encoding: {encoding}')
-    
+
         hasher = hashlib.new(method, force_bString(value))
         if encoding == self.HASHENC_HEX:
             myhash = hasher.hexdigest()
@@ -332,13 +329,12 @@ class Mailattachment(Cachelimits):
             myhash = base64.b32encode(hasher.digest()).decode()
         else:
             myhash = ''
-    
+
         if strip:
             myhash = myhash.rstrip('=')
-    
+
         return myhash
-    
-    
+
     @smart_cached_memberfunc(inputs=['buffer'])
     def get_checksum(self, method, encoding=HASHENC_HEX, strip=False):
         """
@@ -357,11 +353,9 @@ class Mailattachment(Cachelimits):
             try:
                 csum = self._mkchecksum(self.buffer, method, encoding, strip)
             except KeyError:
-                self.logger.error("checksum method %s not valid! Options are [%s]"
-                                  % (method, ",".join(list(hashlib.algorithms_guaranteed))))
+                self.logger.error(f"{self.fugluid} checksum method {method} not valid! Options are [{','.join(list(hashlib.algorithms_guaranteed))}]")
         return csum
-    
-    
+
     @smart_cached_memberfunc(inputs=['buffer'])
     def get_checksumdict(self, methods=(), encoding=HASHENC_HEX, strip=False):
         """Create a dict for all checksum methods given (or sane default if none)"""
@@ -377,8 +371,7 @@ class Mailattachment(Cachelimits):
         for m in mlist:
             checksumdict[m] = self.get_checksum(m, encoding, strip)
         return checksumdict
-    
-    
+
     @smart_cached_property(inputs=['buffer'])
     def parsed(self):
         """
@@ -388,8 +381,7 @@ class Mailattachment(Cachelimits):
             (bool): true if attachment was parsed, so no decoding error
         """
         return DEFECT_ATTPARSEERROR not in self.defects
-    
-    
+
     @smart_cached_property(inputs=['buffer'])
     def contenttype(self):
         """
@@ -405,16 +397,15 @@ class Mailattachment(Cachelimits):
         contenttype_magic = None
         if self.buffer is not None and filetype_handler.available():
             if DEFECT_ATTPARSEERROR in self.defects:
-                # if there are defects then the buffer was not decoded and it doesn't
+                # if there are defects then the buffer was not decoded, and it doesn't
                 # make sense to return a type by filemagic. For example, if a base64
                 # encoded content can not be decoded filemagic will detect it as text
                 return ATTPARSEERROR_CONTENTTYPE
             else:
                 contenttype_magic = filetype_handler.get_buffertype(self.buffer)
         return contenttype_magic
-    
-    
-    @smart_cached_property(inputs=['contenttype','filename'])
+
+    @smart_cached_property(inputs=['contenttype', 'filename'])
     def archive_type(self):
         """
         (Cached Property-Getter)
@@ -429,7 +420,6 @@ class Mailattachment(Cachelimits):
             (str): Archive type if object is an archive, None otherwise
         """
 
-        self._arext = None
 
         # try guessing the archive type based on magic content type first
         archive_type = Archivehandle.archive_type_from_content_type(self.contenttype)
@@ -438,21 +428,18 @@ class Mailattachment(Cachelimits):
         if archive_type is None:
             # sort by length, so tar.gz is checked before .gz
             for arext in Archivehandle.avail_archive_extensions_list:
-
-                if self.filename.lower().endswith('.%s' % arext):
+                if self.filename.lower().endswith(f'.{arext}'):
                     archive_type = Archivehandle.avail_archive_extensions[arext]
                     # store archive extension for internal use
                     self._arext = arext
                     break
         return archive_type
-    
-    
+
     @smart_cached_memberfunc(inputs=['archive_type'])
     def atype_fromext(self):
         """True if extension was used to determine archive type"""
         return self._arext
-    
-    
+
     @smart_cached_property(inputs=['archive_type'])
     def is_archive(self):
         """
@@ -483,21 +470,21 @@ class Mailattachment(Cachelimits):
         """
         # try to extract a file
         if self.is_archive and self.fileslist_archive:
-                fname = self.fileslist_archive[0]
-                noextractinfo = NoExtractInfo()
-                _ = self.get_archive_obj(fname, None, noextractinfo=noextractinfo)
-                noextractlist = noextractinfo.get_filtered(plus_filters=["archivehandle"])
-                return any(["password" in item[1].lower() for item in noextractlist])
+            fname = self.fileslist_archive[0]
+            noextractinfo = NoExtractInfo()
+            _ = self.get_archive_obj(fname, None, noextractinfo=noextractinfo)
+            noextractlist = noextractinfo.get_filtered(plus_filters=["archivehandle"])
+            return any(["password" in item[1].lower() for item in noextractlist])
         return False
 
-    def get_fileslist(self, levelin: tp.Optional[int], levelmax: tp.Optional[int],
-                      maxsize_extract: tp.Optional[int], maxfiles_extract: tp.Optional[int]):
+    def get_fileslist(self, levelin: tp.Optional[int] = None, levelmax: tp.Optional[int] = None,
+                      maxsize_extract: tp.Optional[int] = None, maxfiles_extract: tp.Optional[int] = None):
         """
         Get a list of files contained in this archive (recursively extracting archives)
         or the current filename if this is not an archive.
 
         Don't cache here, "fileslist_archive" is only available for
-        archives. If this is put as dependency then there will be a
+        archives. If this is put as dependency then there will be an
         archive handler applied on a non-archive which throws an
         exception
 
@@ -530,14 +517,14 @@ class Mailattachment(Cachelimits):
 
         return filelist
 
-
-    def get_objectlist(self,levelin, levelmax, maxsize_extract, maxfiles_extract, noextractinfo=None, include_parents=False):
+    def get_objectlist(self, levelin:int=0, levelmax:int=None, maxsize_extract: tp.Optional[int]=None,
+                       maxfiles_extract: tp.Optional[int]=None, noextractinfo:NoExtractInfo=None, include_parents:bool=False):
         """
         Get a list of file objects contained in this archive (recursively extracting archives)
         or the current object if this is not an archive.
 
         Don't cache here, "fileslist_archive" is only available for
-        archives. If this is put as dependency then there will be a
+        archives. If this is put as dependency then there will be an
         archive handler applied on a non-archive which throws an
         exception
 
@@ -545,11 +532,10 @@ class Mailattachment(Cachelimits):
                    Further extraction depends on the input recursion level.
 
         Args:
-            levelin  (in): Current recursive level
-            levelmax (in): Max recursive archive level up to which archives are extracted
+            levelin  (int): Current recursive level
+            levelmax (int): Max recursive archive level up to which archives are extracted
             maxsize_extract (int, None): Maximum size that will be extracted to further go into archive
-
-        Keyword Args:
+            maxfiles_extract (int, None): Maximum number of files that will be extracted
             noextractinfo (NoExtractInfo): stores info why object was not extracted
             include_parents (bool): True means extract the archives (depending on level) but return archive as well,
                                     not only the extracted files
@@ -559,7 +545,7 @@ class Mailattachment(Cachelimits):
         """
 
         if include_parents:
-            # if include_parents is true than the current object has to be part of the list whatever...
+            # if include_parents is true then the current object has to be part of the list whatever...
             newlist = [self]
         else:
             newlist = []
@@ -604,10 +590,9 @@ class Mailattachment(Cachelimits):
             newlist = [self]
 
         return newlist
-    
-    
-    @smart_cached_memberfunc(inputs=['fileslist_archive','archive_handle','is_archive'])
-    def get_archive_flist(self, maxsize_extract=None, inverse=False):
+
+    @smart_cached_memberfunc(inputs=['fileslist_archive', 'archive_handle', 'is_archive'])
+    def get_archive_flist(self, maxsize_extract: tp.Optional[int]=None, inverse:bool=False):
         """
         Get list of all filenames for objects in archive if within size limits. The list
         is consistent with the object list that would be returned by 'get_archive_objlist' or
@@ -634,9 +619,8 @@ class Mailattachment(Cachelimits):
                 else:
                     inverselist.append(fname)
         return inverselist if inverse else matchlist
-    
-    
-    def get_archive_objlist(self, maxsize_extract=None, noextractinfo=None):
+
+    def get_archive_objlist(self, maxsize_extract: tp.Optional[int]=None, noextractinfo:NoExtractInfo=None):
         """
         Get list of all object in archive (extracts the archive) if within size limits.
         If the file is already extracted the file will be returned even if the size is
@@ -649,8 +633,6 @@ class Mailattachment(Cachelimits):
 
         Args:
             maxsize_extract (int): Maximum size that will be extracted
-
-        Keyword Args:
             noextractinfo (NoExtractInfo): stores info why object was not extracted
 
         Returns:
@@ -664,17 +646,14 @@ class Mailattachment(Cachelimits):
                 if attach_obj is not None:
                     newlist.append(attach_obj)
         return newlist
-    
-    
-    def get_archive_obj(self, fname, maxsize_extract, noextractinfo=None):
+
+    def get_archive_obj(self, fname:str, maxsize_extract: tp.Optional[int]=None, noextractinfo:NoExtractInfo=None):
         """
         Get cached archive object or create a new one.
 
         Args:
             fname (str): filename of file object
             maxsize_extract (int): Maximum size that will be extracted
-
-        Keyword Args:
             noextractinfo (NoExtractInfo): stores info why object was not extracted
 
         Returns:
@@ -686,19 +665,31 @@ class Mailattachment(Cachelimits):
         else:
             try:
                 obj = self._buffer_archobj[fname]
+                einfo = self._buffer_noextractinfo.get(fname)
+                if einfo:
+                    if noextractinfo:
+                        noextractinfo.append(*einfo)
             except KeyError:
                 try:
                     filesize = self.archive_handle.filesize(fname)
                 except Exception as e:
                     if noextractinfo is not None:
-                        noextractinfo.append(fname, "archivehandle", "exception: %s" % force_uString(e))
+                        noextractinfo.append(fname, "archivehandle", f"exception: {e.__class__.__name__}: {str(e)}")
+                    # cache answer despite problem
+                    if self._mgr:
+                        self._buffer_archobj[fname] = None
+                        self._buffer_noextractinfo[fname] = fname, "archivehandle", f"exception: {force_uString(e)}"
                     return None
 
                 try:
-                    buffer = self.archive_handle.extract(fname,maxsize_extract)
+                    buffer = self.archive_handle.extract(fname, maxsize_extract)
                 except Exception as e:
                     if noextractinfo is not None:
-                        noextractinfo.append(fname, "archivehandle", "exception: %s" % force_uString(e))
+                        noextractinfo.append(fname, "archivehandle", f"exception: {e.__class__.__name__}: {str(e)}")
+                    # cache answer despite problem
+                    if self._mgr:
+                        self._buffer_archobj[fname] = None
+                        self._buffer_noextractinfo[fname] = fname, "archivehandle", f"exception: {e.__class__.__name__}: {str(e)}"
                     return None
 
                 if buffer is None:
@@ -707,8 +698,15 @@ class Mailattachment(Cachelimits):
                                 and filesize is not None \
                                 and filesize > maxsize_extract:
                             noextractinfo.append(fname, "extractsize", "not extracted: %u > %u" % (filesize, maxsize_extract))
+                            if self._mgr:
+                                self._buffer_noextractinfo[fname] = fname, "extractsize", "not extracted: %u > %u" % (filesize, maxsize_extract)
                         else:
                             noextractinfo.append(fname, "archivehandle", "(no info)")
+                        if self._mgr:
+                            self._buffer_noextractinfo[fname] = fname, "archivehandle", "(no info)"
+                    # cache answer despite problem
+                    if self._mgr:
+                        self._buffer_archobj[fname] = None
                     return None
                 obj = Mailattachment(buffer, fname, self._mgr() if self._mgr else None, self.fugluid, filesize=filesize, in_obj=self,
                                      is_attachment=self.is_attachment, is_inline=self.is_inline)
@@ -718,12 +716,11 @@ class Mailattachment(Cachelimits):
                 if self._mgr and self._mgr().use_caching(filesize):
                     self._buffer_archobj[fname] = obj
             return obj
-    
-    
-    @smart_cached_memberfunc(inputs=['fileslist_archive','archive_handle'])
+
+    @smart_cached_memberfunc(inputs=['fileslist_archive', 'archive_handle'])
     def get_fileslist_arch(self, levelin: int, levelmax: tp.Optional[int],
-                           maxsize_extract: tp.Optional[int],
-                           maxfiles_extract: tp.Optional[int]):
+                           maxsize_extract: tp.Optional[int] = None,
+                           maxfiles_extract: tp.Optional[int] = None):
         """
         Get a list of filenames contained in this archive (recursively extracting archives)
         or the current object filename if this is not an archive.
@@ -756,9 +753,8 @@ class Mailattachment(Cachelimits):
                     # then return filename directly
                     newlist.append(fname)
         return newlist
-    
-    
-    @smart_cached_property(inputs=['archive_type','buffer'])
+
+    @smart_cached_property(inputs=['archive_type', 'buffer'])
     def archive_handle(self):
         """
         (Cached Property-Getter)
@@ -776,10 +772,11 @@ class Mailattachment(Cachelimits):
         # make sure there's no buffered archive object when
         # the archive handle is created (or overwritten)
         self._buffer_archobj = {}
+        self._buffer_noextractinfo = {}
         handle = None
         if self.buffer is not None:
             try:
-                handle = Archivehandle(self.archive_type, BytesIO(self.buffer),archivename=self.filename)
+                handle = Archivehandle(self.archive_type, BytesIO(self.buffer), archivename=self.filename)
             except Exception as e:
                 self.logger.warning(f"{self.fugluid}, Problem creating Archivehandle for file: "
                                     f"{self.filename} using archive handler {str(self.archive_type)} "
@@ -810,14 +807,12 @@ class Mailattachment(Cachelimits):
             return []
         else:
             return self.archive_handle.namelist()
-    
-    
+
     @smart_cached_property(inputs=["in_obj"])
     def in_archive(self):
         """Return if this object is in an archive"""
         return True if self.parent_archives else False
-    
-    
+
     @smart_cached_property(inputs=["in_obj"])
     def parent_archives(self):
         """
@@ -836,31 +831,30 @@ class Mailattachment(Cachelimits):
             parentsList.append(upstream_obj().in_obj)
             upstream_obj = upstream_obj().in_obj
         return parentsList
-    
-    
+
     def location(self):
-        """Print the location of the the file in the archive tree"""
+        """Print the location of the file in the archive tree"""
         element_of = " \u2208 "
         location = self.filename
         if self.parent_archives:
             location += element_of + element_of.join(["{" + obj().filename + "}" for obj in self.parent_archives])
         return location
 
-    
     _re_ws = re.compile(r'\s')
+
     @smart_cached_memberfunc(inputs=["filename"])
     def get_mangled_filename(self, prefix: str = '', maxchars: int = 15, keepending: bool = False):
-        first=self.filename.split('.')[0]
-        #replace spaces with underscore
+        first = self.filename.split('.')[0]
+        # replace spaces with underscore
         first = self._re_ws.sub('_', first)
-        #remove all non alnum chars
-        first=''.join([x for x in first if x in string.ascii_letters or x in string.digits])
-        if first.strip()=='' or first.strip()=='filename': #renattach renames to 'filename'
-            first='upn-%s' % random.randint(1000,9999)
+        # remove all non alnum chars
+        first = ''.join([x for x in first if x in string.ascii_letters or x in string.digits])
+        if first.strip() == '' or first.strip() == 'filename':  # renattach renames to 'filename'
+            first = f'upn-{random.randint(1000, 9999)}'
         elif len(first) < 4:
-            first = 'sh-%s-%s' % (random.randint(1000,9999), first)
-        #strip to a maximum of maxchars characters
-        first=first[:maxchars]
+            first = f'sh-{random.randint(1000, 9999)}-{first}'
+        # strip to a maximum of maxchars characters
+        first = first[:maxchars]
         newfilename = first
         if prefix:
             newfilename = f'{prefix}.{newfilename}'
@@ -870,8 +864,7 @@ class Mailattachment(Cachelimits):
                 # if filename ends with a "." ending is empty
                 newfilename = f'{newfilename}.{ending}'
         return newfilename
-    
-    
+
     def __str__(self):
         """
         String conversion function for object. Creates
@@ -881,16 +874,12 @@ class Mailattachment(Cachelimits):
 
         """
         element_of = " \u2208 "
-        return """
-Filename     : %s        
-Size (bytes) : %s    
-Location     : %s        
-Archive type : %s        
-Content type : %s""" % (self.filename,'(unknown)' if self.filesize is None else str(self.filesize),
-                        self.filename + element_of +
-                        element_of.join(["{" + obj().filename +"}" for obj in self.parent_archives]),
-                        self.archive_type,
-                        self.contenttype)
+        return f"""
+Filename     : {self.filename}
+Size (bytes) : {'(unknown)' if self.filesize is None else str(self.filesize)}
+Location     : {self.filename}{element_of}{element_of.join(["{" + obj().filename + "}" for obj in self.parent_archives])}
+Archive type : {self.archive_type}
+Content type : {self.contenttype}"""
 
 
 class Mailattachment_mgr(object):
@@ -924,10 +913,7 @@ class Mailattachment_mgr(object):
             self.section = self.__class__.__name__
         else:
             self.section = section
-
-        myclass = self.__class__.__name__
-        loggername = "fuglu.%s" % myclass
-        self.logger = logging.getLogger(loggername)
+        self.logger = logging.getLogger(f"fuglu.{self.__class__.__name__}")
 
         # to limit the size of the attachment cache
         self._current_att_cache = 0
@@ -946,7 +932,7 @@ class Mailattachment_mgr(object):
         # limit max number of files extracted from a single archive
         self._default_numfilelimit = default_numfilelimit
         if default_numfilelimit is not None and max_numfilelimit is not None:
-            self._default_numfilelimit = min(max_numfilelimit,default_numfilelimit)
+            self._default_numfilelimit = min(max_numfilelimit, default_numfilelimit)
 
         self._max_numfilelimit = max_numfilelimit
 
@@ -997,7 +983,6 @@ class Mailattachment_mgr(object):
         """
         self._mailatt_obj_counter += 1
 
-
     def use_caching(self, used_size):
         """
         Used to decide if new attachment objects inside other attachments should be cached or noe
@@ -1008,13 +993,12 @@ class Mailattachment_mgr(object):
         """
         self._new_att_cache = self._current_att_cache + (used_size if used_size else 0)
 
-        if  True if self._cache_limit is None else self._cache_limit >= self._new_att_cache:
+        if True if self._cache_limit is None else self._cache_limit >= self._new_att_cache:
             self._current_att_cache += (used_size if used_size else 0)
             return True
         else:
             return False
 
-
     def walk_all_parts(self, message):
         """
         Like email.message.Message's .walk() but also tries to find parts in the message's epilogue.
@@ -1043,7 +1027,7 @@ class Mailattachment_mgr(object):
                     yield message
 
             except Exception as e:
-                self.logger.info("%s, hidden part extraction failed: %s" % (self.fugluid, str(e)))
+                self.logger.info(f"{self.fugluid} hidden part extraction failed: {e.__class__.__name__}: {str(e)}")
 
     @smart_cached_property(inputs=["_msgrep"])
     def att_file_dict(self):
@@ -1127,7 +1111,7 @@ class Mailattachment_mgr(object):
                 #-----------------#
 
                 # process part, extract information needed to create Mailattachment
-                (att_name, buffer, attsize,contenttype_mime,
+                (att_name, buffer, attsize, contenttype_mime,
                  maintype_mime, subtype_mime, ismultipart_mime,
                  content_charset_mime, isattachment, isinline,
                  defects, att_name_generated) = self.process_msg_part(part)
@@ -1149,7 +1133,7 @@ class Mailattachment_mgr(object):
             tuple : tuple containing
 
         -   att_name             (string) : attachment filename
-        -   buffer               (bytes)  : attachment buffer as bytes
+        -   buffer               (bytes)  : attachment buffer (must be type bytes)
         -   attsize              (int)    : attachment size in bytes
         -   contenttype_mime     (string) : content type
         -   maintype_mime        (string) : main content type
@@ -1176,8 +1160,7 @@ class Mailattachment_mgr(object):
         # any error all parts are marked as attachment
         isattachment = True
         isinline = False
-
-        content_disposition = "attachment"
+        
         try:
             content_disposition = part.get_content_disposition()
         except AttributeError:
@@ -1225,7 +1208,7 @@ class Mailattachment_mgr(object):
             except Exception:
                 pass
             # for long filenames (<78 chars) not properly implementing
-            # continuation according to RFC2231 we might end up with with
+            # continuation according to RFC2231 we might end up with
             # line break in the filename. Even tough some operating systems
             # allow line breaks in filenames, better to remove them...
             att_name = att_name.replace('\r', '').replace('\n', '')
@@ -1253,9 +1236,9 @@ class Mailattachment_mgr(object):
                 att_name = "unnamed"
             else:
                 if ext.startswith("."):
-                    att_name = 'unnamed%s' % ext
+                    att_name = f'unnamed{ext}'
                 else:
-                    att_name = 'unnamed.%s' % ext
+                    att_name = f'unnamed.{ext}'
 
         try:
             buffer = part.get_payload(decode=True)  # Py2: string, Py3: bytes
@@ -1374,7 +1357,9 @@ class Mailattachment_mgr(object):
             list[Mailattachment]: list containing attached files with archives extracted to given level
         """
         obj_list = []
-        for att_obj in self.get_mailatt_generator():
+        g = self.get_mailatt_generator()
+
+        for att_obj in g:
             obj_list.extend(att_obj.get_objectlist(0, level,
                                                    self.get_maxsize_extract(maxsize_extract),
                                                    self.get_maxfilenum_extract(maxfiles_extract),
@@ -1384,7 +1369,7 @@ class Mailattachment_mgr(object):
     def get_fileslist_checksum(self, level: int = 0,
                                maxsize_extract: tp.Optional[int] = None,
                                methods: tp.Tuple = (),
-                               noextractinfo: tp.Optional[NoExtractInfo]=None,
+                               noextractinfo: tp.Optional[NoExtractInfo] = None,
                                maxfiles_extract: tp.Optional[int] = None):
         """
         Get a list containg tuples (filanem, checksumdict) for all the extracted files up to a given extraction level
