@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
 #
 __all__ = [
     'a_logging', 'a_statsd', 'antivirus', 'archive', 'attachment', 'bacn', 'bacn', 'call_ahead', 'clamav',
-    'dataprovider', 'decision', 'delay', 'dnsdata', 'domainauth', 'encryptedarchives', 'fuzor', 'geoip',
+    'dataprovider', 'decision', 'delay', 'dnsdata', 'domainauth', 'fuzor', 'geoip',
     'knownsubject', 'mailcopy', 'messagesize', 'originpolicy', 'outpolicy', 'p_blwl', 'p_debug', 'p_fraction',
-    'p_skipper', 'restrictions', 'rspamd', 'sa', 'script', 'sssp', 'tlspolicy', 'uriextract', 'vacation',
+    'p_skipper', 'restrictions', 'rspamd', 'sa', 'script', 'tlspolicy', 'uriextract', 'vacation',
 ]
