@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,7 @@ class PluginTime(AppenderPlugin):
     """EXPERIMENTAL: Send Plugin execution time to a statsd server"""
 
     def __init__(self, config, section=None):
-        AppenderPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.logger = self._logger()
 
         self.requiredvars = {
@@ -42,7 +42,6 @@ class PluginTime(AppenderPlugin):
         self.sock = None
         self.nodename = platform.node().split('.')[0]
 
-
     def process(self, suspect, decision):
         timings = suspect.get_tag('scantimes')
         if timings is None:
@@ -70,7 +69,7 @@ class MessageStatus(AppenderPlugin):
     """EXPERIMENTAL: Send message status to a statsd server"""
 
     def __init__(self, config, section=None):
-        AppenderPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.logger = self._logger()
 
         self.requiredvars = {
@@ -87,7 +86,6 @@ class MessageStatus(AppenderPlugin):
         self.sock = None
         self.nodename = platform.node().split('.')[0]
 
-
     def process(self, suspect, decision):
         buffer = "%s.fuglu.decision.%s:1|c\n" % (self.nodename, actioncode_to_string(decision))
 
@@ -108,7 +106,7 @@ class MessageStatus(AppenderPlugin):
         else:
             buffer = "%s%s.fuglu.message.clean:1|c\n" % (buffer, self.nodename)
 
-        self.sock.sendto(force_bString(buffer), (host,port))
+        self.sock.sendto(force_bString(buffer), (host, port))
 
     def __str__(self):
         return 'Statsd Sender: Global Message Status'
@@ -119,7 +117,7 @@ class MessageStatusPerRecipient(AppenderPlugin):
     """EXPERIMENTAL: Send per recipient stats to a statsd server"""
 
     def __init__(self, config, section=None):
-        AppenderPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.logger = self._logger()
 
         self.requiredvars = {
@@ -140,9 +138,8 @@ class MessageStatusPerRecipient(AppenderPlugin):
         self.sock = None
         self.nodename = platform.node().split('.')[0]
 
-
     def process(self, suspect, decision):
-        recipient = force_uString(suspect.to_domain) # work with unicode string
+        recipient = force_uString(suspect.to_domain)  # work with unicode string
         if self.config.get(self.section, 'level') == 'email':
             recipient = suspect.to_address
         recipient = recipient.replace('.', '-')
@@ -172,7 +169,5 @@ class MessageStatusPerRecipient(AppenderPlugin):
         self.sock.sendto(force_bString(buffer), (host, port))
         #self.logger.info("buffer: %s"%buffer)
 
-
     def __str__(self):
         return 'Statsd Sender: Per Recipient Message Status'
-
