@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,7 +23,8 @@ May be untested and/or deprecated.
 
 from fuglu.shared import AVScannerPlugin, DUNNO, string_to_actioncode, actioncode_to_string, Suspect, FuConfigParser
 from fuglu.stringencode import force_bString, force_uString
-from fuglu import get_version
+from fuglu.protocolbase import set_keepalive_linux
+from fuglu.utils.version import get_main_version
 import socket
 import struct
 import re
@@ -32,6 +33,335 @@ import sys
 import time
 
 
+# Regular Expressions defining some messages from the SSSP server.
+SSSP_acceptsyntax = re.compile(rb"^ACC\s+(.*?)\s*$")
+SSSP_optionsyntax = re.compile(rb"^(\w+):\s*(.*?)\s*$")
+SSSP_virussyntax = re.compile(rb"^VIRUS\s+(\S+)\s+(.*)")
+SSSP_typesyntax = re.compile(rb"^TYPE\s+(\w+)")
+SSSP_donesyntax = re.compile(rb"^DONE\s+(\w+)\s+(\w+)\s+(.*?)\s*$")
+SSSP_eventsyntax = re.compile(rb"^([A-Z]+)\s+(\w+)")
+SSSP_tmpdirsyntax = re.compile(r"(/tmp/savid_tmp[^/]+/)(.+)")
+
+
+class SSSPPlugin(AVScannerPlugin):
+    """ This plugin scans the suspect using the sophos SSSP protocol.
+
+Prerequisites: Requires a running sophos daemon with dynamic interface (SAVDI)
+"""
+
+    def __init__(self, config, section=None):
+        super().__init__(config, section)
+        self.requiredvars = {
+            'host': {
+                'default': 'localhost',
+                'description': 'hostname where the SSSP server runs',
+            },
+
+            'port': {
+                'default': '4010',
+                'description': "tcp port or path to unix socket",
+            },
+
+            'timeout': {
+                'default': '30',
+                'description': 'socket timeout',
+            },
+
+            'maxsize': {
+                'default': '22000000',
+                'description': "maximum message size, larger messages will not be scanned. ",
+            },
+
+            'retries': {
+                'default': '3',
+                'description': 'how often should fuglu retry the connection before giving up',
+            },
+
+            'virusaction': {
+                'default': 'DEFAULTVIRUSACTION',
+                'description': "action if infection is detected (DUNNO, REJECT, DELETE)",
+            },
+
+            'problemaction': {
+                'default': 'DEFER',
+                'description': "action if there is a problem (DUNNO, DEFER)",
+            },
+
+            'rejectmessage': {
+                'default': 'threat detected: ${virusname}',
+                'description': "reject message template if running in pre-queue mode and virusaction=REJECT",
+            },
+
+            'enginename': {
+                'default': '',
+                'description': 'set custom engine name (defaults to %s)' % self.enginename,
+            },
+        }
+        self.logger = self._logger()
+        self.enginename = 'sophos'
+        self._last_line = None
+
+    def __str__(self):
+        return "Sophos AV"
+
+    def examine(self, suspect):
+        if self._check_too_big(suspect):
+            return DUNNO
+
+        content = suspect.get_source()
+
+        for i in range(0, self.config.getint(self.section, 'retries')):
+            try:
+                viruses = self.scan_stream(content)
+                actioncode, message = self._virusreport(suspect, viruses)
+                return actioncode, message
+            except Exception as e:
+                self.logger.warning("Error encountered while contacting SSSP server (try %s of %s): %s" % (
+                    i + 1, self.config.getint(self.section, 'retries'), str(e)))
+                time.sleep(0.05)  # give savdi a few ms before retrying
+        self.logger.error("SSSP scan failed after %s retries" %
+                          self.config.getint(self.section, 'retries'))
+
+        return self._problemcode()
+
+    def scan_stream(self, content, suspectid='(NA)'):
+        """
+        Scan a buffer
+
+        content (string) : buffer to scan
+
+        return either :
+          - (dict) : {filename1: "virusname"}
+          - None if no virus found
+        """
+
+        s = self.__init_socket__()
+        dr = {}
+
+        # Read the welcome message
+
+        if not self._exchange_greetings(s):
+            raise Exception("SSSP Greeting failed: %s" % self._last_line)
+
+        # QUERY to discover the maxclassificationsize
+        s.send(b'SSSP/1.0 QUERY\n')
+
+        if not self._accepted(s):
+            raise Exception("SSSP Query rejected: %s" % self._last_line)
+
+        self._read_options(s)
+
+        # Set the options for classification
+        enableoptions = [
+            b"TnefAttachmentHandling",
+            b"ActiveMimeHandling",
+            b"Mime",
+            b"ZipDecompression",
+            b"DynamicDecompression",
+        ]
+
+        enablegroups = [
+            b'GrpExecutable',
+            b'GrpArchiveUnpack',
+            b'GrpSelfExtract',
+            b'GrpInternet',
+            b'GrpSuper',
+            b'GrpMisc',
+        ]
+
+        sendbuf = "OPTIONS\nreport:all\n"
+        for opt in enableoptions:
+            sendbuf += "savists: %s 1\n" % force_uString(opt)
+
+        for grp in enablegroups:
+            sendbuf += "savigrp: %s 1\n" % force_uString(grp)
+
+        # all sent, add aditional newline
+        sendbuf += "\n"
+
+        s.send(force_bString(sendbuf))
+
+        if not self._accepted(s):
+            raise Exception("SSSP Options not accepted: %s" % self._last_line)
+
+        resp = self._receive_msg(s)
+
+        for l in resp:
+            if SSSP_donesyntax.match(l):
+                parts = SSSP_donesyntax.findall(l)
+                if parts[0][0] != b'OK':
+                    raise Exception("SSSP Options failed")
+                break
+
+        # Send the SCAN request
+
+        s.send(force_bString('SCANDATA ' + str(len(content)) + '\n'))
+        if not self._accepted(s):
+            raise Exception("SSSP Scan rejected: %s" % self._last_line)
+
+        s.sendall(force_bString(content))
+
+        # and read the result
+        events = self._receive_msg(s)
+
+        for l in events:
+            if SSSP_virussyntax.match(l):
+                parts = SSSP_virussyntax.findall(l)
+                virus = force_uString(parts[0][0])
+                filename = force_uString(parts[0][1])
+                try:
+                    filename = SSSP_tmpdirsyntax.findall(filename)[0][1]
+                except IndexError:
+                    pass
+                dr[filename] = virus
+
+        try:
+            self._say_goodbye(s)
+            s.shutdown(socket.SHUT_RDWR)
+        except socket.error as e:
+            self.logger.warning('%s Error terminating connection: %s', (suspectid, str(e)))
+        finally:
+            s.close()
+
+        if dr == {}:
+            return None
+        else:
+            return dr
+
+    def __init_socket__(self):
+        unixsocket = False
+
+        try:
+            self.config.getint(self.section, 'port')
+        except ValueError:
+            unixsocket = True
+
+        if unixsocket:
+            sock = self.config.get(self.section, 'port')
+            if not os.path.exists(sock):
+                raise Exception("unix socket %s not found" % sock)
+            s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+            s.settimeout(self.config.getfloat(self.section, 'timeout'))
+            try:
+                s.connect(sock)
+            except socket.error:
+                raise Exception('Could not reach SSSP server using unix socket %s' % sock)
+        else:
+            host = self.config.get(self.section, 'host')
+            port = self.config.getint(self.section, 'port')
+            timeout = self.config.getfloat(self.section, 'timeout')
+            try:
+                s = socket.create_connection((host, port), timeout)
+            except socket.error:
+                raise Exception('Could not reach SSSP server using network (%s, %s)' % (host, port))
+            set_keepalive_linux(s)
+
+        return s
+
+    def _receive_line(self, s):
+        """
+        Receives a line of text from the socket
+        \r chars are discarded
+        The line is terminated by a \n
+        NUL chars indicate a broken socket
+        :param s: the socket
+        :return: data read from socket
+        """
+        line = b''
+        done = 0
+        while not done:
+            c = s.recv(1)
+            if c == b'':
+                return b''
+            done = (c == b'\n')
+            if not done and c != b'\r':
+                line = line + c
+        self._last_line = line
+        return line
+
+    def _receive_msg(self, s):
+        """
+        Receives a whole message. Messages are terminated by a blank line
+        :param s: the socket
+        :return: list of lines received
+        """
+        response = []
+        finished = 0
+
+        while not finished:
+            msg = self._receive_line(s)
+            finished = (len(msg) == 0)
+            if not finished:
+                response.append(msg)
+
+        return response
+
+    def _accepted(self, s):
+        """
+        Receives the ACC message which is a single line
+        conforming to the acceptsyntax RE.
+        :param s: the socket
+        :return: boolean: does line start with ACC?
+        """
+        acc = self._receive_line(s)
+        return SSSP_acceptsyntax.match(acc)
+
+    def _read_options(self, s):
+        """
+        Reads a message which should be a list of options
+        and transforms them into a dictionary
+        :param s: the socket
+        :return: dict: options supported by sophos
+        """
+        resp = self._receive_msg(s)
+        opts = {}
+
+        for l in resp:
+            parts = SSSP_optionsyntax.findall(l)
+            for p in parts:
+                p0 = force_uString(p[0])
+                if p0 not in opts:
+                    opts[p0] = []
+
+                opts[p0].append(force_uString(p[1]))
+
+        return opts
+
+    def _exchange_greetings(self, s):
+        """
+        Performs the initial exchange of messages.
+        :param s: the socket
+        :return: 1 on success, 0 if greeting failed
+        """
+        line = self._receive_line(s)
+
+        if not line.startswith(b'OK SSSP/1.0'):
+            return 0
+
+        s.send(b'SSSP/1.0\n')
+
+        if not self._accepted(s):
+            print("Greeting Rejected!!")
+            return 0
+
+        return 1
+
+    def _say_goodbye(self, s):
+        """
+        performs the final exchange of messages
+        :param s: the socket
+        :return: None
+        """
+        s.send(b'BYE\n')
+        self._receive_line(s)
+
+    def lint(self):
+        viract = self.config.get(self.section, 'virusaction')
+        print("Virusaction: %s" % actioncode_to_string(string_to_actioncode(viract, self.config)))
+        allok = self.check_config() and self.lint_eicar()
+        return allok
+
+
 # from : https://github.com/AlexeyDemidov/avsmtpd/blob/master/drweb.h
 # Dr. Web daemon commands
 DRWEBD_SCAN_CMD = 0x0001
@@ -79,10 +409,10 @@ Tags:
  * sets ``virus['drweb']`` (boolean)
  * sets ``DrWebPlugin.virus`` (list of strings) - virus names found in message
 """
-    
+
     def __init__(self, config, section=None):
-        AVScannerPlugin.__init__(self, config, section)
-        
+        super().__init__(config, section)
+
         self.requiredvars = {
             'host': {
                 'default': 'localhost',
@@ -124,8 +454,7 @@ Tags:
         self.logger = self._logger()
         self.pattern = re.compile(r'(?:DATA\[\d+\])(.+) infected with (.+)$')
         self.enginename = 'drweb'
-    
-    
+
     def _parse_result(self, lines):
         dr = {}
         for line in lines:
@@ -136,14 +465,13 @@ Tags:
             filename = m.group(1)
             virus = m.group(2)
             dr[filename] = virus
-        
+
         if len(dr) == 0:
             self.logger.warning("could not extract virus information from report: %s" % "\n".join(lines))
             return dict(buffer='infection details unavailable')
         else:
             return dr
-    
-    
+
     def scan_stream(self, content, suspectid='(NA)'):
         """
         Scan a buffer
@@ -154,12 +482,12 @@ Tags:
           - (dict) : {filename: "virusname"}
           - None if no virus found
         """
-        
+
         s = self.__init_socket__()
         buflen = len(content)
-        
+
         self._sendint(s, DRWEBD_SCAN_CMD)
-        
+
         # flags:
         # self._sendint(s, 0) # "flags" # use this to get only the code
         # self._sendint(s, DRWEBD_RETURN_VIRUSES) # use this to get the virus
@@ -177,13 +505,12 @@ Tags:
             line = self._readstr(s)
             lines.append(line)
         s.close()
-        
+
         if retcode & DERR_VIRUS == retcode:
             return self._parse_result(lines)
         else:
             return None
-    
-    
+
     def __init_socket__(self):
         host = self.config.get(self.section, 'host', resolve_env=True)
         port = self.config.getint(self.section, 'port')
@@ -192,19 +519,16 @@ Tags:
             s = socket.create_connection((host, port), timeout)
         except socket.error:
             raise Exception('Could not reach drweb using network (%s, %s)' % (host, port))
-        
+
         return s
-    
-    
+
     def __str__(self):
         return 'DrWeb AV'
-    
-    
+
     def lint(self):
         allok = self.check_config() and self.lint_info() and self.lint_eicar()
         return allok
-    
-    
+
     def lint_info(self):
         try:
             version = self.get_version()
@@ -215,8 +539,7 @@ Tags:
             print("Could not get DrWeb Version info: %s" % str(e))
             return False
         return True
-    
-    
+
     def get_version(self):
         """Return numeric version of the DrWeb daemon"""
         try:
@@ -227,8 +550,7 @@ Tags:
         except Exception as e:
             self.logger.error("Could not get DrWeb Version: %s" % str(e))
         return None
-    
-    
+
     def get_baseinfo(self):
         """return list of tuples (basename,number of virus definitions)"""
         ret = []
@@ -245,18 +567,15 @@ Tags:
                 "Could not get DrWeb Base Information: %s" % str(e))
             return None
         return ret
-    
-    
+
     def _sendint(self, sock, value):
         sock.sendall(struct.pack('!I', value))
-    
-    
+
     def _readint(self, sock):
         res = sock.recv(4)
         ret = struct.unpack('!I', res)[0]
         return ret
-    
-    
+
     def _readstr(self, sock):
         strlength = self._readint(sock)
         buf = sock.recv(strlength)
@@ -267,7 +586,7 @@ Tags:
 
 def drweb_main():
     import logging
-    
+
     logging.basicConfig(level=logging.DEBUG)
     config = FuConfigParser()
     sec = 'dev'
@@ -276,11 +595,11 @@ def drweb_main():
     config.set(sec, 'port', '3000')
     config.set(sec, 'timeout', '5')
     plugin = DrWebPlugin(config, sec)
-    
+
     assert plugin.lint_info()
-    
+
     import sys
-    
+
     if len(sys.argv) > 1:
         counter = 0
         infected = 0
@@ -302,68 +621,67 @@ def drweb_main():
         plugin.lint_eicar()
 
 
-
 class ICAPPlugin(AVScannerPlugin):
     """ICAP Antivirus Plugin
-This plugin allows Antivirus Scanning over the ICAP Protocol (http://tools.ietf.org/html/rfc3507 )
+This plugin allows Antivirus Scanning over the ICAP Protocol ( https://www.rfc-editor.org/rfc/rfc3507 )
 supported by some AV Scanners like Symantec and Sophos. For sophos, however, it is recommended to use the native SSSP Protocol.
 
 Prerequisites: requires an ICAP capable antivirus engine somewhere in your network
 """
-    
+
     def __init__(self, config, section=None):
-        AVScannerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.requiredvars = {
             'host': {
                 'default': 'localhost',
                 'description': 'hostname where the ICAP server runs',
             },
-            
+
             'port': {
                 'default': '1344',
                 'description': "tcp port or path to unix socket",
             },
-            
+
             'timeout': {
                 'default': '10',
                 'description': 'socket timeout',
             },
-            
+
             'maxsize': {
                 'default': '22000000',
                 'description': "maximum message size, larger messages will not be scanned. ",
             },
-            
+
             'retries': {
                 'default': '3',
                 'description': 'how often should fuglu retry the connection before giving up',
             },
-            
+
             'virusaction': {
                 'default': 'DEFAULTVIRUSACTION',
                 'description': "action if infection is detected (DUNNO, REJECT, DELETE)",
             },
-            
+
             'problemaction': {
                 'default': 'DEFER',
                 'description': "action if there is a problem (DUNNO, DEFER)",
             },
-            
+
             'rejectmessage': {
                 'default': 'threat detected: ${virusname}',
                 'description': "reject message template if running in pre-queue mode and virusaction=REJECT",
             },
-            
+
             'service': {
                 'default': 'AVSCAN',
                 'description': 'ICAP Av scan service, usually AVSCAN (sophos, symantec, eset)',
             },
-            
+
             'send_fakeheaders': {
                 'default': 'False',
                 'description': 'send ICAP fake response headers (must be False for eset)',
             },
-            
+
             'enginename': {
                 'default': 'icap-generic',
                 'description': "name of the virus engine behind the icap service. used to inform other plugins. can be anything like 'sophos', 'symantec', ...",
@@ -371,8 +689,7 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
         }
         self.logger = self._logger()
         self.enginename = 'icap-generic'
-    
-    
+
     def __str__(self):
         classname = self.__class__.__name__
         if self.section == classname:
@@ -380,8 +697,6 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
         else:
             return f'ICAP AV - {self.section}'
 
-    
-    
     def scan_stream(self, content, suspectid='(NA)'):
         """
         Scan a buffer
@@ -392,10 +707,10 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
           - (dict) : {filename1: "virusname"}
           - None if no virus found
         """
-        
+
         s = self.__init_socket__()
         dr = {}
-        
+
         content = force_bString(content)
         CRLF = b"\r\n"
         host = self.config.get(self.section, 'host', resolve_env=True)
@@ -408,35 +723,35 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
         bodypart = bodyparthexlen + CRLF
         bodypart += content + CRLF
         bodypart += b"0" + CRLF
-        
+
         if send_fakeheaders:
             fakerequestheader = b"GET http://localhost/message.eml HTTP/1.1%s" + CRLF
             fakerequestheader += b"Host: localhost" + CRLF
             fakerequestheader += CRLF
             fakereqlen = len(fakerequestheader)
-            
+
             fakeresponseheader = b"HTTP/1.1 200 OK" + CRLF
             fakeresponseheader += b"Content-Length: " + str(buflen).encode() + CRLF
             fakeresponseheader += CRLF
             fakeresplen = len(fakeresponseheader)
-            
+
             hdrstart = 0
             responsestart = fakereqlen
             bodystart = fakereqlen + fakeresplen
-        
+
         # now that we know the length of the fake request/response, we can
         # build the ICAP header
         icapheader = b""
         icapheader += b"RESPMOD icap://%s:%s/%s ICAP/1.0%s" % (host.encode(), port.encode(), service.encode(), CRLF)
         icapheader += b"Host: " + host.encode() + CRLF
-        icapheader += b"User-Agent: Fuglu/%s%s" % (get_version().strip().encode(), CRLF)
+        icapheader += b"User-Agent: Fuglu/%s%s" % (get_main_version().strip().encode(), CRLF)
         icapheader += b"Allow: 204" + CRLF
         if send_fakeheaders:
             icapheader += b"Encapsulated: req-hdr=%s, res-hdr=%s, res-body=%s%s" % (hdrstart, responsestart, bodystart, CRLF)
         else:
             icapheader += b"Encapsulated: res-body=0" + CRLF
         icapheader += CRLF
-        
+
         if send_fakeheaders:
             everything = icapheader + fakerequestheader + fakeresponseheader + bodypart + CRLF
         else:
@@ -446,64 +761,62 @@ Prerequisites: requires an ICAP capable antivirus engine somewhere in your netwo
         result = force_uString(s.recv(20000))
         s.close()
         #print(f"result is: {result}")
-        
+
         sheaders = ['X-Infection-Found', "X-Violations-Found:", ]
         for sheader in sheaders:
             if sheader.lower() in result.lower():
                 lines = result.split('\n')
                 lineidx = 0
-                
+
                 for line in lines:
                     if sheader.lower() in line.lower():
-                        if sheader == 'X-Infection-Found': # ESET
+                        if sheader == 'X-Infection-Found':  # ESET
                             # X-Infection-Found: Type=0; Resolution=0; Threat=Eicar;
                             fields = [x.strip().split('=') for x in line.split(';')]
                             for field in fields:
-                                if len(field)==2 and field[0].lower() == 'threat':
+                                if len(field) == 2 and field[0].lower() == 'threat':
                                     dr['stream'] = field[1]
-                                
-                        if sheader == 'X-Violations-Found:': # Sophos, Kaspersky?
+
+                        if sheader == 'X-Violations-Found:':  # Sophos, Kaspersky?
                             numfound = int(line[len(sheader):])
                             # for each found virus, get 4 lines
                             for vircount in range(numfound):
                                 infectedfile = lines[lineidx + vircount * 4 + 1].strip()
                                 infection = lines[lineidx + vircount * 4 + 2].strip()
                                 dr[infectedfile] = infection
-                        
+
                         break
                     lineidx += 1
                 if dr:
                     break
-        
+
         if dr == {}:
             return None
         else:
             return dr
-    
-    
+
     def lint_options(self):
         CRLF = "\r\n"
         host = self.config.get(self.section, 'host', resolve_env=True)
         port = self.config.get(self.section, 'port')
         service = self.config.get(self.section, 'service')
-        
+
         icapheader = ''
         icapheader += "OPTIONS icap://%s:%s/%s ICAP/1.0%s" % (host, port, service, CRLF)
         icapheader += "Host: " + host + CRLF
-        icapheader += "User-Agent: Fuglu/%s%s" % (get_version().strip(), CRLF)
-        
+        icapheader += "User-Agent: Fuglu/%s%s" % (get_main_version().strip(), CRLF)
+
         s = self.__init_socket__()
         s.sendall(force_bString(icapheader))
         result = s.recv(20000)
         s.close()
         print(force_uString(result))
-    
-    
+
     def lint(self):
         viract = self.config.get(self.section, 'virusaction')
         print("Virusaction: %s" % actioncode_to_string(string_to_actioncode(viract, self.config)))
         allok = self.check_config()
-        #self.lint_options()
+        # self.lint_options()
         allok = allok and self.lint_eicar()
         return allok
 
@@ -521,11 +834,11 @@ Tags:
  * sets ``virus['F-Prot']`` (boolean)
  * sets ``FprotPlugin.virus`` (list of strings) - virus names found in message
 """
-    
+
     def __init__(self, config, section=None):
-        AVScannerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.logger = self._logger()
-        
+
         self.requiredvars = {
             'host': {
                 'default': 'localhost',
@@ -572,22 +885,21 @@ Tags:
                 'description': 'set custom engine name (defaults to %s)' % self.enginename,
             },
         }
-        
+
         self.pattern = re.compile(rb'^(\d)+ <(.+)> (.+)$')
         self.enginename = 'F-Prot'
-    
-    
+
     def examine(self, suspect):
         if self._check_too_big(suspect):
             return DUNNO
-        
+
         # use msgrep only to check for Content-Type header
         # use source directly for Fprot to prevent exceptions converting the email-object to bytes
         msgrep = suspect.get_message_rep()
         content = suspect.get_original_source()
-        
+
         networkmode = self.config.getboolean(self.section, 'networkmode')
-        
+
         # this seems to be a bug in f-prot.
         # If no Content-Type header is set, then no scan is performed.
         # However, content of the header does not seem to matter.
@@ -596,7 +908,7 @@ Tags:
             content = Suspect.prepend_header_to_source('Content-Type', 'dummy', content)
             networkmode = True
             self.logger.debug('%s missing Content-Type header... falling back to network mode' % suspect.id)
-        
+
         for i in range(0, self.config.getint(self.section, 'retries')):
             try:
                 if networkmode:
@@ -608,11 +920,10 @@ Tags:
             except Exception as e:
                 self.logger.warning("%s Error encountered while contacting fpscand (try %s of %s): %s" %
                                     (suspect.id, i + 1, self.config.getint(self.section, 'retries'), str(e)))
-        self.logger.error("fpscand failed after %s retries" % self.config.getint(self.section, 'retries'))
-        
+        self.logger.error("%s fpscand failed after %s retries" % (suspect.id, self.config.getint(self.section, 'retries')))
+
         return self._problemcode()
-    
-    
+
     def _parse_result(self, result):
         dr = {}
         result = force_uString(result)
@@ -624,16 +935,16 @@ Tags:
             status = force_uString(m.group(1))
             text = force_uString(m.group(2))
             details = force_uString(m.group(3))
-            
+
             status = int(status)
             self.logger.debug("f-prot scan status: %s" % status)
             self.logger.debug("f-prot scan text: %s" % text)
             if status == 0:
                 continue
-            
+
             if status > 3:
                 self.logger.warning("f-prot: got unusual status %s (result: %s)" % (status, result))
-            
+
             # http://www.f-prot.com/support/helpfiles/unix/appendix_c.html
             if status & 1 == 1 or status & 2 == 2:
                 # we have a infection
@@ -645,27 +956,25 @@ Tags:
                     self.logger.warning("Unexpected reply from f-prot: %s" % text)
                     continue
                 dr[details] = text
-        
+
         if len(dr) == 0:
             return None
         else:
             return dr
-    
-    
+
     def scan_file(self, filename):
         filename = os.path.abspath(filename)
         s = self.__init_socket__()
         s.sendall(force_bString('SCAN %s FILE %s' % (self.config.get(self.section, 'scanoptions'), filename)))
         s.sendall(b'\n')
-        
+
         result = s.recv(20000)
         if len(result) < 1:
             self.logger.error('Got no reply from fpscand')
         s.close()
-        
+
         return self._parse_result(result)
-    
-    
+
     def scan_stream(self, content, suspectid='(NA)'):
         """
         Scan a buffer
@@ -676,7 +985,7 @@ Tags:
           - (dict) : {filename1: "virusname"}
           - None if no virus found
         """
-        
+
         s = self.__init_socket__()
         content = force_bString(content)
         buflen = len(content)
@@ -685,15 +994,14 @@ Tags:
         self.logger.debug('%s Sending buffer (length=%s) to fpscand...' % (suspectid, buflen))
         s.sendall(content)
         self.logger.debug('%s Sent %s bytes to fpscand, waiting for scan result' % (suspectid, buflen))
-        
+
         result = force_uString(s.recv(20000))
         if len(result) < 1:
-            self.logger.error('Got no reply from fpscand')
+            self.logger.error(f'{suspectid} Got no reply from fpscand')
         s.close()
-        
+
         return self._parse_result(result)
-    
-    
+
     def __init_socket__(self):
         host = self.config.get(self.section, 'host', resolve_env=True)
         port = self.config.getint(self.section, 'port')
@@ -703,38 +1011,36 @@ Tags:
         except socket.error:
             raise Exception('Could not reach fpscand using network (%s, %s)' % (host, port))
         return s
-    
-    
+
     def __str__(self):
         return 'F-Prot AV'
-    
-    
+
     def lint(self):
         allok = self.check_config() and self.lint_eicar()
         networkmode = self.config.getboolean(self.section, 'networkmode')
         if not networkmode:
             allok = allok and self.lint_file()
         return allok
-    
+
     def lint_file(self):
         import tempfile
         (handle, tempfilename) = tempfile.mkstemp(prefix='fuglu', dir=self.config.get('main', 'tempdir'))
         tempfilename = tempfilename
-        
+
         with os.fdopen(handle, 'w+b') as fd:
             fd.write(force_bString(self.eicar))
-        
+
         try:
             viruses = self.scan_file(tempfilename)
         except Exception as e:
             print(e)
             return False
-        
+
         try:
             os.remove(tempfilename)
         except Exception:
             pass
-        
+
         try:
             for fname, virus in iter(viruses.items()):
                 print("F-Prot AV (file mode): Found virus: %s in %s" % (virus, fname))
@@ -746,7 +1052,6 @@ Tags:
         except Exception as e:
             print(e)
             return False
-        
 
 
 class CyrenPlugin(AVScannerPlugin):
@@ -754,50 +1059,50 @@ class CyrenPlugin(AVScannerPlugin):
 
 Prerequisites: Cyren Antivirus must be installed and running, not necessarily on the same box as fuglu though.
     """
-    
+
     def __init__(self, config, section=None):
-        AVScannerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.requiredvars = {
             'host': {
                 'default': 'localhost',
                 'description': 'hostname where the ICAP server runs',
             },
-            
+
             'port': {
                 'default': '1344',
                 'description': "tcp port or path to unix socket",
             },
-            
+
             'timeout': {
                 'default': '10',
                 'description': 'socket timeout',
             },
-            
+
             'maxsize': {
                 'default': '22000000',
                 'description': "maximum message size, larger messages will not be scanned. ",
             },
-            
+
             'retries': {
                 'default': '3',
                 'description': 'how often should fuglu retry the connection before giving up',
             },
-            
+
             'virusaction': {
                 'default': 'DEFAULTVIRUSACTION',
                 'description': "action if infection is detected (DUNNO, REJECT, DELETE)",
             },
-            
+
             'problemaction': {
                 'default': 'DEFER',
                 'description': "action if there is a problem (DUNNO, DEFER)",
             },
-            
+
             'rejectmessage': {
                 'default': 'threat detected: ${virusname}',
                 'description': "reject message template if running in pre-queue mode and virusaction=REJECT",
             },
-            
+
             'enginename': {
                 'default': '',
                 'description': 'set custom engine name (defaults to %s)' % self.enginename,
@@ -805,12 +1110,10 @@ Prerequisites: Cyren Antivirus must be installed and running, not necessarily on
         }
         self.enginename = 'cyren'
         self.logger = self._logger()
-    
-    
+
     def __str__(self):
         return "Cyren AV"
-    
-    
+
     def _parse_result(self, result, suspectid):
         dr = {}
         result = force_uString(result.strip())
@@ -825,21 +1128,20 @@ Prerequisites: Cyren Antivirus must be installed and running, not necessarily on
             scan_status = fields[1]
             #obj = fields[2]
             message = ' '.join(fields[3:])
-        
+
         if ':' in daemon_status:
             daemon_status = daemon_status.rsplit(':')[-1]
-        
+
         if daemon_status != 'OK' or scan_status == 'ERROR':
             self.logger.warning(f'{suspectid} scanner errror, got {daemon_status} {scan_status}')
-            return None # or raise exception?
+            return None  # or raise exception?
         if scan_status != 'INFECTED':
             self.logger.debug(f'{suspectid} message not infected, got {scan_status}')
             return None
-        
+
         dr['stream'] = message
         return dr
-    
-    
+
     def scan_stream(self, content, suspectid='(N/A)'):
         """
         Scans given byte buffer (file content). May raise an exception on errors.
@@ -850,13 +1152,13 @@ Prerequisites: Cyren Antivirus must be installed and running, not necessarily on
         sock = self.__init_socket__()
         remainingbytes = force_bString(content)
         buflen = len(remainingbytes)
-        
+
         sock.sendall(force_bString('INSTREAM %s\n' % buflen))
         reply = sock.recv(20000)
         if reply != b'OK SEND_DATA\n':
             self.logger.warning(f'{suspectid} Cyren did not respond OK, got {force_uString(reply.strip())}')
             return None
-        
+
         maxtime = time.time() + self.config.getint(self.section, 'timeout')
         default_chunk_size = min(buflen, 20480)
         iChunk = 0
@@ -871,13 +1173,12 @@ Prerequisites: Cyren Antivirus must be installed and running, not necessarily on
             msg = packsize + chunkdata
             sock.sendall(msg)
         sock.sendall(struct.pack(b'!I', 0))
-        
+
         resp = sock.recv(20000)
         self._close_socket(sock)
         dr = self._parse_result(resp, suspectid)
         return dr
-    
-    
+
     def lint_ping(self):
         try:
             sock = self.__init_socket__()
@@ -891,15 +1192,12 @@ Prerequisites: Cyren Antivirus must be installed and running, not necessarily on
             print("Invalid PONG: %s" % force_uString(result))
         self._close_socket(sock)
         return True
-    
-    
+
     def lint(self):
         allok = self.check_config() and self.lint_ping() and self.lint_eicar()
         return allok
 
 
-
-
 if __name__ == '__main__':
     if 'drweb' in sys.argv:
         drweb_main()
