@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -48,24 +48,23 @@ LOG_MAPPING = {
 }
 
 
-
 class AbstractBackend(object):
     requiredvars = {}
-    
+
     def __init__(self, config, section):
         self.config = config
         self.section = section
         self.logger = logging.getLogger('fuglu.archive.backend.%s' % self.__class__.__name__)
-    
+
     def __str__(self):
         return self.__class__.__name__
-        
+
     def archive(self, suspect, fugluid_override=None):
         raise NotImplementedError
-    
+
     def lint(self):
         raise NotImplementedError
-    
+
     def _get_msg_source(self, suspect, original=True):
         if original:
             content = suspect.get_original_source()
@@ -74,7 +73,6 @@ class AbstractBackend(object):
         return content
 
 
-
 class LocalDirBackend(AbstractBackend):
     """
     This is the default backend. It stores mail in a local directory.
@@ -110,90 +108,86 @@ class LocalDirBackend(AbstractBackend):
             'description': "set file permissions of saved messages",
         },
     }
-    
-    
+
     def archive(self, suspect, fugluid_override=None):
         archivedir = self.config.get(self.section, 'local_archivedir')
         if archivedir == "":
-            self.logger.error('Archivedir is not specified')
+            self.logger.error(f'{suspect.id} Archivedir is not specified')
             return
-        
+
         subdirtemplate = self.config.get(self.section, 'local_subdirtemplate')
-        
+
         # the archive root dir
         startdir = os.path.abspath(archivedir)
-        
+
         # relative dir within archive root
         subdir = apply_template(subdirtemplate, suspect)
         if subdir.endswith('/'):
             subdir = subdir[:-1]
         if subdir.startswith('/'):
             subdir = subdir[1:]
-        
+
         # filename without dir
         filenametemplate = self.config.get(self.section, 'local_filenametemplate')
-        
+
         values = {
             'archiveid': fugluid_override or suspect.id,
         }
         filename = apply_template(filenametemplate, suspect, values)
         # make sure filename can't create new folders
         filename = filename.replace('/', '_')
-        
+
         # absolute final filepath
         requested_path = os.path.join(startdir, subdir, filename)
-        
+
         finaldir = os.path.dirname(requested_path)
         if not os.path.isdir(finaldir):
             os.makedirs(finaldir, 0o755)
-        
+
         if self.config.getboolean(self.section, 'local_useoriginal'):
             shutil.copy(suspect.tempfile, requested_path)
         else:
             with open(requested_path, 'wb') as fp:
                 # write bytes
                 fp.write(suspect.get_source())
-        
+
         chmod = self.config.get(self.section, 'local_chmod')
         chgrp = self.config.get(self.section, 'local_chgrp')
         chown = self.config.get(self.section, 'local_chown')
         if chmod or chgrp or chown:
-            self._setperms(requested_path, chmod, chgrp, chown)
-        
-        self.logger.info('Message from %s to %s archived as %s' % (
-            suspect.from_address, suspect.to_address, requested_path))
+            self._setperms(requested_path, chmod, chgrp, chown, suspect.id)
+
+        self.logger.info(f'{suspect.id} Message from {suspect.from_address} to {suspect.to_address} archived as {requested_path}')
         return dict(filepath=requested_path)
-    
 
     def lint(self):
         archivedir = self.config.get(self.section, 'local_archivedir')
         if archivedir == "":
             print('Archivedir is not specified')
             return False
-    
+
         if not os.path.isdir(archivedir):
             print("Archivedir '%s' does not exist or is not a directory" % archivedir)
             return False
-    
+
         return True
-    
-    
-    def _setperms(self, filename, chmod, chgrp, chown):
+
+    def _setperms(self, filename, chmod, chgrp, chown, fugluid):
         """Set file permissions and ownership
         :param filename The target file
         :param chmod string representing the permissions (example '640')
         :param chgrp groupname or group id of the target group. the user running fuglu must be a member of this group for this to work
         :param chown username or user id of the target user. fuglu must run as root for this to work (which is not recommended for security reasons)
         """
-        
+
         # chmod
         if chmod:
             perm = int(chmod, 8)
             try:
                 os.chmod(filename, perm)
             except Exception:
-                self.logger.error('could not set permission on file %s' % filename)
-        
+                self.logger.error(f'{fugluid }could not set permission on file {filename}')
+
         # chgrp
         changetogroup = -1
         if chgrp:
@@ -202,18 +196,18 @@ class LocalDirBackend(AbstractBackend):
                 group = grp.getgrnam(chgrp)
             except KeyError:
                 pass
-            
+
             if group is None:
                 try:
                     group = grp.getgrgid(int(chgrp))
                 except (KeyError, ValueError):
                     pass
-            
+
             if group is not None:
                 changetogroup = group.gr_gid
             else:
-                self.logger.warning("Group %s not found" % chgrp)
-        
+                self.logger.warning(f'{fugluid} Group {chgrp} not found')
+
         # chown
         changetouser = -1
         if chown:
@@ -222,23 +216,23 @@ class LocalDirBackend(AbstractBackend):
                 user = pwd.getpwnam(chown)
             except KeyError:
                 pass
-            
+
             if user is None:
                 try:
                     user = pwd.getpwuid(int(chown))
                 except (KeyError, ValueError):
                     pass
-            
+
             if user is not None:
                 changetouser = user.pw_uid
             else:
-                self.logger.warning("User %s not found" % chown)
-        
+                self.logger.warning(f'{fugluid} User {chown} not found')
+
         if changetogroup != -1 or changetouser != -1:
             try:
                 os.chown(filename, changetouser, changetogroup)
             except Exception as e:
-                self.logger.error("Could not change user/group of file %s : %s" % (filename, str(e)))
+                self.logger.error(f'{fugluid} Could not change user/group of file {filename} : {e.__class__.__name__}: {str(e)}')
 
 
 class ElasticBackend(AbstractBackend):
@@ -247,9 +241,9 @@ class ElasticBackend(AbstractBackend):
     Status: experimental
     """
     requiredvars = {
-        'elastic_uris':{
+        'elastic_uris': {
             'default': '',
-            'description':'comma separated list of ElasticSearch host definition (hostname, hostname:port, https://user:pass@hostname:port/)',
+            'description': 'comma separated list of ElasticSearch host definition (hostname, hostname:port, https://user:pass@hostname:port/)',
         },
         'elastic_verify_certs': {
             'default': 'True',
@@ -259,13 +253,13 @@ class ElasticBackend(AbstractBackend):
             'default': '30',
             'description': 'set elastic connection timeout to this value',
         },
-        'elastic_index':{
+        'elastic_index': {
             'default': 'fugluquar-${date}',
-            'description':'Name of ElasticSearch index in which document will be stored. Template vars (e.g. ${to_domain} or ${date}) can be used.',
+            'description': 'Name of ElasticSearch index in which document will be stored. Template vars (e.g. ${to_domain} or ${date}) can be used.',
         },
-        'elastic_extrafields':{
-            'default':'',
-            'description':'comma separated list of additional fields to be added to document. Any fuglu Suspect variable is permitted (e.g. to_address)',
+        'elastic_extrafields': {
+            'default': '',
+            'description': 'comma separated list of additional fields to be added to document. Any fuglu Suspect variable is permitted (e.g. to_address)',
         },
         'elastic_useoriginal': {
             'default': 'True',
@@ -275,7 +269,6 @@ class ElasticBackend(AbstractBackend):
     }
     elastic_connection = {}
 
-
     def get_elastic_connection(self, reload=False):
         es = None
         elastic_uris = self.config.getlist(self.section, 'elastic_uris', resolve_env=True)
@@ -291,14 +284,12 @@ class ElasticBackend(AbstractBackend):
                 es = ElasticClient(hosts=elastic_uris, verify_certs=verify_certs, ssl_show_warn=False, timeout=timeout)
                 self.elastic_connection[elastic_uri_string] = es
         return es
-    
-    
+
     def _get_elastic_index(self, suspect):
-        indextmpl = self.config.get(self.section,'elastic_index')
+        indextmpl = self.config.get(self.section, 'elastic_index')
         indexname = apply_template(indextmpl, suspect)
         return indexname
-    
-    
+
     @smart_cached_memberfunc(inputs=[])
     def _set_index_mapping(self, indexname: str) -> None:
         self.logger.info(f'checking mapping of index {indexname}')
@@ -344,54 +335,59 @@ class ElasticBackend(AbstractBackend):
 
     def archive(self, suspect, fugluid_override=None, reload=False, retry=3):
         result = {}
-        try:
-            original = self.config.getboolean(self.section, 'elastic_useoriginal')
-            content = self._get_msg_source(suspect, original)
 
-            # store b64-encoded content if possible to account for malformed sources
-            try:
-                blob = b64encode(content).decode('ascii')
-            except Exception as e:
-                self.logger.warning(f"{suspect.id} Could not b64 encode blob due to: {str(e)}")
-                blob = None
-            doc = {
-                'fugluid': fugluid_override or suspect.id,
-                'fugluid_original': suspect.id,
-                'content': None if blob else force_uString(content),
-                'timestamp': datetime.datetime.utcfromtimestamp(suspect.timestamp),
-                'to_address': suspect.to_address,
-                'to_domain': suspect.to_domain,
-                'blob': blob,
-            }
-            
-            extrafields = self.config.getlist(self.section, 'elastic_extrafields')
-            for field in extrafields:
-                doc[field] = apply_template('${%s}' % field, suspect)
-            
-            es = self.get_elastic_connection(reload)
-            indexname = self._get_elastic_index(suspect)
-            self._set_index_mapping(indexname)
-            r = es.index(index=indexname, id=suspect.id, body=doc)
-            for key in ['_id', 'result']:
-                try:
-                    result[key] = r[key]
-                except KeyError:
-                    self.logger.error('%s key %s not found in result %s' % (suspect.id, key, r))
-            self.logger.info('%s indexed in elastic: %s' % (suspect.id, r))
+        original = self.config.getboolean(self.section, 'elastic_useoriginal')
+        content = self._get_msg_source(suspect, original)
+
+        # store b64-encoded content if possible to account for malformed sources
+        try:
+            blob = b64encode(content).decode('ascii')
         except Exception as e:
-            if retry>0:
-                self.logger.debug(f'{suspect.id} failed to index in elastic, retry={retry} reason={e.__class__.__name__}: {str(e)}')
-                time.sleep(0.2*(4-retry))
-                result = self.archive(suspect, fugluid_override, reload=True, retry=retry-1)
-            else:
-                raise
+            self.logger.warning(f"{suspect.id} Could not b64 encode blob due to: {str(e)}")
+            blob = None
+        doc = {
+            'fugluid': fugluid_override or suspect.id,
+            'fugluid_original': suspect.id,
+            'content': None if blob else force_uString(content),
+            'timestamp': datetime.datetime.utcfromtimestamp(suspect.timestamp),
+            'to_address': suspect.to_address.lower() if suspect.to_address else '',
+            'to_domain': suspect.to_domain.lower() if suspect.to_domain else '',
+            'blob': blob,
+        }
+
+        extrafields = self.config.getlist(self.section, 'elastic_extrafields')
+        for field in extrafields:
+            doc[field] = apply_template('${%s}' % field, suspect)
+
+        while retry > 0:
+            retry -= 1
+            try:
+                es = self.get_elastic_connection(reload)
+                indexname = self._get_elastic_index(suspect)
+                self._set_index_mapping(indexname)
+                r = es.index(index=indexname, id=suspect.id, body=doc, request_timeout=30)
+                for key in ['_id', 'result']:
+                    try:
+                        result[key] = r[key]
+                    except KeyError:
+                        self.logger.error('%s key %s not found in result %s' % (suspect.id, key, r))
+
+                self.logger.info('%s indexed in elastic: %s' % (suspect.id, r))
+                retry = 0  # stop while loop
+            except Exception as e:
+                if retry > 0:
+                    self.logger.debug(f'{suspect.id} failed to index in elastic, retry={retry} reason={e.__class__.__name__}: {str(e)}')
+                    time.sleep(0.2*(4-retry))
+                    reload = True
+                    #result = self.archive(suspect, fugluid_override, reload=True, retry=retry-1)
+                else:
+                    raise
         return result
-    
-    
+
     def lint(self):
         if not lint_elastic():
             return False
-        
+
         es = self.get_elastic_connection()
         if es is None:
             print('WARNING: elastic_uris not defined, this backend will do nothing')
@@ -400,9 +396,8 @@ class ElasticBackend(AbstractBackend):
             elastic_uris = self.config.getlist(self.section, 'elastic_uris', resolve_env=True)
             print(f'ERROR: failed to connect to elasticsearch {", ".join(elastic_uris)}, connection info: {str(es)}')
             return False
-        
+
         return True
-        
 
 
 class CassandraBackend(AbstractBackend):
@@ -411,17 +406,17 @@ class CassandraBackend(AbstractBackend):
     Status: abandoned (last used in 2018)
     """
     requiredvars = {
-        'cassandra_hosts':{
+        'cassandra_hosts': {
             'default': '',
-            'description':"quarantine cassandra hostnames, separated by comma",
+            'description': "quarantine cassandra hostnames, separated by comma",
         },
-        'cassandra_keyspace':{
+        'cassandra_keyspace': {
             'default': 'fugluquar',
-            'description':"quarantine cassandra keyspace",
+            'description': "quarantine cassandra keyspace",
         },
-        'cassandra_ttl':{
+        'cassandra_ttl': {
             'default': str(14*24*3600),
-            'description':"ttl for quarantined files in seconds",
+            'description': "ttl for quarantined files in seconds",
         },
         'cassandra_useoriginal': {
             'default': 'True',
@@ -429,30 +424,27 @@ class CassandraBackend(AbstractBackend):
                                 current state in fuglu (which might have been altered by previous plugins)""",
         },
     }
-    
-    
+
     def __init__(self, config, section=None):
         super().__init__(config, section)
         self.cassandra_session = None
         self.cass_prep_quarantine = None
-    
-    
+
     def get_cassandra_session(self):
         if self.cassandra_session is None:
-            keyspace=self.config.get(self.section,'cassandra_keyspace')
+            keyspace = self.config.get(self.section, 'cassandra_keyspace')
             # default idle_heartbeat_interval=30, we lower it to reduce write errors
-            hosts = self.config.getlist(self.section,'cassandra_hosts')
+            hosts = self.config.getlist(self.section, 'cassandra_hosts')
             lbp = RoundRobinPolicy()
             cassandra_cluster = Cluster(hosts, protocol_version=4, idle_heartbeat_interval=10, load_balancing_policy=lbp)
             self.cassandra_session = cassandra_cluster.connect(keyspace)
-            
+
             self.cass_prep_quarantine = self.cassandra_session.prepare("INSERT INTO quarantine (fugluid, messagecontent) VALUES (?, ?)  USING TTL ?")
         return self.cassandra_session
-    
-    
+
     def _cassandra_store(self, suspect, fugluid_override, retry=3):
         """Store message source into cassandra"""
-        ttl=self.config.getint(self.section,'cassandra_ttl')
+        ttl = self.config.getint(self.section, 'cassandra_ttl')
         original = self.config.getboolean(self.section, 'cassandra_useoriginal')
         content = self._get_msg_source(suspect, original)
         try:
@@ -460,41 +452,39 @@ class CassandraBackend(AbstractBackend):
             fugluid = fugluid_override or suspect.id
             session.execute(self.cass_prep_quarantine, (fugluid, bytearray(content), ttl), timeout=20)
         except Exception as e:
-            if retry>0:
+            if retry > 0:
                 self.logger.info('%s failed to write to cassandra. try %s/3. error was: %s' % (suspect.id, retry, str(e)))
                 time.sleep(0.1)
                 self._cassandra_store(suspect, fugluid_override, retry-1)
             else:
                 raise
-    
-    
+
     def archive(self, suspect, fugluid_override=None):
         result = {}
         self._cassandra_store(suspect, fugluid_override)
         self.logger.info('%s stored in cassandra' % suspect.id)
         result['success'] = True
         return result
-        
-        
+
     def lint(self):
         if not HAVE_CASSANDRA:
             print('ERROR: cassandra driver not available')
             return False
-        
+
         print("INFO: Cassandra Driver Version: %s" % cassandraversion)
-        
+
         try:
             self.get_cassandra_session().execute("SELECT * FROM quarantine WHERE fugluid='dummy'")
         except Exception as e:
-            print("ERROR: cassandra connection failed: %s"%str(e))
+            print("ERROR: cassandra connection failed: %s" % str(e))
             return False
-        
+
         return True
 
 
-
 dovecot_username_chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ01234567890.-_@'
 
+
 def dovecot_username(username):
     legal_username = ''
     for char in username:
@@ -505,17 +495,16 @@ def dovecot_username(username):
 
 class MyLMTP(smtplib.LMTP):
     LMTP_PORT = 24
-    
+
     # versions before 3.9 don't pass timeout to SMTP
     # https://github.com/python/cpython/blob/3.9/Lib/smtplib.py#L1079
     def __init__(self, host='', port=LMTP_PORT, local_hostname=None, source_address=None, timeout=socket._GLOBAL_DEFAULT_TIMEOUT):
         """Initialize a new instance."""
         #super().__init__(host, port, local_hostname=local_hostname, source_address=source_address)
         smtplib.SMTP.__init__(self, host, port, local_hostname=local_hostname, source_address=source_address, timeout=timeout)
-    
-    
-    
+
     # https://github.com/python/cpython/blob/3.9/Lib/smtplib.py#L803
+
     def sendmail(self, from_addr, to_addrs, msg, mail_options=(), rcpt_options=()):
         self.ehlo_or_helo_if_needed()
         esmtp_opts = []
@@ -559,7 +548,6 @@ class MyLMTP(smtplib.LMTP):
         return senderrs, resp
 
 
-
 class LMTPBackend(AbstractBackend):
     """
     This backend passes mail via LMTP to any LMTP server (however, only tested with dovecot)
@@ -593,7 +581,6 @@ class LMTPBackend(AbstractBackend):
         },
     }
 
-
     def _parse_host_port(self, hostdefinition, defaultport=None):
         port = defaultport
         if ':' in hostdefinition:
@@ -605,11 +592,10 @@ class LMTPBackend(AbstractBackend):
         else:
             host = hostdefinition
         return host, port
-    
-    
+
     def __init_socket(self):
         lmtp_hosts = self.config.getlist(self.section, 'lmtp_hosts')
-        
+
         s = None
         if not lmtp_hosts:
             return None
@@ -627,23 +613,21 @@ class LMTPBackend(AbstractBackend):
             if s is None and hosttuples:
                 raise Exception('no LMTP server is reachable')
         return s
-    
-    
+
     def __auth(self, lmtp):
         user = self.config.get(self.section, 'lmtp_user')
         password = self.config.get(self.section, 'lmtp_password')
         if user and password:
             lmtp.login(user, password)
-    
-    
+
     def __do_quarantine(self, suspect, content, retry=3):
         host = 'unknown'
         lmtp_id = None
-        
+
         lmtpfrom = self.config.get(self.section, 'lmtp_sender')
         if not lmtpfrom:
             lmtpfrom = suspect.from_address
-        
+
         try:
             lmtp = self.__init_socket()
             # noinspection PyProtectedMember
@@ -668,10 +652,9 @@ class LMTPBackend(AbstractBackend):
                 self.__do_quarantine(suspect, content, retry - 1)
             else:
                 raise
-        
+
         return host, lmtp_id
-    
-    
+
     def _lmtp_queueid(self, resp):
         """
         sample response:
@@ -696,8 +679,7 @@ class LMTPBackend(AbstractBackend):
         if len(v) > 3:
             qid = v[2]
         return qid
-    
-    
+
     def archive(self, suspect, fugluid_override=None):
         original = self.config.getboolean(self.section, 'lmtp_useoriginal')
         content = self._get_msg_source(suspect, original)
@@ -707,13 +689,12 @@ class LMTPBackend(AbstractBackend):
             content = Suspect.prepend_header_to_source(headername, fugluid, force_bString(content))
         host, lmtp_id = self.__do_quarantine(suspect, content)
         return dict(host=host, lmtp_id=lmtp_id)
-    
-    
+
     def lint(self):
         if not self.config.get(self.section, 'lmtp_hosts'):
             print('WARNING: LMTP quarantine disabled')
             return True
-        
+
         try:
             lmtp = self.__init_socket()
             if lmtp is None:
@@ -728,9 +709,8 @@ class LMTPBackend(AbstractBackend):
         except Exception as e:
             print('ERROR: LMTP connection error: %s' % str(e))
             success = False
-        
-        return success
 
+        return success
 
 
 ARCHIVE_BACKENDS = OrderedDict()
@@ -742,10 +722,10 @@ ARCHIVE_BACKENDS['cassandra'] = CassandraBackend
 
 class ArchivePlugin(ScannerPlugin):
     r"""
-This plugins stores a copy of the message if it matches certain criteria (Suspect Filter).
+This plugin stores a copy of the message if it matches certain criteria (Suspect Filter).
 You can use this if you want message archives for your domains, need a quarantine or to debug problems occuring only for certain recipients.
 The architecture allows to store data in various backends (databases), either simultaneously or using one backend as main and others as fallback in case the main backend is unavailable.
-Currently the following backends are supported:
+Currently, the following backends are supported:
 - localdir: save mail to a local directory structure. This backend is stable and in active use.
 - lmtp: pass message to LMTP server. This backend is stable and in active use.
 - elastic: store message in elasticsearch. This backend is in development (2021).
@@ -783,7 +763,7 @@ Note: The first rule to match in a message is the only rule that will be applied
 """
 
     def __init__(self, config, section=None):
-        ScannerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
 
         self.requiredvars = {
             'archiverules': {
@@ -817,12 +797,10 @@ Note: The first rule to match in a message is the only rule that will be applied
         self.filter = None
         self.logger = self._logger()
         self.backends = OrderedDict()
-    
-    
+
     def __str__(self):
         return "Archive"
-    
-    
+
     def _load_backends(self):
         self.backends = OrderedDict()
         backend_names = self.config.getlist(self.section, 'archivebackends')
@@ -833,17 +811,17 @@ Note: The first rule to match in a message is the only rule that will be applied
                 self.backends[backend_name] = backend
             except KeyError:
                 self.logger.error(f'invalid backend name {backend_name}')
-    
-    
+
     def do_archive(self, suspect):
         result = {}
         errors = {}
-        
+
         headername = self.config.get(self.section, 'fugluid_headername')
         if not headername:
             fugluid_override = None
         else:
-            fugluid_override = suspect.get_message_rep().get(headername)
+            # use headeronly message rep in case message is too large
+            fugluid_override = suspect.build_headeronly_message_rep().get(headername)
             if fugluid_override is not None:
                 fugluid_override = fugluid_override.strip()
             skipmissing = self.config.getboolean(self.section, 'fugluid_headername_skipmissing')
@@ -853,11 +831,12 @@ Note: The first rule to match in a message is the only rule that will be applied
             elif suspect.check_id(fugluid_override) is None:
                 errors['preparation'] = f'previous fugluid is not valid: {fugluid_override}'
                 return result, errors
-        
+
         self._load_backends()
         multibackend = self.config.getboolean(self.section, 'multibackend')
-        
+
         for backend_name in self.backends:
+            self.logger.debug(f"{suspect.id} try to archive using backend {backend_name}...")
             backend = self.backends[backend_name]
             try:
                 r = backend.archive(suspect, fugluid_override)
@@ -865,20 +844,20 @@ Note: The first rule to match in a message is the only rule that will be applied
                 if not multibackend:
                     break
             except Exception as e:
-                #self.logger.error(f'{suspect.id} failed to archive using backend {backend_name} due to {e.__class__.__name__}: {str(e)}')
+                self.logger.debug(f'{suspect.id} failed to archive using backend {backend_name} due to {e.__class__.__name__}: {str(e)}')
                 errors[backend_name] = f'{e.__class__.__name__}: {str(e)}'
         return result, errors
-    
-    
+
     def _set_archive_tags(self, suspect):
         suspect.set_tag('archive.ham', suspect.is_ham())
         suspect.set_tag('archive.spam', suspect.is_spam())
         suspect.set_tag('archive.blocked', suspect.is_blocked())
         suspect.set_tag('archive.virus', suspect.is_virus())
-    
-    
+
     def _archive(self, suspect):
+        self.logger.debug(f"{suspect.id} archive -> set archived tag to false")
         suspect.set_tag('archived', False)
+        self.logger.debug(f"{suspect.id} archive -> loading rules")
         archiverules = self.config.get(self.section, 'archiverules')
         if archiverules is None or archiverules == "":
             return DUNNO
@@ -887,55 +866,57 @@ Note: The first rule to match in a message is the only rule that will be applied
             self.logger.error('%s Archive Rules file does not exist: %s' % (suspect.id, archiverules))
             return DUNNO
 
+        self.logger.debug(f"{suspect.id} archive -> setup SuspectFilter with arhive rules")
         if self.filter is None:
             self.filter = SuspectFilter(archiverules)
-        
+
+        self.logger.debug(f"{suspect.id} archive -> set_archive_tags")
         self._set_archive_tags(suspect)
+        self.logger.debug(f"{suspect.id} archive -> filter match")
         match, arg = self.filter.matches(suspect)
+        self.logger.debug(f"{suspect.id} archive -> match:{bool(match)}")
         if match:
             if arg is not None and arg.lower().strip() == 'no':
                 suspect.debug("Suspect matches archive exception rule")
-                self.logger.debug("%s Header matches archive exception rule - not archiving" % suspect.id)
+                self.logger.debug(f"{suspect.id} Header matches archive exception rule - not archiving")
             else:
                 if arg is not None and arg.lower().strip() != 'yes':
-                    self.logger.warning("%s Unknown archive action '%s' assuming 'yes'" % (suspect.id, arg))
-                self.logger.debug("%s Header matches archive rule" % suspect.id)
+                    self.logger.warning(f"{suspect.id} Unknown archive action '{arg}' assuming 'yes'")
+                self.logger.debug(f"{suspect.id} Header matches archive rule")
                 if suspect.get_tag('debug'):
                     suspect.debug("Suspect matches archiving rule (would be archived it if we weren't in debug mode)" % suspect.id)
                 else:
                     result, errors = self.do_archive(suspect)
                     if result:
                         suspect.set_tag('archived', True)
-                        self.logger.info('%s archived successfully: %s' % (suspect.id, result))
+                        self.logger.info(f"{suspect.id} archived successfully: {result}")
                     if errors:
-                        self.logger.warning('%s archiving errors in backend %s' % (suspect.id, ','.join(list(errors.keys()))))
+                        self.logger.warning(f"{suspect.id} archiving errors in backend {','.join(list(errors.keys()))}")
                         for key in errors:
-                            self.logger.error('%s archiving errors in backend %s: %s' % (suspect.id, key, errors[key]))
+                            self.logger.error(f"{suspect.id} archiving errors in backend {key}: {errors[key]}")
                     suspect.set_tag('archived.result', result)
                     suspect.set_tag('archived.errors', errors)
-                    
+
         else:
             suspect.debug("No archive rule/exception rule applies to this message")
-            self.logger.debug("%s No archive rule/exception rule applies to this message" % suspect.id)
-    
-    
+            self.logger.debug(f"{suspect.id} No archive rule/exception rule applies to this message")
+
     def examine(self, suspect):
+        self.logger.debug(f"{suspect.id} running archive plugin")
         self._archive(suspect)
         if not suspect.get_tag('archived') and suspect.get_tag('archived.errors'):
             return self._problemcode()
         return DUNNO
-    
-    
-    def process(self, suspect, decision):
-        self._archive(suspect)
 
+    def process(self, suspect, decision):
+        self.logger.debug(f"{suspect.id} running archive plugin")
+        self._archive(suspect)
 
     def lint_filter(self):
         filterfile = self.config.get(self.section, 'archiverules')
         sfilter = SuspectFilter(filterfile)
         return sfilter.lint()
-    
-    
+
     def lint(self):
         ok = self.check_config()
         if ok:
@@ -947,4 +928,4 @@ Note: The first rule to match in a message is the only rule that will be applied
                 backend_ok = backend.lint()
                 if not backend_ok:
                     ok = False
-        return ok
\ No newline at end of file
+        return ok
