@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -52,7 +52,7 @@ Tags:
 """
 
     def __init__(self, config, section=None):
-        AVScannerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.requiredvars = {
             'host': {
                 'default': 'localhost',
@@ -87,7 +87,7 @@ Tags:
                 'default': 'DEFAULTVIRUSACTION',
                 'description': "action if infection is detected (DUNNO, REJECT, DELETE)",
             },
-            
+
             'virusaction_unofficial': {
                 'default': '',
                 'description': "action if infection is detected by UNOFFICIAL (3rd party) signature (DUNNO, REJECT, DELETE). if left empty use standard virusaction",
@@ -102,49 +102,48 @@ Tags:
                 'default': 'threat detected: ${virusname}',
                 'description': "reject message template if running in pre-queue mode and virusaction=REJECT",
             },
-            
+
             'clamscanfallback': {
                 'default': 'False',
                 'description': "*EXPERIMENTAL*: fallback to clamscan if clamd is unavailable. YMMV, each scan can take 5-20 seconds and massively increase load on a busy system.",
             },
-            
+
             'clamscan': {
                 'default': '/usr/bin/clamscan',
                 'description': "the path to clamscan executable",
             },
-            
+
             'clamscantimeout': {
                 'default': '30',
                 'description': "process timeout",
             },
-            
+
             'skip_on_previous_virus': {
                 'default': 'none',
                 'description': 'define AVScanner engine names causing current plugin to skip if they found already a virus',
             },
-            
+
             'skiplist_file': {
                 'default': '',
                 'description': 'path to file with signature names that can be skipped if message is welcomelisted. list one signature per line, signature is case sensitive',
             },
-            
+
             'enginename': {
                 'default': '',
-                'description': 'set custom engine name (defaults to %s)' % self.enginename,
+                'description': f'set custom engine name (defaults to {self.enginename})',
             },
         }
         self.enginename = 'ClamAV'
         # logger after engine so we can use the enginename in the logger...
         self.logger = self._logger()
         self.skiplist = None
-    
-    
+
     def _init_skiplist(self):
         if self.skiplist is None:
             skiplist_file = self.config.get(self.section, 'skiplist_file')
             if skiplist_file:
                 self.skiplist = FileList(skiplist_file)
-    
+
     def _check_skiplist(self, suspect, viruses):
         """
         if message is welcomelisted, skip defined signature hits
@@ -157,10 +156,10 @@ Tags:
             skiplist = self.skiplist.get_list()
             virusnames = list(viruses.values())
             if len(virusnames) == 1 and virusnames[0] in skiplist:
-                self.logger.info('%s got hit on %s but is skiplisted' % (suspect.id, ','.join(virusnames)))
+                self.logger.info(f"{suspect.id} got hit on {','.join(virusnames)} but is skiplisted")
                 return True
         return False
-    
+
     def __str__(self):
         classname = self.__class__.__name__
         if self.section == classname:
@@ -171,18 +170,18 @@ Tags:
     def examine(self, suspect):
         if self._check_too_big(suspect):
             return DUNNO
-        
+
         skip = self._skip_on_previous_virus(suspect)
         if skip:
             self.logger.info("%s %s" % (suspect.id, skip))
             return DUNNO
-        
+
         viruses = {}
         success = False
         content = suspect.get_source()
 
         attempts = max_attempts = max(self.config.getint(self.section, 'retries'), 1)
-        #for i in range(0, self.config.getint(self.section, 'retries')):
+        # for i in range(0, self.config.getint(self.section, 'retries')):
         while attempts:
             attempts -= 1
             try:
@@ -217,7 +216,7 @@ Tags:
                 viruses = self.scan_shell(content)
                 success = True
             except Exception:
-                self.logger.error('%s failed to scan using fallback clamscan' % suspect.id)
+                self.logger.error(f'{suspect.id} failed to scan using fallback clamscan')
 
         if not success:
             return self._problemcode()
@@ -228,8 +227,7 @@ Tags:
             if viruses and self.config.get(self.section, 'virusaction_unofficial'):
                 actioncode = self._virusaction_unofficial(viruses, actioncode)
             return actioncode, message
-    
-        
+
     def _virusaction_unofficial(self, viruses, actioncode):
         # override virusaction if one of the virus hits is caused by "UNOFFICIAL" signature
         for filename in viruses:
@@ -239,18 +237,17 @@ Tags:
                 actioncode = string_to_actioncode(virusaction, self.config)
                 break
         return actioncode
-    
-    
+
     def scan_shell(self, content):
         clamscan = self.config.get(self.section, 'clamscan')
         timeout = self.config.getint(self.section, 'clamscantimeout')
-        
+
         if not os.path.exists(clamscan):
-            raise Exception('could not find clamscan executable in %s' % clamscan)
-        
+            raise Exception(f'could not find clamscan executable in {clamscan}')
+
         try:
-            process = subprocess.Popen([clamscan, u'-'], stdin=subprocess.PIPE, stdout=subprocess.PIPE) # file data by pipe
-            kill_proc = lambda p: p.kill()
+            process = subprocess.Popen([clamscan, u'-'], stdin=subprocess.PIPE, stdout=subprocess.PIPE)  # file data by pipe
+            def kill_proc(p): return p.kill()
             timer = threading.Timer(timeout, kill_proc, [process])
             timer.start()
             stdout = process.communicate(force_bString(content))[0]
@@ -260,14 +257,14 @@ Tags:
         except Exception:
             exitcode = -1
             stdout = b''
-        
-        if exitcode > 1: # 0: no virus, 1: virus, >1: error, -1 subprocess error
+
+        if exitcode > 1:  # 0: no virus, 1: virus, >1: error, -1 subprocess error
             raise Exception('clamscan error')
         elif exitcode < 0:
-            raise Exception('clamscan timeout after %ss' % timeout)
-        
+            raise Exception(f'clamscan timeout after {timeout}s')
+
         dr = {}
-        
+
         for line in stdout.splitlines():
             line = line.strip()
             if line.endswith(b'FOUND'):
@@ -279,9 +276,8 @@ Tags:
             return None
         else:
             return dr
-    
 
-    def scan_stream(self, content, suspectid ="(NA)"):
+    def scan_stream(self, content, suspectid="(NA)"):
         """
         Scan byte buffer
 
@@ -300,11 +296,11 @@ Tags:
         numChunksToSend = math.ceil(len(remainingbytes)/default_chunk_size)
         iChunk = 0
         chunklength = 0
-        self.logger.debug('%s: sending message in %u chunks of size %u bytes' % (suspectid, numChunksToSend, default_chunk_size))
+        self.logger.debug(f'{suspectid} sending message in {numChunksToSend} chunks of size {default_chunk_size} bytes')
 
         while len(remainingbytes) > 0:
             if time.time() > maxtime:
-                raise TimeoutError('Timeout sending to AV daemon')
+                raise TimeoutError(f'{suspectid}Timeout sending to AV daemon')
             iChunk = iChunk + 1
             chunklength = min(default_chunk_size, len(remainingbytes))
             #self.logger.debug('sending chunk %u/%u' % (iChunk,numChunksToSend))
@@ -313,32 +309,31 @@ Tags:
             remainingbytes = remainingbytes[chunklength:]
             s.sendall(struct.pack(b'!L', chunklength))
             s.sendall(chunkdata)
-        self.logger.debug('%s: sent chunk %u/%u, last number of bytes sent was %u' % (suspectid, iChunk, numChunksToSend, chunklength))
-        self.logger.debug('%s: All chunks sent, sending 0 - size to tell ClamAV the whole message has been sent' % suspectid)
+        self.logger.debug(f'{suspectid} sent chunk {iChunk}/{numChunksToSend}, last number of bytes sent was {chunklength}')
+        self.logger.debug(f'{suspectid} All chunks sent, sending 0 - size to tell ClamAV the whole message has been sent')
         s.sendall(struct.pack(b'!L', 0))
-        self.logger.debug('%s: 0 has been sent, now wait for answer' % suspectid)
+        self.logger.debug(f'{suspectid} 0 has been sent, now wait for answer')
         dr = {}
 
-
         result = force_uString(self._read_until_delimiter(s, suspectid)).strip()
-        self.logger.debug('%s: got result' % suspectid)
+        self.logger.debug(f'{suspectid} got result {result}')
 
         if result.startswith('INSTREAM size limit exceeded'):
-            raise Exception("%s: Clamd size limit exeeded. Make sure fuglu's clamd maxsize config is not larger than clamd's StreamMaxLength" % suspectid)
+            raise Exception(f"{suspectid} Clamd size limit exeeded. Make sure fuglu's clamd maxsize config is not larger than clamd's StreamMaxLength")
         if result.startswith('UNKNOWN'):
-            raise Exception("%s: Clamd doesn't understand INSTREAM command. very old version?" % suspectid)
+            raise Exception(f"{suspectid} Clamd doesn't understand INSTREAM command. very old version?")
 
         if pipelining:
             try:
                 ans_id, filename, virusinfo = result.split(':', 2)
                 filename = force_uString(filename.strip())  # use unicode for filename
-                virusinfo = force_uString(virusinfo.strip())  # lets use unicode for the info
+                virusinfo = force_uString(virusinfo.strip())  # use unicode for virusinfo
             except Exception:
-                raise Exception("%s: Protocol error, could not parse result: %s" % (suspectid, result))
+                raise Exception(f"{suspectid} Protocol error, could not parse result: {result}")
 
             threadLocal.expectedID += 1
             if threadLocal.expectedID != int(ans_id):
-                raise Exception("Commands out of sync - expected ID %s - got %s" % (threadLocal.expectedID, ans_id))
+                raise Exception(f"{suspectid} Commands out of sync - expected ID {threadLocal.expectedID} - got {ans_id}")
 
             if virusinfo[-5:] == 'ERROR':
                 raise Exception(virusinfo)
@@ -365,15 +360,14 @@ Tags:
             return None
         else:
             return dr
-    
-    
-    def _read_until_delimiter(self, sock, suspectID = "(NA)"):
+
+    def _read_until_delimiter(self, sock, suspectID="(NA)"):
         data = b''
         maxFailedAttempts = 40
         failedAttempt = 0
         readtimeout = self.config.getint(self.section, 'timeout') * 3  # extra timeout condition
         starttime = time.time()
-        
+
         while True:
             try:
                 #self.logger.debug(f"{suspectID}: try to receive chunk")
@@ -385,14 +379,14 @@ Tags:
                     # swarm setup with multiple clam services during an update
                     runtime = time.time() - starttime
                     if runtime > readtimeout:
-                        raise ClamavTimeout(f"{suspectID}: 0-chunk-length read timeout after {runtime:.2f}s")
+                        raise ClamavTimeout(f"{suspectID} 0-chunk-length read timeout after {runtime:.2f}s")
                     continue
                 data += chunk
                 if chunk.endswith(b'\0'):
-                    self.logger.debug(f"{suspectID}: Got all chunks... data length {len(data)}")
+                    self.logger.debug(f"{suspectID} Got all chunks... data length {len(data)}")
                     break
                 if b'\0' in chunk:
-                    raise Exception("%s: Protocol error: got unexpected additional data after delimiter"%suspectID)
+                    raise Exception(f"{suspectID} Protocol error: got unexpected additional data after delimiter")
             except socket.error as e:
                 # looks like there can be a socket error when we try to connect too quickly after sending, so
                 # better retry several times
@@ -402,10 +396,10 @@ Tags:
                 # maximum of 1 second by 40*0.025 [s] if this helps to avoid a complete rescan of the message
                 time.sleep(0.025)
                 failedAttempt += 1
-                self.logger.warning("%s: Failed receive attempt %u/%u: %s" % (suspectID,failedAttempt, maxFailedAttempts, str(e)))
+                self.logger.warning(f"{suspectID} Failed receive attempt {failedAttempt}/{maxFailedAttempts}: {str(e)}")
                 if failedAttempt == maxFailedAttempts:
-                    raise ClamavTimeout(f"{suspectID}, Max failed received attempts({maxFailedAttempts}): {str(e)}")
-            
+                    raise ClamavTimeout(f"{suspectID} Max failed received attempts({maxFailedAttempts}): {str(e)}")
+
             # Sometimes we get one socket error and after that we never get a proper answer to satisfy
             # break or exception condition in try block, neither are there subsequent socket errors.
             # This can lead to an endless loop where a worker remains forever in this while loop,
@@ -416,13 +410,11 @@ Tags:
                 raise ClamavTimeout(f"{suspectID}: Read timeout after {runtime:.2f}s > {readtimeout:.2f}s")
 
         return data[:-1]  # remove \0 at the end
-    
-    
+
     def __invalidate_socket(self):
         threadLocal.clamdsocket = None
         threadLocal.expectedID = 0
-    
-    
+
     def __init_socket__(self, oneshot=False):
         """initialize a socket connection to clamd using host/port/file defined in the configuration
         this connection is initialized with clamd's "IDSESSION" and cached per thread
@@ -449,13 +441,13 @@ Tags:
         if unixsocket:
             sock = self.config.get(self.section, 'port')
             if not os.path.exists(sock):
-                raise Exception("unix socket %s not found" % sock)
+                raise socket.error(f"unix socket {sock} not found")
             s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
             s.settimeout(socktimeout)
             try:
                 s.connect(sock)
             except socket.error:
-                raise Exception('Could not reach clamd using unix socket %s' % sock)
+                raise socket.error(f'Could not reach clamd using unix socket {sock}')
         else:
             clamd_PORT = self.config.getint(self.section, 'port')
             proto = socket.AF_INET
@@ -466,7 +458,7 @@ Tags:
             try:
                 s.connect((clamd_HOST, clamd_PORT))
             except socket.error:
-                raise Exception('Could not reach clamd using network (%s, %s)' % (clamd_HOST, clamd_PORT))
+                raise socket.error(f'Could not reach clamd using network ({clamd_HOST}:{clamd_PORT})')
             # set keepalive options
             set_keepalive_linux(s)
 
@@ -476,14 +468,13 @@ Tags:
             threadLocal.clamdsocket = s
             threadLocal.expectedID = 0
         return s
-    
-    
+
     def lint(self):
         viract = self.config.get(self.section, 'virusaction')
         print("Virusaction: %s" % actioncode_to_string(
             string_to_actioncode(viract, self.config)))
-        allok = self.checkConfig() and self.lint_ping() and self.lint_version() and self.lint_eicar()
-        
+        allok = self.check_config() and self.lint_ping() and self.lint_version() and self.lint_eicar()
+
         if self.config.getboolean(self.section, 'clamscanfallback'):
             print('WARNING: Fallback to clamscan enabled')
             starttime = time.time()
@@ -495,8 +486,7 @@ Tags:
         # print lint info for skip
         self.lintinfo_skip()
         return allok
-    
-    
+
     def lint_ping(self):
         try:
             s = self.__init_socket__(oneshot=True)
@@ -510,8 +500,7 @@ Tags:
         if result != 'PONG':
             print(f"Invalid PONG: {result}")
         return True
-    
-    
+
     def lint_version(self):
         try:
             s = self.__init_socket__(oneshot=True)
