@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -34,21 +34,19 @@ try:
     # requires kafka-python
     import kafka
     CONN_EXC.append(kafka.errors.KafkaError)
-    KAFKA_AVAILABLE=True
+    KAFKA_AVAILABLE = True
 except ImportError:
-    KAFKA_AVAILABLE=False
+    KAFKA_AVAILABLE = False
 CONN_EXC = tuple(CONN_EXC)
 
 
-
-
 class FuzorMixin(object):
     requiredvars = {
         'backend': {
             'default': 'redis',
             'description': 'storage backend to be used. currently supported: redis (check/report), kafka (report only)'
         },
-        
+
         'redis_conn': {
             'default': 'redis://localhost:6379/0',
             'description': 'redis config: redis://host:port/db',
@@ -61,7 +59,7 @@ class FuzorMixin(object):
             'default': '604800',
             'description': 'hash redis ttl in seconds',
         },
-        
+
         'kafkahosts': {
             'default': '',
             'description:': 'kafka bootstrap hosts: host1:port host2:port'
@@ -78,7 +76,7 @@ class FuzorMixin(object):
             'default': '',
             'description': 'kafka sasl password for this producer'
         },
-        
+
         'maxsize': {
             'default': '600000',
             'description':
@@ -94,25 +92,24 @@ class FuzorMixin(object):
             'description': 'timeout in seconds'
         },
     }
-    
-    def __init__(self):
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self.backend = None
-        
-        
+
     def _init_backend(self):
         backend = self.config.get(self.section, 'backend').lower()
-        
+
         try:
             if backend == 'redis':
                 self._init_backend_redis()
             elif backend == 'kafka':
                 self._init_backend_kafka()
             else:
-                raise BackendError('backend=%s, error=invalid backend' % backend)
+                raise BackendError(f'backend={backend}, error=invalid backend')
         except CONN_EXC as e:
-            raise BackendError('backend=%s, error=%s' % (backend, str(e)))
-        
-        
+            raise BackendError(f'backend={backend}, error={str(e)}')
+
     def _init_backend_redis(self):
         if self.backend is not None:
             return
@@ -121,8 +118,7 @@ class FuzorMixin(object):
         redis_pool = RedisPooledConn(redis_url, password=password)
         self.backend = RedisBackend(redis_pool)
         self.backend.ttl = self.config.getint(self.section, 'redis_ttl')
-    
-    
+
     def _init_backend_kafka(self):
         if self.backend is not None:
             return
@@ -133,8 +129,6 @@ class FuzorMixin(object):
         timeout = self.config.getint(self.section, 'timeout')
         self.backend = KafkaBackend(hosts, topic, timeout, username, password, self.config)
 
-
-
     def lint(self):
         if not self.check_config():
             return False
@@ -142,7 +136,7 @@ class FuzorMixin(object):
         if self.config.getboolean(self.section, 'stripoversize'):
             maxsize = self.config.getint(self.section, 'maxsize')
             print("Stripping oversize messages (size > %u) to calculate a fuzor hash..." % maxsize)
-        
+
         backend = self.config.get(self.section, 'backend').lower()
         if backend == 'redis':
             ok = self._lint_redis()
@@ -154,14 +148,13 @@ class FuzorMixin(object):
             print('ERROR: invalid backend %s' % backend)
             ok = False
         return ok
-    
-    
+
     def _lint_redis(self):
         ok = True
         if not REDIS_ENABLED:
             print('ERROR: redis extension not available. This plugin will do nothing.')
             return False
-        
+
         # check redis version
         # 2.10.a has a bug which can make the connections stuck with python3
         py3minversion = [2, 10, 2]
@@ -190,67 +183,65 @@ class FuzorMixin(object):
                 else:
                     ok = False
                     print('ERROR: redis server did not reply to ping')
-    
+
             except redis.exceptions.ConnectionError as e:
                 ok = False
                 print('ERROR: failed to talk to redis server: %s' % str(e))
-                
+
         return ok
-    
-    
+
     def _lint_kafka(self):
         ok = True
         if not KAFKA_AVAILABLE:
             print('ERROR: kafka module not available. This plugin will do nothing.')
             ok = False
-            
+
         if ok:
             try:
                 self._init_backend()
             except Exception as e:
                 print('ERROR: failed to connect to kafka: %s' % str(e))
-                
+
         return ok
-    
-    
+
     def report(self, suspect):
         digest = None
         count = 0
         if not REDIS_ENABLED:
             return digest, count
-        
+
         maxsize = self.config.getint(self.section, 'maxsize')
         if suspect.size > maxsize:
             if self.config.getboolean(self.section, 'stripoversize'):
-                suspect.debug('Fuzor: message too big (%u), stripping down to %u' % (suspect.size, maxsize))
-                msg = message_from_bytes(
+                suspect.debug(f'Fuzor: message size={suspect.size} too big, stripping down to maxsize={maxsize}')
+                self.logger.debug(f'{suspect.id} message size={suspect.size} too big, stripping down to maxsize={maxsize}')
+                msgrep = message_from_bytes(
                     suspect.source_stripped_attachments(maxsize=maxsize),
                     _class=PatchedMessage
                 )
             else:
-                self.logger.debug('%s Size Skip, %s > %s' % (suspect.id, suspect.size, maxsize))
+                self.logger.debug(f'{suspect.id} message too big, {suspect.size} > {maxsize}')
                 return digest, count
         else:
-            msg = suspect.get_message_rep()
-        
-        fuhash = FuzorDigest(msg)
-        
+            msgrep = suspect.get_message_rep()
+
+        fuhash = FuzorDigest(msgrep)
+
         try:
             self.logger.debug(
-                "suspect %s to=%s hash=%s usable_body=%s predigest=%s subject=%s" %
-                (suspect.id, suspect.to_address, fuhash.digest, fuhash.bodytext_size, fuhash.predigest[:50], msg.get('Subject')))
+                f"{suspect.id} to={suspect.to_address} hash={fuhash.digest} usable_body={fuhash.bodytext_size} predigest={fuhash.predigest[:50]} subject={msg.get('Subject')}")
         except Exception:
             pass
-        
+
         if fuhash.digest is not None:
             digest = fuhash.digest
             try:
                 self._init_backend()
                 have_backend = True
             except BackendError as e:
-                self.logger.error('failed to connect to backend: %s' % str(e))
+                self.logger.error(f'{suspect.id} failed to connect to backend: {str(e)}')
                 have_backend = False
-            
+
             amount = len(suspect.recipients)
             attempts = 2
             while attempts:
@@ -258,7 +249,7 @@ class FuzorMixin(object):
                 try:
                     self._init_backend()
                     count = self.backend.increase(fuhash.digest, amount)
-                    self.logger.info("suspect %s hash %s seen %s times before" % (suspect.id, fuhash.digest, count - 1))
+                    self.logger.info(f"{suspect.id} hash {fuhash} seen {count-1} times before")
                     attempts = 0
                 except redis.exceptions.TimeoutError as e:
                     msg = f'{suspect.id} failed increasing count due to {str(e)}'
@@ -272,81 +263,71 @@ class FuzorMixin(object):
                     self.backend = None
                     self.logger.warning(msg) if attempts else self.logger.error(msg)
         else:
-            self.logger.info("suspect %s not enough data for a digest" % suspect.id)
-        
+            self.logger.debug(f"{suspect.id} not enough data for a digest")
+
         if fuhash.digest is not None:
             return digest, count
         else:
             return digest, count
 
 
-
 class FuzorReport(ScannerPlugin, FuzorMixin):
     """ Report all messages to the fuzor redis backend"""
-    
+
     def __init__(self, config, section=None):
-        ScannerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         FuzorMixin.__init__(self)
         self.requiredvars = FuzorMixin.requiredvars
         self.logger = self._logger()
-    
-    
 
     def examine(self, suspect):
         if not REDIS_ENABLED and not KAFKA_AVAILABLE:
             return DUNNO
-        
+
         digest, count = self.report(suspect)
         if digest is not None and suspect.get_tag('FuZor') is None:
             suspect.set_tag('FuZor', (digest, count))
             suspect.set_tag('fuzor.digest', digest)
             suspect.set_tag('fuzor.count', count)
         return DUNNO
-    
-    
+
     def lint(self):
         return FuzorMixin.lint(self)
 
 
-
 class FuzorReportAppender(AppenderPlugin, FuzorMixin):
     """ Report all messages to the fuzor redis backend"""
-    
+
     def __init__(self, config, section=None):
-        AppenderPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         FuzorMixin.__init__(self)
         self.requiredvars = FuzorMixin.requiredvars
         self.logger = self._logger()
-    
-    
 
     def process(self, suspect, decision):
         if not REDIS_ENABLED and not KAFKA_AVAILABLE:
             return
-        
+
         if decision not in [DUNNO, ACCEPT]:
             self.logger.debug(f'{suspect.id} not increasing count decision {ALLCODES.get(decision)}')
             return
-        
+
         digest, count = self.report(suspect)
         if digest is not None and suspect.get_tag('FuZor') is None:
             suspect.set_tag('FuZor', (digest, count))
             suspect.set_tag('fuzor.digest', digest)
             suspect.set_tag('fuzor.count', count)
             self.logger.debug(f'{suspect.id} increased fuzor count for digest {digest} by {count}')
-    
-    
-    
+
     def lint(self):
         return FuzorMixin.lint(self)
 
 
-
 class FuzorCheck(ScannerPlugin, FuzorMixin):
     """Check messages against the redis database and write spamassassin pseudo-headers"""
-    
+
     def __init__(self, config, section=None):
-        ScannerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         FuzorMixin.__init__(self)
         self.logger = self._logger()
         self.requiredvars = FuzorMixin.requiredvars
@@ -355,15 +336,13 @@ class FuzorCheck(ScannerPlugin, FuzorMixin):
                 'default': 'X-FuZor',
                 'description': 'header name',
             },
-            'ignorelist':{
-                'default':'',
-                'description':'path to file containing fuzor sums which should not be considered. one hash per line',
+            'ignorelist': {
+                'default': '',
+                'description': 'path to file containing fuzor sums which should not be considered. one hash per line',
             },
         })
         self.ignorelist = None
-        
-        
-        
+
     def _init_ignorelist(self):
         if self.ignorelist is None:
             filename = self.config.get(self.section, 'ignorelist')
@@ -371,45 +350,44 @@ class FuzorCheck(ScannerPlugin, FuzorMixin):
                 self.ignorelist = FileList(
                     filename=filename,
                     lowercase=True, additional_filters=[FileList.inline_comments_filter])
-    
-    
-    
+
     def examine(self, suspect):
         if not REDIS_ENABLED:
             return DUNNO
-        
+
         # self.logger.info("%s: FUZOR START"%suspect.id)
         # start=time.time()
         maxsize = self.config.getint(self.section, 'maxsize')
         if suspect.size > maxsize:
 
             if self.config.getboolean(self.section, 'stripoversize'):
-                suspect.debug('Fuzor: message too big (%u), stripping down to %u' % (suspect.size, maxsize))
-                msg = message_from_bytes(
+                suspect.debug(f'Fuzor: message size={suspect.size} too big, stripping down to maxsize={maxsize}')
+                self.logger.debug(f'{suspect.id} message size={suspect.size} too big, stripping down to maxsize={maxsize}')
+                msgrep = message_from_bytes(
                     suspect.source_stripped_attachments(maxsize=maxsize),
                     _class=PatchedMessage
                 )
             else:
                 suspect.debug('Fuzor: message too big, not digesting')
-                self.logger.debug('suspect %s message too big, not digesting' % suspect.id)
+                self.logger.debug(f'{suspect.id} message too big, {suspect.size} > {maxsize}')
                 # self.logger.info("%s: FUZOR END (SIZE SKIP)"%suspect.id)
                 return DUNNO
         else:
-            msg = suspect.get_message_rep()
+            msgrep = suspect.get_message_rep()
         # self.logger.info("%s: FUZOR PRE-HASH"%suspect.id)
-        fuhash = FuzorDigest(msg)
+        fuhash = FuzorDigest(msgrep)
         digest = fuhash.digest
         # self.logger.info("%s: FUZOR POST-HASH"%suspect.id)
         if digest is not None:
-            suspect.debug('Fuzor digest = %s' % digest)
-            
+            suspect.debug(f'Fuzor digest {digest}')
+
             self._init_ignorelist()
             if self.ignorelist is not None:
                 ignorelist = self.ignorelist.get_list()
                 if digest in ignorelist:
-                    self.logger.info('%s ignoring fuzor hash %s' % (suspect.id, digest))
+                    self.logger.info(f'{suspect.id} ignoring fuzor hash {digest}')
                     return DUNNO
-            
+
             count = 0
             attempts = 2
             while attempts:
@@ -455,58 +433,53 @@ class FuzorCheck(ScannerPlugin, FuzorMixin):
             suspect.set_tag('fuzor.count', count)
             if count > 0:
                 # self.logger.info("%s: FUZOR WRITE HEADER"%suspect.id)
-                suspect.write_sa_temp_header("%s-ID" % headername, digest)
-                suspect.write_sa_temp_header("%s-Lvl" % headername, count)
+                suspect.write_sa_temp_header(f"{headername}-ID", digest)
+                suspect.write_sa_temp_header(f"{headername}-Lvl", count)
                 self.logger.info("%s digest %s seen %s times" % (suspect.id, digest, count))
             else:
-                self.logger.debug("%s digest %s not seen before" % (suspect.id, digest))
+                self.logger.debug(f"{suspect.id} digest {digest} not seen before")
         else:
-            suspect.debug('%s not enough data for a digest' % suspect.id)
-        
+            suspect.debug(f'{suspect.id} not enough data for a digest')
+
         # diff=time.time()-start
         # self.logger.info("%s: FUZOR END (NORMAL), time =
         # %.4f"%(suspect.id,diff))
         return DUNNO
-    
-    
+
     def lint(self):
         ok = True
         backend = self.config.get(self.section, 'backend').lower()
         if backend == 'kafka':
             print('ERROR: kafka backend is only supported for reports')
             ok = False
-        
+
         filename = self.config.get(self.section, 'ignorelist')
         if filename and not os.path.exists(filename):
             ok = False
             print('cannot find ignorelist file %s' % filename)
-            
+
         if ok:
             ok = FuzorMixin.lint(self)
         return ok
-    
-    
+
 
 class FuzorPrint(ScannerPlugin):
     """Just print out the fuzor hash (for debugging) """
-    
+
     def __init__(self, config, section=None):
-        ScannerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.logger = self._logger()
-    
-    
-    
+
     def examine(self, suspect):
         msg = suspect.get_message_rep()
         fuhash = FuzorDigest(msg)
         if fuhash.digest is not None:
-            self.logger.info("Predigest: %s" % fuhash.predigest)
-            self.logger.info('%s: hash %s' % (suspect.id, fuhash.digest))
+            self.logger.info(f'{suspect.id} Predigest: {fuhash.predigest}')
+            self.logger.info(f'{suspect.id} hash {fuhash.digest}')
         else:
-            self.logger.info('%s does not produce enough data for a unique hash' % suspect.id)
-        
-        return DUNNO
+            self.logger.info(f'{suspect.id} does not produce enough data for a unique hash')
 
+        return DUNNO
 
 
 class FuzorDigest(object):
@@ -517,16 +490,18 @@ class FuzorDigest(object):
         self.bodytext_size = 0
         self.filter = SuspectFilter(None)
         self.logger = logging.getLogger('fuglu.plugins.fuzor.Digest')
-        
+
         # digest config
         self.LONG_WORD_THRESHOLD = 10  # what is considered a long word
         self.REPLACE_LONG_WORD = '[LONG]'  # Replace long words in pre-digest with... None to disable
         self.REPLACE_EMAIL = '[EMAIL]'  # Replace email addrs in pre-digest with... None to disable
         self.REPLACE_URL = '[LINK]'  # Replace urls in pre-digest with... None to disable
-        self.INCLUDE_ATTACHMENT_CONTENT = False  # should non-text attachment contents be included in digest (not recommended, there are better attachment hash systems)
+        # should non-text attachment contents be included in digest (not recommended, there are better attachment hash systems)
+        self.INCLUDE_ATTACHMENT_CONTENT = False
         self.INCLUDE_ATTACHMENT_COUNT = True  # should the number of non-text-attachments be included in the digest
         self.MINIMUM_PREDIGEST_SIZE = 27  # if the predigest is smaller than this, ignore this message
-        self.MINIMUM_UNMODIFIED_CONTENT = 27  # minimum unmodified content after stripping, eg. [SOMETHING] removed from the predigest (27>'von meinem Iphone gesendet')
+        # minimum unmodified content after stripping, eg. [SOMETHING] removed from the predigest (27>'von meinem Iphone gesendet')
+        self.MINIMUM_UNMODIFIED_CONTENT = 27
         self.MINIMUM_BODYTEXT_SIZE = 27  # if the body text content is smaller than this, ignore this message
         self.STRIP_WHITESPACE = True  # remove all whitespace from the pre-digest
         self.STRIP_HTML_MARKUP = True  # remove html tags (but keep content)
@@ -534,12 +509,10 @@ class FuzorDigest(object):
             'script',
             'style',
         ]  # strip tags (including content)
-        
+
         self.predigest = self._make_predigest(msg)
         self.digest = self._make_hash(self.predigest)
-    
-    
-    
+
     def _make_hash(self, predigest):
         if self.bodytext_size < self.MINIMUM_BODYTEXT_SIZE:
             return None
@@ -549,12 +522,10 @@ class FuzorDigest(object):
         unmodified = re.sub(r'\[[A-Z0-9:]+\]', '', predigest)
         if len(unmodified) < self.MINIMUM_UNMODIFIED_CONTENT:
             return None
-        
+
         predigest = predigest.encode('utf-8', errors='ignore')
         return hashlib.sha1(predigest).hexdigest()
-    
-    
-    
+
     def _handle_text_part(self, part):
         payload = part.get_payload(decode=True)
         charset = part.get_content_charset()
@@ -563,42 +534,40 @@ class FuzorDigest(object):
             charset = "ascii"
         elif charset.lower().replace("_", "-") in ("quopri-codec", "quopri", "quoted-printable", "quotedprintable"):
             errors = "strict"
-        
+
         try:
             payload = payload.decode(charset, errors)
         except (LookupError, UnicodeError, AssertionError):
             payload = payload.decode("ascii", "ignore")
-        
+
         if self.STRIP_HTML_MARKUP:
             payload = self.filter.strip_text(
                 payload,
                 remove_tags=self.REMOVE_HTML_TAGS,
                 use_bfs=True)
-        
+
         if self.REPLACE_EMAIL is not None:
             payload = re.sub(r'\S{1,50}@\S{1,30}', self.REPLACE_EMAIL, payload)
-        
+
         if self.REPLACE_URL is not None:
             payload = re.sub(r'[a-z]+:\S{1,100}', self.REPLACE_URL, payload)
-        
+
         if self.REPLACE_LONG_WORD is not None:
             patt = r'\S{%s,}' % self.LONG_WORD_THRESHOLD
             payload = re.sub(patt, self.REPLACE_LONG_WORD, payload)
-        
+
         if self.STRIP_WHITESPACE:
             payload = re.sub(r'\s', '', payload)
         payload = payload.strip()
         return payload
-    
-    
-    
+
     def _make_predigest(self, msg):
         attachment_count = 0
         predigest = ''
         for part in msg.walk():
             if part.is_multipart():
                 continue
-            
+
             if part.get_content_maintype() == "text":
                 try:
                     normalized_text_part = self._handle_text_part(part)
@@ -609,16 +578,15 @@ class FuzorDigest(object):
             else:
                 attachment_count += 1
                 if self.INCLUDE_ATTACHMENT_CONTENT:
-                    predigest += "[ATTH:%s]" % hashlib.sha1(part.get_payload()).hexdigest()
-        
+                    predigest += f"[ATTH:{hashlib.sha1(part.get_payload()).hexdigest()}]"
+
         if self.INCLUDE_ATTACHMENT_COUNT and attachment_count:
-            predigest += "[ATTC:%s]" % attachment_count
-        
+            predigest += f"[ATTC:{attachment_count}]"
+
         if self.STRIP_WHITESPACE:
             predigest = re.sub(r'\s', '', predigest)
-        
-        return predigest
 
+        return predigest
 
 
 class BackendError(Exception):
@@ -628,17 +596,17 @@ class BackendError(Exception):
 class FuzorBackend(object):
     def increase(self, digest, amount=1):
         raise NotImplementedError
+
     def get(self, digest):
         raise NotImplementedError
 
 
-
 class RedisBackend(FuzorBackend):
     def __init__(self, redispool=None):
         self.redis_pool = redispool or RedisPooledConn()
         self.ttl = 7 * 24 * 3600
         self.logger = logging.getLogger("fuglu.fuzor.RedisBackend")
-    
+
     def increase(self, digest, amount=1):
         redisconn = self.redis_pool.get_conn()
         pipe = redisconn.pipeline()
@@ -648,9 +616,9 @@ class RedisBackend(FuzorBackend):
             result = pipe.execute()
         except (socket.timeout, redis.exceptions.TimeoutError):
             self.logger.info("Socket timeout in increase")
-            return 0 # counter did not get increased
+            return 0  # counter did not get increased
         return result[0]
-    
+
     def get(self, digest):
         try:
             redisconn = self.redis_pool.get_conn()
@@ -662,7 +630,6 @@ class RedisBackend(FuzorBackend):
             return 0
 
 
-
 class KafkaBackend(FuzorBackend):
     def __init__(self, bootstrap_servers, topic, timeout, username, password, config=None):
         self.logger = logging.getLogger("fuglu.fuzor.KafkaBackend")
@@ -672,18 +639,17 @@ class KafkaBackend(FuzorBackend):
             request_timeout_ms=timeout*1000, sasl_plain_username=username, sasl_plain_password=password
         )
         self.topic = topic
-    
+
     def increase(self, digest, amount=1):
         messagebytes = force_bString(digest)
         for x in range(0, amount):
             self.producer.send(self.topic, value=messagebytes, key=messagebytes)
-        return amount # stay compatible to redis backend, return 1 as in "counter increased by 1"
-    
+        return amount  # stay compatible to redis backend, return 1 as in "counter increased by 1"
+
     def get(self, digest):
         raise BackendError('Kafka backend can only be used for reporting')
 
 
-
 if __name__ == '__main__':
     import email
     import sys
