@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Fumail Project
+#   Copyright Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,17 +22,17 @@ from fuglu.stringencode import force_uString, force_bString
 from fuglu.extensions.redisext import RedisPooledConn, redis, ENABLED as REDIS_AVAILABLE, ExpiringCounter
 try:
     from domainmagic.util import unconfuse
-    DOMAINMAGIC_AVAILABLE=True
+    DOMAINMAGIC_AVAILABLE = True
 except ImportError:
-    DOMAINMAGIC_AVAILABLE=False
+    DOMAINMAGIC_AVAILABLE = False
+
     def unconfuse(value):
         return value
 try:
     import kafka
-    KAFKA_AVAILABLE=True
+    KAFKA_AVAILABLE = True
 except ImportError:
-    KAFKA_AVAILABLE=False
-
+    KAFKA_AVAILABLE = False
 
 
 class KnownSubjectMixin(object):
@@ -82,23 +82,17 @@ class KnownSubjectMixin(object):
         },
     }
 
-
-
-    def __init__(self):
-        self.backend_redis=None
+    def __init__(self, *args, **kwargs):
+        self.backend_redis = None
         self.backend_kafka = None
         self.skiplist = None
-    
-    
-    
+
     def _init_skiplist(self):
         if self.skiplist is None:
             filepath = self.config.get(self.section, 'skiplist')
             if filepath and os.path.exists(filepath):
                 self.skiplist = FileList(filename=filepath, additional_filters=[self._normalise_subject])
-    
-    
-    
+
     def _init_backend_redis(self):
         """
         Init Redis backend if not yet setup.
@@ -112,19 +106,17 @@ class KnownSubjectMixin(object):
             pinginterval = self.config.getint(self.section, 'pinginterval')
             redis_pool = RedisPooledConn(redis_conn, socket_keepalive=True, socket_timeout=socket_timeout, pinginterval=pinginterval)
             self.backend_redis = ExpiringCounter(redis_pool, ttl)
-    
-    
+
     def _insert_redis(self, suspect, subject):
         self._init_backend_redis()
         if self.backend_redis:
             multiplicator = self.config.getint(self.section, 'multiplicator')
             try:
                 self.backend_redis.increase(subject, multiplicator)
-                self.logger.info('%s subject %s registered in redis' % (suspect.id, subject))
-            except redis.exceptions.TimeoutError as e:
-                self.logger.error('%s failed to register subject in redis %s due to %s' % (suspect.id, subject, str(e)))
-    
-    
+                self.logger.info(f'{suspect.id} subject {subject} registered in redis')
+            except (redis.exceptions.TimeoutError, redis.exceptions.ConnectionError) as e:
+                self.logger.error(f'{suspect.id} failed to register subject in redis {subject} due to {str(e)}')
+
     def _init_backend_kafka(self):
         if self.backend_kafka is not None:
             return
@@ -134,11 +126,10 @@ class KnownSubjectMixin(object):
             timeout = self.config.getint(self.section, 'timeout')
             username = self.config.get(self.section, 'kafkausername')
             password = self.config.get(self.section, 'kafkapassword')
-            clientid = 'prod-fuglu-%s-%s' % (self.__class__.__name__, get_outgoing_helo(self.config))
+            clientid = f'prod-fuglu-{self.__class__.__name__}-{get_outgoing_helo(self.config)}'
             self.backend_kafka = kafka.KafkaProducer(bootstrap_servers=bootstrap_servers, api_version=(0, 10, 1), client_id=clientid,
                                                      request_timeout_ms=timeout*1000, sasl_plain_username=username, sasl_plain_password=password)
-    
-    
+
     def _insert_kafka(self, suspect, subject):
         if self.backend_kafka is None:
             self._init_backend_kafka()
@@ -146,21 +137,20 @@ class KnownSubjectMixin(object):
             messagebytes = force_bString(subject)
             try:
                 self.backend_kafka.send(self.kafkatopic, value=messagebytes, key=messagebytes)
-                self.logger.info('%s subject %s registered in kafka' % (suspect.id, subject))
+                self.logger.info(f'{suspect.id} subject {subject} registered in kafka')
             except Exception as e:
-                self.logger.error('%s failed to send message %s: %s' % (suspect.id, subject, str(e)))
+                self.logger.error(f'{suspect.id} failed to send message {suspect}: {e.__class__.__name__} {str(e)}')
                 self.producer = None
-        
-    
+
     def _normalise_subject(self, subject, to_addr=None):
         s = Suspect.decode_msg_header(subject)
         s = force_uString(s)
-        s = s.lower() # small caps only
-        s = unconfuse(s) # replace all non-ascii characters by their ascii lookalike
-        
+        s = s.lower()  # small caps only
+        s = unconfuse(s)  # replace all non-ascii characters by their ascii lookalike
+
         if to_addr:
             to_addr = force_uString(to_addr.lower())
-            to_addr_lhs, to_addr_dom = to_addr.split('@',1)
+            to_addr_lhs, to_addr_dom = to_addr.split('@', 1)
             repl_map = [
                 (to_addr, 'E'),
                 (to_addr_lhs, 'L'),
@@ -168,38 +158,35 @@ class KnownSubjectMixin(object):
             ]
             if '.' in to_addr_lhs:
                 # maybe handle cases with more than one . in lhs
-                fn, ln = to_addr_lhs.split('.',1)
+                fn, ln = to_addr_lhs.split('.', 1)
                 repl_map.append((fn, 'F'))
                 repl_map.append((ln, 'S'))
-            for k, v in repl_map: # remove to_addr,  to_addr lhs and domain name from subject
+            for k, v in repl_map:  # remove to_addr,  to_addr lhs and domain name from subject
                 s = s.replace(k, v)
 
-        s = re.sub(r'^((?:re|fwd?|aw|tr|sv|rv)\W?:?\s?)+', '',  s) # strip re/fwd prefix
-        s = re.sub(r"(?:^|\s|\b|\w)([0-9'.,-]{2,16})(?:\s|\b|\w|$)", 'N', s) # replace all number groups
-        s = re.sub(r'[0-9]', 'X', s) # replace individual numbers
-        s = re.sub(r'[^\x00-\x7F]','U', s)  # replace non-ascii chars (note: unconfuse has already replaced special
-                                           #                                chars we care by an ascii representation)
+        s = re.sub(r'^((?:re|fwd?|aw|tr|sv|rv)\W?:?\s?)+', '',  s)  # strip re/fwd prefix
+        s = re.sub(r"(?:^|\s|\b|\w)([0-9'.,-]{2,16})(?:\s|\b|\w|$)", 'N', s)  # replace all number groups
+        s = re.sub(r'[0-9]', 'X', s)  # replace individual numbers
+        s = re.sub(r'[^\x00-\x7F]', 'U', s)  # replace non-ascii chars (note: unconfuse has already replaced special
+        #                                chars we care by an ascii representation)
         s = re.sub(r'\W', '', s, re.UNICODE)  # remove all whitespaces and punctuations
-        s = s[:32] # shorten to a maximum of 32 chars
+        s = s[:32]  # shorten to a maximum of 32 chars
         return s
-    
-    
+
     def _get_subject(self, suspect):
-        msgrep = suspect.get_message_rep()
-        subject = suspect.decode_msg_header(msgrep.get('subject', ''))
+        subject = suspect.decode_msg_header(suspect.get_header('subject', ''))
         if not subject:
             return ''
         subject = self._normalise_subject(subject, suspect.to_address)
-        if subject.isupper(): # ignore subjects that only consist of placeholders
+        if subject.isupper():  # ignore subjects that only consist of placeholders
             return ''
-        if len(subject)<3: # ignore too short subjects
+        if len(subject) < 3:  # ignore too short subjects
             return ''
         self._init_skiplist()
         if self.skiplist and subject in self.skiplist.get_list():
             return ''
         return subject
-    
-    
+
     def _lint_redis(self, fc):
         ok = True
         try:
@@ -209,8 +196,8 @@ class KnownSubjectMixin(object):
             else:
                 ok = False
                 print(fc.strcolor("ERROR: ", "red"), "redis server did not reply to ping")
-    
-        except redis.exceptions.ConnectionError as e:
+
+        except (redis.exceptions.TimeoutError, redis.exceptions.ConnectionError) as e:
             ok = False
             print(fc.strcolor("ERROR: ", "red"), f"failed to talk to redis server: {str(e)}")
         except Exception as e:
@@ -219,78 +206,76 @@ class KnownSubjectMixin(object):
             import traceback
             traceback.print_exc()
         return ok
-    
-    
+
     def _lint_kafka(self, fc):
         ok = True
         try:
             self._init_backend_kafka()
         except kafka.errors.KafkaError as e:
-            print('ERROR: failed to connect to kafka: %s' % str(e))
+            print(f'ERROR: failed to connect to kafka: {str(e)}')
             ok = False
         except Exception as e:
-            print('ERROR: Error connecting to kafka: %s' % str(e))
+            print(f'ERROR: Error connecting to kafka: {e.__class__.__name__} {str(e)}')
             self.logger.exception(e)
             ok = False
         return ok
-    
-    
+
     def lint(self):
         from fuglu.funkyconsole import FunkyConsole
         fc = FunkyConsole()
         if not REDIS_AVAILABLE and not KAFKA_AVAILABLE:
             print(fc.strcolor("ERROR: ", "red"), 'neither redis nor kafka python module not available - this plugin will do nothing')
             return False
-        
+
         ok = self.check_config()
         if ok:
             self._init_skiplist()
             if self.skiplist is None:
                 print(fc.strcolor("WARNING: ", "red"), "skiplist not initialised")
                 ok = False
-        
+
         redis_conn = self.config.get(self.section, 'redis_conn')
         kafkahosts = self.config.get(self.section, 'kafkahosts')
         if not redis_conn and not kafkahosts:
             print(fc.strcolor("WARNING: ", "red"), "neither redis nor kafka specified, this plugin will have no effect")
-        
+
         if ok:
             if redis_conn:
                 ok = self._lint_redis(fc)
-        
+
         if ok:
             if kafkahosts:
                 ok = self._lint_kafka(fc)
-        
+
         return ok
 
 
-
 class KnownSubject(ScannerPlugin, KnownSubjectMixin):
-    
-    def __init__(self,config,section=None):
-        ScannerPlugin.__init__(self,config,section)
-        KnownSubjectMixin.__init__(self)
+    """
+    Check if normalised subject is found in redis database
+    """
+
+    def __init__(self, config, section=None):
+        super().__init__(config, section)
+        KnownSubjectMixin.__init__(self, config, section)
         self.logger = self._logger()
         self.requiredvars = KnownSubjectMixin.requiredvars
-    
-    
-    
-    def examine(self,suspect):
+
+    def examine(self, suspect):
         if not REDIS_AVAILABLE and not KAFKA_AVAILABLE:
             return DUNNO
-        
+
         if not suspect.from_address:
-            self.logger.debug('%s skipping bounce message' % suspect.id)
+            self.logger.debug(f'{suspect.id} skipping bounce message')
             return DUNNO
-        
+
         suspect.set_tag('KnownSubjectRun', True)
-        
+
         subject = self._get_subject(suspect)
         if not subject:
-            self.logger.debug('%s skipping empty normalised subject' % suspect.id)
+            self.logger.debug(f'{suspect.id} skipping empty normalised subject')
             return DUNNO
-        
+
         suspect.set_tag('KnownSubject', subject)
 
         attempts = 2
@@ -316,28 +301,29 @@ class KnownSubject(ScannerPlugin, KnownSubjectMixin):
                     suspect.set_tag('KnownSubjectScore', count)
                     headername = self.config.get(self.section, 'headername')
                     suspect.write_sa_temp_header(headername, count)
-                    self.logger.info('%s subject %s seen %s times' % (suspect.id, subject, count))
+                    self.logger.info(f'{suspect.id} subject {subject} seen {count} times')
                 else:
-                    self.logger.debug('%s subject %s seen %s times' % (suspect.id, subject, 0))
+                    self.logger.debug(f'{suspect.id} subject {subject} seen 0 times')
                 attempts = 0
-        
+
         return DUNNO
-    
-    
+
     def lint(self):
         return KnownSubjectMixin.lint(self)
-    
-    
-    
+
+
 class KnownSubjectAppender(AppenderPlugin, KnownSubjectMixin):
-    
-    def __init__(self,config,section=None):
-        AppenderPlugin.__init__(self,config,section)
-        KnownSubjectMixin.__init__(self)
+    """
+    Learn normalised subject to redis or kafka. Training plugin for KnownSubject.
+    """
+
+    def __init__(self, config, section=None):
+        super().__init__(config, section)
+        KnownSubjectMixin.__init__(self, config, section)
         self.logger = self._logger()
 
         self.requiredvars = KnownSubjectMixin.requiredvars
-        requiredvars={
+        requiredvars = {
             'multiplicator': {
                 'default': '1',
                 'description': 'how many times does each entry count. you may want to set a higher value for trap processors'
@@ -348,42 +334,34 @@ class KnownSubjectAppender(AppenderPlugin, KnownSubjectMixin):
             },
         }
         self.requiredvars.update(requiredvars)
-    
-    
-    
+
     def process(self, suspect, decision):
         if not REDIS_AVAILABLE and not KAFKA_AVAILABLE:
             return
-        
+
         if not suspect.from_address:
-            self.logger.debug('%s skipping bounce message' % suspect.id)
+            self.logger.debug(f'{suspect.id} skipping bounce message')
             return
-        
+
         reportall = self.config.getboolean(self.section, 'reportall')
         if not reportall and not (suspect.is_spam() or suspect.is_virus()):
-            self.logger.debug('%s skipped: not spam or virus' % suspect.id)
+            self.logger.debug(f'{suspect.id} skipped: not spam or virus')
             return
-        
+
         has_run = suspect.get_tag('KnownSubjectRun', False)
         if has_run:
             subject = suspect.get_tag('KnownSubject')
         else:
             subject = self._get_subject(suspect)
             if not subject:
-                self.logger.debug('%s skipping empty normalised subject' % suspect.id)
+                self.logger.debug(f'{suspect.id} skipping empty normalised subject')
                 return
-            
+
         if subject is not None:
             self._insert_redis(suspect, subject)
             self._insert_kafka(suspect, subject)
-            
+
         return
-    
-    
+
     def lint(self):
         return KnownSubjectMixin.lint(self)
-    
-    
-    
-    
-    
