@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,7 +26,7 @@ This plugin enables the fuglu_debug functionality. Make sure fuglu listens on th
 """
 
     def __init__(self, config, section=None):
-        PrependerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         if self.section == 'MessageDebugger':
             self.section = 'debug'
 
@@ -67,8 +67,7 @@ This plugin enables the fuglu_debug functionality. Make sure fuglu listens on th
         allok = self.check_config()
 
         if debugport not in incomingport.split(','):
-            print(
-                "Debug port %s not specified in [main]::incomingport - messages can't be debugged" % debugport)
+            print("Debug port %s not specified in [main]::incomingport - messages can't be debugged" % debugport)
             allok = False
 
         return allok
@@ -79,8 +78,7 @@ This plugin enables the fuglu_debug functionality. Make sure fuglu listens on th
     def pluginlist(self, suspect, pluginlist):
         debugport = self.config.getint(self.section, 'debugport')
         if suspect.get_tag('incomingport') == debugport:
-            self.logger.info(
-                'Enabling debug mode for message on incoming port %s' % debugport)
+            self.logger.info(f'{suspect.id} Enabling debug mode for message on incoming port {debugport}')
             if self.config.getboolean(self.section, 'nobounce'):
                 suspect.tags['nobounce'] = True
             if self.config.getboolean(self.section, 'noreinject'):
@@ -90,5 +88,4 @@ This plugin enables the fuglu_debug functionality. Make sure fuglu listens on th
             fp = open(self.config.get(self.section, 'debugfile'), 'w')
             suspect.tags['debug'] = True
             suspect.tags['debugfile'] = fp
-        self.logger.debug('Debugport: %s , Incoming port: %s' %
-                          (debugport, suspect.get_tag('incomingport')))
+        self.logger.debug(f"{suspect.id} debugport={debugport} incomingport={suspect.get_tag('incomingport')}")
