@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,12 +24,12 @@ class PluginSkipper(PrependerPlugin):
     """Skips plugins based on standard filter file
     This can be used for example to skip spam filters on outgoing messages.
     e.g. put this in /etc/fuglu/skipplugins.regex:
-    
+
     @incomingport    1099    SAPlugin
     """
 
     def __init__(self, config, section=None):
-        PrependerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.filter = None
         self.requiredvars = {
             'filterfile': {
@@ -38,12 +38,10 @@ class PluginSkipper(PrependerPlugin):
             }
         }
         self.logger = self._logger()
-    
-    
+
     def __str__(self):
         return "Plugin Skipper"
-    
-    
+
     def pluginlist(self, suspect, pluginlist):
         """Removes scannerplugins based on filter file"""
         if not self._initfilter():
@@ -66,8 +64,7 @@ class PluginSkipper(PrependerPlugin):
                 suspect.set_tag(f'{name}.skipreason', skipplugindict[name])
                 self.logger.debug(f'{suspect.id} skipping plugin {name} due to {skipplugindict[name]}')
         return listcopy
-    
-    
+
     def _initfilter(self):
         if self.filter is not None:
             return True
@@ -82,12 +79,10 @@ class PluginSkipper(PrependerPlugin):
 
         self.filter = SuspectFilter(filename)
         return True
-    
-    
+
     def lint(self):
         return self.check_config() and self.lint_filter()
-    
-    
+
     def lint_filter(self):
         filterfile = self.config.get(self.section, 'filterfile')
         sfilter = SuspectFilter(filterfile)
@@ -98,7 +93,7 @@ class AppenderSkipper(PrependerPlugin):
 
     """
     Skips appender plugins based on standard filter file
-    
+
     This can be used for example to skip appenders on outgoing messages.
     e.g. put this in /etc/fuglu/skipappenders.regex:
 
@@ -106,7 +101,7 @@ class AppenderSkipper(PrependerPlugin):
     """
 
     def __init__(self, config, section=None):
-        PrependerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.filter = None
         self.requiredvars = {
             'filterfile': {
@@ -115,12 +110,10 @@ class AppenderSkipper(PrependerPlugin):
             }
         }
         self.logger = self._logger()
-    
-    
+
     def __str__(self):
         return "Appender Plugin Skipper"
-    
-    
+
     def appenderlist(self, suspect, pluginlist):
         """Removes appender plugins based on filter file"""
         if not self._initfilter():
@@ -141,8 +134,7 @@ class AppenderSkipper(PrependerPlugin):
                 listcopy.remove(plug)
                 self.logger.debug(f'{suspect.id} skipping appender {plug}')
         return listcopy
-    
-    
+
     def _initfilter(self):
         if self.filter is not None:
             return True
@@ -157,12 +149,10 @@ class AppenderSkipper(PrependerPlugin):
 
         self.filter = SuspectFilter(filename)
         return True
-    
-    
+
     def lint(self):
         return self.check_config() and self.lint_filter()
-    
-    
+
     def lint_filter(self):
         filterfile = self.config.get(self.section, 'filterfile')
         sfilter = SuspectFilter(filterfile)
