@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Fumail Project
+#   Copyright Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,10 +27,10 @@ from fuglu.plugins.uriextract import EXCLUDE_DOMAIN, EXCLUDE_FQDN
 from fuglu.extensions.dnsquery import revlookup, FuNXDOMAIN, FuTIMEOUT, FuSERVFAIL, FuNoNameserver, FuNoAnswer
 from fuglu.connectors.asyncmilterconnector import MilterSession
 
-BE_VERBOSE = False # for debugging
+BE_VERBOSE = False  # for debugging
 
 
-def ip2network(ipstring: str, prefixlen: int=32) -> str:
+def ip2network(ipstring: str, prefixlen: int = 32) -> str:
     """
     Take an ip string and the prefixlen to calculate the network string
     which can be used as a key in the RateLimitPlugin
@@ -46,6 +46,7 @@ def ip2network(ipstring: str, prefixlen: int=32) -> str:
 # Singleton implementation for Domainmagic
 class DomainMagic(object):
     _instance = None
+
     @classmethod
     def instance(cls):
         if cls._instance is None:
@@ -159,6 +160,7 @@ def asn(ipaddr: tp.Union[str, ipaddress.IPv4Address, ipaddress.IPv6Address], geo
         response_network,
     )
 
+
 def match_key_in_array(inputarray: tp.Optional[tp.List[str]], searchstring: str = "", suspect=None) -> tp.Optional[str]:
     """Search for string in array"""
     if inputarray is None:
@@ -200,7 +202,7 @@ def convert_truefalse(input: tp.Optional[str], suspect=None) -> str:
     return str(bool(input))
 
 
-def create_sudomain_list(domain: str, reverse: bool=False, suspect=None) -> tp.Optional[tp.List[str]]:
+def create_sudomain_list(domain: str, reverse: bool = False, suspect=None) -> tp.Optional[tp.List[str]]:
     """
     Create subdomain list, from domain to smallest subdomain
     unless reversed.
@@ -269,6 +271,7 @@ def get_nameservers(idomain: tp.Union[str, tp.List[str]], suspect=None) -> tp.Op
 
     return None
 
+
 def filter4left_tuple(tuplelist: tp.Union[tp.Tuple, tp.List],
                       filter: str,
                       lowercase: bool = True,
@@ -327,8 +330,8 @@ def decode_from_type_headers(tuplelist: tp.Union[tp.Tuple, tp.List], name="From"
 
 
 def select_tuple_index(tuplelist: tp.Union[tp.Tuple, tp.List],
-                 index: int,
-                 suspect=None) -> tp.Optional[tp.List[str]]:
+                       index: int,
+                       suspect=None) -> tp.Optional[tp.List[str]]:
     suspectid = suspect.id if suspect else '<>'
     if isinstance(tuplelist, tuple):
         tuplelist = tuplelist[0]
@@ -338,6 +341,18 @@ def select_tuple_index(tuplelist: tp.Union[tp.Tuple, tp.List],
         return None
 
 
+def select_element_by_index(iterable: tp.Union[tp.Tuple, tp.List],
+                            index: int,
+                            suspect=None) -> tp.Optional[tp.List[str]]:
+    suspectid = suspect.id if suspect else '<>'
+    try:
+        return iterable[index]
+    except Exception as e:
+        logger = logging.getLogger(f'fuglu.helperfuncs.get_element_by_index(id={suspectid})')
+        logger.error(f"Can't select element {index} of iterable {iterable}: {str(e)}")
+    return None
+
+
 def domain_from_email(emaillist: tp.List[str], suspect=None) -> tp.Optional[tp.List[str]]:
     if emaillist:
         out = list(set([m.rsplit("@", 1)[1] for m in emaillist if m and "@" in m]))
@@ -387,3 +402,41 @@ def get_ptr(ipaddress: str, suspect=None) -> tp.Optional[str]:
         logger.error(f"got: {e.__class__.__name__}: {str(e)} querying PTR for: {ipaddress}")
     logger.debug(f"no ptr defined in miltersession: return unknown")
     return "unknown"
+
+
+def suspecttagsummary(inputdictstring: tp.Optional[tp.Dict[str, bool]], suspect=None):
+    try:
+        for key in list(inputdictstring.keys()):
+            val = inputdictstring[key]
+            if val:
+                return True
+    except Exception as e:
+        suspectid = f"{suspect.id if suspect else '<>'}"
+        logger = logging.getLogger(f'fuglu.helperfuncs.suspecttagsummary(id={suspectid})')
+        logger.error(f"({type(e)}) {str(e)}")
+        pass
+    return None
+
+
+def sender_recipient_match(recipient: str, suspect: tp.Optional[MilterSession] = None) -> tp.Optional[str]:
+    """Return email if recipient matches sender of suspect, None otherwise"""
+    suspectid = f"{suspect.id if suspect else '<>'}"
+    logger = logging.getLogger(f'fuglu.helperfuncs.sender_recipient_match(id={suspectid})')
+    if not suspect:
+        logger.debug("no suspect -> return")
+        return None
+    sender = force_uString(suspect.sender, convert_none=True)
+    if not (sender and sender.strip()):
+        logger.debug(f"no sender({suspect.sender}) -> return")
+        return None
+    if not (recipient and recipient.strip()):
+        logger.debug(f"no recipient({recipient}) suspect.recipients=[{force_uString(suspect.recipients)}]-> return")
+        return None
+    sender_norm = sender.strip().lower()
+    recipient_norm = recipient.strip().lower()
+    if sender_norm == recipient_norm:
+        logger.debug(f"sender({sender_norm}) == recipient({recipient})")
+        return sender_norm
+    else:
+        logger.debug(f"sender({sender_norm}) != recipient({recipient}) -> return")
+        return None
