@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Fumail Project
+#   Copyright Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ import ipaddress as ia
 import asyncio
 import copy
 import time
+from copy import deepcopy
 try:
     _ = tp.OrderedDict
 except AttributeError:
@@ -62,7 +63,7 @@ except ImportError:
     HAVE_YAML = False
 
 ALLSTATES = list(BasicMilterPlugin.ALL_STATES.keys()) + [EOM]
-BE_VERBOSE = False # enable for debugging
+BE_VERBOSE = False  # enable for debugging
 
 
 SKIP = -100
@@ -89,6 +90,7 @@ class CounterInt(object):
     targetSuspects = ()
 
     """Interface for a counter"""
+
     def __init__(self, fieldname: str, setupdict: tp.Dict):
         self.fieldname = fieldname
         self.setupdict = setupdict
@@ -110,30 +112,27 @@ class CounterInt(object):
 
         self.invert = bool(setupdict.get("invert"))
         self._lint = setupdict.get('lint', [])
-        
+
         loggername = "%s.limiter.%s" % (__package__ if __package__ else "fuglu", self.__class__.__name__)
         self.logger = logging.getLogger(loggername)
-    
-    
+
     def lint(self) -> int:
         """Lint counter rules if defined"""
 
         from fuglu.funkyconsole import FunkyConsole
         fc = FunkyConsole()
-        
+
         if not HAVE_YAML:
             print(fc.strcolor("ERROR", "red"), 'yaml module not found, this plugin will not do anything')
             return False
-
+        
         try:
-            from mock import patch, MagicMock
-        except ModuleNotFoundError:
-            try:
-                from unittest.mock import patch, MagicMock
-            except (ModuleNotFoundError, AttributeError):
-                print(fc.strcolor('WARNING', 'yellow'), f"LINT \"RateLimitPlugin -> Limiter -> Counter\" requires python package 'mock' or 'unittest' for proper linting capabilities")
-                MagicMock = None
-                patch = None
+            from unittest.mock import patch, MagicMock
+        except (ModuleNotFoundError, AttributeError):
+            print(fc.strcolor('WARNING', 'yellow'),
+                  f"LINT \"RateLimitPlugin -> Limiter -> Counter\" requires python package 'mock' or 'unittest' for proper linting capabilities")
+            MagicMock = None
+            patch = None
 
         try:
             from collections import defaultdict
@@ -274,7 +273,7 @@ class CounterInt(object):
         elif self.ipmatch:
             # If there is one (or several) network ranges defined,
             # only count if ip is in net
-            
+
             try:
                 # create ip
                 ip = ia.ip_address(field)
@@ -285,7 +284,7 @@ class CounterInt(object):
                 for net in self.ipmatch:
                     if ip in net:
                         return self._invert(True)
-            
+
             return self._invert(False)
         else:
             # default is True, always count
@@ -335,6 +334,7 @@ class CounterSuspectAttr(CounterInt):
     targetSuspects = (Suspect,)
 
     """Use an attribute of the Suspect"""
+
     def _get_field(self, suspect: Suspect, ignore_errors: bool = True):
         try:
             return force_uString(getattr(suspect, self.fieldname))
@@ -348,17 +348,47 @@ class CounterSuspectAttr(CounterInt):
         return default_template_values(suspect=suspect)
 
 
+class CounterSuspectAttrRaw(CounterInt):
+    """Like CounterSuspectAttr but _get_field doesn't return pure string"""
+    targetSuspects = (Suspect,)
+
+    """Use an attribute of the Suspect"""
+
+    def _get_field(self, suspect: Suspect, ignore_errors: bool = True):
+        try:
+            return getattr(suspect, self.fieldname)
+        except AttributeError:
+            if ignore_errors:
+                return None
+            else:
+                raise
+
+    def get_suspect_dict(self, suspect: Suspect) -> tp.Dict:
+        return default_template_values(suspect=suspect)
+
+
 class CounterSuspectTag(CounterInt):
     targetSuspects = (Suspect,)
     """Use a tag of the Suspect"""
+
     def _get_field(self, suspect: Suspect, ignore_errors: bool = True):
         return force_uString(suspect.get_tag(self.fieldname, None))
 
 
+class CounterSuspectTagRaw(CounterInt):
+    """Like CounterSuspectTag but return raw content of tag without converting to str"""
+    targetSuspects = (Suspect,)
+    """Use a tag of the Suspect"""
+
+    def _get_field(self, suspect: Suspect, ignore_errors: bool = True):
+        return suspect.get_tag(self.fieldname, None)
+
+
 class MilterSessionAttr(CounterInt):
     targetSuspects = (sm.MilterSession, asm.MilterSession)
 
     """Use an attribute of the Suspect"""
+
     def get_suspect_dict(self, suspect: tp.Union[asm.MilterSession, sm.MilterSession]) -> tp.Dict:
         return suspect.get_templ_dict()
 
@@ -376,10 +406,13 @@ class Limiter(object):
     """Rule for RateLimiter"""
     CTYPE = {'suspect': CounterSuspectAttr,
              'miltersession': MilterSessionAttr,
-             'suspecttag': CounterSuspectTag
+             'suspecttag': CounterSuspectTag,
+             'suspectraw': CounterSuspectAttrRaw,
+             'suspecttagraw': CounterSuspectTagRaw,
              }
 
     """Rule for ratelimit"""
+
     def __init__(self, name: str, setupdict: tp.Dict):
         # set priority, default is 0
         self.priority = setupdict.get('priority', None)
@@ -398,7 +431,7 @@ class Limiter(object):
             self.bounceaction = SKIP if action == "skip" else string_to_actioncode(bounceaction)
         else:
             self.bounceaction = self.action
-        
+
         loggername = "%s.limiter.%s" % (__package__ if __package__ else "fuglu", self.__class__.__name__)
         self.logger = logging.getLogger(loggername)
 
@@ -408,7 +441,6 @@ class Limiter(object):
         assert self.strategy in AVAILABLE_STRATEGIES, \
             f"{name}: \"{self.strategy}\" is not in available strategies \"{AVAILABLE_STRATEGIES}\""
 
-
         # extract rate
         self.number, self.frame = Limiter._parse_rate(setupdict.get('rate', ""))
         self.states = Limiter._parse_state(setupdict.get('state'))
@@ -571,20 +603,20 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
 
             'backendtype': {
                 'default': 'memory',
-                'description': 'type of backend where the events are stored. memory is only recommended for low traffic standalone systems. alternatives are: redis, sqlalchemy, aioredis' #pylint: disable=C0301
+                'description': 'type of backend where the events are stored. memory is only recommended for low traffic standalone systems. alternatives are: redis, sqlalchemy, aioredis'  # pylint: disable=C0301
             },
 
             'backendconfig': {
                 'default': '',
-                'description': 'backend specific configuration. sqlalchemy: the database url, redis: redis://:[password]@hostname:6379/0' #pylint: disable=C0301
+                'description': 'backend specific configuration. sqlalchemy: the database url, redis: redis://:[password]@hostname:6379/0'  # pylint: disable=C0301
             },
-            
+
             'state': {
                 'default': ','.join(BasicMilterPlugin.ALL_STATES.keys()),
                 'description': f'comma/space separated list of milter states this plugin should be '
                                f'applied ({",".join(BasicMilterPlugin.ALL_STATES.keys())})'
             },
-            
+
             'timeout': {
                 'default': "10",
                 'description': 'Processing timeout, abort further lookups when exceeded'
@@ -683,25 +715,25 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
             if not catch_exceptions:
                 raise Exception(str(e)).with_traceback(e.__traceback__)
         return limiterdict
-    
-    
+
     def examine(self, suspect: Suspect):
         try:
-            return asyncio.run(self.core(suspect=suspect, state=EOM))
+            out = asyncio.run(self.core(suspect=suspect, state=EOM))
+            return out
         except OSError as e:
             self.logger.warning(f"EOM asyncio-run(EOM) plugin got error {str(e)} -> retrying")
 
         return asyncio.run(self.core(suspect=suspect, state=EOM))
-    
-    
+
     async def core(self, suspect: tp.Union[Suspect, asm.MilterSession, sm.MilterSession], state: str = EOM) -> tp.Union[int, tp.Tuple[int, str]]:
         """Main routine examining suspect, returncode from config if one limiter is exceeded"""
         if not HAVE_YAML:
             return DUNNO
-        
-        self.logger.debug(f"{suspect.id} -> state={state} -> limiters to check {[lname for lname,l in self.limiters.items() if state in l.states and l.number ]}")
+
+        self.logger.debug(
+            f"{suspect.id} -> state={state} -> limiters to check {[lname for lname,l in self.limiters.items() if state in l.states and l.number ]}")
         timeout = self.config.getfloat(self.section, 'timeout')
-        starttime= time.time()
+        starttime = time.time()
         errors = 0
         for lname, limiter in self.limiters.items():
             if BE_VERBOSE:
@@ -741,7 +773,8 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
                     (allow, count) = curbackend.check_allowed(eventname, limiter.number, timespan, increment)
                 if count < 0:
                     errors += 1
-                self.logger.debug(f'{suspect.id} {state}-Limiter event {eventname} (allow={allow},DUNNO={limiter.action == DUNNO}),SKIP={limiter.action == SKIP} count: {count}')
+                self.logger.debug(
+                    f'{suspect.id} {state}-Limiter event {eventname} (allow={allow},DUNNO={limiter.action == DUNNO}),SKIP={limiter.action == SKIP} count: {count}')
 
                 limiteraction = limiter.action if suspect.from_address else limiter.bounceaction
                 if not allow and limiteraction == SKIP:
@@ -775,15 +808,15 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
             except ConnectionResetError as ex:
                 error = type(ex).__name__, str(ex)
                 self.logger.warning(f'{suspect.id} (state={state}) Failed to run limiter backend for strategy '
-                                  f'"{limiter.strategy}" eventname {eventname} error {error}'
-                                  )
+                                    f'"{limiter.strategy}" eventname {eventname} error {error}'
+                                    )
             except Exception as ex:
                 error = type(ex).__name__, str(ex)
                 self.logger.error(f'{suspect.id} (state={state}) Failed to run limiter backend for strategy '
                                   f'"{limiter.strategy}" eventname {eventname} error {error}'
                                   )
                 self.logger.debug(f"{suspect.id} (state={state}) error traceback...", exc_info=ex)
-            
+
             timediff = time.time() - starttime
             if timediff > timeout:
                 self.logger.warning(f'{suspect.id} timeout exceeded after %.2fs, aborting further lookups, errors={errors}' % timediff)
@@ -791,7 +824,7 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
             elif errors > 3:
                 self.logger.warning(f'{suspect.id} error count exceeded after %.2fs, aborting further lookups, errors={errors}' % timediff)
                 break
-            
+
         return DUNNO
 
     @property
@@ -925,7 +958,20 @@ class RateLimitPlugin(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin,
         return self._convert_return2milter(await self.core(suspect=sess, state=MAILFROM))
 
     async def examine_rcpt(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], recipient: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
-        return self._convert_return2milter(await self.core(suspect=sess, state=RCPT))
+        # called before recipient is added to sess.recipients, we need a special temp object to make ratelimit work...
+        original_recipients = list(sess.recipients)
+        try:
+            if recipient:
+                sess.recipients.append(recipient)
+            out = self._convert_return2milter(await self.core(suspect=sess, state=RCPT))
+        except Exception:
+            if recipient:
+                sess.recipients = original_recipients
+            raise
+        else:
+            if recipient:
+                sess.recipients = original_recipients
+        return out
 
     async def examine_header(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], key: bytes, value: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
         return self._convert_return2milter(await self.core(suspect=sess, state=HEADER))
