@@ -1,4 +1,4 @@
-#   Copyright 2009-2022 Fumail Project
+#   Copyright Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,12 +27,9 @@ import re
 import logging
 from collections import defaultdict
 try:
-    from mock import MagicMock
+    from unittest.mock import MagicMock
 except ModuleNotFoundError:
-    try:
-        from unittest.mock import MagicMock
-    except ModuleNotFoundError:
-        MagicMock = None
+    MagicMock = None
 
 from domainmagic.rbl import RBLLookup
 from domainmagic.tld import TLDMagic
@@ -63,7 +60,8 @@ except ImportError:
     HAVE_YAML = False
 
 BE_VERBOSE = False
-regexrule = re.compile(r"^/(?P<regex>.{1,200}[^\\])\/(?P<flag>[a-z])?[ \t]{1,150}((?P<astring>[A-Z]{1,8})|(?P<acode>[0-9]{3}))([ \t]{1,150}(?P<msg>.{1,100}))?$")
+regexrule = re.compile(
+    r"^/(?P<regex>.{1,200}[^\\])\/(?P<flag>[a-z])?[ \t]{1,150}((?P<astring>[A-Z]{1,8})|(?P<acode>[0-9]{3}))([ \t]{1,150}(?P<msg>.{1,100}))?$")
 ifregex = re.compile(r"if /(?P<regex>.{1,200}[^\\])\/(?P<flag>[a-z])?[ \t]{0,150}$")
 hashrule = re.compile(r"^(?P<key>\S{1,200})[ \t]{1,150}((?P<astring>[A-Z]{1,8})|(?P<acode>[0-9]{3}))([ \t]{1,150}(?P<msg>.{1,100}))?$")
 
@@ -76,7 +74,7 @@ class CreateSubdomainListMixin(object):
         self.tldmagic = TLDMagic()
         super().__init__(**kwargs)
 
-    def create_sudomain_list(self, domain: str, reverse: bool=False) -> tp.List[str]:
+    def create_sudomain_list(self, domain: str, reverse: bool = False) -> tp.List[str]:
         """Create subdomain list, from domain to smallest subdomain
         unless reversed.
 
@@ -123,7 +121,7 @@ class MatchRestriction(object):
 class RegexRestriction(MatchRestriction):
 
     @staticmethod
-    def lines2regex(lines: tp.List[str], ignoreerrors: bool=True, logger=None) -> tp.List[MatchRestriction]:
+    def lines2regex(lines: tp.List[str], ignoreerrors: bool = True, logger=None) -> tp.List[MatchRestriction]:
         """Parse lines from pcre file, create reges options"""
         restrictions = []
         multilineconditions = []
@@ -266,7 +264,7 @@ class HashTable(MatchRestriction):
         return hashcore_objects
 
     @staticmethod
-    def lines2hashcores(lines: tp.List[str], ignoreerrors: bool=True, logger=None) -> tp.Dict[str, HashCore]:
+    def lines2hashcores(lines: tp.List[str], ignoreerrors: bool = True, logger=None) -> tp.Dict[str, HashCore]:
         """Parse lines from hash file, create HashCore objects, put then in dict"""
         restrictions = {}
         for line in lines:
@@ -330,6 +328,7 @@ class HashTable(MatchRestriction):
         else:
             return ()
 
+
 class RBLRestriction(MatchRestriction):
     def __init__(self,
                  providertype: str,
@@ -352,7 +351,7 @@ class RBLRestriction(MatchRestriction):
             self.rbllookup.providers[0].descriptiontemplate = self.message
             self.message = None
         assert bool(self.rbllookup)
-        self.checksubdomains=bool(checksubdomains)
+        self.checksubdomains = bool(checksubdomains)
         self.tldmagic = TLDMagic() if self.checksubdomains else None
 
     def _create_rbl(self):
@@ -363,9 +362,7 @@ class RBLRestriction(MatchRestriction):
 
         providers = []
         providerclass = rbl.providermap[self.providertype]
-
         providerinstance = providerclass(self.searchdomain, timeout=rbl.timeout, lifetime=rbl.lifetime)
-        providerinstance.resolver = rbl.resolver
 
         # set bitmasks and filters
         for res in self.resultconfig:
@@ -394,13 +391,13 @@ class RBLRestriction(MatchRestriction):
 
     def match(self, inputstring: str) -> str:
         if self.checksubdomains:
-            tldcount=self.tldmagic.get_tld_count(inputstring)
-            parts=inputstring.split('.')
+            tldcount = self.tldmagic.get_tld_count(inputstring)
+            parts = inputstring.split('.')
 
             subrange = range(tldcount + 1, len(parts) + 1)
             checkstrings = []
             for subindex in subrange:
-                subdomain='.'.join(parts[-subindex:])
+                subdomain = '.'.join(parts[-subindex:])
                 checkstrings.append(subdomain)
         else:
             checkstrings = [inputstring]
@@ -429,6 +426,7 @@ class RBLRestriction(MatchRestriction):
     def _str_(self) -> str:
         return f"{self.__class__.__name__}({self.providertype}/{self.searchdomain})"
 
+
 class RestrictionSet(object):
     # default variables to test in the stages if not given differently
     DEFAULT_STAGE_ATTRS = {
@@ -441,7 +439,8 @@ class RestrictionSet(object):
     }
 
     """Stores a set of restrictions, for example regex list defined in a file"""
-    def __init__(self, name: str, config: tp.Dict, ignoreerrors: bool=True):
+
+    def __init__(self, name: str, config: tp.Dict, ignoreerrors: bool = True):
         self.name = name
         self.config = config
         self.restrictions = []
@@ -506,6 +505,11 @@ class RestrictionSet(object):
 
 class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCPTMixin,
                          BMPHeaderMixin, BMPEOHMixin, ScannerPlugin, BasicMilterPlugin):
+    """
+    Restrict mail access based on complex rulesets. Upon rule hit, mail can be rejected or deferred.
+
+    Rules are read from yaml-file, thus python-yaml is a required dependency.
+    """
 
     DEFAULTSTATES = [sm.CONNECT, sm.HELO, sm.MAILFROM, sm.RCPT, sm.HEADER, sm.EOH]
 
@@ -522,9 +526,8 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
     DELAY_REJ_STATE = 2
     DELAY_REJ_AFTER = 3
     DELAY_REJ_TAG = 'AccessRestrictions.delayedreject'
-    
-    get_ptr = get_ptr_helper # define here for easy monkey patching in unit tests
 
+    get_ptr = get_ptr_helper  # define here for easy monkey patching in unit tests
 
     @staticmethod
     def _load_yamlfile(filename: str) -> tp.Union[tp.Dict, tp.OrderedDict]:
@@ -631,7 +634,7 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
             else:
                 raise ValueError(f"No setup to load defined in {filename}!")
             return restrictions_sets, setupdict
-        
+
         force_state = self.config.get(self.section, 'force_state', fallback=None)
         if force_state:
             oldsetup = setup
@@ -704,7 +707,7 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
                     inputs_possible = list(self._gen_suspect_attrdict(
                         Suspect("from@fuglu.org", "to@fuglu.org", "/dev/null"), setall=True).keys())
                     inputs_possible.append("headers")
-                elif state== asm.EOH:
+                elif state == asm.EOH:
                     inputs_possible = ["headers"]
                 else:
                     if MagicMock:
@@ -775,7 +778,7 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
         return attrdict
 
     def _examine_general(self, sess: tp.Union[sm.MilterSession, asm.MilterSession, Suspect], state: str,
-                         inputstring: tp.Optional[str]=None)\
+                         inputstring: tp.Optional[str] = None)\
             -> tp.Union[bytes, int, tp.Tuple[tp.Union[bytes, int], str]]:
         """
         - delay_rejects is enabled
@@ -791,7 +794,7 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
         """
         if not HAVE_YAML:
             return sm.CONTINUE
-        
+
         position = self.delay_rejects_statedict.get(state, AccessRestrictions.DELAY_REJ_AFTER)
         previous_reject_tag = sess.tags.get(self.DELAY_REJ_TAG)
 
@@ -826,7 +829,7 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
                     checkstrings = []
                     for k, v in sess.get_message_rep().items():
                         checkstrings.append(f"{k}: {str(v)}")
-                    self.logger.debug(f"Created {len(checkstrings)} headers to check")
+                    self.logger.debug(f"{sess.id} Created {len(checkstrings)} headers to check")
                 else:
                     checkstrings = attrdict.get(rinput)
 
@@ -834,7 +837,7 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
                     checkstrings = [checkstrings]
 
                 if BE_VERBOSE:
-                    self.logger.debug(f"Test:{rinput} for Restriction({rname}) ->teststring={checkstrings}")
+                    self.logger.debug(f"{sess.id} Test:{rinput} for Restriction({rname}) ->teststring={checkstrings}")
 
                 if checkstrings:
                     for checkstring in checkstrings:
@@ -856,11 +859,11 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
                                 res, msg = res
                         else:
                             msg = None
-                        self.logger.debug(f"Test:{rinput} with input:{checkstring} for Restriction({rname}): {res}")
+                        self.logger.debug(f"{sess.id} Test:{rinput} with input:{checkstring} for Restriction({rname}): {res}")
                         try:
                             sess.tags[RESTRICTIONS_TAG][rname] = res
                         except KeyError as e:
-                            sess.tags[RESTRICTIONS_TAG] = dict(rname = res)
+                            sess.tags[RESTRICTIONS_TAG] = dict(rname=res)
 
                         # --- #
                         # hit #
@@ -881,7 +884,7 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
                                     # replace double quotes by single quotes since
                                     # we need double quotes for logstash parsing
                                     attrmap = {}
-                                    for k,v in attrdict.items():
+                                    for k, v in attrdict.items():
                                         attrmap[k] = v.replace('"', "'") if isinstance(v, str) else v
                                     msg = template.safe_substitute(attrmap)
                                     # clean message and remove newlines (might be in header parsing output):
@@ -944,7 +947,7 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
                 self.logger.warning(f"{suspect.id} Block because milter AccessRestrictions would reject with:{msg}")
                 blockinfo = {'AccessRestrictions': msg}
                 self._blockreport(suspect, blockinfo, enginename='AccessRestrictions')
-                suspect.tags['AccessRestrictions.errormessage'] = msg # deprecated
+                suspect.tags['AccessRestrictions.errormessage'] = msg  # deprecated
         return DUNNO
 
     def lint(self, state=EOM) -> bool:
@@ -958,7 +961,7 @@ class AccessRestrictions(BMPConnectMixin, BMPHeloMixin, BMPMailFromMixin, BMPRCP
         if not HAVE_YAML:
             print(fc.strcolor("ERROR", "red"), 'yaml module not found, this plugin will not do anything')
             return False
-        
+
         check_config = super().lint()
         if not check_config:
             print(fc.strcolor("ERROR - config check", "red"))
