@@ -1,5 +1,5 @@
-# -*- coding: UTF-8 -*- 
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+# -*- coding: UTF-8 -*-
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -37,18 +37,17 @@ GLOBALSCOPE = '$GLOBAL'
 if SQL_EXTENSION_ENABLED:
     from sqlalchemy import Column
     from sqlalchemy.types import Unicode, Integer
-    
+
     class UserPref(DeclarativeBase):
         __tablename__ = 'userpref'
         prefid = Column(Integer, primary_key=True)
-        username = Column(Unicode(100),nullable=False)
-        preference = Column(Unicode(30),nullable=False)
-        value = Column(Unicode(100),nullable=False)
+        username = Column(Unicode(100), nullable=False)
+        preference = Column(Unicode(30), nullable=False)
+        value = Column(Unicode(100), nullable=False)
 else:
     UserPref = None
 
 
-
 class SAPlugin(ScannerPlugin):
 
     """This plugin passes suspects to spamassassin daemon.
@@ -70,7 +69,7 @@ Tags:
 """
 
     def __init__(self, config, section=None):
-        ScannerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.requiredvars = {
             'host': {
                 'default': 'localhost',
@@ -92,7 +91,7 @@ Tags:
                 'description': "maximum size in bytes. larger messages will be skipped",
             },
 
-            'strip_oversize':{
+            'strip_oversize': {
                 'default': '1',
                 'description': "enable scanning of messages larger than maxsize. all attachments will be stripped and only headers, plaintext and html part will be scanned. If message is still oversize it will be truncated.",
             },
@@ -121,17 +120,17 @@ Tags:
                 'default': 'X-Spam-Status',
                 'description': """what header does SA set to indicate the spam status\nNote that fuglu requires a standard header template configuration for spamstatus and score extraction\nif 'forwardoriginal' is set to 0\neg. start with _YESNO_ or _YESNOCAPS_ and contain score=_SCORE_""",
             },
-            
+
             'spamheader_prepend': {
                 'default': 'X-Spam-',
                 'description': 'tells fuglu what spamassassin prepends to its headers. Set this according to your spamassassin config especially if you forwardoriginal=0 and strip_oversize=1',
             },
-            
+
             'peruserconfig': {
                 'default': 'True',
                 'description': 'enable user_prefs in SA. This hands the recipient address over the spamd connection which allows SA to search for configuration overrides',
             },
-            
+
             'lowercase_user': {
                 'default': 'True',
                 'description': 'lowercase user (envelope rcpt) before passing it to spamd'
@@ -166,7 +165,7 @@ Tags:
                 'default': '',
                 'description': "sqlalchemy db connect string, e.g. mysql:///localhost/spamassassin",
             },
-            
+
             'attach_suspect_tags': {
                 'default': '',
                 'description': "Suspect tags to attach as text part to message for scanning",
@@ -181,16 +180,13 @@ Tags:
         }
         self.logger = self._logger()
 
-
     def __str__(self):
         return "SpamAssassin"
 
-
     def lint(self):
         allok = self.check_config() and self._lint_ping() and self._lint_spam() and self._lint_blocklist()
         return allok
 
-
     def _lint_blocklist(self):
         dbconnectionstring = self.config.get(self.section, 'sql_blocklist_dbconnectstring')
         if not dbconnectionstring:
@@ -199,7 +195,7 @@ Tags:
         if not SQL_EXTENSION_ENABLED:
             print("SQL Blacklist requested but SQLALCHEMY is not enabled")
             return False
-        
+
         suspect = Suspect('dummy@example.com', 'dummy@example.com', '/dev/null')
         try:
             self._get_sql_blocklist(suspect)
@@ -209,7 +205,6 @@ Tags:
             print("SQL UserPref blocklist query failed: %s" % str(e))
             return False
 
-
     def _lint_ping(self):
         """ping sa"""
         retries = self.config.getint(self.section, 'retries')
@@ -246,17 +241,20 @@ Tags:
             time.sleep(retry_sleep)
         return False
 
-
     def _lint_spam(self):
-        spamflag, score, rules = self.safilter_symbols(GTUBE, 'test', fid="<no-suspect>")
+        values = self.safilter_symbols(GTUBE, 'test', fid="<no-suspect>")
+        if len(values) != 3:
+            print(f'Invalid SPAMD response to GTUBE: {values}')
+            return False
+
+        spamflag, score, rules = values
         if 'GTUBE' in rules:
             print("GTUBE Has been detected correctly")
             return True
         else:
             print("SA did not detect GTUBE")
             return False
-    
-    
+
     def _get_sql_blocklist(self, suspect: Suspect):
         dbconnectionstring = self.config.get(self.section, 'sql_blocklist_dbconnectstring')
         if not dbconnectionstring:
@@ -267,24 +265,22 @@ Tags:
         scopes = [
             '$GLOBAL',
             f'%{suspect.to_domain}',  # sa domain wide
-            f'*@{suspect.to_domain}', # roundcube sauserprefs plugin
+            f'*@{suspect.to_domain}',  # roundcube sauserprefs plugin
             suspect.to_address,
         ]
         query = query.filter(UserPref.username.in_(scopes))
         results = query.all()
         return results
-        
-
 
     def check_sql_blocklist(self, suspect, runtimeconfig=None):
         """Check this message against the SQL blocklist. returns highspamaction on hit, DUNNO otherwise"""
-        
+
         action = DUNNO
         message = None
         if not SQL_EXTENSION_ENABLED:
             self.logger.error('Cannot check sql blocklist, SQLALCHEMY extension is not available')
             return action, message
-        
+
         try:
             blocklistings = self._get_sql_blocklist(suspect)
         except Exception as e:
@@ -292,7 +288,7 @@ Tags:
             action = self._problemcode()
             message = 'Internal Server Error'
             return action, message
-        
+
         for result in blocklistings:
             self.logger.debug('%s checking blocklist value %s' % (suspect.id, result.value))
             # build regex
@@ -301,7 +297,7 @@ Tags:
             regexp = re.escape(result.value).replace(r'\?', '.').replace(r'\*', '.*?')
             self.logger.debug(regexp)
             pattern = re.compile(regexp)
-            
+
             if pattern.search(suspect.from_address):
                 self.logger.debug('%s Blocklist match : %s for sa pref %s' % (suspect.id, suspect.from_address, result.value))
                 confcheck = self.config
@@ -317,13 +313,12 @@ Tags:
                     message = 'Internal Server Error'
                 highspamlevel = self.config.getfloat(self.section, 'highspamlevel')
                 self._spamreport(suspect, True, True, None, highspamlevel, enginename='SpamAssassin')
-                prependheader = self.config.get( 'main', 'prependaddedheaders')
-                suspect.add_header("%sBlacklisted" % prependheader, result.value) # deprecated
+                prependheader = self.config.get('main', 'prependaddedheaders')
+                suspect.add_header("%sBlacklisted" % prependheader, result.value)  # deprecated
                 suspect.add_header("%sBlocklisted" % prependheader, result.value)
                 suspect.debug('Sender is Blocklisted: %s' % result.value)
-        
-        return action, message
 
+        return action, message
 
     def _extract_spamstatus(self, msgrep, spamheadername, suspect):
         """
@@ -356,14 +351,13 @@ Tags:
                 suspect.debug('Spamscore: %s' % spamscore)
             else:
                 self.logger.warning('%s Could not extract spam score from header: %s' % (suspect.id, spamheader))
-                suspect.debug( 'Could not read spam score from header %s' % spamheader)
+                suspect.debug('Could not read spam score from header %s' % spamheader)
             return isspam, spamscore, spamheader
         return isspam, spamscore, spamheader
-    
-    
+
     def _get_content(self, suspect):
         maxsize = self.config.getint(self.section, 'maxsize')
-        
+
         if self.config.getboolean(self.section, 'scanoriginal'):
             content = suspect.get_original_source()
         else:
@@ -371,40 +365,40 @@ Tags:
 
         # keep copy of original content before stripping
         content_orig = content
-        
+
         stripped = False
         if suspect.size > maxsize:
             stripped = True
             # send maxsize-1 to be consistent with previous implementation
             content = suspect.source_stripped_attachments(content=content, maxsize=maxsize - 1, with_mime_headers=True)
-            suspect.set_tag('SAPlugin.stripped', True) # deprecated
+            suspect.set_tag('SAPlugin.stripped', True)  # deprecated
             suspect.set_tag('SpamAssassin.stripped', True)
             suspect.write_sa_temp_header('X-Fuglu-OrigSize', str(len(content_orig)))
             self.logger.info('%s stripped attachments, body size reduced from %s to %s bytes' % (
-            suspect.id, len(content_orig), len(content)))
+                suspect.id, len(content_orig), len(content)))
         # stick to bytes
         content = force_bString(content)
-    
+
         # write fugluid
         suspect.write_sa_temp_header('X-Fuglu-Suspect', str(suspect.id))
-    
+
         # prepend temporary headers set by other plugins
         tempheaders = suspect.get_sa_temp_headers()
         if tempheaders != b'':
             content = tempheaders + content
-    
+
         # add incoming port information
         try:
             portheader = b'X-Fuglu-Incomingport: %i\r\n' % int(suspect.get_tag('incomingport', 0))
             content = portheader + content
         except (TypeError, ValueError) as e:
             self.logger.error('%s could not add incomingport header: %s' % (suspect.id, str(e)))
-    
+
         # add envelope sender information
         msgrep = suspect.get_message_rep()
         if not 'Return-Path' in msgrep.keys():
             from_address = suspect.from_address or '<>'  # bounce address should be <>
-            content = force_bString('Return-Path: %s' % from_address + '\r\n') + content
+            content = force_bString(f'Return-Path: {from_address}\r\n') + content
             self.logger.info(f'{suspect.id} Set temp Return-Path header as {from_address}')
         else:
             self.logger.debug(f'{suspect.id} Return-Path already set as {msgrep["Return-Path"]}')
@@ -470,26 +464,26 @@ Tags:
         # check if someone wants to skip sa checks
         if suspect.get_tag('SAPlugin.skip') is True or suspect.get_tag('SpamAssassin.skip') is True:
             self.logger.debug('%s Skipping SA Plugin (requested by previous plugin)' % suspect.id)
-            suspect.set_tag('SAPlugin.skipreason', 'requested by previous plugin') # deprecated
+            suspect.set_tag('SAPlugin.skipreason', 'requested by previous plugin')  # deprecated
             suspect.set_tag('SpamAssassin.skipreason', 'requested by previous plugin')
             return DUNNO
-        
+
         runtimeconfig = DBConfig(self.config, suspect)
-        
+
         maxsize = self.config.getint(self.section, 'maxsize')
         strip_oversize = self.config.getboolean(self.section, 'strip_oversize')
-        
+
         if suspect.size > maxsize and not strip_oversize:
             self.logger.info('%s Size Skip, %s > %s' % (suspect.id, suspect.size, maxsize))
             suspect.debug('Too big for spamchecks. %s > %s' % (suspect.size, maxsize))
             prependheader = self.config.get('main', 'prependaddedheaders')
             suspect.add_header("%sSA-SKIP" % prependheader, 'Too big for spamchecks. %s > %s' % (suspect.size, maxsize))
-            suspect.set_tag('SAPlugin.skipreason', 'size skip') # deprecated
+            suspect.set_tag('SAPlugin.skipreason', 'size skip')  # deprecated
             suspect.set_tag('SpamAssassin.skipreason', 'size skip')
             return self.check_sql_blocklist(suspect)
-        
+
         content, content_orig, stripped = self._get_content(suspect)
-        
+
         forwardoriginal = self.config.getboolean(self.section, 'forwardoriginal')
         if forwardoriginal:
             ret = self.safilter_report(content, suspect.to_address, fid=suspect.id)
@@ -497,30 +491,30 @@ Tags:
                 suspect.debug('SA report Scan failed - please check error log')
                 self.logger.warning('%s SA report scan FAILED' % suspect.id)
                 suspect.add_header('%sSA-SKIP' % self.config.get('main', 'prependaddedheaders'), 'SA scan failed')
-                suspect.set_tag('SAPlugin.skipreason', 'scan failed') # deprecated
+                suspect.set_tag('SAPlugin.skipreason', 'scan failed')  # deprecated
                 suspect.set_tag('SpamAssassin.skipreason', 'scan failed')
                 return self._problemcode()
             isspam, spamscore, report = ret
-            suspect.tags['SAPlugin.report'] = report # deprecated
-        
+            suspect.tags['SAPlugin.report'] = report  # deprecated
+
         else:
             filtered = self.safilter(content, suspect.to_address, fid=suspect.id)
             if filtered is None:
                 suspect.debug('SA Scan failed - please check error log')
                 self.logger.error('%s SA scan FAILED' % suspect.id)
                 suspect.add_header('%sSA-SKIP' % self.config.get('main', 'prependaddedheaders'), 'SA scan failed')
-                suspect.set_tag('SAPlugin.skipreason', 'scan failed') # deprecated
+                suspect.set_tag('SAPlugin.skipreason', 'scan failed')  # deprecated
                 suspect.set_tag('SpamAssassin.skipreason', 'scan failed')
                 return self._problemcode()
             else:
                 if stripped:
                     # create msgrep of filtered msg
-                    if isinstance(content,str):
+                    if isinstance(content, str):
                         msgrep_filtered = email.message_from_string(filtered, _class=PatchedMessage)
                     else:
                         msgrep_filtered = email.message_from_bytes(filtered, _class=PatchedMessage)
                     header_new = []
-                    for h,v in msgrep_filtered.items():
+                    for h, v in msgrep_filtered.items():
                         header_new.append(force_uString(h).strip() + ': ' + force_uString(v).strip())
                     # add headers to msg
                     sa_prepend = self.config.get(self.section, 'spamheader_prepend')
@@ -535,20 +529,20 @@ Tags:
                     content = content_orig
                 else:
                     content = filtered
-                    
-            if isinstance(content,str):
+
+            if isinstance(content, str):
                 newmsgrep = email.message_from_string(content, _class=PatchedMessage)
             else:
                 newmsgrep = email.message_from_bytes(content, _class=PatchedMessage)
-            
+
             # if original content is forwarded there's no need to reset the attachmant
             # manager. Only header have been changed.
-            suspect.set_source(content,att_mgr_reset=(not forwardoriginal))
+            suspect.set_source(content, att_mgr_reset=(not forwardoriginal))
             spamheadername = self.config.get(self.section, 'spamheader')
             isspam, spamscore, report = self._extract_spamstatus(newmsgrep, spamheadername, suspect)
-            suspect.tags['SAPlugin.report'] = report # deprecated
+            suspect.tags['SAPlugin.report'] = report  # deprecated
             self.logger.debug('suspect %s %s %s %s' % (suspect.id, isspam, spamscore, report))
-        
+
         action = DUNNO
         message = None
         try:
@@ -556,7 +550,7 @@ Tags:
             if isspam:
                 values = dict(spamscore=spamscore)
                 message = apply_template(self.config.get(self.section, 'rejectmessage'), suspect, values)
-            
+
             if isspam and not ishighspam:
                 self.logger.debug('%s Message is spam' % suspect.id)
                 suspect.debug('Message is spam')
@@ -566,31 +560,29 @@ Tags:
             else:
                 self.logger.debug('%s Message is not spam' % suspect.id)
                 suspect.debug('Message is not spam')
-            
+
             if spamscore is not None:
-                suspect.tags['SAPlugin.spamscore'] = spamscore # deprecated
+                suspect.tags['SAPlugin.spamscore'] = spamscore  # deprecated
             if ishighspam:
                 configaction = string_to_actioncode(runtimeconfig.get(self.section, 'highspamaction'), self.config)
                 if configaction is not None:
                     action = configaction
-                    
+
             self._spamreport(suspect, isspam, ishighspam, report, spamscore, enginename='SpamAssassin')
         except RESTAPIError as e:
             self.logger.warning(f'{suspect.id} failed to query REST API: {str(e)}')
             action = self._problemcode()
             message = 'Internal Server Error'
-        
+
         return action, message
-    
-    
+
     def _is_highspam(self, runtimeconfig, spamscore):
         if spamscore is not None:
             highspamlevel = runtimeconfig.getfloat(self.section, 'highspamlevel')
             if spamscore >= highspamlevel:
                 return True
         return False
-    
-    
+
     def __init_socket(self):
         unixsocket = False
 
@@ -602,7 +594,7 @@ Tags:
         if unixsocket:
             sock = self.config.get(self.section, 'port')
             if not os.path.exists(sock):
-                raise Exception("unix socket %s not found" % sock)
+                raise socket.error(f"unix socket {sock} not found")
             s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
             s.settimeout(self.config.getint(self.section, 'timeout'))
             try:
@@ -620,7 +612,6 @@ Tags:
 
         return s
 
-
     def safilter(self, messagecontent, user, fid: str = ""):
         """pass content to sa, return sa-processed mail"""
         retries = self.config.getint(self.section, 'retries')
@@ -692,7 +683,6 @@ Tags:
                 time.sleep(retry_sleep)
         return None
 
-
     def safilter_symbols(self, messagecontent, user, fid: str = ""):
         """Pass content to sa, return spamflag, score, rules"""
         ret = self._safilter_content(messagecontent, user, 'SYMBOLS', fid=fid)
@@ -705,11 +695,9 @@ Tags:
         rules = content.split(',')
         return status, score, rules
 
-
     def safilter_report(self, messagecontent, user, fid: str = ""):
         return self._safilter_content(messagecontent, user, 'REPORT', fid=fid)
 
-
     def _safilter_content(self, messagecontent, user, command, fid: str = ""):
         """pass content to sa, return body"""
         assert command in ['SYMBOLS', 'REPORT', ]
@@ -790,7 +778,6 @@ Tags:
                 time.sleep(retry_sleep)
         return None
 
-
     def debug_proto(self, messagecontent, command='SYMBOLS'):
         """proto debug.. only used for development"""
         command = command.upper()
@@ -816,7 +803,6 @@ Tags:
         print(gotback)
 
 
-
 SALEARN_LOCAL = 'local'
 SALEARN_REMOTE = 'remote'
 SALEARN_HAM = 'ham'
@@ -835,10 +821,9 @@ Tags:
  * reads ``salearn.class``, (text or None) set to 'ham' or 'spam' for learning, any other value will disable learning
  * reads ``SAPlugin.action``, (text) set to 'Set' or 'Remove' for respective action. If unset, defaults to 'Set'.
 """
-    
-    
+
     def __init__(self, config, section):
-        SAPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.requiredvars = {
             'host': {
                 'default': 'localhost',
@@ -864,7 +849,7 @@ Tags:
                 'default': '3',
                 'description': 'how often should fuglu retry the connection before giving up',
             },
-            
+
             'peruserconfig': {
                 'default': 'True',
                 'description': 'enable user_prefs in SA. This hands the recipient address over the spamd connection which allows SA to search for configuration overrides',
@@ -874,25 +859,24 @@ Tags:
                 'default': 'DUNNO',
                 'description': "action if there is a problem (DUNNO, DEFER)",
             },
-            
+
             'learn_local': {
                 'default': 'True',
                 'description': 'learn to local database',
             },
-            
+
             'learn_remote': {
                 'default': 'True',
                 'description': 'learn to remote database',
             },
-            
+
             'learn_default': {
                 'default': SALEARN_SPAM,
                 'description': 'default learn action (ham, spam or leave empty)'
             }
-        
+
         }
-        
-    
+
     def _get_databases(self):
         databases = []
         if self.config.getboolean(self.section, 'learn_local'):
@@ -900,8 +884,7 @@ Tags:
         if self.config.getboolean(self.section, 'learn_remote'):
             databases.append(SALEARN_REMOTE)
         return databases
-        
-    
+
     def examine(self, suspect):
         spamsize = suspect.size
         maxsize = self.config.getint(self.section, 'maxsize')
@@ -910,10 +893,10 @@ Tags:
             self.logger.info('%s Size Skip, %s > %s' % (suspect.id, spamsize, maxsize))
             suspect.debug('Too big for spamchecks. %s > %s' % (spamsize, maxsize))
             return DUNNO
-        
+
         messageclass = suspect.get_tag('salearn.class', self.config.get(self.section, 'learn_default'))
         learnaction = suspect.get_tag('salearn.action', SALEARN_SET)
-        
+
         if messageclass not in [SALEARN_SPAM, SALEARN_HAM]:
             self.logger.debug('%s not learning message, message class tag value=%s' % (suspect.id, messageclass))
         else:
@@ -922,16 +905,15 @@ Tags:
             if not ret:
                 return self._problemcode()
         return DUNNO
-    
-    
+
     def _salearn_content(self, messagecontent, user, messageclass=SALEARN_SPAM, learnaction=SALEARN_SET, databases=(SALEARN_LOCAL, SALEARN_REMOTE)):
         """pass content to sa, return body"""
         assert messageclass in [SALEARN_HAM, SALEARN_SPAM]
         assert learnaction in [SALEARN_SET, SALEARN_REMOVE]
-        assert 1<=len(databases)<=2
+        assert 1 <= len(databases) <= 2
         for db in databases:
             assert db in [SALEARN_LOCAL, SALEARN_REMOTE]
-            
+
         retries = self.config.getint(self.section, 'retries')
         peruserconfig = self.config.getboolean(self.section, 'peruserconfig')
         if self.config.getboolean(self.section, 'lowercase_user') and user:
@@ -960,25 +942,25 @@ Tags:
                 line1_info = force_uString(socketfile.readline())
                 self.logger.debug(f"Answer line1: {line1_info}")
                 line2_spaminfo = force_uString(socketfile.readline())
-                
+
                 answer = line1_info.strip().split()
                 if len(answer) != 3:
                     self.logger.warning("Got invalid status line from spamd: %s" % line1_info)
                     continue
-                
+
                 version, number, status = answer
                 if status != 'EX_OK':
                     self.logger.error("Got bad status from spamd: %s" % status)
                     continue
-                
+
                 self.logger.debug('Spamd said: %s' % line2_spaminfo)
                 hdr, status = line2_spaminfo.split(':')
-                if (learnaction==SALEARN_SET and hdr=='DidSet') \
-                    or (learnaction==SALEARN_REMOVE and hdr=='DidRemove'):
+                if (learnaction == SALEARN_SET and hdr == 'DidSet') \
+                        or (learnaction == SALEARN_REMOVE and hdr == 'DidRemove'):
                     success = True
                 else:
                     success = False
-                
+
                 return success
             except socket.timeout:
                 self.logger.error('SPAMD Socket timed out.')
@@ -993,19 +975,17 @@ Tags:
 
             time.sleep(1)
         return None
-    
-    
+
     def lint(self):
         allok = self.check_config() and self._lint_ping()
         if allok:
             databases = self._get_databases()
-            allok = 1<=len(databases)<=2
+            allok = 1 <= len(databases) <= 2
             if not allok:
                 print('ERROR: Enable at least one of learn_local, learn_remote')
         return allok
 
 
-
 if __name__ == '__main__':
     import sys
     if len(sys.argv) <= 1:
@@ -1016,5 +996,3 @@ if __name__ == '__main__':
     print("--------------")
     plugin.debug_proto(GTUBE, sys.argv[1])
     print("--------------")
-
-
