@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -67,7 +67,7 @@ Example script:
     """
 
     def __init__(self, config, section=None):
-        ScannerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.logger = self._logger()
         self.requiredvars = {
             'scriptdir': {
@@ -128,9 +128,9 @@ Example script:
     def exec_script(self, suspect, filename):
         action = DUNNO
         message = ''
-        debug = lambda message: self._debug(suspect, message)
-        info = lambda message: self.logger.info(message)
-        warn = lambda message: self.logger.warning(message)
+        def debug(message): return self._debug(suspect, message)
+        def info(message): return self.logger.info(message)
+        def warn(message): return self.logger.warning(message)
 
         def stop(action=DUNNO, message=''):
             raise Stopped(action, message)
@@ -166,7 +166,7 @@ Example script:
         if os.path.isdir(scriptdir):
             filelist = os.listdir(scriptdir)
             scripts = [os.path.join(scriptdir, f)
-                for f in filelist if f.endswith('.fgf')]
+                       for f in filelist if f.endswith('.fgf')]
             scripts = sorted(scripts)
             return scripts
         else:
