@@ -19,7 +19,7 @@ import os
 import typing as tp
 import fuglu.connectors.asyncmilterconnector as asm
 import fuglu.connectors.milterconnector as sm
-from fuglu.mshared import  BMPRCPTMixin, BasicMilterPlugin
+from fuglu.mshared import BMPRCPTMixin, BasicMilterPlugin
 from fuglu.shared import _SuspectTemplate, FileList, get_default_cache
 from fuglu.stringencode import force_uString
 from fuglu.extensions.sql import get_domain_setting, SQL_EXTENSION_ENABLED, get_session
@@ -30,7 +30,7 @@ class EnforceTLS(BMPRCPTMixin, BasicMilterPlugin):
     """
     set TLS policy by recipient domain. Allows to enforce TLS on per-recipient base
     """
-    
+
     def __init__(self, config, section=None):
         super().__init__(config, section)
         self.logger = self._logger()
@@ -63,16 +63,15 @@ class EnforceTLS(BMPRCPTMixin, BasicMilterPlugin):
                                f'applied ({",".join(BasicMilterPlugin.ALL_STATES.keys())})'
             }
         }
-    
-    
+
     def enforce_domain(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], to_domain: str) -> bool:
         dbconnection = self.config.get(self.section, 'dbconnection').strip()
         datasource = self.config.get(self.section, 'datasource')
         enforce = False
-        
+
         if datasource.strip() == '':
             enforce = True
-        
+
         elif datasource.startswith('txt:'):
             domainfile = datasource[4:]
             if self.selective_domain_loader is None:
@@ -81,13 +80,13 @@ class EnforceTLS(BMPRCPTMixin, BasicMilterPlugin):
                 if to_domain == domain or domain.startswith('*.') and fnmatch.fnmatch(to_domain, domain):
                     enforce = True
                     break
-        
+
         # use DBConfig instead of get_domain_setting
         elif SQL_EXTENSION_ENABLED and datasource.startswith('sql:') and dbconnection != '':
             cache = get_default_cache()
             sqlquery = datasource[4:]
             enforce = get_domain_setting(to_domain, dbconnection, sqlquery, cache, self.section, False, self.logger)
-        
+
         elif datasource.startswith('tag:'):
             keylist = datasource.split(':')[1:]
             value = sess.tags
@@ -95,19 +94,18 @@ class EnforceTLS(BMPRCPTMixin, BasicMilterPlugin):
                 value = value.get(key, {})
                 if isinstance(value, bool):
                     enforce = value
-        
+
         return enforce
-    
-    
+
     def examine_rcpt(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], recipient: bytes) -> tp.Union[bytes, tp.Tuple[bytes, str]]:
         """Use asnyc routine because tls info needs async patched milter lib"""
-        encryption_protocol = sess.tls_version
         if not recipient:
             return sm.CONTINUE
-        
+
         recipient = force_uString(recipient)
         enforce = self.enforce_domain(sess, sm.MilterSession.extract_domain(recipient))
-        
+        encryption_protocol = sess.milter_macros.get('tls_version')
+
         action = sm.CONTINUE
         message = None
         if enforce and not encryption_protocol:
@@ -116,27 +114,26 @@ class EnforceTLS(BMPRCPTMixin, BasicMilterPlugin):
             templ_dict = sess.get_templ_dict()
             templ_dict['recipient'] = recipient
             message = template.safe_substitute(templ_dict)
-        self.logger.debug(f"{sess.id} recpient:{recipient}, enforce:{enforce}, encprot:{encryption_protocol}")
+        self.logger.info(f"{sess.id} recpient={recipient} enforce={enforce} encproto={force_uString(encryption_protocol)} action={sm.RETCODE2STR.get(action)}")
         return action, message
-    
-    
+
     def lint(self, state=None) -> bool:
         if state and state not in self.state:
             # not active in current state
             return True
-        
+
         lint_ok = True
-        if not self.checkConfig():
+        if not self.check_config():
             print('Error checking config')
             lint_ok = False
-        
+
         action = sm.STR2RETCODE.get(self.config.get(self.section, 'action'), None)
-        
+
         if not action:
             lint_ok = False
             print(f"Action value '{self.config.get(self.section, 'action')}' not in allowed "
                   f"choices: 'continue', 'tempfail', 'reject'")
-        
+
         if lint_ok:
             datasource = self.config.get(self.section, 'datasource')
             if datasource.strip() == '':
@@ -166,19 +163,18 @@ class EnforceTLS(BMPRCPTMixin, BasicMilterPlugin):
             else:
                 lint_ok = False
                 print('Could not determine domain list backend type')
-        
+
         return lint_ok
-    
+
     def __str__(self):
         return "EnforceTLS"
 
 
-
 class TLSSender(BMPRCPTMixin, BasicMilterPlugin):
     """
     set TLS policy by sender domain. Allows to enforce TLS on per-sender base
     """
-    
+
     def __init__(self, config, section=None):
         super().__init__(config, section)
         self.logger = self._logger()
@@ -202,66 +198,62 @@ class TLSSender(BMPRCPTMixin, BasicMilterPlugin):
                                f'applied ({",".join(BasicMilterPlugin.ALL_STATES.keys())})'
             }
         }
-    
-    
+
     def enforce_domain(self, from_domain: str) -> bool:
         enforce = False
         domainfile = self.config.get(self.section, 'domains_file')
-        
+
         if domainfile.strip() == '':
             enforce = True
-        
+
         if self.selective_domain_loader is None:
             self.selective_domain_loader = FileList(domainfile, lowercase=True)
         for domain in self.selective_domain_loader.get_list():
             if from_domain == domain or domain.startswith('*.') and fnmatch.fnmatch(from_domain, domain):
                 enforce = True
                 break
-        
+
         return enforce
-    
-    
+
     def examine_rcpt(self, sess: tp.Union[asm.MilterSession, sm.MilterSession], recipient: bytes) -> tp.Union[
-        bytes, tp.Tuple[bytes, str]]:
+            bytes, tp.Tuple[bytes, str]]:
         """Note this routine is not async atm..."""
-        encryption_protocol = sess.tls_version
         sender = sess.sender
-        
+
         if not sender:
             return sm.CONTINUE
-        
+
         sender = force_uString(sender)
         sender_domain = sm.MilterSession.extract_domain(sender)
         enforce = self.enforce_domain(sender_domain)
-        
+        encryption_protocol = sess.milter_macros.get('tls_version')
+
         action = sm.CONTINUE
         message = None
         if enforce and not encryption_protocol:
             action = sm.STR2RETCODE.get(self.config.get(self.section, 'action'), action)
             template = _SuspectTemplate(self.config.get(self.section, 'messagetemplate'))
             message = template.safe_substitute(sess.get_templ_dict())
-        self.logger.debug(f"{sess.id} sender: {sender}, domain: {sender_domain}, enforce: {enforce}, "
-                          f"encprot: {force_uString(encryption_protocol)}, action: {sm.RETCODE2STR[action]}")
+        self.logger.info(f"{sess.id} sender={sender} enforce={enforce} encproto={force_uString(encryption_protocol)} action={sm.RETCODE2STR.get(action)}")
         return action, message
-    
-    
+
     def lint(self, state=None) -> bool:
         if state and state not in self.state:
             # not active in current state
             return True
-        
+
         lint_ok = True
-        if not self.checkConfig():
+        if not self.check_config():
             print('Error checking config')
             lint_ok = False
-        
+
         action = sm.STR2RETCODE.get(self.config.get(self.section, 'action'), None)
-        
+
         if not action:
             lint_ok = False
             print(f"Action value '{self.config.get(self.section, 'action')}' not in allowed "
                   f"choices: 'continue', 'tempfail', 'reject'")
-        
+
         if lint_ok:
             domainfile = self.config.get(self.section, 'domains_file')
             if domainfile.strip() == '':
@@ -270,10 +262,8 @@ class TLSSender(BMPRCPTMixin, BasicMilterPlugin):
                 if not os.path.exists(domainfile):
                     print('Cannot find domain file %s' % domainfile)
                     lint_ok = False
-        
+
         return lint_ok
-    
-    
+
     def __str__(self):
         return "TLSSender"
-
