@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -125,7 +125,7 @@ class VacationReply(object):
 
 if SQL_EXTENSION_ENABLED:
     from sqlalchemy import Table, Column, TEXT, TIMESTAMP, Integer, ForeignKey, Unicode, Boolean
-    from sqlalchemy.orm import mapper, relation
+    from sqlalchemy.orm import mapper, relationship
     metadata = DeclarativeBase.metadata
 
     vacation_table = Table(
@@ -147,13 +147,13 @@ if SQL_EXTENSION_ENABLED:
         Column('vacation_id', None, ForeignKey('vacation.id')),
         Column('sent', TIMESTAMP, nullable=False),
         Column('recipient', Unicode(255), nullable=False),
-        )
+    )
     vacation_mapper = mapper(Vacation, vacation_table)
 
     vacationreply_mapper = mapper(
         VacationReply,
         vacationreply_table,
-        properties=dict(vacation=relation(Vacation, backref='replies'),)
+        properties=dict(vacation=relationship(Vacation, backref='replies'),)
     )
 
 
@@ -268,7 +268,7 @@ SQL Example for mysql:
 """
 
     def __init__(self, config, section=None):
-        ScannerPlugin.__init__(self, config, section)
+        super().__init__(config, section)
         self.requiredvars = {
             'dbconnectstring': {
                 'default': '',
@@ -465,7 +465,7 @@ SQL Example for mysql:
         if not SQL_EXTENSION_ENABLED:
             print("Vacation requires the fuglu sql extension to be enabled")
             return False
-        
+
         dbconnectstring = self.config.get(self.section, 'dbconnectstring')
         try:
             dbsession = get_session(dbconnectstring)
