@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -45,8 +45,8 @@ except ImportError:
 
 
 class ProcManager(object):
-    def __init__(self, logQueue, numprocs = None, queuesize=100, config = None):
-        self._child_id_counter=0
+    def __init__(self, logQueue, numprocs=None, queuesize=100, config=None):
+        self._child_id_counter = 0
         self._logQueue = logQueue
         self.manager = multiprocessing.Manager()
         self.shared_state = self._init_shared_state()
@@ -109,8 +109,8 @@ class ProcManager(object):
             self.logger.exception(e)
 
     def _create_worker(self):
-        self._child_id_counter +=1
-        worker_name = "Worker-%s"%self._child_id_counter
+        self._child_id_counter += 1
+        worker_name = "Worker-%s" % self._child_id_counter
         worker = multiprocessing.Process(target=fuglu_process_worker, name=worker_name,
                                          args=(self.tasks, self.config, self.shared_state, self.child_to_server_messages, self._logQueue))
         return worker
@@ -210,23 +210,23 @@ class ProcManager(object):
         self.manager.shutdown()
         self.logger.debug("done...")
 
+
 class MessageListener(threading.Thread):
     def __init__(self, message_queue):
-        threading.Thread.__init__(self)
+        super().__init__()
         self.name = "Process Message Listener"
         self.message_queue = message_queue
         self.stayalive = True
         self.statskeeper = Statskeeper()
         self.daemon = True
 
-
     def run(self):
         while self.stayalive:
             message = self.message_queue.get()
             if message is None:
                 break
             event_type = message['event_type']
-            if event_type == 'statsdelta': # increase statistics counters
+            if event_type == 'statsdelta':  # increase statistics counters
                 try:
                     delta = StatDelta(**message)
                     self.statskeeper.increase_counter_values(delta)
@@ -241,17 +241,17 @@ def fuglu_process_worker(queue, config, shared_state, child_to_server_messages,
 
     logtools.client_configurer(logQueue)
     logging.basicConfig(level=logging.DEBUG)
-    workerstate = WorkerStateWrapper(shared_state,'loading configuration')
+    pidinfo = logtools.createPIDinfo()
+    workerstate = WorkerStateWrapper(shared_state, 'loading configuration')
     logger = logging.getLogger('fuglu.process.%s(%u)' % (workerstate.process.name, workerstate.process.pid))
-    logger.debug("New worker: %s" % logtools.createPIDinfo())
-
+    logger.debug(f"{pidinfo}: New worker")
 
     # Setup address compliance checker
     # -> Due to default linux forking behavior this should already
-    #    have the correct setup but it's better not to rely on this
-    address_check = config.get('main','address_compliance_checker', fallback='Default')
-    Addrcheck().set(address_check)
-
+    #    have the correct setup, but it's better not to rely on this
+    address_check = config.get('main', 'address_compliance_checker', fallback='Default')
+    maxsize = config.getint('main', 'address_compliance_checker_maxsize', fallback=64)
+    Addrcheck().set(address_check, maxsize)
 
     # load config and plugins
     logger.debug("Create MainController")
@@ -287,29 +287,30 @@ def fuglu_process_worker(queue, config, shared_state, child_to_server_messages,
     # forward statistics counters to parent process
     stats = Statskeeper()
     stats.stat_listener_callback.append(lambda event: child_to_server_messages.put(event.as_message()))
-
-    logger.debug("%s: Enter service loop..." % logtools.createPIDinfo())
+    
+    
+    logger.debug(f"{pidinfo}: Enter service loop...")
 
     try:
-        logger.info(f"{logtools.createPIDinfo()}: Child worker process ready")
+        logger.info(f"{pidinfo}: Child worker process ready")
         while True:
             workerstate.workerstate = 'waiting for task'
-            logger.debug("%s: Child process waiting for task" % logtools.createPIDinfo())
+            logger.debug(f"{pidinfo}: Child process waiting for task")
             task = queue.get()
-            if task is None: # poison pill
-                logger.debug("%s: Child process received poison pill - shut down" % logtools.createPIDinfo())
+            if task is None:  # poison pill
+                logger.debug(f"{pidinfo}: Child process received poison pill - shut down")
                 try:
                     # it might be possible it does not work to properly set the workerstate
                     # since this is a shared variable -> prevent exceptions
                     workerstate.workerstate = 'ended (poison pill)'
                 except Exception as e:
-                    logger.debug("Exception setting workstate while getting poison pill")
+                    logger.debug(f"{pidinfo}: Exception setting workstate while getting poison pill")
                     logger.exception(e)
                     pass
                 finally:
                     return
             workerstate.workerstate = 'starting scan session'
-            logger.debug("%s: Child process starting scan session" % logtools.createPIDinfo())
+            logger.debug(f"{pidinfo}: Child process starting scan session")
             sock, handler_modulename, handler_classname, port = uncompress_task(task)
             handler_class = getattr(importlib.import_module(handler_modulename), handler_classname)
             handler_instance = handler_class(sock, config)
@@ -331,15 +332,16 @@ def fuglu_process_worker(queue, config, shared_state, child_to_server_messages,
 
     except KeyboardInterrupt:
         workerstate.workerstate = 'ended (keyboard interrupt)'
-        logger.debug("Keyboard interrupt")
+        logger.debug(f"{pidinfo}: Keyboard interrupt")
     except Exception as e:
-        logger.error("Exception in worker process: %s" % str(e), exc_info=e)
+        logger.error(f"{pidinfo}: Exception in worker process: {str(e)}", exc_info=e)
         workerstate.workerstate = 'crashed'
     finally:
         # this process will not put any object in queue
         queue.close()
         controller.shutdown()
 
+
 def debug_procpoolworkermemory(logger, config):
     """
     Debug memory usage using the objgraph library, eventually
@@ -361,7 +363,7 @@ def debug_procpoolworkermemory(logger, config):
     suspectobjects = objgraph.by_type('Suspect')
     if len(suspectobjects) > 0:
         if writedebuggraphs:
-            objgraph.show_backrefs(suspectobjects[-1], max_depth=5,refcounts=True,
+            objgraph.show_backrefs(suspectobjects[-1], max_depth=5, refcounts=True,
                                    filename=os.path.join(config.get('main', 'tempdir'), 'suspects.dot'))
         logger.info("Refcounts on last subject: %u" % sys.getrefcount(suspectobjects[-1]))
     mailattachmentobjects = objgraph.by_type('Mailattachment')
@@ -382,14 +384,15 @@ def debug_procpoolworkermemory(logger, config):
     allobjects = suspectobjects + mailattachmentobjects + mailattachmentmanagerobjects
     if len(allobjects) > 0:
         logger.error('objects in memory: Suspect: %u, MailAttachments: %u, MailAttachment_mgr: %u'
-                     % (len(suspectobjects),len(mailattachmentobjects),len(mailattachmentmanagerobjects)))
+                     % (len(suspectobjects), len(mailattachmentobjects), len(mailattachmentmanagerobjects)))
     else:
         logger.debug('objects in memory: Suspect: %u, MailAttachments: %u, MailAttachment_mrt: %u'
-                     % (len(suspectobjects),len(mailattachmentobjects),len(mailattachmentmanagerobjects)))
+                     % (len(suspectobjects), len(mailattachmentobjects), len(mailattachmentmanagerobjects)))
     del suspectobjects
     del mailattachmentobjects
     del mailattachmentmanagerobjects
 
+
 class WorkerStateWrapper(object):
     def __init__(self, shared_state_dict, initial_state='created', process=None):
         self._state = initial_state
