@@ -1,4 +1,4 @@
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -101,7 +101,7 @@ class BasicTCPServer(object):
         self.logger.debug('Starting incoming Server on Port %s, protocol=%s' % (
             port, self.protohandlerclass.protoname))
         self.logger.debug('Incoming server process info:  %s' % createPIDinfo())
-        self.logger.debug('(%s) Logger id is %s' % (createPIDinfo(),id(self)))
+        self.logger.debug('(%s) Logger id is %s' % (createPIDinfo(), id(self)))
         self.port = port
         self.controller = controller
         self.stayalive = True
@@ -147,12 +147,13 @@ class BasicTCPServer(object):
             try:
                 self.logger.debug('Waiting for connection...')
                 nsd = self._socket.accept()
-                sock,addr = nsd
+                sock, addr = nsd
                 if not self.stayalive:
                     break
                 handler_classname = self.protohandlerclass.__name__
                 handler_modulename = self.protohandlerclass.__module__
-                self.logger.debug('(%s) Incoming connection  [incoming server port: %s, prot: %s]' % (createPIDinfo(),self.port,self.protohandlerclass.protoname))
+                self.logger.debug('(%s) Incoming connection  [incoming server port: %s, prot: %s]' %
+                                  (createPIDinfo(), self.port, self.protohandlerclass.protoname))
                 if self.controller.threadpool:
                     # this will block if queue is full
                     self.controller.threadpool.add_task_from_socket(sock, handler_modulename, handler_classname, self.port)
@@ -168,8 +169,8 @@ class BasicTCPServer(object):
                     engine.handlesession()
             except ConnectionAbortedError as e:
                 if self.stayalive:
-                   self.logger.warning('Connection aborted - break and setting stayalive to False')
-                   self.stayalive = False
+                    self.logger.warning('Connection aborted - break and setting stayalive to False')
+                    self.stayalive = False
                 break
             except Exception as e:
                 exc = traceback.format_exc()
@@ -215,4 +216,3 @@ def uncompress_task(task):
     pickled_socket, handler_modulename, handler_classname, port = task
     sock = pickle.loads(pickled_socket)
     return sock, handler_modulename, handler_classname, port
-
