@@ -1,4 +1,4 @@
-#   Copyright 2009-2022 Oli Schacher, Fumail Project #
+#   Copyright Oli Schacher, Fumail Project #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -27,7 +27,6 @@ from fuglu.stats import Statskeeper, StatDelta
 from fuglu.shared import DUNNO, ACCEPT, REJECT, DEFER, DELETE, utcnow
 from fuglu.connectors.check import HealthCheckSuspect
 from fuglu.debug import CrashStore
-from fuglu.stringencode import force_uString
 from collections import defaultdict
 
 
@@ -40,10 +39,10 @@ class TrackTimings(object):
             enable (bool): enable/disable time tracker
             port (int): incoming port
         """
-        self.timings_logger = logging.getLogger("%s.Timings" % __package__)
+        self.timings_logger = logging.getLogger(f"{__package__}.Timings")
 
         # timings
-        self._enabletimetracker= False
+        self._enabletimetracker = False
         self.enabletimetracker = enable
 
         self.timetracker = time.time()
@@ -54,7 +53,7 @@ class TrackTimings(object):
         self.asynctime = defaultdict(lambda: 0.0)
 
         self.port = port if port is not None else -1
-        # name is used as hash so it's possible to add to an existing timer by same name
+        # name is used as hash, so it's possible to add to an existing timer by same name
         self.timingshash = {}
 
     def resettimer(self):
@@ -189,38 +188,31 @@ class TrackTimings(object):
 
         total_asynctime = 0.0
         if withrealtime:
-            self.timings_logger.info('port: %u, id: %s, real: %.6f' % (self.port, suspectid, self.realtime))
+            self.timings_logger.info(f'port: {self.port}, id: {suspectid}, real: {self.realtime:.6f}')
             for k, val in self.asynctime.items():
                 self.timings_logger.info(f'port: {self.port}, id: {suspectid}, async{"("+k+")" if k else ""}: {val:.6f}')
                 total_asynctime += val
-            self.timings_logger.info('port: %u, id: %s, total(async): %.3f' % (self.port, suspectid,total_asynctime))
+            self.timings_logger.info(f'port: {self.port}, id: {suspectid}, total(async): {total_asynctime:.3f}')
 
-        self.timings_logger.info('port: %u, id: %s, total: %.6f' % (self.port, suspectid, self.sumtime() - total_asynctime))
-        self.timings_logger.info('port: %u, id: %s, overhead: %.3f' % (self.port, suspectid,
-                                                                       self.sumtime(plugins=False,
-                                                                                    prependers=False,
-                                                                                    appenders=False,
-                                                                                    mplugins=False)))
+        self.timings_logger.info(f'port: {self.port}, id: {suspectid}, total: {self.sumtime()-total_asynctime:.6f}')
+        sumtime = self.sumtime(plugins=False, prependers=False, appenders=False, mplugins=False)
+        self.timings_logger.info(f'port: {self.port}, id: {suspectid}, overhead: {sumtime:.3f}')
+        
         all_milterplugs = self.gettime(mplugins=True)
         for mplugintime in all_milterplugs:
-            self.timings_logger.info('port: %u, id: %s, (MPL) %s: %.3f' % (self.port, suspectid, mplugintime[0],
-                                                                           mplugintime[1]))
+            self.timings_logger.info(f'port: {self.port}, id: {suspectid}, (MPL) {mplugintime[0]}: {mplugintime[1]:.3f}')
         all_prependertimes = self.gettime(prependers=True)
         for prependertime in all_prependertimes:
-            self.timings_logger.info('port: %u, id: %s, (PRE) %s: %.3f' % (self.port, suspectid, prependertime[0],
-                                                                           prependertime[1]))
+            self.timings_logger.info(f'port: {self.port}, id: {suspectid}, (PRE) {prependertime[0]}: {prependertime[1]:.3f}')
         all_plugintimes = self.gettime(plugins=True)
         for plugintime in all_plugintimes:
-            self.timings_logger.info('port: %u, id: %s, (PLG) %s: %.3f' % (self.port, suspectid, plugintime[0],
-                                                                           plugintime[1]))
+            self.timings_logger.info(f'port: {self.port}, id: {suspectid}, (PLG) {plugintime[0]}: {plugintime[1]:.3f}')
         all_appendertimes = self.gettime(appenders=True)
         for appendertime in all_appendertimes:
-            self.timings_logger.info('port: %u, id: %s, (APP) %s: %.3f' % (self.port, suspectid, appendertime[0],
-                                                                           appendertime[1]))
+            self.timings_logger.info(f'port: {self.port}, id: {suspectid}, (APP) {appendertime[0]}: {appendertime[1]:.3f}')
         all_overheadtimes = self.gettime(plugins=False, prependers=False, appenders=False)
         for overheadtime in all_overheadtimes:
-            self.timings_logger.debug('port: %u, id: %s, %s: %.3f' % (self.port, suspectid, overheadtime[0],
-                                                                      overheadtime[1]))
+            self.timings_logger.debug(f'port: {self.port}, id: {suspectid}, {overheadtime[0]}: {overheadtime[1]:.3f}')
 
     def _report_plugintime(self, suspectid, pluginname, end=True):
         """
@@ -237,8 +229,7 @@ class TrackTimings(object):
             self.tracktime('end')
         all_timings = self.gettime()
         for itime in all_timings:
-            self.timings_logger.debug('port: %u, id: %s, [%s] %s: %.3f' %
-                                      (self.port, suspectid, pluginname, itime[0], itime[1]))
+            self.timings_logger.debug(f'port: {self.port}, id: {suspectid}, [{pluginname}] {itime[0]}: {itime[1]:.3f}')
 
 
 class SessionHandler(TrackTimings):
@@ -246,7 +237,7 @@ class SessionHandler(TrackTimings):
     """thread handling one message"""
 
     def __init__(self, protohandler, config, prependers, plugins, appenders, port, milterplugins):
-        TrackTimings.__init__(self, port=port)
+        super().__init__(port=port)
         self.logger = logging.getLogger("fuglu.SessionHandler")
         self.prependers = prependers
         self.plugins = plugins
@@ -263,7 +254,7 @@ class SessionHandler(TrackTimings):
         self.action = None
         self.message = None
         self.enabletimetracker = config.getboolean('main', 'scantimelogger', fallback=False)
-        self.logger.debug("enabletimetracker = %s" % self.enabletimetracker)
+        self.logger.debug(f"enabletimetracker={self.enabletimetracker}")
 
     def resetvars(self):
         """Reset and initialise variables that are valid per message"""
@@ -276,7 +267,7 @@ class SessionHandler(TrackTimings):
     def set_workerstate(self, status):
         if self.worker is not None:
             self.worker.workerstate = status
-            
+
     def _gen_retmessage(self, suspect, msg):
         if self.config.getboolean('main', 'suspectidreturn') and suspect.id not in msg:
             msg = f'{msg} ({suspect.id})'
@@ -290,7 +281,7 @@ class SessionHandler(TrackTimings):
         keep_connection = True
         isuspect = 0  # in case connection is reused (milter) keep track of suspect for workerstate
         message_prefix = 'undef'
-        
+
         while keep_connection:
             suspect = None
             try:
@@ -298,14 +289,14 @@ class SessionHandler(TrackTimings):
                 # reset variables to make sure per-message variables are re-initialised in case of handling
                 # multiple messages in one session
                 self.resetvars()
-                
-                message_prefix = "(#%u)" % isuspect  # identifier (receiving multiple suspects in one session)
 
-                self.set_workerstate(message_prefix+'receiving message')
+                message_prefix = f"(#{isuspect})"  # identifier (receiving multiple suspects in one session)
+
+                self.set_workerstate(f'{message_prefix}receiving message')
                 self.tracktime("SessionHandler-Setup")
                 suspect = self.protohandler.get_suspect(milterplugins=self.milterplugins)
                 if suspect is None:
-                    self.logger.debug(message_prefix+u'No Suspect retrieved, ending session')
+                    self.logger.debug(f'{message_prefix}No Suspect retrieved, ending session')
                     try:
                         self.protohandler.endsession()
                         if isuspect > 2:
@@ -315,7 +306,7 @@ class SessionHandler(TrackTimings):
                         pass
                     return
                 elif isinstance(suspect, HealthCheckSuspect):
-                    self.logger.debug(message_prefix+'Health Check Suspect retrieved')
+                    self.logger.debug(f'{message_prefix}Health Check Suspect retrieved')
                     self.protohandler.healthcheck_reply()
                     self.resetvars()
                     return
@@ -324,11 +315,10 @@ class SessionHandler(TrackTimings):
                 self.stats.increase_counter_values(StatDelta(in_=1))
 
                 if len(suspect.recipients) != 1:
-                    self.logger.warning(message_prefix+'Notice: Message from %s has %s recipients. Plugins supporting only one recipient will see: %s' % (
-                        suspect.from_address, len(suspect.recipients), suspect.to_address))
-                self.logger.debug(message_prefix+"Message from %s to %s: %s bytes stored to %s" % (
-                    suspect.from_address, suspect.to_address, suspect.size, suspect.tempfile))
-                self.set_workerstate(message_prefix+"Handling message %s" % suspect)
+                    self.logger.warning(
+                        f'{message_prefix}Notice: Message from {suspect.from_address} has {len(suspect.recipients)} recipients. Plugins supporting only one recipient will see: {suspect.to_address}')
+                self.logger.debug(f'{message_prefix}Message from {suspect.from_address} to {suspect.to_address}: {suspect.size} bytes stored to {suspect.tempfile}')
+                self.set_workerstate(f'{message_prefix}Handling message {suspect}')
                 # store incoming port to tag, could be used to disable plugins
                 # based on port
                 try:
@@ -336,7 +326,7 @@ class SessionHandler(TrackTimings):
                     if port is not None:
                         suspect.tags['incomingport'] = port
                 except Exception as e:
-                    self.logger.warning(message_prefix+'Could not get incoming port: %s' % str(e))
+                    self.logger.warning(f'{message_prefix}Could not get incoming port: {e.__class__.__name__}: {str(e)}')
 
                 pluglist, applist = self.run_prependers(suspect)
 
@@ -346,9 +336,9 @@ class SessionHandler(TrackTimings):
                 # Set fuglu spam status if wanted
                 if self.config.getboolean('main', 'spamstatusheader'):
                     if suspect.is_spam():
-                        suspect.add_header("%sSpamstatus" % prependheader, 'YES')
+                        suspect.add_header(f'{prependheader}Spamstatus', 'YES')
                     else:
-                        suspect.add_header("%sSpamstatus" % prependheader, 'NO')
+                        suspect.add_header(f'{prependheader}Spamstatus', 'NO')
 
                 # how long did it all take?
                 difftime = time.time() - starttime
@@ -357,14 +347,14 @@ class SessionHandler(TrackTimings):
                 # Debug info to mail
                 if self.config.getboolean('main', 'debuginfoheader'):
                     debuginfo = str(suspect)
-                    suspect.add_header("%sDebuginfo" % prependheader, debuginfo)
+                    suspect.add_header(f"{prependheader}Debuginfo", debuginfo)
 
                 # add suspect id for tracking
                 if self.config.getboolean('main', 'suspectidheader'):
-                    suspect.add_header('%sSuspect' % prependheader, suspect.id)
+                    suspect.add_header(f'{prependheader}Suspect', suspect.id)
                 self.tracktime("Adding-Headers")
 
-                # checks done.. print out suspect status
+                # checks done... print out suspect status
                 logformat = self.config.get('main', 'logtemplate')
                 if logformat.strip() != '':
                     self.logger.info(suspect.log_format(logformat))
@@ -374,7 +364,7 @@ class SessionHandler(TrackTimings):
                 # check if one of the plugins made a decision
                 result = self.action
 
-                self.set_workerstate(message_prefix+"Finishing message %s" % suspect)
+                self.set_workerstate(f"{message_prefix}Finishing message {suspect.id}")
 
                 message_is_deferred = False
                 if result == ACCEPT or result == DUNNO:
@@ -389,7 +379,7 @@ class SessionHandler(TrackTimings):
                     except Exception as e:
                         message_is_deferred = True
                         trb = traceback.format_exc()
-                        self.logger.error("Could not commit message. Error: %s" % trb)
+                        self.logger.error(f'{suspect.id} Could not commit message. Error: {trb}')
                         self.logger.exception(e)
                         self._defer()
 
@@ -424,35 +414,37 @@ class SessionHandler(TrackTimings):
                 if suspect.tempfilename() != "(buffer-only)":
                     try:
                         os.remove(suspect.tempfile)
-                        self.logger.debug(message_prefix+'Removed tempfile %s' % (suspect.tempfile if suspect.tempfile else "(not available)"))
+                        self.logger.debug(f'{message_prefix}Removed tempfile {suspect.tempfile if suspect.tempfile else "(not available)"}')
                         suspect.tempfile = None
                         self.tracktime("Remove-tempfile")
                     except OSError:
-                        self.logger.warning(message_prefix+'Could not remove tempfile %s' % suspect.tempfile)
+                        self.logger.warning(f'{message_prefix}Could not remove tempfile {suspect.tempfile}')
             except KeyboardInterrupt:
                 sys.exit(0)
             except ValueError as e:
                 # Error in envelope send/receive address
                 try:
-                    self.logger.warning(message_prefix+"Invalid send/receive address -> %s" % force_uString(e))
+                    self.logger.warning(f'{message_prefix}Invalid send/receive address -> {str(e)}')
                 except Exception:
                     pass
 
-                address_compliance_fail_action = self.config.get('main','address_compliance_fail_action', fallback='defer').lower()
-                message = self.config.get('main','address_compliance_fail_message', fallback='invalid sender or recipient address')
+                address_compliance_fail_action = self.config.get('main', 'address_compliance_fail_action', fallback='defer').lower()
+                message = self.config.get('main', 'address_compliance_fail_message', fallback='invalid sender or recipient address')
 
-                if address_compliance_fail_action   == "defer":
+                if address_compliance_fail_action == "defer":
                     self._defer(message)
                 elif address_compliance_fail_action == "reject":
                     self._reject(message)
                 elif address_compliance_fail_action == "discard":
                     self._discard(message)
+                elif address_compliance_fail_action == "dunno":
+                    self.logger.warning(f'{message_prefix} accepting invalid sender or recipient')
                 else:
                     self._defer(message)
 
             except Exception as e:
                 exc = traceback.format_exc()
-                self.logger.error('Exception %s: %s' % (e, exc))
+                self.logger.error(f'{suspect.id if suspect else "<>"} Exception {e}: {exc}')
                 self._defer()
                 # don't continue, might end in an infinite loop
                 keep_connection = False
@@ -470,28 +462,28 @@ class SessionHandler(TrackTimings):
 
                     if remove_tmpfiles_on_error:
                         if tmpfilename:
-                            self.logger.debug(message_prefix+'Remove tmpfile: %s for failed message' % tmpfilename)
+                            self.logger.debug(f'{message_prefix}Remove tmpfile: {tmpfilename} for failed message')
                         self.protohandler.remove_tmpfile()
                     else:
                         if tmpfilename:
-                            self.logger.warning(message_prefix+'Keep tmpfile: %s for failed message' % tmpfilename)
+                            self.logger.warning(f'{message_prefix}Keep tmpfile: {tmpfilename} for failed message')
                         else:
-                            self.logger.warning(message_prefix+'No tmpfile to keep for failed message')
+                            self.logger.warning(f'{message_prefix}No tmpfile to keep for failed message')
 
                 elif suspect.tempfilename() != "(buffer-only)":
                     # suspect was created but not stopped cleanly
                     if remove_tmpfiles_on_error:
                         try:
                             os.remove(suspect.tempfilename())
-                            self.logger.debug(message_prefix+'Removed tempfile %s' % suspect.tempfilename())
+                            self.logger.debug(f'{message_prefix}Removed tempfile {suspect.tempfilename()}')
                         except OSError:
-                            self.logger.warning(message_prefix+'Could not remove tempfile %s' % suspect.tempfilename())
+                            self.logger.warning(f'{message_prefix}Could not remove tempfile {suspect.tempfilename()}')
                     else:
-                        self.logger.warning(message_prefix+'Keep tempfile %s for failed message' % suspect.tempfilename())
+                        self.logger.warning(f'{message_prefix}Keep tempfile {suspect.tempfilename()} for failed message')
 
                 # try to remove the suspect
                 try:
-                    self.logger.debug(message_prefix+'Remove suspect (current refs): %u' % sys.getrefcount(suspect))
+                    self.logger.debug(f'{message_prefix}Remove suspect (current refs): {sys.getrefcount(suspect)}')
                     suspectid = suspect.id
                     del suspect
                 except Exception:
@@ -512,12 +504,11 @@ class SessionHandler(TrackTimings):
                 self.logger.debug(message_prefix+'Session finished')
 
         if isuspect > 2:
-            self.logger.info(message_prefix+'Session finished for %u sessions in same connection' % (isuspect-1))
-
+            self.logger.info(f'{message_prefix}Session finished for {isuspect-1} sessions in same connection')
 
     def _discard(self, message=None):
         if message is None:
-            message="internal problem - discard"
+            message = "internal problem - discard"
 
         # try to end the session gracefully, but this might cause the same exception again,
         # in case of a broken pipe for example
@@ -528,7 +519,7 @@ class SessionHandler(TrackTimings):
 
     def _reject(self, message=None):
         if message is None:
-            message="internal problem - reject"
+            message = "internal problem - reject"
 
         # try to end the session gracefully, but this might cause the same exception again,
         # in case of a broken pipe for example
@@ -539,7 +530,7 @@ class SessionHandler(TrackTimings):
 
     def _defer(self, message=None):
         if message is None:
-            message="internal problem - message deferred"
+            message = "internal problem - message deferred"
 
         # try to end the session gracefully, but this might cause the same exception again,
         # in case of a broken pipe for example
@@ -548,7 +539,6 @@ class SessionHandler(TrackTimings):
         except Exception:
             pass
 
-
     def trash(self, suspect, killerplugin=None):
         """copy suspect to trash if this is enabled"""
         trashdir = self.config.get('main', 'trashdir').strip()
@@ -563,10 +553,10 @@ class SessionHandler(TrackTimings):
                 return
             self.logger.info(f'{suspect.id }Created trashdir {trashdir}')
 
+        trashdir = self.config.get('main', 'trashdir')
         trashfilename = ''
         try:
-            handle, trashfilename = tempfile.mkstemp(
-                prefix=suspect.id, dir=self.config.get('main', 'trashdir'))
+            handle, trashfilename = tempfile.mkstemp(prefix=suspect.id, dir=trashdir)
             with os.fdopen(handle, 'w+b') as trashfile:
                 trashfile.write(suspect.get_source())
             self.logger.debug(f'{suspect.id} Message stored to trash: {trashfilename}')
@@ -574,26 +564,26 @@ class SessionHandler(TrackTimings):
             self.logger.error(f"{suspect.id} could not create file {trashfilename}: {e.__class__.__name__}: {str(e)}")
 
         # TODO: document main.trashlog
-        if self.config.has_option('main', 'trashlog') and self.config.getboolean('main', 'trashlog'):
+        if self.config.getboolean('main', 'trashlog', fallback=False):
             try:
-                with open('%s/00-fuglutrash.log' % self.config.get('main', 'trashdir'), 'a') as handle:
+                trashlogpath = os.path.join(trashdir, '00-fuglutrash.log')
+                with open(trashlogpath, 'a') as handle:
                     # <date> <time> <from address> <to address> <plugin that said "DELETE"> <filename>
                     now = utcnow().isoformat()
-                    handle.write("%s %s %s %s %s" % (
-                        now, suspect.from_address, suspect.to_address, killerplugin, trashfilename))
+                    handle.write(f'{now} {suspect.from_address} {suspect.to_address} {killerplugin} {trashfilename}')
                     handle.write("\n")
             except Exception as e:
                 self.logger.error(f'{suspect.id} Could not update trash log: {e.__class__.__name__}: {str(e)}')
 
     def run_plugins(self, suspect, pluglist):
         """Run scannerplugins on suspect"""
-        suspect.debug('Will run plugins: %s' % pluglist)
+        suspect.debug(f'{suspect.id} Will run plugins: {", ".join([p.__class__.__name__ for p in pluglist])}')
         self.tracktime("Before-Plugins")
         for plugin in pluglist:
             try:
                 self.logger.debug(f'{suspect.id} Running plugin {str(plugin)}')
-                self.set_workerstate("%s : Running Plugin %s" % (suspect, plugin))
-                suspect.debug('Running plugin %s' % str(plugin))
+                self.set_workerstate(f"{suspect.id} Running Plugin {str(plugin)}")
+                suspect.debug(f'Running plugin {str(plugin)}')
                 starttime = time.time()
                 ans = plugin.run_examine(suspect)
                 plugintime = time.time() - starttime
@@ -642,9 +632,9 @@ class SessionHandler(TrackTimings):
                 CrashStore.store_exception()
                 exc = traceback.format_exc()
                 self.logger.error(f'{suspect.id} Plugin {str(plugin)} failed: {str(exc)}')
-                suspect.debug('Plugin failed : %s . Please check fuglu log for more details' % e)
+                suspect.debug(f'Plugin failed : {e} . Please check fuglu log for more details')
                 ptag = suspect.get_tag("processingerrors", defaultvalue=[])
-                ptag.append("Plugin %s failed: %s" % (str(plugin), str(e)))
+                ptag.append(f"Plugin {str(plugin)} failed: {e.__class__.__name__}: {str(e)}")
                 suspect.set_tag("processingerrors", ptag)
             finally:
                 self.tracktime(str(plugin), plugin=True)
@@ -653,20 +643,20 @@ class SessionHandler(TrackTimings):
         """Run prependers on suspect"""
         plugcopy = self.plugins[:]
         appcopy = self.appenders[:]
-        
+
         self.tracktime("Before-Prependers")
         for plugin in self.prependers:
             try:
                 self.logger.debug(f'{suspect.id} Running prepender {str(plugin)}')
-                self.set_workerstate("%s : Running Prepender %s" % (suspect, plugin))
+                self.set_workerstate(f"{suspect.id} Running Prepender {str(plugin)}")
                 starttime = time.time()
-                
+
                 out_plugins = plugin.pluginlist(suspect, plugcopy)
                 out_appenders = plugin.appenderlist(suspect, appcopy)
-                
+
                 plugintime = time.time() - starttime
                 suspect.tags['scantimes'].append((plugin.section, plugintime))
-                
+
                 # Plugins
                 if out_plugins is not None:
                     plugcopyset = set(plugcopy)
@@ -678,7 +668,7 @@ class SessionHandler(TrackTimings):
                     if len(added) > 0:
                         self.logger.debug(f'{suspect.id} Prepender {str(plugin)} added plugins: {", ".join(list(map(str, added)))}')
                     plugcopy = out_plugins
-                    
+
                 # Appenders
                 if out_appenders is not None:
                     appcopyset = set(appcopy)
@@ -696,10 +686,13 @@ class SessionHandler(TrackTimings):
                 exc = traceback.format_exc()
                 self.logger.error(f'{suspect.id} Prepender plugin {str(plugin)} failed: {str(exc)}')
                 ptag = suspect.get_tag("processingerrors", defaultvalue=[])
-                ptag.append("Prepender %s failed: %s" % (str(plugin), str(e)))
+                ptag.append(f"Prepender {str(plugin)} failed: {e.__class__.__name__}: {str(e)}")
                 suspect.set_tag("processingerrors", ptag)
             finally:
                 self.tracktime(str(plugin), prepender=True)
+                if suspect.get_tag('skip_prependers'):
+                    self.logger.debug(f'{suspect.id} Prepender {str(plugin)} skips all remaining prependers')
+                    break
         return plugcopy, appcopy
 
     def run_appenders(self, suspect, finaldecision, applist):
@@ -711,8 +704,8 @@ class SessionHandler(TrackTimings):
         for plugin in applist:
             try:
                 self.logger.debug(f'{suspect.id} Running appender {str(plugin)}')
-                suspect.debug('Running appender %s' % plugin)
-                self.set_workerstate("%s : Running appender %s" % (suspect, plugin))
+                suspect.debug(f'Running appender {str(plugin)}')
+                self.set_workerstate(f"{suspect.id} Running Appender {str(plugin)}")
                 starttime = time.time()
                 plugin.process(suspect, finaldecision)
                 plugintime = time.time() - starttime
@@ -722,7 +715,7 @@ class SessionHandler(TrackTimings):
                 exc = traceback.format_exc()
                 self.logger.error(f'{suspect.id} Appender plugin {str(plugin)} failed: {str(exc)}')
                 ptag = suspect.get_tag("processingerrors", defaultvalue=[])
-                ptag.append("Appender %s failed: %s" % (str(plugin), str(e)))
+                ptag.append(f"Appender {str(plugin)} failed: {e.__class__.__name__}: {str(e)}")
                 suspect.set_tag("processingerrors", ptag)
             finally:
                 self.tracktime(str(plugin), appender=True)
