@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#   Copyright 2009-2022 Fumail Project
+#   Copyright Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,10 +16,11 @@
 #
 #
 import logging
+from encodings.aliases import aliases
 
 try:
     import chardet
-    
+
     CHARDET_AVAILABLE = True
 except ImportError:
     CHARDET_AVAILABLE = False
@@ -41,12 +42,12 @@ def try_encoding(u_inputstring, encoding="utf-8"):
     """
     if u_inputstring is None:
         return None
-    
+
     # make sure encoding is not None or empty
     if not encoding:
         encoding = "utf-8"
-    
-    logger = logging.getLogger("%s.stringencode.try_encoding" % __package__)
+
+    logger = logging.getLogger(f"{__package__ or 'fuglu'}.stringencode.try_encoding")
     try:
         return u_inputstring.encode(encoding, "strict")
     except UnicodeEncodeError as e:
@@ -74,7 +75,7 @@ def try_decoding(b_inputstring, encodingGuess="utf-8", errors="strict"):
     """
     if b_inputstring is None:
         return None
-    
+
     # make sure encoding is not None or empty
     if not encodingGuess:
         encodingGuess = "utf-8"
@@ -85,7 +86,7 @@ def try_decoding(b_inputstring, encodingGuess="utf-8", errors="strict"):
         firsterrors = errors
         seconderrors = None
 
-    logger = logging.getLogger("%s.stringencode.try_decoding" % __package__)
+    logger = logging.getLogger(f"{__package__ or 'fuglu'}.stringencode.try_decoding")
     u_outputstring = None
     try:
         u_outputstring = b_inputstring.decode(encodingGuess, errors=firsterrors)
@@ -97,33 +98,32 @@ def try_decoding(b_inputstring, encodingGuess="utf-8", errors="strict"):
                 e = f
         if not u_outputstring:
             # if we get here we will also print either the chardet or trial&error decoding message anyway
-            logger.debug("found non %s encoding or encoding not found (msg: %s), try to detect encoding"
-                         % (str(e), encodingGuess))
-    
+            logger.debug(f"found non {str(e)} encoding or encoding not found (msg: {encodingGuess}), try to detect encoding")
+
     if u_outputstring is None:
         if CHARDET_AVAILABLE:
             # limit to analyse max 10'000 characters because it might become very expensive
             encoding = chardet.detect(b_inputstring[:10000])['encoding']
-            logger.info("chardet -> encoding estimated as %s" % encoding)
+            logger.info(f"chardet -> encoding estimated as {encoding}")
             try:
                 u_outputstring = b_inputstring.decode(encoding, errors=firsterrors)
             except (UnicodeDecodeError, LookupError):
-                logger.info("encoding found by chardet (%s) does not work" % encoding)
+                logger.info(f"encoding found by chardet ({encoding}) does not work")
         else:
             logger.debug("module chardet not available -> skip autodetect")
-    
+
     if u_outputstring is None:
-        trialerrorencoding = EncodingTrialError.test_all(b_inputstring, returnimmediately=True)
-        logger.info("trial&error -> encoding estimated as one of (selecting first) %s" % trialerrorencoding)
+        trialerrorencoding = EncodingTrialError.test_all(b_inputstring)
+        logger.info(f"trial&error -> encoding estimated as one of (selecting first) {trialerrorencoding}")
         if trialerrorencoding:
             try:
                 u_outputstring = b_inputstring.decode(trialerrorencoding[0], errors=firsterrors)
             except (UnicodeDecodeError, LookupError):
-                logger.info("encoding found by trial & error (%s) does not work" % trialerrorencoding)
-    
+                logger.info(f"encoding found by trial & error ({trialerrorencoding}) does not work")
+
     if u_outputstring is None:
         raise UnicodeDecodeError
-    
+
     return u_outputstring
 
 
@@ -134,7 +134,7 @@ def force_uString(inputstring, encodingGuess="utf-8", errors="strict", convert_n
         inputstring (str, unicode, list): input string or list of strings to be checked
         encodingGuess (str): guess for encoding used, default assume unicode
         errors (str): error handling as in standard bytes.decode -> strict, ignore, replace
-                        or strict+ignore, strict+replace (try strict first, on certain errors try ignore/replace)
+                        or strict+ignore, strict+replace (try strict first, on certain errors try to ignore/replace)
         convert_none (bool): convert None to empty string if True
 
     Raises:
@@ -151,7 +151,7 @@ def force_uString(inputstring, encodingGuess="utf-8", errors="strict", convert_n
         return "" if convert_none else None
     elif isinstance(inputstring, list):
         return [force_uString(item, encodingGuess=encodingGuess, errors=errors) for item in inputstring]
-    
+
     try:
         if isinstance(inputstring, str):
             return inputstring
@@ -160,33 +160,30 @@ def force_uString(inputstring, encodingGuess="utf-8", errors="strict", convert_n
     except (AttributeError, TypeError) as e:
         # Input might not be bytes but a number which is then
         # expected to be converted to unicode
-        
+
         logger = logging.getLogger("fuglu.force_uString")
         if isinstance(inputstring, (int, float)):
             pass
         elif not isinstance(inputstring, (str, bytes)):
-            logger.debug("object is not string/bytes/int/float but %s" % str(type(inputstring)))
+            logger.debug(f"object is not string/bytes/int/float but {str(type(inputstring))}")
         else:
-            logger.debug("decoding failed using guess %s for object of type %s with message %s"
-                         % (encodingGuess, str(type(inputstring)), str(e)))
-        
+            logger.debug(f"decoding failed using guess {encodingGuess} for object of type {str(type(inputstring))} with message {str(e)}")
+
         try:
             return str(inputstring)
         except (NameError, ValueError, TypeError, UnicodeEncodeError, UnicodeDecodeError) as e:
-            logger.debug("Could not convert using 'str' -> error %s" % str(e))
+            logger.debug(f"Could not convert using 'str' -> error {e.__class__.__name__}: {str(e)}")
             pass
         except Exception as e:
-            logger.debug("Could not convert using 'str' -> error %s" % str(e))
+            logger.debug(f"Could not convert using 'str' -> error {e.__class__.__name__}: {str(e)}")
             pass
-        
+
         try:
             representation = str(repr(inputstring))
         except Exception as e:
-            representation = "(%s)" % str(e)
-        
-        errormsg = "Could not transform input object of type %s with repr: %s" % \
-                   (str(type(inputstring)), representation)
-        
+            representation = f"({e.__class__.__name__}: {str(e)})"
+
+        errormsg = f"Could not transform input object of type {str(type(inputstring))} with repr: {representation}"
         logger.error(errormsg)
         raise ForceUStringError(errormsg)
 
@@ -206,7 +203,7 @@ def force_bString(inputstring, encoding="utf-8", checkEncoding=False):
         return None
     elif isinstance(inputstring, list):
         return [force_bString(item) for item in inputstring]
-    
+
     try:
         if isinstance(inputstring, bytes):
             # string is already a byte string
@@ -220,7 +217,7 @@ def force_bString(inputstring, encoding="utf-8", checkEncoding=False):
         # just try to first create a string and then encode it
         inputstring = force_uString(inputstring)
         b_outString = try_encoding(inputstring, encoding)
-    
+
     if checkEncoding:
         # re-encode to make sure it matches input encoding
         return try_encoding(try_decoding(b_outString, encodingGuess=encoding), encoding=encoding)
@@ -232,7 +229,7 @@ def force_bfromc(chars_iteratable):
     """Python 2 like bytes from char for Python 3
 
     Implemented to have the same char-byte conversion in Python 3 as in Python 2
-    for special applications. In general it is recommended to use the real
+    for special applications. In general, it is recommended to use the real
     str.encode() function for Python3
 
     Args:
@@ -254,7 +251,7 @@ def force_cfromb(bytes_iteratable):
     """Python 2 like chars from bytes for Python 3
 
     Implemented to have the same byte-char conversion in Python 3 as in Python 2
-    for special applications. In general it is recommended to use the real
+    for special applications. In general, it is recommended to use the real
     bytes.decode() function for Python3
 
     Args:
@@ -273,42 +270,36 @@ def force_cfromb(bytes_iteratable):
     elif isinstance(bytes_iteratable, list):
         return [force_cfromb(b) for b in bytes_iteratable]
     else:
-        raise AttributeError("Type: %s is not str and not bytes" % (type(bytes_iteratable)))
+        raise AttributeError(f"Type: {type(bytes_iteratable)} is not str and not bytes")
+
+
+def _gen_all_encodings():
+    """
+    generate a sorted list of all encodings supported by current python version that are suitable for mail content encoding
+    :return: list(str) list containing all encodings we want to test
+    """
+    enclist = ['utf_8', 'iso8859_15'] # encodings we want to test first
+    excludes = {'rot_13', 'base64_codec', 'hex_codec', 'quopri_codec', 'bz2_codec', 'zlib_codec', 'uu_codec'} # encodings we don't want to test
+    alias = set(aliases.values())
+    alias = alias - set(enclist) - excludes
+    alias = sorted(list(alias))
+    enclist.extend(alias)
+    return enclist
+ALL_ENCODINGS = _gen_all_encodings()
 
 
 class EncodingTrialError(object):
-    # list of Py-3.7 encodings
-    all_encodings_list = ['utf-8', 'ascii', 'big5', 'big5hkscs', 'cp037',
-                          'cp273', 'cp424', 'cp437', 'cp500',
-                          'cp720', 'cp737', 'cp775', 'cp850',
-                          'cp852', 'cp855', 'cp856', 'cp857',
-                          'cp858', 'cp860', 'cp861', 'cp862',
-                          'cp863', 'cp864', 'cp865', 'cp866',
-                          'cp869', 'cp874', 'cp875', 'cp932',
-                          'cp949', 'cp950', 'cp1006', 'cp1026',
-                          'cp1125', 'cp1140', 'cp1250', 'cp1251',
-                          'cp1252', 'cp1253', 'cp1254', 'cp1255',
-                          'cp1256', 'cp1257', 'cp1258', 'cp65001',
-                          'euc_jp', 'euc_jis_2004', 'euc_jisx0213',
-                          'euc_kr', 'gb2312', 'gbk', 'gb18030',
-                          'hz', 'iso2022_jp', 'iso2022_jp_1', 'iso2022_jp_2',
-                          'iso2022_jp_2004', 'iso2022_jp_3', 'iso2022_jp_ext', 'iso2022_kr',
-                          'latin_1', 'iso8859_2', 'iso8859_3', 'iso8859_4',
-                          'iso8859_5', 'iso8859_6', 'iso8859_7', 'iso8859_8',
-                          'iso8859_9', 'iso8859_10', 'iso8859_11', 'iso8859_13',
-                          'iso8859_14', 'iso8859_15', 'iso8859_16', 'johab',
-                          'koi8_r', 'koi8_t', 'koi8_u', 'kz1048',
-                          'mac_cyrillic', 'mac_greek', 'mac_iceland', 'mac_latin2',
-                          'mac_roman', 'mac_turkish', 'ptcp154', 'shift_jis',
-                          'shift_jis_2004', 'shift_jisx0213', 'utf_32', 'utf_32_be',
-                          'utf_32_le', 'utf_16', 'utf_16_be', 'utf_16_le',
-                          'utf_7', 'utf_8_sig']
     
+    @staticmethod
+    def get_encodings_list():
+        return ALL_ENCODINGS
+    
+
     @staticmethod
     def test_all(bytestring, returnimmediately=False):
         """
         Test all known codecs if they can be used to decode an encoded string.
-        A codec can be used if it it possible to decode the string without exception.
+        A codec can be used if it is possible to decode the string without exception.
         Then after reencoding the string it should be the same as the original string.
 
         Args:
@@ -320,20 +311,22 @@ class EncodingTrialError(object):
 
         """
         assert isinstance(bytestring, bytes)
-        
+
         positive = []
-        for enc in EncodingTrialError.all_encodings_list:
+        for enc in ALL_ENCODINGS:
             try:
                 # encode and decode
                 test_decoded = bytestring.decode(enc, "strict")
                 test_reencoded = test_decoded.encode(enc, "strict")
-                
+
                 if not (isinstance(test_decoded, str) and isinstance(test_reencoded, bytes)):
                     raise TypeError()
-                
+
                 if bytestring == test_reencoded:
                     positive.append(enc)
+                    if returnimmediately:
+                        break
             except Exception:
                 pass
-        
+
         return positive
