@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-#   Copyright 2009-2022 Oli Schacher, Fumail Project
+#   Copyright Oli Schacher, Fumail Project
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -45,7 +45,7 @@ class ThreadPool(threading.Thread):
         self.laststats = 0
         self.statinverval = 60
         self.controller = weakref.ref(controller)  # keep a weak reference to controller
-        threading.Thread.__init__(self)
+        super().__init__()
         self.name = 'Threadpool'
         self.daemon = False
         self.start()
@@ -258,7 +258,7 @@ class ThreadPool(threading.Thread):
                 mark_defer_counter += 1
                 sesshandler.protohandler.defer(return_message)
 
-            self.logger.info("Marked %s messages as '%s' to close queue" % (mark_defer_counter,return_message))
+            self.logger.info("Marked %s messages as '%s' to close queue" % (mark_defer_counter, return_message))
 
         # remove all the workers (joins them also)
         if newmanager:
@@ -272,10 +272,11 @@ class ThreadPool(threading.Thread):
             worker.stayalive = False
             worker.join(join_timeout)
 
+
 class Worker(threading.Thread):
 
     def __init__(self, workerid, pool):
-        threading.Thread.__init__(self, name='Pool worker %s' % workerid)
+        super().__init__(name=f'Pool worker {workerid}')
         self.workerid = workerid
         self.birth = time.time()
         self.pool = pool
